@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
 /* vim: set ts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
@@ -134,14 +134,9 @@ const DownloadsPanel = {
 
     window.addEventListener("unload", this.onWindowUnload, false);
 
-    // Ensure that the Download Manager service is running.  This resumes
-    // active downloads if required.  If there are downloads to be shown in the
-    // panel, starting the service will make us load their data asynchronously.
-    if (DownloadsCommon.useJSTransfer) {
-      DownloadsCommon.initializeAllDataLinks();
-    } else {
-      Services.downloads;
-    }
+    // Load and resume active downloads if required.  If there are downloads to
+    // be shown in the panel, they will be loaded asynchronously.
+    DownloadsCommon.initializeAllDataLinks();
 
     // Now that data loading has eventually started, load the required XUL
     // elements and initialize our views.
@@ -510,8 +505,7 @@ const DownloadsPanel = {
 
       let uri = NetUtil.newURI(url);
       DownloadsCommon.log("Pasted URL seems valid. Starting download.");
-      saveURL(uri.spec, name || uri.spec, null, true, true,
-              undefined, document);
+      DownloadURL(uri.spec, name, document);
     } catch (ex) {}
   },
 
@@ -646,11 +640,6 @@ const DownloadsOverlayLoader = {
       this._overlayLoading = false;
       this._loadedOverlays[aOverlay] = true;
 
-      // Loading the overlay causes all the persisted XUL attributes to be
-      // reapplied, including "iconsize" on the toolbars.  Until bug 640158 is
-      // fixed, we must recalculate the correct "iconsize" attributes manually.
-      retrieveToolbarIconsizesFromTheme();
-
       this.processPendingRequests();
     }
 
@@ -789,26 +778,6 @@ const DownloadsView = {
     DownloadsPanel.onViewLoadCompleted();
   },
 
-  /**
-   * Called when the downloads database becomes unavailable (for example,
-   * entering Private Browsing Mode).  References to existing data should be
-   * discarded.
-   */
-  onDataInvalidated: function DV_onDataInvalidated()
-  {
-    DownloadsCommon.log("Downloads data has been invalidated. Cleaning up",
-                        "DownloadsView.");
-
-    DownloadsPanel.terminate();
-
-    // Clear the list by replacing with a shallow copy.
-    let emptyView = this.richListBox.cloneNode(false);
-    this.richListBox.parentNode.replaceChild(emptyView, this.richListBox);
-    this.richListBox = emptyView;
-    this._viewItems = {};
-    this._dataItems = [];
-  },
-
   /**
    * Called when a new download data item is available, either during the
    * asynchronous data load or when a new download is started.
@@ -988,8 +957,7 @@ const DownloadsView = {
       return;
     }
 
-    if (aEvent.keyCode == KeyEvent.DOM_VK_ENTER ||
-        aEvent.keyCode == KeyEvent.DOM_VK_RETURN) {
+    if (aEvent.keyCode == KeyEvent.DOM_VK_RETURN) {
       goDoCommand("downloadsCmd_doDefault");
     }
   },
@@ -1486,7 +1454,8 @@ DownloadsViewItemController.prototype = {
 
     downloadsCmd_open: function DVIC_downloadsCmd_open()
     {
-      this.dataItem.openLocalFile(window);
+      this.dataItem.openLocalFile();
+
       // We explicitly close the panel here to give the user the feedback that
       // their click has been received, and we're handling the action.
       // Otherwise, we'd have to wait for the file-type handler to execute
@@ -1678,7 +1647,6 @@ const DownloadsSummary = {
   onKeyDown: function DS_onKeyDown(aEvent)
   {
     if (aEvent.charCode == " ".charCodeAt(0) ||
-        aEvent.keyCode == KeyEvent.DOM_VK_ENTER ||
         aEvent.keyCode == KeyEvent.DOM_VK_RETURN) {
       DownloadsPanel.showDownloadsHistory();
     }
