@@ -1,8 +1,9 @@
-/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim: set ts=8 sts=4 sw=4 tw=99 et: */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+
 #ifndef nsFrameMessageManager_h__
 #define nsFrameMessageManager_h__
 
@@ -25,13 +26,14 @@
 #include "nsWeakPtr.h"
 #include "mozilla/Attributes.h"
 #include "js/RootingAPI.h"
+#include "nsTObserverArray.h"
+#include "mozilla/dom/StructuredCloneUtils.h"
 
 namespace mozilla {
 namespace dom {
 
-class ContentParent;
-class ContentChild;
-struct StructuredCloneData;
+class nsIContentParent;
+class nsIContentChild;
 class ClonedMessageData;
 class MessageManagerReporter;
 
@@ -51,15 +53,16 @@ class MessageManagerCallback
 public:
   virtual ~MessageManagerCallback() {}
 
-  virtual bool DoLoadFrameScript(const nsAString& aURL)
+  virtual bool DoLoadFrameScript(const nsAString& aURL, bool aRunInGlobalScope)
   {
     return true;
   }
 
   virtual bool DoSendBlockingMessage(JSContext* aCx,
                                      const nsAString& aMessage,
-                                     const mozilla::dom::StructuredCloneData& aData,
-                                     JS::Handle<JSObject *> aCpows,
+                                     const StructuredCloneData& aData,
+                                     JS::Handle<JSObject*> aCpows,
+                                     nsIPrincipal* aPrincipal,
                                      InfallibleTArray<nsString>* aJSONRetVal,
                                      bool aIsSync)
   {
@@ -68,8 +71,9 @@ public:
 
   virtual bool DoSendAsyncMessage(JSContext* aCx,
                                   const nsAString& aMessage,
-                                  const mozilla::dom::StructuredCloneData& aData,
-                                  JS::Handle<JSObject *> aCpows)
+                                  const StructuredCloneData& aData,
+                                  JS::Handle<JSObject*> aCpows,
+                                  nsIPrincipal* aPrincipal)
   {
     return true;
   }
@@ -95,10 +99,10 @@ public:
   }
 
 protected:
-  bool BuildClonedMessageDataForParent(ContentParent* aParent,
+  bool BuildClonedMessageDataForParent(nsIContentParent* aParent,
                                        const StructuredCloneData& aData,
                                        ClonedMessageData& aClonedData);
-  bool BuildClonedMessageDataForChild(ContentChild* aChild,
+  bool BuildClonedMessageDataForChild(nsIContentChild* aChild,
                                       const StructuredCloneData& aData,
                                       ClonedMessageData& aClonedData);
 };
@@ -114,6 +118,11 @@ class nsAXPCNativeCallContext;
 
 struct nsMessageListenerInfo
 {
+  bool operator==(const nsMessageListenerInfo& aOther) const
+  {
+    return &aOther == this;
+  }
+
   // Exactly one of mStrongListener and mWeakListener must be non-null.
   nsCOMPtr<nsIMessageListener> mStrongListener;
   nsWeakPtr mWeakListener;
@@ -121,22 +130,22 @@ struct nsMessageListenerInfo
 
 class CpowHolder
 {
-  public:
-    virtual bool ToObject(JSContext* cx, JS::MutableHandleObject objp) = 0;
+public:
+  virtual bool ToObject(JSContext* cx, JS::MutableHandle<JSObject*> objp) = 0;
 };
 
 class MOZ_STACK_CLASS SameProcessCpowHolder : public CpowHolder
 {
-  public:
-    SameProcessCpowHolder(JSRuntime *aRuntime, JS::Handle<JSObject *> aObj)
-      : mObj(aRuntime, aObj)
-    {
-    }
+public:
+  SameProcessCpowHolder(JSRuntime *aRuntime, JS::Handle<JSObject*> aObj)
+    : mObj(aRuntime, aObj)
+  {
+  }
 
-    bool ToObject(JSContext* aCx, JS::MutableHandleObject aObjp);
+  bool ToObject(JSContext* aCx, JS::MutableHandle<JSObject*> aObjp);
 
-  private:
-    JS::Rooted<JSObject*> mObj;
+private:
+  JS::Rooted<JSObject*> mObj;
 };
 
 class nsFrameMessageManager MOZ_FINAL : public nsIContentFrameMessageManager,
@@ -164,10 +173,10 @@ public:
     NS_ASSERTION(!mIsBroadcaster || !mCallback,
                  "Broadcasters cannot have callbacks!");
     // This is a bit hackish. When parent manager is global, we want
-    // to attach the window message manager to it immediately.
+    // to attach the message manager to it immediately.
     // Is it just the frame message manager which waits until the
     // content process is running.
-    if (mParentManager && (mCallback || IsWindowLevel())) {
+    if (mParentManager && (mCallback || IsBroadcaster())) {
       mParentManager->AddChildManager(this);
     }
     if (mOwnsCallback) {
@@ -175,6 +184,7 @@ public:
     }
   }
 
+private:
   ~nsFrameMessageManager()
   {
     for (int32_t i = mChildManagers.Count(); i > 0; --i) {
@@ -196,6 +206,7 @@ public:
     }
   }
 
+public:
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
   NS_DECL_CYCLE_COLLECTION_CLASS_AMBIGUOUS(nsFrameMessageManager,
                                            nsIContentFrameMessageManager)
@@ -208,23 +219,22 @@ public:
   NS_DECL_NSIPROCESSCHECKER
 
   static nsFrameMessageManager*
-  NewProcessMessageManager(mozilla::dom::ContentParent* aProcess);
+  NewProcessMessageManager(mozilla::dom::nsIContentParent* aProcess);
 
   nsresult ReceiveMessage(nsISupports* aTarget, const nsAString& aMessage,
                           bool aIsSync, const StructuredCloneData* aCloneData,
-                          CpowHolder* aCpows,
+                          CpowHolder* aCpows, nsIPrincipal* aPrincipal,
                           InfallibleTArray<nsString>* aJSONRetVal);
 
-  void AddChildManager(nsFrameMessageManager* aManager,
-                       bool aLoadScripts = true);
+  void AddChildManager(nsFrameMessageManager* aManager);
   void RemoveChildManager(nsFrameMessageManager* aManager)
   {
     mChildManagers.RemoveObject(aManager);
   }
   void Disconnect(bool aRemoveFromParent = true);
 
-  void SetCallback(mozilla::dom::ipc::MessageManagerCallback* aCallback,
-                   bool aLoadScripts = true);
+  void InitWithCallback(mozilla::dom::ipc::MessageManagerCallback* aCallback);
+  void SetCallback(mozilla::dom::ipc::MessageManagerCallback* aCallback);
   mozilla::dom::ipc::MessageManagerCallback* GetCallback()
   {
     return mCallback;
@@ -233,12 +243,14 @@ public:
   nsresult DispatchAsyncMessage(const nsAString& aMessageName,
                                 const JS::Value& aJSON,
                                 const JS::Value& aObjects,
+                                nsIPrincipal* aPrincipal,
                                 JSContext* aCx,
                                 uint8_t aArgc);
   nsresult DispatchAsyncMessageInternal(JSContext* aCx,
                                         const nsAString& aMessage,
                                         const StructuredCloneData& aData,
-                                        JS::Handle<JSObject *> aCpows);
+                                        JS::Handle<JSObject*> aCpows,
+                                        nsIPrincipal* aPrincipal);
   void RemoveFromParent();
   nsFrameMessageManager* GetParentManager() { return mParentManager; }
   void SetParentManager(nsFrameMessageManager* aParent)
@@ -248,7 +260,7 @@ public:
     mParentManager = aParent;
   }
   bool IsGlobal() { return mGlobal; }
-  bool IsWindowLevel() { return mParentManager && mParentManager->IsGlobal(); }
+  bool IsBroadcaster() { return mIsBroadcaster; }
 
   static nsFrameMessageManager* GetParentProcessManager()
   {
@@ -260,17 +272,19 @@ public:
   }
 private:
   nsresult SendMessage(const nsAString& aMessageName,
-                       const JS::Value& aJSON,
-                       const JS::Value& aObjects,
+                       JS::Handle<JS::Value> aJSON,
+                       JS::Handle<JS::Value> aObjects,
+                       nsIPrincipal* aPrincipal,
                        JSContext* aCx,
                        uint8_t aArgc,
-                       JS::Value* aRetval,
+                       JS::MutableHandle<JS::Value> aRetval,
                        bool aIsSync);
 protected:
   friend class MMListenerRemover;
   // We keep the message listeners as arrays in a hastable indexed by the
   // message name. That gives us fast lookups in ReceiveMessage().
-  nsClassHashtable<nsStringHashKey, nsTArray<nsMessageListenerInfo>> mListeners;
+  nsClassHashtable<nsStringHashKey,
+                   nsAutoTObserverArray<nsMessageListenerInfo, 1>> mListeners;
   nsCOMArray<nsIContentFrameMessageManager> mChildManagers;
   bool mChrome;     // true if we're in the chrome process
   bool mGlobal;     // true if we're the global frame message manager
@@ -283,6 +297,10 @@ protected:
   nsAutoPtr<mozilla::dom::ipc::MessageManagerCallback> mOwnedCallback;
   nsFrameMessageManager* mParentManager;
   nsTArray<nsString> mPendingScripts;
+  nsTArray<bool> mPendingScriptsGlobalStates;
+
+  void LoadPendingScripts(nsFrameMessageManager* aManager,
+                          nsFrameMessageManager* aChildMM);
 public:
   static nsFrameMessageManager* sParentProcessManager;
   static nsFrameMessageManager* sChildProcessManager;
@@ -299,17 +317,64 @@ private:
                                  bool* aValid);
 };
 
+/* A helper class for taking care of many details for async message sending
+   within a single process.  Intended to be used like so:
+
+   class MyAsyncMessage : public nsSameProcessAsyncMessageBase, public nsRunnable
+   {
+     // Initialize nsSameProcessAsyncMessageBase...
+
+     NS_IMETHOD Run() {
+       ReceiveMessage(..., ...);
+       return NS_OK;
+     }
+   };
+ */
+class nsSameProcessAsyncMessageBase
+{
+  typedef mozilla::dom::StructuredCloneClosure StructuredCloneClosure;
+
+public:
+  typedef mozilla::dom::StructuredCloneData StructuredCloneData;
+
+  nsSameProcessAsyncMessageBase(JSContext* aCx,
+                                const nsAString& aMessage,
+                                const StructuredCloneData& aData,
+                                JS::Handle<JSObject*> aCpows,
+                                nsIPrincipal* aPrincipal);
+
+  void ReceiveMessage(nsISupports* aTarget, nsFrameMessageManager* aManager);
+
+private:
+  nsSameProcessAsyncMessageBase(const nsSameProcessAsyncMessageBase&);
+
+  JSRuntime* mRuntime;
+  nsString mMessage;
+  JSAutoStructuredCloneBuffer mData;
+  StructuredCloneClosure mClosure;
+  JS::PersistentRooted<JSObject*> mCpows;
+  nsCOMPtr<nsIPrincipal> mPrincipal;
+};
+
 class nsScriptCacheCleaner;
 
-struct nsFrameJSScriptExecutorHolder
+struct nsFrameScriptObjectExecutorHolder
 {
-  nsFrameJSScriptExecutorHolder(JSScript* aScript) : mScript(aScript)
-  { MOZ_COUNT_CTOR(nsFrameJSScriptExecutorHolder); }
-  ~nsFrameJSScriptExecutorHolder()
-  { MOZ_COUNT_DTOR(nsFrameJSScriptExecutorHolder); }
-  JSScript* mScript;
+  nsFrameScriptObjectExecutorHolder(JSContext* aCx, JSScript* aScript, bool aRunInGlobalScope)
+   : mScript(aCx, aScript), mRunInGlobalScope(aRunInGlobalScope)
+  { MOZ_COUNT_CTOR(nsFrameScriptObjectExecutorHolder); }
+
+  ~nsFrameScriptObjectExecutorHolder()
+  { MOZ_COUNT_DTOR(nsFrameScriptObjectExecutorHolder); }
+
+  bool WillRunInGlobalScope() { return mRunInGlobalScope; }
+
+  JS::PersistentRooted<JSScript*> mScript;
+  bool mRunInGlobalScope;
 };
 
+class nsFrameScriptObjectExecutorStackHolder;
+
 class nsFrameScriptExecutor
 {
 public:
@@ -321,24 +386,30 @@ public:
   }
 protected:
   friend class nsFrameScriptCx;
-  nsFrameScriptExecutor()
-  { MOZ_COUNT_CTOR(nsFrameScriptExecutor); }
-  ~nsFrameScriptExecutor()
-  { MOZ_COUNT_DTOR(nsFrameScriptExecutor); }
+  nsFrameScriptExecutor() { MOZ_COUNT_CTOR(nsFrameScriptExecutor); }
+  ~nsFrameScriptExecutor() { MOZ_COUNT_DTOR(nsFrameScriptExecutor); }
+
   void DidCreateGlobal();
-  void LoadFrameScriptInternal(const nsAString& aURL);
-  enum CacheFailedBehavior { EXECUTE_IF_CANT_CACHE, DONT_EXECUTE };
+  void LoadFrameScriptInternal(const nsAString& aURL, bool aRunInGlobalScope);
   void TryCacheLoadAndCompileScript(const nsAString& aURL,
-                                    CacheFailedBehavior aBehavior = DONT_EXECUTE);
+                                    bool aRunInGlobalScope,
+                                    bool aShouldCache,
+                                    JS::MutableHandle<JSScript*> aScriptp);
+  void TryCacheLoadAndCompileScript(const nsAString& aURL,
+                                    bool aRunInGlobalScope);
   bool InitTabChildGlobalInternal(nsISupports* aScope, const nsACString& aID);
   nsCOMPtr<nsIXPConnectJSObjectHolder> mGlobal;
   nsCOMPtr<nsIPrincipal> mPrincipal;
-  static nsDataHashtable<nsStringHashKey, nsFrameJSScriptExecutorHolder*>* sCachedScripts;
+  nsAutoTArray<JS::Heap<JSObject*>, 2> mAnonymousGlobalScopes;
+
+  static nsDataHashtable<nsStringHashKey, nsFrameScriptObjectExecutorHolder*>* sCachedScripts;
   static nsScriptCacheCleaner* sScriptCacheCleaner;
 };
 
 class nsScriptCacheCleaner MOZ_FINAL : public nsIObserver
 {
+  ~nsScriptCacheCleaner() {}
+
   NS_DECL_ISUPPORTS
 
   nsScriptCacheCleaner()
@@ -350,7 +421,7 @@ class nsScriptCacheCleaner MOZ_FINAL : public nsIObserver
 
   NS_IMETHODIMP Observe(nsISupports *aSubject,
                         const char *aTopic,
-                        const PRUnichar *aData)
+                        const char16_t *aData)
   {
     nsFrameScriptExecutor::Shutdown();
     return NS_OK;
