@@ -11,13 +11,11 @@ const Cr = Components.results;
 Cu.import('resource://gre/modules/CSPUtils.jsm');
 Cu.import('resource://gre/modules/NetUtil.jsm');
 
-// load the HTTP server
-Cu.import("resource://testing-common/httpd.js");
-
 var httpServer = new HttpServer();
+httpServer.start(-1);
 
-const POLICY_FROM_URI = "allow 'self'; img-src *";
-const POLICY_PORT = 9000;
+const POLICY_FROM_URI = "default-src 'self'; img-src *";
+const POLICY_PORT = httpServer.identity.primaryPort;
 const POLICY_URI = "http://localhost:" + POLICY_PORT + "/policy";
 const POLICY_URI_RELATIVE = "/policy";
 
@@ -37,7 +35,7 @@ function do_check_in_array(arr, val, stack) {
   var text = val + " in [" + arr.join(",") + "]";
 
   for(var i in arr) {
-    dump(".......... " + i + "> " + arr[i] + "\n");
+    //dump(".......... " + i + "> " + arr[i] + "\n");
     if(arr[i] == val) {
       //succeed
       ++_passedChecks;
@@ -116,8 +114,12 @@ test(
     h = CSPHost.fromString("foo-bar.com");
     do_check_neq(null, h); // "dashes in hosts should work"
 
+
     h = CSPHost.fromString("foo!bar.com");
     do_check_eq(null, h); // "special chars in hosts should fail"
+
+    h = CSPHost.fromString("{app-url-is-uid}");
+    do_check_neq(null, h); // "Packaged apps URLs failed"
   });
 
 test(
@@ -143,15 +145,6 @@ test(
     do_check_true( h2.permits("a.b.c")); //"CSPHost a.b.c should allow string a.b.c"
   });
 
-test(
-    function test_CSPHost_intersectWith() {
-      var h = CSPHost.fromString("*.b.c");
-      //"*.a.b.c ^ *.b.c should be *.a.b.c"
-      do_check_eq("*.a.b.c", h.intersectWith(CSPHost.fromString("*.a.b.c")).toString());
-
-      //"*.b.c ^ *.d.e should not work (null)"
-      do_check_eq(null, h.intersectWith(CSPHost.fromString("*.d.e")));
-    });
 
 ///////////////////// Test the Source object //////////////////////
 
@@ -188,6 +181,9 @@ test(
       //Port parsing should work for all schemes
       do_check_neq(null, CSPSource.create("data:"));
       do_check_neq(null, CSPSource.create("javascript:"));
+
+      //"app:// URLs should work, including the {} characters.");
+      do_check_neq(null, CSPSource.fromString("{app-host-is-uid}", undefined, "app://{app-host-is-uid}"));
     });
 
 test(
@@ -229,6 +225,12 @@ test(
       //"nothing else should be allowed"
       do_check_false(src.permits("https://foobar.com"));
 
+      src = CSPSource.create("{app-host-is-uid}", undefined, "app://{app-host-is-uid}");
+      //"src should inherit and require 'app' scheme"
+      do_check_false(src.permits("https://{app-host-is-uid}"));
+      //"src should inherit scheme 'app'"
+      do_check_true(src.permits("app://{app-host-is-uid}"));
+
     });
 
 ///////////////////// Test the source list //////////////////////
@@ -259,7 +261,7 @@ test(
 test(
     function test_CSPSourceList_fromString_twohost() {
       var str = "foo.bar:21 https://ras.bar";
-      var parsed = "http://foo.bar:21 https://ras.bar:443";
+      var parsed = "http://foo.bar:21 https://ras.bar";
       var sd = CSPSourceList.fromString(str, undefined, URI("http://self.com:80"));
       //"two-host list should parse"
       do_check_neq(null,sd);
@@ -319,98 +321,25 @@ test(
       do_check_false(wildcardHostSourceList.permits("http://barbaz.com"));
     });
 
-test(
-    function test_CSPSourceList_intersect() {
-      // for this test, 'self' values are irrelevant
-      // policy a /\ policy b intersects policies, not context (where 'self'
-      // values come into play)
-      var nullSourceList = CSPSourceList.fromString("'none'");
-      var simpleSourceList = CSPSourceList.fromString("http://a.com");
-      var doubleSourceList = CSPSourceList.fromString("https://foo.com http://bar.com:88");
-      var singleFooSourceList = CSPSourceList.fromString("https://foo.com");
-      var allSourceList = CSPSourceList.fromString("*");
-
-      //"Intersection of one source with 'none' source list should be none.");
-      do_check_true(nullSourceList.intersectWith(simpleSourceList).isNone());
-      //"Intersection of two sources with 'none' source list should be none.");
-      do_check_true(nullSourceList.intersectWith(doubleSourceList).isNone());
-      //"Intersection of '*' with 'none' source list should be none.");
-      do_check_true(nullSourceList.intersectWith(allSourceList).isNone());
-
-      //"Intersection of one source with '*' source list should be one source.");
-      do_check_equivalent(allSourceList.intersectWith(simpleSourceList),
-                          simpleSourceList);
-      //"Intersection of two sources with '*' source list should be two sources.");
-      do_check_equivalent(allSourceList.intersectWith(doubleSourceList),
-                          doubleSourceList);
-
-      //"Non-overlapping source lists should intersect to 'none'");
-      do_check_true(simpleSourceList.intersectWith(doubleSourceList).isNone());
-
-      //"subset and superset should intersect to subset.");
-      do_check_equivalent(singleFooSourceList,
-                          doubleSourceList.intersectWith(singleFooSourceList));
-
-      //TODO: write more tests?
-
-    });
-
-///////////////////// Test the Whole CSP rep object //////////////////////
-
+//////////////// TEST CSP REP SPEC COMPLIANT PARSER ////////////
 test(
     function test_CSPRep_fromString() {
 
-      // check default init
-      //ASSERT(!(new CSPRep())._isInitialized, "Uninitialized rep thinks it is.")
-
       var cspr;
       var cspr_allowval;
       var SD = CSPRep.SRC_DIRECTIVES;
+      var DEFAULTS = [SD.STYLE_SRC, SD.MEDIA_SRC, SD.IMG_SRC, SD.SCRIPT_SRC, SD.FONT_SRC,
+                      SD.OBJECT_SRC, SD.FRAME_SRC, SD.CONNECT_SRC];
 
-      // check default policy "allow *"
-      cspr = CSPRep.fromString("allow *", URI("http://self.com:80"));
-      // "DEFAULT_SRC directive is missing when specified in fromString"
-      do_check_has_key(cspr._directives, SD.DEFAULT_SRC);
-
-      // ... and check that the other directives were auto-filled with the
-      // DEFAULT_SRC one.
-      cspr_allowval = cspr._directives[SD.DEFAULT_SRC];
-      for(var d in SD) {
-        //"Missing key " + d
-        do_check_has_key(cspr._directives, SD[d]);
-        //"Implicit directive " + d + " has non-allow value."
-        do_check_eq(cspr._directives[SD[d]].toString(), cspr_allowval.toString());
-      }
-    });
-
-
-test(
-    function test_CSPRep_defaultSrc() {
-      var cspr, cspr_default_val, cspr_allow;
-      var SD = CSPRep.SRC_DIRECTIVES;
-
-      // apply policy of "default-src *" (e.g. "allow *")
+      // check default policy "default-src *"
       cspr = CSPRep.fromString("default-src *", URI("http://self.com:80"));
       // "DEFAULT_SRC directive is missing when specified in fromString"
       do_check_has_key(cspr._directives, SD.DEFAULT_SRC);
 
-      // check that the other directives were auto-filled with the
-      // DEFAULT_SRC one.
-      cspr_default_val = cspr._directives[SD.DEFAULT_SRC];
-      for (var d in SD) {
-        do_check_has_key(cspr._directives, SD[d]);
-        // "Implicit directive " + d + " has non-default-src value."
-        do_check_eq(cspr._directives[SD[d]].toString(), cspr_default_val.toString());
-      }
-
-      // check that |allow *| and |default-src *| are parsed equivalently and
-      // result in the same set of explicit policy directives
-      cspr = CSPRep.fromString("default-src *", URI("http://self.com:80"));
-      cspr_allow = CSPRep.fromString("allow *", URI("http://self.com:80"));
-
-      for (var d in SD) {
-        do_check_equivalent(cspr._directives[SD[d]],
-                            cspr_allow._directives[SD[d]]);
+      for(var x in DEFAULTS) {
+        // each of these should be equivalent to DEFAULT_SRC
+        //DEFAULTS[x] + " does not use default rule."
+        do_check_true(cspr.permits("http://bar.com", DEFAULTS[x]));
       }
     });
 
@@ -420,10 +349,11 @@ test(
 
       var cspr;
       var SD = CSPRep.SRC_DIRECTIVES;
-      var DEFAULTS = [SD.STYLE_SRC, SD.MEDIA_SRC, SD.IMG_SRC, SD.FRAME_SRC];
+      var DEFAULTS = [SD.STYLE_SRC, SD.MEDIA_SRC, SD.IMG_SRC,
+                      SD.FRAME_SRC, SD.CONNECT_SRC];
 
       // check one-directive policies
-      cspr = CSPRep.fromString("allow bar.com; script-src https://foo.com",
+      cspr = CSPRep.fromString("default-src bar.com; script-src https://foo.com",
                                URI("http://self.com"));
 
       for(var x in DEFAULTS) {
@@ -443,15 +373,18 @@ test(
     });
 
 test(
-    function test_CSPRep_fromString_twodir() {
+    function test_CSPRep_fromString_twoDir() {
       var cspr;
+
       var SD = CSPRep.SRC_DIRECTIVES;
-      var DEFAULTS = [SD.STYLE_SRC, SD.MEDIA_SRC, SD.FRAME_SRC];
+
+      var DEFAULTS = [SD.STYLE_SRC, SD.MEDIA_SRC, SD.FRAME_SRC,
+                      SD.CONNECT_SRC];
 
       // check two-directive policies
-      var polstr = "allow allow.com; "
-                  + "script-src https://foo.com; "
-                  + "img-src bar.com:*";
+      var polstr = "default-src allow.com; " +
+                   "script-src https://foo.com; " +
+                   "img-src bar.com:*";
       cspr = CSPRep.fromString(polstr, URI("http://self.com"));
 
       for(var x in DEFAULTS) {
@@ -479,12 +412,12 @@ test(
 
 test(function test_CSPRep_fromString_withself() {
       var cspr;
-      var SD = CSPRep.SRC_DIRECTIVES;
       var self = "https://self.com:34";
+      var SD = CSPRep.SRC_DIRECTIVES;
 
       // check one-directive policies
-      cspr = CSPRep.fromString("allow 'self'; script-src 'self' https://*:*",
-                              URI(self));
+      cspr = CSPRep.fromString("default-src 'self'; script-src 'self' https://*:*",
+                               URI(self));
       //"img-src does not enforce default rule, 'self'.
       do_check_false(cspr.permits("https://foo.com:400", SD.IMG_SRC));
       //"img-src does not allow self
@@ -497,23 +430,24 @@ test(function test_CSPRep_fromString_withself() {
       do_check_true(cspr.permits("https://evil.com:100", SD.SCRIPT_SRC));
      });
 
+
 //////////////// TEST FRAME ANCESTOR DEFAULTS /////////////////
 // (see bug 555068)
 test(function test_FrameAncestor_defaults() {
       var cspr;
-      var SD = CSPRep.SRC_DIRECTIVES;
       var self = "http://self.com:34";
+      var SD = CSPRep.SRC_DIRECTIVES;
 
-      cspr = CSPRep.fromString("allow 'none'", URI(self));
+      cspr = CSPRep.fromString("default-src 'none'", URI(self));
 
-      //"frame-ancestors should default to * not 'allow' value"
+      //"frame-ancestors should default to * not 'default-src' value"
       do_check_true(cspr.permits("https://foo.com:400", SD.FRAME_ANCESTORS));
       do_check_true(cspr.permits("http://self.com:34", SD.FRAME_ANCESTORS));
       do_check_true(cspr.permits("https://self.com:34", SD.FRAME_ANCESTORS));
       do_check_true(cspr.permits("http://self.com", SD.FRAME_ANCESTORS));
       do_check_true(cspr.permits("http://subd.self.com:34", SD.FRAME_ANCESTORS));
 
-      cspr = CSPRep.fromString("allow 'none'; frame-ancestors 'self'", URI(self));
+      cspr = CSPRep.fromString("default-src 'none'; frame-ancestors 'self'", URI(self));
 
       //"frame-ancestors should only allow self"
       do_check_true(cspr.permits("http://self.com:34", SD.FRAME_ANCESTORS));
@@ -523,12 +457,13 @@ test(function test_FrameAncestor_defaults() {
       do_check_false(cspr.permits("http://subd.self.com:34", SD.FRAME_ANCESTORS));
      });
 
+
 test(function test_FrameAncestor_TLD_defaultPorts() {
       var cspr;
       var SD = CSPRep.SRC_DIRECTIVES;
       var self = "http://self"; //TLD only, no .com or anything.
 
-      cspr = CSPRep.fromString("allow 'self'; frame-ancestors 'self' http://foo:80 bar:80 http://three", URI(self));
+      cspr = CSPRep.fromString("default-src 'self'; frame-ancestors 'self' http://foo:80 bar:80 http://three", URI(self));
 
       //"frame-ancestors should default to * not 'allow' value"
       do_check_true(cspr.permits("http://self", SD.FRAME_ANCESTORS));
@@ -545,6 +480,54 @@ test(function test_FrameAncestor_TLD_defaultPorts() {
       do_check_false(cspr.permits("https://three:81", SD.FRAME_ANCESTORS));
      });
 
+test(function test_FrameAncestor_ignores_userpass_bug779918() {
+      var cspr;
+      var SD = CSPRep.SRC_DIRECTIVES;
+      var self = "http://self.com/bar";
+      var testPolicy = "default-src 'self'; frame-ancestors 'self'";
+
+      cspr = CSPRep.fromString(testPolicy, URI(self));
+
+      // wrapped in URI() because of source parsing
+      do_check_true(cspr.permits(URI("http://username:password@self.com/foo"), SD.FRAME_ANCESTORS));
+      do_check_true(cspr.permits(URI("http://other:pass1@self.com/foo"), SD.FRAME_ANCESTORS));
+      do_check_true(cspr.permits(URI("http://self.com:80/foo"), SD.FRAME_ANCESTORS));
+      do_check_true(cspr.permits(URI("http://self.com/foo"), SD.FRAME_ANCESTORS));
+
+      // construct fake ancestry with CSP applied to the child.
+      // [aChildUri] -> [aParentUri] -> (root/top)
+      // and then test "permitsAncestry" on the child/self docshell.
+      function testPermits(aChildUri, aParentUri, aContentType) {
+        let cspObj = Cc["@mozilla.org/contentsecuritypolicy;1"]
+                       .createInstance(Ci.nsIContentSecurityPolicy);
+        cspObj.appendPolicy(testPolicy, aChildUri, false);
+        let docshellparent = Cc["@mozilla.org/docshell;1"]
+                               .createInstance(Ci.nsIDocShell);
+        let docshellchild  = Cc["@mozilla.org/docshell;1"]
+                               .createInstance(Ci.nsIDocShell);
+        docshellparent.setCurrentURI(aParentUri);
+        docshellchild.setCurrentURI(aChildUri);
+        docshellparent.addChild(docshellchild);
+        return cspObj.permitsAncestry(docshellchild);
+      };
+
+      // check parent without userpass
+      do_check_true(testPermits(URI("http://username:password@self.com/foo"),
+                                URI("http://self.com/bar")));
+      do_check_true(testPermits(URI("http://user1:pass1@self.com/foo"),
+                                URI("http://self.com/bar")));
+      do_check_true(testPermits(URI("http://self.com/foo"),
+                                URI("http://self.com/bar")));
+
+      // check parent with userpass
+      do_check_true(testPermits(URI("http://username:password@self.com/foo"),
+                                URI("http://username:password@self.com/bar")));
+      do_check_true(testPermits(URI("http://user1:pass1@self.com/foo"),
+                                URI("http://username:password@self.com/bar")));
+      do_check_true(testPermits(URI("http://self.com/foo"),
+                                URI("http://username:password@self.com/bar")));
+     });
+
 test(function test_CSP_ReportURI_parsing() {
       var cspr;
       var SD = CSPRep.SRC_DIRECTIVES;
@@ -552,41 +535,57 @@ test(function test_CSP_ReportURI_parsing() {
       var parsedURIs = [];
 
       var uri_valid_absolute = self + "/report.py";
-      var uri_invalid_host_absolute = "http://foo.org:34/report.py";
+      var uri_other_host_absolute = "http://foo.org:34/report.py";
       var uri_valid_relative = "/report.py";
       var uri_valid_relative_expanded = self + uri_valid_relative;
       var uri_valid_relative2 = "foo/bar/report.py";
       var uri_valid_relative2_expanded = self + "/" + uri_valid_relative2;
       var uri_invalid_relative = "javascript:alert(1)";
+      var uri_other_scheme_absolute = "https://self.com/report.py";
+      var uri_other_scheme_and_host_absolute = "https://foo.com/report.py";
 
-      cspr = CSPRep.fromString("allow *; report-uri " + uri_valid_absolute, URI(self));
+      cspr = CSPRep.fromString("default-src *; report-uri " + uri_valid_absolute, URI(self));
       parsedURIs = cspr.getReportURIs().split(/\s+/);
       do_check_in_array(parsedURIs, uri_valid_absolute);
       do_check_eq(parsedURIs.length, 1);
 
-      cspr = CSPRep.fromString("allow *; report-uri " + uri_invalid_host_absolute, URI(self));
+      cspr = CSPRep.fromString("default-src *; report-uri " + uri_other_host_absolute, URI(self));
       parsedURIs = cspr.getReportURIs().split(/\s+/);
-      do_check_in_array(parsedURIs, "");
+      do_check_in_array(parsedURIs, uri_other_host_absolute);
       do_check_eq(parsedURIs.length, 1); // the empty string is in there.
 
-      cspr = CSPRep.fromString("allow *; report-uri " + uri_invalid_relative, URI(self));
+      cspr = CSPRep.fromString("default-src *; report-uri " + uri_invalid_relative, URI(self));
       parsedURIs = cspr.getReportURIs().split(/\s+/);
       do_check_in_array(parsedURIs, "");
       do_check_eq(parsedURIs.length, 1);
 
-      cspr = CSPRep.fromString("allow *; report-uri " + uri_valid_relative, URI(self));
+      cspr = CSPRep.fromString("default-src *; report-uri " + uri_valid_relative, URI(self));
       parsedURIs = cspr.getReportURIs().split(/\s+/);
       do_check_in_array(parsedURIs, uri_valid_relative_expanded);
       do_check_eq(parsedURIs.length, 1);
 
-      cspr = CSPRep.fromString("allow *; report-uri " + uri_valid_relative2, URI(self));
+      cspr = CSPRep.fromString("default-src *; report-uri " + uri_valid_relative2, URI(self));
       parsedURIs = cspr.getReportURIs().split(/\s+/);
       dump(parsedURIs.length);
       do_check_in_array(parsedURIs, uri_valid_relative2_expanded);
       do_check_eq(parsedURIs.length, 1);
 
+      // make sure cross-scheme reporting works
+      cspr = CSPRep.fromString("default-src *; report-uri " + uri_other_scheme_absolute, URI(self));
+      parsedURIs = cspr.getReportURIs().split(/\s+/);
+      dump(parsedURIs.length);
+      do_check_in_array(parsedURIs, uri_other_scheme_absolute);
+      do_check_eq(parsedURIs.length, 1);
+
+      // make sure cross-scheme, cross-host reporting works
+      cspr = CSPRep.fromString("default-src *; report-uri " + uri_other_scheme_and_host_absolute, URI(self));
+      parsedURIs = cspr.getReportURIs().split(/\s+/);
+      dump(parsedURIs.length);
+      do_check_in_array(parsedURIs, uri_other_scheme_and_host_absolute);
+      do_check_eq(parsedURIs.length, 1);
+
       // combination!
-      cspr = CSPRep.fromString("allow *; report-uri " +
+      cspr = CSPRep.fromString("default-src *; report-uri " +
                                uri_valid_relative2 + " " +
                                uri_valid_absolute, URI(self));
       parsedURIs = cspr.getReportURIs().split(/\s+/);
@@ -594,14 +593,15 @@ test(function test_CSP_ReportURI_parsing() {
       do_check_in_array(parsedURIs, uri_valid_absolute);
       do_check_eq(parsedURIs.length, 2);
 
-      cspr = CSPRep.fromString("allow *; report-uri " +
+      cspr = CSPRep.fromString("default-src *; report-uri " +
                                uri_valid_relative2 + " " +
-                               uri_invalid_host_absolute + " " +
+                               uri_other_host_absolute + " " +
                                uri_valid_absolute, URI(self));
       parsedURIs = cspr.getReportURIs().split(/\s+/);
       do_check_in_array(parsedURIs, uri_valid_relative2_expanded);
+      do_check_in_array(parsedURIs, uri_other_host_absolute);
       do_check_in_array(parsedURIs, uri_valid_absolute);
-      do_check_eq(parsedURIs.length, 2);
+      do_check_eq(parsedURIs.length, 3);
     });
 
 test(
@@ -614,25 +614,15 @@ test(
       var thirdDomain = "http://third.com";
 
       // check for duplicate "default-src" directives
+      // Spec says first directive persists (subsequent re-statement is
+      // ignored)
       cspr = CSPRep.fromString("default-src " + self + "; default-src " +
                               firstDomain, URI(self));
       do_check_true(cspr.permits(self, SD.DEFAULT_SRC));
       do_check_false(cspr.permits(firstDomain, SD.DEFAULT_SRC));
 
-      // check for duplicate "allow" directives
-      cspr = CSPRep.fromString("allow " + self + "; allow " + firstDomain,
-                              URI(self));
-      do_check_true(cspr.permits(self, SD.DEFAULT_SRC));
-      do_check_false(cspr.permits(firstDomain, SD.DEFAULT_SRC));
-
-      // check for duplicate "allow" + "default-src" directives
-      cspr = CSPRep.fromString("allow " + self + "; default-src " + firstDomain,
-                              URI(self));
-      do_check_true(cspr.permits(self, SD.DEFAULT_SRC));
-      do_check_false(cspr.permits(firstDomain, SD.DEFAULT_SRC));
-
       // check for duplicate report-uri directives
-      cspr = CSPRep.fromString("allow *; report-uri " + self + "/report.py; report-uri "
+      cspr = CSPRep.fromString("default-src *; report-uri " + self + "/report.py; report-uri "
                               + firstDomain + "/report.py", URI(self));
       parsedURIs = cspr.getReportURIs().split(/\s+/);
       do_check_in_array(parsedURIs, self + "/report.py");
@@ -687,9 +677,9 @@ test(
        *
        *   Document with CSP: https://foobar.com:4443
        *   Transmitted policy:
-       *       "allow 'self' a.com"
+       *       "default-src 'self' a.com"
        *   Explicit policy:
-       *       "allow https://foobar.com:4443 https://a.com:443"
+       *       "default-src https://foobar.com:4443 https://a.com:443"
        *
        * This test examines scheme and nonstandard port inheritance.
        */
@@ -726,7 +716,7 @@ test(
 test(
     function test_bug634773_noneAndStarAreDifferent() {
       /**
-       * Bug 634773 is that allow * and allow 'none' end up "equal" via
+       * Bug 634773 is that default-src * and default-src 'none' end up "equal" via
        * CSPSourceList.prototype.equals(), which is wrong.  This tests that
        * doesn't happen.
        */
@@ -745,70 +735,111 @@ test(
       do_check_false(p_none.permits("http://bar.com"));
     });
 
-test(
-    function test_bug783497_refinePolicyIssues() {
 
-      const firstPolicy = "allow 'self'; img-src 'self'; script-src 'self'; options 'bogus-option'";
-      const secondPolicy = "default-src 'none'; script-src 'self'";
+test(
+    function test_bug764937_defaultSrcMissing() {
       var cspObj = Cc["@mozilla.org/contentsecuritypolicy;1"]
                      .createInstance(Ci.nsIContentSecurityPolicy);
       var selfURI = URI("http://self.com/");
 
-      function testPermits(aUri, aContext) {
-        return cspObj.shouldLoad(aContext, aUri, null, null, null, null)
+      function testPermits(cspObj, aUri, aContentType) {
+        return cspObj.shouldLoad(aContentType, aUri, null, null, null, null)
                == Ci.nsIContentPolicy.ACCEPT;
       };
 
-      // everything is allowed by the default policy
-      do_check_true(testPermits(URI("http://self.com/foo.js"),
-                    Ci.nsIContentPolicy.TYPE_SCRIPT));
-      do_check_true(testPermits(URI("http://other.com/foo.js"),
-                    Ci.nsIContentPolicy.TYPE_SCRIPT));
-      do_check_true(testPermits(URI("http://self.com/foo.png"),
-                    Ci.nsIContentPolicy.TYPE_IMAGE));
-      do_check_true(testPermits(URI("http://other.com/foo.png"),
-                    Ci.nsIContentPolicy.TYPE_IMAGE));
+      const policy = "script-src 'self'";
+      cspObj.appendPolicy(policy, selfURI, false);
 
-      // fold in the first policy
-      cspObj.refinePolicy(firstPolicy, selfURI);
+      // Spec-Compliant policy default-src defaults to *.
+      // This means all images are allowed, and only 'self'
+      // script is allowed.
+      do_check_true(testPermits(cspObj,
+                                URI("http://bar.com/foo.png"),
+                                Ci.nsIContentPolicy.TYPE_IMAGE));
+      do_check_true(testPermits(cspObj,
+                                URI("http://self.com/foo.png"),
+                                Ci.nsIContentPolicy.TYPE_IMAGE));
+      do_check_true(testPermits(cspObj,
+                                URI("http://self.com/foo.js"),
+                                Ci.nsIContentPolicy.TYPE_SCRIPT));
+      do_check_false(testPermits(cspObj,
+                                 URI("http://bar.com/foo.js"),
+                                 Ci.nsIContentPolicy.TYPE_SCRIPT));
 
-      // script-src and img-src are limited to self after the first policy
-      do_check_true(testPermits(URI("http://self.com/foo.js"),
-                    Ci.nsIContentPolicy.TYPE_SCRIPT));
-      do_check_false(testPermits(URI("http://other.com/foo.js"),
-                     Ci.nsIContentPolicy.TYPE_SCRIPT));
-      do_check_true(testPermits(URI("http://self.com/foo.png"),
-                    Ci.nsIContentPolicy.TYPE_IMAGE));
-      do_check_false(testPermits(URI("http://other.com/foo.png"),
-                     Ci.nsIContentPolicy.TYPE_IMAGE));
-
-      // fold in the second policy
-      cspObj.refinePolicy(secondPolicy, selfURI);
-
-      // script-src is self and img-src is none after the merge
-      do_check_true(testPermits(URI("http://self.com/foo.js"),
-                    Ci.nsIContentPolicy.TYPE_SCRIPT));
-      do_check_false(testPermits(URI("http://other.com/foo.js"),
-                     Ci.nsIContentPolicy.TYPE_SCRIPT));
-      do_check_false(testPermits(URI("http://self.com/foo.png"),
-                     Ci.nsIContentPolicy.TYPE_IMAGE));
-      do_check_false(testPermits(URI("http://other.com/foo.png"),
-                     Ci.nsIContentPolicy.TYPE_IMAGE));
     });
 
+test(function test_equals_does_case_insensitive_comparison() {
+      // NOTE: For scheme, host and keyword-host:
+      // (1) compare the same lower-case in two distinct objects
+      // (2) compare upper-case with lower-case inputs
+      // to test case insensitivity.
 
+      // CSPSource equals ignores case
+      var upperCaseHost = "http://FOO.COM";
+      var lowerCaseHost = "http://foo.com";
+      var src1 = CSPSource.fromString(lowerCaseHost);
+      var src2 = CSPSource.fromString(lowerCaseHost);
+      do_check_true(src1.equals(src2))
+      var src3 = CSPSource.fromString(upperCaseHost);
+      do_check_true(src1.equals(src3))
+
+      // CSPHost equals ignores case
+      var upperCaseScheme = "HTTP";
+      var lowerCaseScheme = "http";
+      src1 = CSPHost.fromString(lowerCaseScheme);
+      src2 = CSPHost.fromString(lowerCaseScheme);
+      do_check_true(src1.equals(src2));
+      src3 = CSPHost.fromString(upperCaseScheme);
+      do_check_true(src1.equals(src3));
+
+      // CSPSourceList equals (mainly for testing keywords)
+      var upperCaseKeywords = "'SELF'";
+      var lowerCaseKeywords = "'self'";
+      src1 = CSPSourceList.fromString(lowerCaseKeywords);
+      src2 = CSPSourceList.fromString(lowerCaseKeywords);
+      do_check_true(src1.equals(src2))
+      src3 = CSPSourceList.fromString(upperCaseKeywords);
+      do_check_true(src1.equals(src3))
+
+  });
+
+test(function test_csp_permits_case_insensitive() {
+      var cspr;
+      var SD = CSPRep.SRC_DIRECTIVES;
+
+      // checks directives can be case-insensitive
+      var selfHost = "http://self.com";
+      var testPolicy1 = "DEFAULT-src 'self';";
+      cspr = CSPRep.fromString(testPolicy1, URI(selfHost));
+      do_check_true(cspr.permits(URI("http://self.com"), SD.DEFAULT_SRC));
+
+      // checks hosts can be case-insensitive
+      var testPolicy2 = "default-src 'self' http://FOO.COM";
+      cspr = CSPRep.fromString(testPolicy2, URI(selfHost));
+      do_check_true(cspr.permits(URI("http://foo.com"), SD.DEFAULT_SRC));
+
+      // checks schemes can be case-insensitive
+      var testPolicy3 = "default-src 'self' HTTP://foo.com";
+      cspr = CSPRep.fromString(testPolicy3, URI(selfHost));
+      do_check_true(cspr.permits(URI("http://foo.com"), SD.DEFAULT_SRC));
+
+      // checks keywords can be case-insensitive
+      var testPolicy4 = "default-src 'NONE'";
+      cspr = CSPRep.fromString(testPolicy4, URI(selfHost));
+      do_check_false(cspr.permits(URI("http://foo.com"), SD.DEFAULT_SRC));
+  });
 /*
 
 test(function test_CSPRep_fromPolicyURI_failswhenmixed() {
         var cspr;
         var self = "http://localhost:" + POLICY_PORT;
-        var closed_policy = CSPRep.fromString("allow 'none'");
+        var closed_policy = CSPRep.fromString("default-src 'none'");
         var my_uri_policy = "policy-uri " + POLICY_URI;
 
         //print(" --- Ignore the following two errors if they print ---");
-        cspr = CSPRep.fromString("allow *; " + my_uri_policy, URI(self));
+        cspr = CSPRep.fromString("default-src *; " + my_uri_policy, URI(self));
 
-        //"Parsing should fail when 'policy-uri' is mixed with allow directive"
+        //"Parsing should fail when 'policy-uri' is mixed with default-src directive"
         do_check_equivalent(cspr, closed_policy);
         cspr = CSPRep.fromString("img-src 'self'; " + my_uri_policy, URI(self));
 
@@ -831,15 +862,17 @@ function run_test() {
   }
   //server.registerDirectory("/", nsILocalFileForBasePath);
   httpServer.registerPathHandler("/policy", policyresponder);
-  httpServer.start(POLICY_PORT);
 
   for(let i in tests) {
-    tests[i]();
+    add_task(tests[i]);
   }
 
-  //teardown
-  httpServer.stop(function() { });
-  do_test_finished();
+  do_register_cleanup(function () {
+    //teardown
+    httpServer.stop(function() { });
+  });
+
+  run_next_test();
 }
 
 
