@@ -13,13 +13,23 @@
 #include "mozilla/Mutex.h"
 #include <algorithm>
 #include "mozilla/Preferences.h"
+#include "mozilla/Telemetry.h"
 #include "soundtouch/SoundTouch.h"
 #include "Latency.h"
+#include "nsPrintfCString.h"
 
 namespace mozilla {
 
+#ifdef LOG
+#undef LOG
+#endif
+
 #ifdef PR_LOGGING
 PRLogModuleInfo* gAudioStreamLog = nullptr;
+// For simple logs
+#define LOG(x) PR_LOG(gAudioStreamLog, PR_LOG_DEBUG, x)
+#else
+#define LOG(x)
 #endif
 
 /**
@@ -42,6 +52,88 @@ double AudioStream::sVolumeScale;
 uint32_t AudioStream::sCubebLatency;
 bool AudioStream::sCubebLatencyPrefSet;
 
+
+/**
+ * Keep a list of frames sent to the audio engine in each DataCallback along
+ * with the playback rate at the moment. Since the playback rate and number of
+ * underrun frames can vary in each callback. We need to keep the whole history
+ * in order to calculate the playback position of the audio engine correctly.
+ */
+class FrameHistory {
+  struct Chunk {
+    uint32_t servicedFrames;
+    uint32_t totalFrames;
+    int rate;
+  };
+
+  template <typename T>
+  static T FramesToUs(uint32_t frames, int rate) {
+    return static_cast<T>(frames) * USECS_PER_S / rate;
+  }
+public:
+  FrameHistory()
+    : mBaseOffset(0), mBasePosition(0) {}
+
+  void Append(uint32_t aServiced, uint32_t aUnderrun, int aRate) {
+    /* In most case where playback rate stays the same and we don't underrun
+     * frames, we are able to merge chunks to avoid lose of precision to add up
+     * in compressing chunks into |mBaseOffset| and |mBasePosition|.
+     */
+    if (!mChunks.IsEmpty()) {
+      Chunk& c = mChunks.LastElement();
+      // 2 chunks (c1 and c2) can be merged when rate is the same and
+      // adjacent frames are zero. That is, underrun frames in c1 are zero
+      // or serviced frames in c2 are zero.
+      if (c.rate == aRate &&
+          (c.servicedFrames == c.totalFrames ||
+           aServiced == 0)) {
+        c.servicedFrames += aServiced;
+        c.totalFrames += aServiced + aUnderrun;
+        return;
+      }
+    }
+    Chunk* p = mChunks.AppendElement();
+    p->servicedFrames = aServiced;
+    p->totalFrames = aServiced + aUnderrun;
+    p->rate = aRate;
+  }
+
+  /**
+   * @param frames The playback position in frames of the audio engine.
+   * @return The playback position in microseconds of the audio engine,
+   *         adjusted by playback rate changes and underrun frames.
+   */
+  int64_t GetPosition(int64_t frames) {
+    // playback position should not go backward.
+    MOZ_ASSERT(frames >= mBaseOffset);
+    while (true) {
+      if (mChunks.IsEmpty()) {
+        return mBasePosition;
+      }
+      const Chunk& c = mChunks[0];
+      if (frames <= mBaseOffset + c.totalFrames) {
+        uint32_t delta = frames - mBaseOffset;
+        delta = std::min(delta, c.servicedFrames);
+        return static_cast<int64_t>(mBasePosition) +
+               FramesToUs<int64_t>(delta, c.rate);
+      }
+      // Since the playback position of the audio engine will not go backward,
+      // we are able to compress chunks so that |mChunks| won't grow unlimitedly.
+      // Note that we lose precision in converting integers into floats and
+      // inaccuracy will accumulate over time. However, for a 24hr long,
+      // sample rate = 44.1k file, the error will be less than 1 microsecond
+      // after playing 24 hours. So we are fine with that.
+      mBaseOffset += c.totalFrames;
+      mBasePosition += FramesToUs<double>(c.servicedFrames, c.rate);
+      mChunks.RemoveElementAt(0);
+    }
+  }
+private:
+  nsAutoTArray<Chunk, 7> mChunks;
+  int64_t mBaseOffset;
+  double mBasePosition;
+};
+
 /*static*/ void AudioStream::PrefChanged(const char* aPref, void* aClosure)
 {
   if (strcmp(aPref, PREF_VOLUME_SCALE) == 0) {
@@ -64,6 +156,16 @@ bool AudioStream::sCubebLatencyPrefSet;
   }
 }
 
+/*static*/ bool AudioStream::GetFirstStream()
+{
+  static bool sFirstStream = true;
+
+  StaticMutexAutoLock lock(sMutex);
+  bool result = sFirstStream;
+  sFirstStream = false;
+  return result;
+}
+
 /*static*/ double AudioStream::GetVolumeScale()
 {
   StaticMutexAutoLock lock(sMutex);
@@ -110,25 +212,25 @@ bool AudioStream::sCubebLatencyPrefSet;
 }
 
 #if defined(__ANDROID__) && defined(MOZ_B2G)
-static cubeb_stream_type ConvertChannelToCubebType(dom::AudioChannelType aType)
+static cubeb_stream_type ConvertChannelToCubebType(dom::AudioChannel aChannel)
 {
-  switch(aType) {
-    case dom::AUDIO_CHANNEL_NORMAL:
+  switch(aChannel) {
+    case dom::AudioChannel::Normal:
       return CUBEB_STREAM_TYPE_SYSTEM;
-    case dom::AUDIO_CHANNEL_CONTENT:
+    case dom::AudioChannel::Content:
       return CUBEB_STREAM_TYPE_MUSIC;
-    case dom::AUDIO_CHANNEL_NOTIFICATION:
+    case dom::AudioChannel::Notification:
       return CUBEB_STREAM_TYPE_NOTIFICATION;
-    case dom::AUDIO_CHANNEL_ALARM:
+    case dom::AudioChannel::Alarm:
       return CUBEB_STREAM_TYPE_ALARM;
-    case dom::AUDIO_CHANNEL_TELEPHONY:
+    case dom::AudioChannel::Telephony:
       return CUBEB_STREAM_TYPE_VOICE_CALL;
-    case dom::AUDIO_CHANNEL_RINGER:
+    case dom::AudioChannel::Ringer:
       return CUBEB_STREAM_TYPE_RING;
-    // Currently Android openSLES library doesn't support FORCE_AUDIBLE yet.
-    case dom::AUDIO_CHANNEL_PUBLICNOTIFICATION:
+    case dom::AudioChannel::Publicnotification:
+      return CUBEB_STREAM_TYPE_SYSTEM_ENFORCED;
     default:
-      NS_ERROR("The value of AudioChannelType is invalid");
+      NS_ERROR("The value of AudioChannel is invalid");
       return CUBEB_STREAM_TYPE_MAX;
   }
 }
@@ -144,11 +246,12 @@ AudioStream::AudioStream()
   , mAudioClock(MOZ_THIS_IN_INITIALIZER_LIST())
   , mLatencyRequest(HighLatency)
   , mReadPoint(0)
-  , mLostFrames(0)
   , mDumpFile(nullptr)
   , mVolume(1.0)
   , mBytesPerFrame(0)
   , mState(INITIALIZED)
+  , mNeedsStart(false)
+  , mPendingAudioInitTask(false)
 {
   // keep a ref in case we shut down later than nsLayoutStatics
   mLatencyLog = AsyncLatencyLogger::Get(true);
@@ -156,12 +259,30 @@ AudioStream::AudioStream()
 
 AudioStream::~AudioStream()
 {
-  Shutdown();
+  LOG(("AudioStream: delete %p, state %d", this, mState));
+  MOZ_ASSERT(mState == SHUTDOWN && !mCubebStream,
+             "Should've called Shutdown() before deleting an AudioStream");
   if (mDumpFile) {
     fclose(mDumpFile);
   }
 }
 
+size_t
+AudioStream::SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const
+{
+  size_t amount = aMallocSizeOf(this);
+
+  // Possibly add in the future:
+  // - mTimeStretcher
+  // - mLatencyLog
+  // - mCubebStream
+
+  amount += mInserts.SizeOfExcludingThis(aMallocSizeOf);
+  amount += mBuffer.SizeOfExcludingThis(aMallocSizeOf);
+
+  return amount;
+}
+
 /*static*/ void AudioStream::InitLibrary()
 {
 #ifdef PR_LOGGING
@@ -189,10 +310,6 @@ nsresult AudioStream::EnsureTimeStretcherInitializedUnlocked()
 {
   mMonitor.AssertCurrentThreadOwns();
   if (!mTimeStretcher) {
-    // SoundTouch does not support a number of channels > 2
-    if (mOutChannels > 2) {
-      return NS_ERROR_FAILURE;
-    }
     mTimeStretcher = new soundtouch::SoundTouch();
     mTimeStretcher->setSampleRate(mInRate);
     mTimeStretcher->setChannels(mOutChannels);
@@ -203,6 +320,10 @@ nsresult AudioStream::EnsureTimeStretcherInitializedUnlocked()
 
 nsresult AudioStream::SetPlaybackRate(double aPlaybackRate)
 {
+  // MUST lock since the rate transposer is used from the cubeb callback,
+  // and rate changes can cause the buffer to be reallocated
+  MonitorAutoLock mon(mMonitor);
+
   NS_ASSERTION(aPlaybackRate > 0.0,
                "Can't handle negative or null playbackrate in the AudioStream.");
   // Avoid instantiating the resampler if we are not changing the playback rate.
@@ -211,9 +332,6 @@ nsresult AudioStream::SetPlaybackRate(double aPlaybackRate)
     return NS_OK;
   }
 
-  // MUST lock since the rate transposer is used from the cubeb callback,
-  // and rate changes can cause the buffer to be reallocated
-  MonitorAutoLock mon(mMonitor);
   if (EnsureTimeStretcherInitializedUnlocked() != NS_OK) {
     return NS_ERROR_FAILURE;
   }
@@ -233,14 +351,15 @@ nsresult AudioStream::SetPlaybackRate(double aPlaybackRate)
 
 nsresult AudioStream::SetPreservesPitch(bool aPreservesPitch)
 {
+  // MUST lock since the rate transposer is used from the cubeb callback,
+  // and rate changes can cause the buffer to be reallocated
+  MonitorAutoLock mon(mMonitor);
+
   // Avoid instantiating the timestretcher instance if not needed.
   if (aPreservesPitch == mAudioClock.GetPreservesPitch()) {
     return NS_OK;
   }
 
-  // MUST lock since the rate transposer is used from the cubeb callback,
-  // and rate changes can cause the buffer to be reallocated
-  MonitorAutoLock mon(mMonitor);
   if (EnsureTimeStretcherInitializedUnlocked() != NS_OK) {
     return NS_ERROR_FAILURE;
   }
@@ -260,6 +379,7 @@ nsresult AudioStream::SetPreservesPitch(bool aPreservesPitch)
 
 int64_t AudioStream::GetWritten()
 {
+  MonitorAutoLock mon(mMonitor);
   return mWritten;
 }
 
@@ -352,19 +472,22 @@ WriteDumpFile(FILE* aDumpFile, AudioStream* aStream, uint32_t aFrames,
   fflush(aDumpFile);
 }
 
+// NOTE: this must not block a LowLatency stream for any significant amount
+// of time, or it will block the entirety of MSG
 nsresult
 AudioStream::Init(int32_t aNumChannels, int32_t aRate,
-                  const dom::AudioChannelType aAudioChannelType,
+                  const dom::AudioChannel aAudioChannel,
                   LatencyRequest aLatencyRequest)
 {
-  cubeb* cubebContext = GetCubebContext();
+  mStartTime = TimeStamp::Now();
+  mIsFirst = GetFirstStream();
 
-  if (!cubebContext || aNumChannels < 0 || aRate < 0) {
+  if (!GetCubebContext() || aNumChannels < 0 || aRate < 0) {
     return NS_ERROR_FAILURE;
   }
 
   PR_LOG(gAudioStreamLog, PR_LOG_DEBUG,
-    ("%s  channels: %d, rate: %d", __FUNCTION__, aNumChannels, aRate));
+    ("%s  channels: %d, rate: %d for %p", __FUNCTION__, aNumChannels, aRate, this));
   mInRate = mOutRate = aRate;
   mChannels = aNumChannels;
   mOutChannels = (aNumChannels > 2) ? 2 : aNumChannels;
@@ -377,7 +500,7 @@ AudioStream::Init(int32_t aNumChannels, int32_t aRate,
   params.channels = mOutChannels;
 #if defined(__ANDROID__)
 #if defined(MOZ_B2G)
-  params.stream_type = ConvertChannelToCubebType(aAudioChannelType);
+  params.stream_type = ConvertChannelToCubebType(aAudioChannel);
 #else
   params.stream_type = CUBEB_STREAM_TYPE_MUSIC;
 #endif
@@ -395,12 +518,59 @@ AudioStream::Init(int32_t aNumChannels, int32_t aRate,
 
   mAudioClock.Init();
 
+  // Size mBuffer for one second of audio.  This value is arbitrary, and was
+  // selected based on the observed behaviour of the existing AudioStream
+  // implementations.
+  uint32_t bufferLimit = FramesToBytes(aRate);
+  NS_ABORT_IF_FALSE(bufferLimit % mBytesPerFrame == 0, "Must buffer complete frames");
+  mBuffer.SetCapacity(bufferLimit);
+
+  if (aLatencyRequest == LowLatency) {
+    // Don't block this thread to initialize a cubeb stream.
+    // When this is done, it will start callbacks from Cubeb.  Those will
+    // cause us to move from INITIALIZED to RUNNING.  Until then, we
+    // can't access any cubeb functions.
+    // Use a RefPtr to avoid leaks if Dispatch fails
+    mPendingAudioInitTask = true;
+    RefPtr<AudioInitTask> init = new AudioInitTask(this, aLatencyRequest, params);
+    nsresult rv = init->Dispatch();
+    if (NS_FAILED(rv)) {
+      mPendingAudioInitTask = false;
+    }
+    return rv;
+  }
+  // High latency - open synchronously
+  nsresult rv = OpenCubeb(params, aLatencyRequest);
+  NS_ENSURE_SUCCESS(rv, rv);
+  // See if we need to start() the stream, since we must do that from this
+  // thread for now (cubeb API issue)
+  {
+    MonitorAutoLock mon(mMonitor);
+    CheckForStart();
+  }
+  return NS_OK;
+}
+
+// This code used to live inside AudioStream::Init(), but on Mac (others?)
+// it has been known to take 300-800 (or even 8500) ms to execute(!)
+nsresult
+AudioStream::OpenCubeb(cubeb_stream_params &aParams,
+                       LatencyRequest aLatencyRequest)
+{
+  cubeb* cubebContext = GetCubebContext();
+  if (!cubebContext) {
+    NS_WARNING("Can't get cubeb context!");
+    MonitorAutoLock mon(mMonitor);
+    mState = AudioStream::ERRORED;
+    return NS_ERROR_FAILURE;
+  }
+
   // If the latency pref is set, use it. Otherwise, if this stream is intended
   // for low latency playback, try to get the lowest latency possible.
   // Otherwise, for normal streams, use 100ms.
   uint32_t latency;
   if (aLatencyRequest == LowLatency && !CubebLatencyPrefSet()) {
-    if (cubeb_get_min_latency(cubebContext, params, &latency) != CUBEB_OK) {
+    if (cubeb_get_min_latency(cubebContext, aParams, &latency) != CUBEB_OK) {
       latency = GetCubebLatency();
     }
   } else {
@@ -409,42 +579,85 @@ AudioStream::Init(int32_t aNumChannels, int32_t aRate,
 
   {
     cubeb_stream* stream;
-    if (cubeb_stream_init(cubebContext, &stream, "AudioStream", params,
+    if (cubeb_stream_init(cubebContext, &stream, "AudioStream", aParams,
                           latency, DataCallback_S, StateCallback_S, this) == CUBEB_OK) {
+      MonitorAutoLock mon(mMonitor);
+      MOZ_ASSERT(mState != SHUTDOWN);
       mCubebStream.own(stream);
+
+      // We can't cubeb_stream_start() the thread from a transient thread due to
+      // cubeb API requirements (init can be called from another thread, but
+      // not start/stop/destroy/etc)
+    } else {
+      MonitorAutoLock mon(mMonitor);
+      mState = ERRORED;
+      NS_WARNING(nsPrintfCString("AudioStream::OpenCubeb() %p failed to init cubeb", this).get());
+      return NS_ERROR_FAILURE;
     }
   }
 
-  if (!mCubebStream) {
-    return NS_ERROR_FAILURE;
-  }
-
-  // Size mBuffer for one second of audio.  This value is arbitrary, and was
-  // selected based on the observed behaviour of the existing AudioStream
-  // implementations.
-  uint32_t bufferLimit = FramesToBytes(aRate);
-  NS_ABORT_IF_FALSE(bufferLimit % mBytesPerFrame == 0, "Must buffer complete frames");
-  mBuffer.SetCapacity(bufferLimit);
-
-  // Start the stream right away when low latency has been requested. This means
-  // that the DataCallback will feed silence to cubeb, until the first frames
-  // are writtent to this AudioStream.
-  if (mLatencyRequest == LowLatency) {
-    Start();
+  if (!mStartTime.IsNull()) {
+		TimeDuration timeDelta = TimeStamp::Now() - mStartTime;
+    LOG(("AudioStream creation time %sfirst: %u ms", mIsFirst ? "" : "not ",
+         (uint32_t) timeDelta.ToMilliseconds()));
+		Telemetry::Accumulate(mIsFirst ? Telemetry::AUDIOSTREAM_FIRST_OPEN_MS :
+                          Telemetry::AUDIOSTREAM_LATER_OPEN_MS, timeDelta.ToMilliseconds());
   }
 
   return NS_OK;
 }
 
 void
-AudioStream::Shutdown()
+AudioStream::AudioInitTaskFinished()
 {
-  if (mState == STARTED) {
-    Pause();
+  MonitorAutoLock mon(mMonitor);
+  mPendingAudioInitTask = false;
+  mon.NotifyAll();
+}
+
+void
+AudioStream::CheckForStart()
+{
+  mMonitor.AssertCurrentThreadOwns();
+  if (mState == INITIALIZED) {
+    // Start the stream right away when low latency has been requested. This means
+    // that the DataCallback will feed silence to cubeb, until the first frames
+    // are written to this AudioStream.  Also start if a start has been queued.
+    if (mLatencyRequest == LowLatency || mNeedsStart) {
+      StartUnlocked(); // mState = STARTED or ERRORED
+      mNeedsStart = false;
+      PR_LOG(gAudioStreamLog, PR_LOG_WARNING,
+             ("Started waiting %s-latency stream",
+              mLatencyRequest == LowLatency ? "low" : "high"));
+    } else {
+      // high latency, not full - OR Pause() was called before we got here
+      PR_LOG(gAudioStreamLog, PR_LOG_DEBUG,
+             ("Not starting waiting %s-latency stream",
+              mLatencyRequest == LowLatency ? "low" : "high"));
+    }
   }
-  if (mCubebStream) {
-    mCubebStream.reset();
+}
+
+NS_IMETHODIMP
+AudioInitTask::Run()
+{
+  MOZ_ASSERT(mThread);
+  if (NS_IsMainThread()) {
+    mThread->Shutdown(); // can't Shutdown from the thread itself, darn
+    // Don't null out mThread!
+    // See bug 999104.  We must hold a ref to the thread across Dispatch()
+    // since the internal mThread ref could be released while processing
+    // the Dispatch(), and Dispatch/PutEvent itself doesn't hold a ref; it
+    // assumes the caller does.
+    return NS_OK;
   }
+
+  nsresult rv = mAudioStream->OpenCubeb(mParams, mLatencyRequest);
+  mAudioStream->AudioInitTaskFinished();
+
+  // and now kill this thread
+  NS_DispatchToMainThread(this);
+  return rv;
 }
 
 // aTime is the time in ms the samples were inserted into MediaStreamGraph
@@ -452,12 +665,15 @@ nsresult
 AudioStream::Write(const AudioDataValue* aBuf, uint32_t aFrames, TimeStamp *aTime)
 {
   MonitorAutoLock mon(mMonitor);
-  if (!mCubebStream || mState == ERRORED) {
+  if (mState == ERRORED) {
     return NS_ERROR_FAILURE;
   }
-  NS_ASSERTION(mState == INITIALIZED || mState == STARTED,
+  NS_ASSERTION(mState == INITIALIZED || mState == STARTED || mState == RUNNING,
     "Stream write in unexpected state.");
 
+  // See if we need to start() the stream, since we must do that from this thread
+  CheckForStart();
+
   // Downmix to Stereo.
   if (mChannels > 2 && mChannels <= 8) {
     DownmixAudioToStereo(const_cast<AudioDataValue*> (aBuf), mChannels, aFrames);
@@ -495,15 +711,33 @@ AudioStream::Write(const AudioDataValue* aBuf, uint32_t aFrames, TimeStamp *aTim
     bytesToCopy -= available;
 
     if (bytesToCopy > 0) {
-      // If we are not playing, but our buffer is full, start playing to make
-      // room for soon-to-be-decoded data.
-      if (mState != STARTED) {
-        StartUnlocked();
-        if (mState != STARTED) {
-          return NS_ERROR_FAILURE;
+      // Careful - the CubebInit thread may not have gotten to STARTED yet
+      if ((mState == INITIALIZED || mState == STARTED) && mLatencyRequest == LowLatency) {
+        // don't ever block MediaStreamGraph low-latency streams
+        uint32_t remains = 0; // we presume the buffer is full
+        if (mBuffer.Length() > bytesToCopy) {
+          remains = mBuffer.Length() - bytesToCopy; // Free up just enough space
         }
+        // account for dropping samples
+        PR_LOG(gAudioStreamLog, PR_LOG_WARNING, ("Stream %p dropping %u bytes (%u frames)in Write()",
+            this, mBuffer.Length() - remains, BytesToFrames(mBuffer.Length() - remains)));
+        mReadPoint += BytesToFrames(mBuffer.Length() - remains);
+        mBuffer.ContractTo(remains);
+      } else { // RUNNING or high latency
+        // If we are not playing, but our buffer is full, start playing to make
+        // room for soon-to-be-decoded data.
+        if (mState != STARTED && mState != RUNNING) {
+          PR_LOG(gAudioStreamLog, PR_LOG_WARNING, ("Starting stream %p in Write (%u waiting)",
+                                                 this, bytesToCopy));
+          StartUnlocked();
+          if (mState == ERRORED) {
+            return NS_ERROR_FAILURE;
+          }
+        }
+        PR_LOG(gAudioStreamLog, PR_LOG_WARNING, ("Stream %p waiting in Write() (%u waiting)",
+                                                 this, bytesToCopy));
+        mon.Wait();
       }
-      mon.Wait();
     }
   }
 
@@ -527,12 +761,21 @@ AudioStream::SetVolume(double aVolume)
   mVolume = aVolume;
 }
 
+void
+AudioStream::Cancel()
+{
+  MonitorAutoLock mon(mMonitor);
+  mState = ERRORED;
+  mon.NotifyAll();
+}
+
 void
 AudioStream::Drain()
 {
   MonitorAutoLock mon(mMonitor);
-  if (mState != STARTED) {
-    NS_ASSERTION(mBuffer.Available() == 0, "Draining with unplayed audio");
+  LOG(("AudioStream::Drain() for %p, state %d, avail %u", this, mState, mBuffer.Available()));
+  if (mState != STARTED && mState != RUNNING) {
+    NS_ASSERTION(mState == ERRORED || mBuffer.Available() == 0, "Draining without full buffer of unplayed audio");
     return;
   }
   mState = DRAINING;
@@ -552,18 +795,18 @@ void
 AudioStream::StartUnlocked()
 {
   mMonitor.AssertCurrentThreadOwns();
-  if (!mCubebStream || mState != INITIALIZED) {
+  if (!mCubebStream) {
+    mNeedsStart = true;
     return;
   }
-  if (mState != STARTED) {
+  if (mState == INITIALIZED) {
     int r;
     {
       MonitorAutoUnlock mon(mMonitor);
       r = cubeb_stream_start(mCubebStream);
     }
-    if (mState != ERRORED) {
-      mState = r == CUBEB_OK ? STARTED : ERRORED;
-    }
+    mState = r == CUBEB_OK ? STARTED : ERRORED;
+    LOG(("AudioStream: started %p, state %s", this, mState == STARTED ? "STARTED" : "ERRORED"));
   }
 }
 
@@ -571,7 +814,14 @@ void
 AudioStream::Pause()
 {
   MonitorAutoLock mon(mMonitor);
-  if (!mCubebStream || mState != STARTED) {
+
+  if (mState == ERRORED) {
+    return;
+  }
+
+  if (!mCubebStream || (mState != STARTED && mState != RUNNING)) {
+    mNeedsStart = false;
+    mState = STOPPED; // which also tells async OpenCubeb not to start, just init
     return;
   }
 
@@ -603,6 +853,28 @@ AudioStream::Resume()
   }
 }
 
+void
+AudioStream::Shutdown()
+{
+  MonitorAutoLock mon(mMonitor);
+  LOG(("AudioStream: Shutdown %p, state %d", this, mState));
+
+  while (mPendingAudioInitTask) {
+    mon.Wait();
+  }
+
+  if (mCubebStream) {
+    MonitorAutoUnlock mon(mMonitor);
+    // Force stop to put the cubeb stream in a stable state before deletion.
+    cubeb_stream_stop(mCubebStream);
+    // Must not try to shut down cubeb from within the lock!  wasapi may still
+    // call our callback after Pause()/stop()!?! Bug 996162
+    mCubebStream.reset();
+  }
+
+  mState = SHUTDOWN;
+}
+
 int64_t
 AudioStream::GetPosition()
 {
@@ -618,19 +890,12 @@ int64_t
 AudioStream::GetPositionInFrames()
 {
   MonitorAutoLock mon(mMonitor);
-  return mAudioClock.GetPositionInFramesUnlocked();
+  return mAudioClock.GetPositionInFrames();
 }
 #ifdef _MSC_VER
 #pragma optimize("", on)
 #endif
 
-int64_t
-AudioStream::GetPositionInFramesInternal()
-{
-  MonitorAutoLock mon(mMonitor);
-  return GetPositionInFramesUnlocked();
-}
-
 int64_t
 AudioStream::GetPositionInFramesUnlocked()
 {
@@ -648,13 +913,7 @@ AudioStream::GetPositionInFramesUnlocked()
     }
   }
 
-  // Adjust the reported position by the number of silent frames written
-  // during stream underruns.
-  uint64_t adjustedPosition = 0;
-  if (position >= mLostFrames) {
-    adjustedPosition = position - mLostFrames;
-  }
-  return std::min<uint64_t>(adjustedPosition, INT64_MAX);
+  return std::min<uint64_t>(position, INT64_MAX);
 }
 
 int64_t
@@ -678,6 +937,7 @@ AudioStream::IsPaused()
 void
 AudioStream::GetBufferInsertTime(int64_t &aTimeMs)
 {
+  mMonitor.AssertCurrentThreadOwns();
   if (mInserts.Length() > 0) {
     // Find the right block, but don't leave the array empty
     while (mInserts.Length() > 1 && mReadPoint >= mInserts[0].mFrames) {
@@ -695,6 +955,7 @@ AudioStream::GetBufferInsertTime(int64_t &aTimeMs)
 long
 AudioStream::GetUnprocessed(void* aBuffer, long aFrames, int64_t &aTimeMs)
 {
+  mMonitor.AssertCurrentThreadOwns();
   uint8_t* wpos = reinterpret_cast<uint8_t*>(aBuffer);
 
   // Flush the timestretcher pipeline, if we were playing using a playback rate
@@ -726,6 +987,7 @@ AudioStream::GetUnprocessed(void* aBuffer, long aFrames, int64_t &aTimeMs)
 long
 AudioStream::GetUnprocessedWithSilencePadding(void* aBuffer, long aFrames, int64_t& aTimeMs)
 {
+  mMonitor.AssertCurrentThreadOwns();
   uint32_t toPopBytes = FramesToBytes(aFrames);
   uint32_t available = std::min(toPopBytes, mBuffer.Length());
   uint32_t silenceOffset = toPopBytes - available;
@@ -750,6 +1012,7 @@ AudioStream::GetUnprocessedWithSilencePadding(void* aBuffer, long aFrames, int64
 long
 AudioStream::GetTimeStretched(void* aBuffer, long aFrames, int64_t &aTimeMs)
 {
+  mMonitor.AssertCurrentThreadOwns();
   long processedFrames = 0;
 
   // We need to call the non-locking version, because we already have the lock.
@@ -759,7 +1022,7 @@ AudioStream::GetTimeStretched(void* aBuffer, long aFrames, int64_t &aTimeMs)
 
   uint8_t* wpos = reinterpret_cast<uint8_t*>(aBuffer);
   double playbackRate = static_cast<double>(mInRate) / mOutRate;
-  uint32_t toPopBytes = FramesToBytes(ceil(aFrames / playbackRate));
+  uint32_t toPopBytes = FramesToBytes(ceil(aFrames * playbackRate));
   uint32_t available = 0;
   bool lowOnBufferedData = false;
   do {
@@ -792,6 +1055,7 @@ long
 AudioStream::DataCallback(void* aBuffer, long aFrames)
 {
   MonitorAutoLock mon(mMonitor);
+  MOZ_ASSERT(mState != SHUTDOWN, "No data callback after shutdown");
   uint32_t available = std::min(static_cast<uint32_t>(FramesToBytes(aFrames)), mBuffer.Length());
   NS_ABORT_IF_FALSE(available % mBytesPerFrame == 0, "Must copy complete frames");
   AudioDataValue* output = reinterpret_cast<AudioDataValue*>(aBuffer);
@@ -799,6 +1063,42 @@ AudioStream::DataCallback(void* aBuffer, long aFrames)
   uint32_t servicedFrames = 0;
   int64_t insertTime;
 
+  // NOTE: wasapi (others?) can call us back *after* stop()/Shutdown() (mState == SHUTDOWN)
+  // Bug 996162
+
+  // callback tells us cubeb succeeded initializing
+  if (mState == STARTED) {
+    // For low-latency streams, we want to minimize any built-up data when
+    // we start getting callbacks.
+    // Simple version - contract on first callback only.
+    if (mLatencyRequest == LowLatency) {
+#ifdef PR_LOGGING
+      uint32_t old_len = mBuffer.Length();
+#endif
+      available = mBuffer.ContractTo(FramesToBytes(aFrames));
+#ifdef PR_LOGGING
+      TimeStamp now = TimeStamp::Now();
+      if (!mStartTime.IsNull()) {
+        int64_t timeMs = (now - mStartTime).ToMilliseconds();
+        PR_LOG(gAudioStreamLog, PR_LOG_WARNING,
+               ("Stream took %lldms to start after first Write() @ %u", timeMs, mOutRate));
+      } else {
+        PR_LOG(gAudioStreamLog, PR_LOG_WARNING,
+          ("Stream started before Write() @ %u", mOutRate));
+      }
+
+      if (old_len != available) {
+        // Note that we may have dropped samples in Write() as well!
+        PR_LOG(gAudioStreamLog, PR_LOG_WARNING,
+               ("AudioStream %p dropped %u + %u initial frames @ %u", this,
+                 mReadPoint, BytesToFrames(old_len - available), mOutRate));
+        mReadPoint += BytesToFrames(old_len - available);
+      }
+#endif
+    }
+    mState = RUNNING;
+  }
+
   if (available) {
     // When we are playing a low latency stream, and it is the first time we are
     // getting data from the buffer, we prefer to add the silence for an
@@ -827,20 +1127,25 @@ AudioStream::DataCallback(void* aBuffer, long aFrames)
 
   underrunFrames = aFrames - servicedFrames;
 
+  // Always send audible frames first, and silent frames later.
+  // Otherwise it will break the assumption of FrameHistory.
   if (mState != DRAINING) {
+    mAudioClock.UpdateFrameHistory(servicedFrames, underrunFrames);
     uint8_t* rpos = static_cast<uint8_t*>(aBuffer) + FramesToBytes(aFrames - underrunFrames);
     memset(rpos, 0, FramesToBytes(underrunFrames));
     if (underrunFrames) {
       PR_LOG(gAudioStreamLog, PR_LOG_WARNING,
              ("AudioStream %p lost %d frames", this, underrunFrames));
     }
-    mLostFrames += underrunFrames;
     servicedFrames += underrunFrames;
+  } else {
+    mAudioClock.UpdateFrameHistory(servicedFrames, 0);
   }
 
   WriteDumpFile(mDumpFile, this, aFrames, aBuffer);
   // Don't log if we're not interested or if the stream is inactive
   if (PR_LOG_TEST(GetLatencyLog(), PR_LOG_DEBUG) &&
+      mState != SHUTDOWN &&
       insertTime != INT64_MAX && servicedFrames > underrunFrames) {
     uint32_t latency = UINT32_MAX;
     if (cubeb_stream_get_latency(mCubebStream, &latency)) {
@@ -854,7 +1159,6 @@ AudioStream::DataCallback(void* aBuffer, long aFrames)
                      (latency * 1000) / mOutRate, now);
   }
 
-  mAudioClock.UpdateWritePosition(servicedFrames);
   return servicedFrames;
 }
 
@@ -862,9 +1166,12 @@ void
 AudioStream::StateCallback(cubeb_state aState)
 {
   MonitorAutoLock mon(mMonitor);
+  MOZ_ASSERT(mState != SHUTDOWN, "No state callback after shutdown");
+  LOG(("AudioStream: StateCallback %p, mState=%d cubeb_state=%d", this, mState, aState));
   if (aState == CUBEB_STATE_DRAINED) {
     mState = DRAINED;
   } else if (aState == CUBEB_STATE_ERROR) {
+    LOG(("AudioStream::StateCallback() state %d cubeb error", mState));
     mState = ERRORED;
   }
   mon.NotifyAll();
@@ -872,94 +1179,42 @@ AudioStream::StateCallback(cubeb_state aState)
 
 AudioClock::AudioClock(AudioStream* aStream)
  :mAudioStream(aStream),
-  mOldOutRate(0),
-  mBasePosition(0),
-  mBaseOffset(0),
-  mOldBaseOffset(0),
-  mOldBasePosition(0),
-  mPlaybackRateChangeOffset(0),
-  mPreviousPosition(0),
-  mWritten(0),
   mOutRate(0),
   mInRate(0),
   mPreservesPitch(true),
-  mCompensatingLatency(false)
+  mFrameHistory(new FrameHistory())
 {}
 
 void AudioClock::Init()
 {
   mOutRate = mAudioStream->GetRate();
   mInRate = mAudioStream->GetRate();
-  mOldOutRate = mOutRate;
 }
 
-void AudioClock::UpdateWritePosition(uint32_t aCount)
+void AudioClock::UpdateFrameHistory(uint32_t aServiced, uint32_t aUnderrun)
 {
-  mWritten += aCount;
+  mFrameHistory->Append(aServiced, aUnderrun, mOutRate);
 }
 
-uint64_t AudioClock::GetPositionUnlocked()
+int64_t AudioClock::GetPositionUnlocked() const
 {
   // GetPositionInFramesUnlocked() asserts it owns the monitor
-  int64_t position = mAudioStream->GetPositionInFramesUnlocked();
-  int64_t diffOffset;
-  NS_ASSERTION(position < 0 || (mInRate != 0 && mOutRate != 0), "AudioClock not initialized.");
-  if (position >= 0) {
-    if (position < mPlaybackRateChangeOffset) {
-      // See if we are still playing frames pushed with the old playback rate in
-      // the backend. If we are, use the old output rate to compute the
-      // position.
-      mCompensatingLatency = true;
-      diffOffset = position - mOldBaseOffset;
-      position = static_cast<uint64_t>(mOldBasePosition +
-        static_cast<float>(USECS_PER_S * diffOffset) / mOldOutRate);
-      mPreviousPosition = position;
-      return position;
-    }
-
-    if (mCompensatingLatency) {
-      diffOffset = position - mPlaybackRateChangeOffset;
-      mCompensatingLatency = false;
-      mBasePosition = mPreviousPosition;
-    } else {
-      diffOffset = position - mPlaybackRateChangeOffset;
-    }
-    position =  static_cast<uint64_t>(mBasePosition +
-      (static_cast<float>(USECS_PER_S * diffOffset) / mOutRate));
-    return position;
-  }
-  return UINT64_MAX;
+  int64_t frames = mAudioStream->GetPositionInFramesUnlocked();
+  NS_ASSERTION(frames < 0 || (mInRate != 0 && mOutRate != 0), "AudioClock not initialized.");
+  return frames >= 0 ? mFrameHistory->GetPosition(frames) : -1;
 }
 
-uint64_t AudioClock::GetPositionInFramesUnlocked()
+int64_t AudioClock::GetPositionInFrames() const
 {
-  return (GetPositionUnlocked() * mOutRate) / USECS_PER_S;
+  return (GetPositionUnlocked() * mInRate) / USECS_PER_S;
 }
 
 void AudioClock::SetPlaybackRateUnlocked(double aPlaybackRate)
 {
-  // GetPositionInFramesUnlocked() asserts it owns the monitor
-  int64_t position = mAudioStream->GetPositionInFramesUnlocked();
-  if (position > mPlaybackRateChangeOffset) {
-    mOldBasePosition = mBasePosition;
-    mBasePosition = GetPositionUnlocked();
-    mOldBaseOffset = mPlaybackRateChangeOffset;
-    mBaseOffset = position;
-    mPlaybackRateChangeOffset = mWritten;
-    mOldOutRate = mOutRate;
-    mOutRate = static_cast<int>(mInRate / aPlaybackRate);
-  } else {
-    // The playbackRate has been changed before the end of the latency
-    // compensation phase. We don't update the mOld* variable. That way, the
-    // last playbackRate set is taken into account.
-    mBasePosition = GetPositionUnlocked();
-    mBaseOffset = position;
-    mPlaybackRateChangeOffset = mWritten;
-    mOutRate = static_cast<int>(mInRate / aPlaybackRate);
-  }
+  mOutRate = static_cast<int>(mInRate / aPlaybackRate);
 }
 
-double AudioClock::GetPlaybackRate()
+double AudioClock::GetPlaybackRate() const
 {
   return static_cast<double>(mInRate) / mOutRate;
 }
@@ -969,7 +1224,7 @@ void AudioClock::SetPreservesPitch(bool aPreservesPitch)
   mPreservesPitch = aPreservesPitch;
 }
 
-bool AudioClock::GetPreservesPitch()
+bool AudioClock::GetPreservesPitch() const
 {
   return mPreservesPitch;
 }
