@@ -7,33 +7,41 @@
 #ifndef mozilla_dom_MediaSource_h_
 #define mozilla_dom_MediaSource_h_
 
-#include "AsyncEventRunner.h"
+#include "MediaSourceDecoder.h"
+#include "js/RootingAPI.h"
+#include "mozilla/Assertions.h"
 #include "mozilla/Attributes.h"
-#include "mozilla/Monitor.h"
+#include "mozilla/DOMEventTargetHelper.h"
 #include "mozilla/dom/MediaSourceBinding.h"
-#include "mozilla/dom/TypedArray.h"
+#include "mozilla/Monitor.h"
 #include "nsCOMPtr.h"
+#include "nsCycleCollectionNoteChild.h"
 #include "nsCycleCollectionParticipant.h"
-#include "nsDOMEventTargetHelper.h"
-#include "nsWrapperCache.h"
+#include "nsID.h"
+#include "nsISupports.h"
 #include "nscore.h"
 
-class nsIInputStream;
+struct JSContext;
+class JSObject;
+class nsPIDOMWindow;
 
 namespace mozilla {
+
+class ErrorResult;
+template <typename T> class AsyncEventRunner;
+
 namespace dom {
 
-class HTMLMediaElement;
-class MediaSourceInputAdapter;
-class SourceBufferList;
+class GlobalObject;
 class SourceBuffer;
-class TimeRanges;
+class SourceBufferList;
+template <typename T> class Optional;
 
 #define MOZILLA_DOM_MEDIASOURCE_IMPLEMENTATION_IID \
   { 0x3839d699, 0x22c5, 0x439f, \
   { 0x94, 0xca, 0x0e, 0x0b, 0x26, 0xf9, 0xca, 0xbf } }
 
-class MediaSource MOZ_FINAL : public nsDOMEventTargetHelper
+class MediaSource MOZ_FINAL : public DOMEventTargetHelper
 {
 public:
   /** WebIDL Methods. */
@@ -52,88 +60,69 @@ public:
   void RemoveSourceBuffer(SourceBuffer& aSourceBuffer, ErrorResult& aRv);
 
   void EndOfStream(const Optional<MediaSourceEndOfStreamError>& aError, ErrorResult& aRv);
-  static bool IsTypeSupported(const GlobalObject& aGlobal,
-                              const nsAString& aType);
+  static bool IsTypeSupported(const GlobalObject&, const nsAString& aType);
   /** End WebIDL Methods. */
 
   NS_DECL_ISUPPORTS_INHERITED
-  NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(MediaSource, nsDOMEventTargetHelper)
+  NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(MediaSource, DOMEventTargetHelper)
   NS_DECLARE_STATIC_IID_ACCESSOR(MOZILLA_DOM_MEDIASOURCE_IMPLEMENTATION_IID)
 
   nsPIDOMWindow* GetParentObject() const;
 
-  JSObject* WrapObject(JSContext* aCx, JS::Handle<JSObject*> aScope) MOZ_OVERRIDE;
+  JSObject* WrapObject(JSContext* aCx) MOZ_OVERRIDE;
 
-  const nsString& GetType()
-  {
-    return mContentType;
-  }
-
-  already_AddRefed<nsIInputStream> CreateInternalStream();
-
-
-  void AppendData(const uint8_t* aData, uint32_t aLength, ErrorResult& aRv);
-
-  // Semi-private, for MediaSourceInputAdapter only.
-  nsTArray<uint8_t> const& GetData()
-  {
-    return mData;
-  }
-
-  Monitor& GetMonitor()
-  {
-    return mMonitor;
-  }
-
-  bool AppendDone() const
-  {
-    return mReadyState == MediaSourceReadyState::Closed || mReadyState == MediaSourceReadyState::Ended;
-  }
-
-  // Attach this MediaSource to MediaElement aElement.  Returns false if already attached.
-  bool AttachElement(HTMLMediaElement* aElement);
-  void DetachElement();
+  // Attach this MediaSource to Decoder aDecoder.  Returns false if already attached.
+  bool Attach(MediaSourceDecoder* aDecoder);
+  void Detach();
 
   // Set mReadyState to aState and fire the required events at the MediaSource.
   void SetReadyState(MediaSourceReadyState aState);
 
-  void GetBuffered(TimeRanges* aRanges);
+ // Used by SourceBuffer to call CreateSubDecoder.
+  MediaSourceDecoder* GetDecoder()
+  {
+    return mDecoder;
+  }
+
+  // Called by SourceBuffers to notify this MediaSource that data has
+  // been evicted from the buffered data. The start and end times
+  // that were evicted are provided.
+  void NotifyEvicted(double aStart, double aEnd);
+
+  // Block thread waiting for data to be appended to a SourceBuffer.
+  void WaitForData();
+
+  // Unblock threads waiting for data to be appended to a SourceBuffer.
+  void NotifyGotData();
 
 private:
+  ~MediaSource();
+
   explicit MediaSource(nsPIDOMWindow* aWindow);
 
-  friend class AsyncEventRunnner<MediaSource>;
+  friend class AsyncEventRunner<MediaSource>;
   void DispatchSimpleEvent(const char* aName);
   void QueueAsyncSimpleEvent(const char* aName);
 
-  void NotifyListeners();
-
   void DurationChange(double aNewDuration, ErrorResult& aRv);
-  void EndOfStreamInternal(const Optional<MediaSourceEndOfStreamError>& aError, ErrorResult& aRv);
-
-  static bool IsTypeSupportedInternal(const nsAString& aType, ErrorResult& aRv);
 
   double mDuration;
 
-  nsTArray<nsRefPtr<MediaSourceInputAdapter> > mAdapters;
-
-  // Protected by monitor.
-  nsTArray<uint8_t> mData;
-
-  // Protects access to mData.
-  Monitor mMonitor;
-
   nsRefPtr<SourceBufferList> mSourceBuffers;
   nsRefPtr<SourceBufferList> mActiveSourceBuffers;
 
-  nsRefPtr<HTMLMediaElement> mElement;
+  nsRefPtr<MediaSourceDecoder> mDecoder;
 
-  nsString mContentType;
   MediaSourceReadyState mReadyState;
+
+  // Monitor for waiting for when new data is appended to
+  // a Source Buffer.
+  Monitor mWaitForDataMonitor;
 };
 
 NS_DEFINE_STATIC_IID_ACCESSOR(MediaSource, MOZILLA_DOM_MEDIASOURCE_IMPLEMENTATION_IID)
 
 } // namespace dom
+
 } // namespace mozilla
 #endif /* mozilla_dom_MediaSource_h_ */
