@@ -10,13 +10,14 @@
 #include "AudioNodeStream.h"
 #include "AudioDestinationNode.h"
 #include "WebAudioUtils.h"
-#include "DelayProcessor.h"
+#include "DelayBuffer.h"
+#include "PlayingRefChangeHandler.h"
 
 namespace mozilla {
 namespace dom {
 
-NS_IMPL_CYCLE_COLLECTION_INHERITED_1(DelayNode, AudioNode,
-                                     mDelay)
+NS_IMPL_CYCLE_COLLECTION_INHERITED(DelayNode, AudioNode,
+                                   mDelay)
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION_INHERITED(DelayNode)
 NS_INTERFACE_MAP_END_INHERITING(AudioNode)
@@ -26,19 +27,22 @@ NS_IMPL_RELEASE_INHERITED(DelayNode, AudioNode)
 
 class DelayNodeEngine : public AudioNodeEngine
 {
-  typedef PlayingRefChangeHandler<DelayNode> PlayingRefChanged;
+  typedef PlayingRefChangeHandler PlayingRefChanged;
 public:
   DelayNodeEngine(AudioNode* aNode, AudioDestinationNode* aDestination,
-                  int aMaxDelayFrames)
+                  double aMaxDelayTicks)
     : AudioNodeEngine(aNode)
     , mSource(nullptr)
     , mDestination(static_cast<AudioNodeStream*> (aDestination->Stream()))
     // Keep the default value in sync with the default value in DelayNode::DelayNode.
     , mDelay(0.f)
     // Use a smoothing range of 20ms
-    , mProcessor(aMaxDelayFrames,
-                 WebAudioUtils::ComputeSmoothingRate(0.02,
-                                                     mDestination->SampleRate()))
+    , mBuffer(std::max(aMaxDelayTicks,
+                       static_cast<double>(WEBAUDIO_BLOCK_SIZE)),
+              WebAudioUtils::ComputeSmoothingRate(0.02,
+                                                  mDestination->SampleRate()))
+    , mMaxDelay(aMaxDelayTicks)
+    , mHaveProducedBeforeInput(false)
     , mLeftOverData(INT32_MIN)
   {
   }
@@ -71,97 +75,111 @@ public:
     }
   }
 
-  virtual void ProduceAudioBlock(AudioNodeStream* aStream,
-                                 const AudioChunk& aInput,
-                                 AudioChunk* aOutput,
-                                 bool* aFinished)
+  virtual void ProcessBlock(AudioNodeStream* aStream,
+                            const AudioChunk& aInput,
+                            AudioChunk* aOutput,
+                            bool* aFinished) MOZ_OVERRIDE
   {
     MOZ_ASSERT(mSource == aStream, "Invalid source stream");
     MOZ_ASSERT(aStream->SampleRate() == mDestination->SampleRate());
 
-    const uint32_t numChannels = aInput.IsNull() ?
-                                 mProcessor.BufferChannelCount() :
-                                 aInput.mChannelData.Length();
-
-    bool playedBackAllLeftOvers = false;
     if (!aInput.IsNull()) {
       if (mLeftOverData <= 0) {
         nsRefPtr<PlayingRefChanged> refchanged =
           new PlayingRefChanged(aStream, PlayingRefChanged::ADDREF);
-        NS_DispatchToMainThread(refchanged);
+        aStream->Graph()->
+          DispatchToMainThreadAfterStreamStateUpdate(refchanged.forget());
       }
-      mLeftOverData = mProcessor.MaxDelayFrames();
-    } else if (mLeftOverData != INT32_MIN) {
+      mLeftOverData = mBuffer.MaxDelayTicks();
+    } else if (mLeftOverData > 0) {
       mLeftOverData -= WEBAUDIO_BLOCK_SIZE;
-      if (mLeftOverData <= 0) {
-        // Continue spamming the main thread with messages until we are destroyed.
-        // This isn't great, but it ensures a message will get through even if
-        // some are ignored by DelayNode::AcceptPlayingRefRelease
-        mLeftOverData = 0;
-        playedBackAllLeftOvers = true;
+    } else {
+      if (mLeftOverData != INT32_MIN) {
+        mLeftOverData = INT32_MIN;
+        // Delete our buffered data now we no longer need it
+        mBuffer.Reset();
 
         nsRefPtr<PlayingRefChanged> refchanged =
           new PlayingRefChanged(aStream, PlayingRefChanged::RELEASE);
-        NS_DispatchToMainThread(refchanged);
+        aStream->Graph()->
+          DispatchToMainThreadAfterStreamStateUpdate(refchanged.forget());
       }
+      *aOutput = aInput;
+      return;
     }
 
-    AllocateAudioBlock(numChannels, aOutput);
+    mBuffer.Write(aInput);
 
-    AudioChunk input = aInput;
-    if (!aInput.IsNull() && aInput.mVolume != 1.0f) {
-      // Pre-multiply the input's volume
-      AllocateAudioBlock(numChannels, &input);
-      for (uint32_t i = 0; i < numChannels; ++i) {
-        const float* src = static_cast<const float*>(aInput.mChannelData[i]);
-        float* dest = static_cast<float*>(const_cast<void*>(input.mChannelData[i]));
-        AudioBlockCopyChannelWithScale(src, aInput.mVolume, dest);
-      }
+    // Skip output update if mLastChunks has already been set by
+    // ProduceBlockBeforeInput() when in a cycle.
+    if (!mHaveProducedBeforeInput) {
+      UpdateOutputBlock(aOutput, 0.0);
     }
+    mHaveProducedBeforeInput = false;
+    mBuffer.NextBlock();
+  }
 
-    const float* const* inputChannels = input.IsNull() ? nullptr :
-      reinterpret_cast<const float* const*>(input.mChannelData.Elements());
-    float* const* outputChannels = reinterpret_cast<float* const*>
-      (const_cast<void* const*>(aOutput->mChannelData.Elements()));
-
-
-    bool inCycle = aStream->AsProcessedStream()->InCycle();
-    double sampleRate = aStream->SampleRate();
+  void UpdateOutputBlock(AudioChunk* aOutput, double minDelay)
+  {
+    double maxDelay = mMaxDelay;
+    double sampleRate = mSource->SampleRate();
+    ChannelInterpretation channelInterpretation =
+      mSource->GetChannelInterpretation();
     if (mDelay.HasSimpleValue()) {
       // If this DelayNode is in a cycle, make sure the delay value is at least
-      // one block.
-      float delayFrames = mDelay.GetValue() * sampleRate;
-      float delayFramesClamped = inCycle ? std::max(static_cast<float>(WEBAUDIO_BLOCK_SIZE), delayFrames) :
-                                           delayFrames;
-      mProcessor.Process(delayFramesClamped, inputChannels, outputChannels,
-                         numChannels, WEBAUDIO_BLOCK_SIZE);
+      // one block, even if that is greater than maxDelay.
+      double delayFrames = mDelay.GetValue() * sampleRate;
+      double delayFramesClamped =
+        std::max(minDelay, std::min(delayFrames, maxDelay));
+      mBuffer.Read(delayFramesClamped, aOutput, channelInterpretation);
     } else {
       // Compute the delay values for the duration of the input AudioChunk
       // If this DelayNode is in a cycle, make sure the delay value is at least
       // one block.
+      TrackTicks tick = mSource->GetCurrentPosition();
       double computedDelay[WEBAUDIO_BLOCK_SIZE];
-      TrackTicks tick = aStream->GetCurrentPosition();
       for (size_t counter = 0; counter < WEBAUDIO_BLOCK_SIZE; ++counter) {
-        float delayAtTick = mDelay.GetValueAtTime(tick, counter) * sampleRate;
-        float delayAtTickClamped = inCycle ? std::max(static_cast<float>(WEBAUDIO_BLOCK_SIZE), delayAtTick) :
-                                             delayAtTick;
+        double delayAtTick = mDelay.GetValueAtTime(tick, counter) * sampleRate;
+        double delayAtTickClamped =
+          std::max(minDelay, std::min(delayAtTick, maxDelay));
         computedDelay[counter] = delayAtTickClamped;
       }
-      mProcessor.Process(computedDelay, inputChannels, outputChannels,
-                         numChannels, WEBAUDIO_BLOCK_SIZE);
+      mBuffer.Read(computedDelay, aOutput, channelInterpretation);
     }
+  }
 
-
-    if (playedBackAllLeftOvers) {
-      // Delete our buffered data once we no longer need it
-      mProcessor.Reset();
+  virtual void ProduceBlockBeforeInput(AudioChunk* aOutput) MOZ_OVERRIDE
+  {
+    if (mLeftOverData <= 0) {
+      aOutput->SetNull(WEBAUDIO_BLOCK_SIZE);
+    } else {
+      UpdateOutputBlock(aOutput, WEBAUDIO_BLOCK_SIZE);
     }
+    mHaveProducedBeforeInput = true;
+  }
+
+  virtual size_t SizeOfExcludingThis(MallocSizeOf aMallocSizeOf) const MOZ_OVERRIDE
+  {
+    size_t amount = AudioNodeEngine::SizeOfExcludingThis(aMallocSizeOf);
+    // Not owned:
+    // - mSource - probably not owned
+    // - mDestination - probably not owned
+    // - mDelay - shares ref with AudioNode, don't count
+    amount += mBuffer.SizeOfExcludingThis(aMallocSizeOf);
+    return amount;
+  }
+
+  virtual size_t SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const MOZ_OVERRIDE
+  {
+    return aMallocSizeOf(this) + SizeOfExcludingThis(aMallocSizeOf);
   }
 
   AudioNodeStream* mSource;
   AudioNodeStream* mDestination;
   AudioParamTimeline mDelay;
-  DelayProcessor mProcessor;
+  DelayBuffer mBuffer;
+  double mMaxDelay;
+  bool mHaveProducedBeforeInput;
   // How much data we have in our buffer which needs to be flushed out when our inputs
   // finish.
   int32_t mLeftOverData;
@@ -172,21 +190,38 @@ DelayNode::DelayNode(AudioContext* aContext, double aMaxDelay)
               2,
               ChannelCountMode::Max,
               ChannelInterpretation::Speakers)
-  , mMediaStreamGraphUpdateIndexAtLastInputConnection(0)
   , mDelay(new AudioParam(MOZ_THIS_IN_INITIALIZER_LIST(),
                           SendDelayToStream, 0.0f))
 {
   DelayNodeEngine* engine =
     new DelayNodeEngine(this, aContext->Destination(),
-                        ceil(aContext->SampleRate() * aMaxDelay));
+                        aContext->SampleRate() * aMaxDelay);
   mStream = aContext->Graph()->CreateAudioNodeStream(engine, MediaStreamGraph::INTERNAL_STREAM);
   engine->SetSourceStream(static_cast<AudioNodeStream*> (mStream.get()));
 }
 
-JSObject*
-DelayNode::WrapObject(JSContext* aCx, JS::Handle<JSObject*> aScope)
+DelayNode::~DelayNode()
 {
-  return DelayNodeBinding::Wrap(aCx, aScope, this);
+}
+
+size_t
+DelayNode::SizeOfExcludingThis(MallocSizeOf aMallocSizeOf) const
+{
+  size_t amount = AudioNode::SizeOfExcludingThis(aMallocSizeOf);
+  amount += mDelay->SizeOfIncludingThis(aMallocSizeOf);
+  return amount;
+}
+
+size_t
+DelayNode::SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const
+{
+  return aMallocSizeOf(this) + SizeOfExcludingThis(aMallocSizeOf);
+}
+
+JSObject*
+DelayNode::WrapObject(JSContext* aCx)
+{
+  return DelayNodeBinding::Wrap(aCx, this);
 }
 
 void
