@@ -9,15 +9,14 @@
 #include "WMFUtils.h"
 #include "WMFByteStream.h"
 #include "WMFSourceReaderCallback.h"
+#include "mozilla/ArrayUtils.h"
 #include "mozilla/dom/TimeRanges.h"
+#include "mozilla/dom/HTMLMediaElement.h"
 #include "mozilla/Preferences.h"
 #include "DXVA2Manager.h"
 #include "ImageContainer.h"
 #include "Layers.h"
 #include "mozilla/layers/LayersTypes.h"
-#include "WinUtils.h"
-
-using namespace mozilla::widget;
 
 #ifndef MOZ_SAMPLE_TYPE_FLOAT32
 #error We expect 32bit float audio samples on desktop for the Windows Media Foundation media backend.
@@ -25,7 +24,9 @@ using namespace mozilla::widget;
 
 #include "MediaDecoder.h"
 #include "VideoUtils.h"
+#include "gfx2DGlue.h"
 
+using namespace mozilla::gfx;
 using mozilla::layers::Image;
 using mozilla::layers::LayerManager;
 using mozilla::layers::LayersBackend;
@@ -34,9 +35,9 @@ namespace mozilla {
 
 #ifdef PR_LOGGING
 extern PRLogModuleInfo* gMediaDecoderLog;
-#define LOG(...) PR_LOG(gMediaDecoderLog, PR_LOG_DEBUG, (__VA_ARGS__))
+#define DECODER_LOG(...) PR_LOG(gMediaDecoderLog, PR_LOG_DEBUG, (__VA_ARGS__))
 #else
-#define LOG(...)
+#define DECODER_LOG(...)
 #endif
 
 // Uncomment to enable verbose per-sample logging.
@@ -55,7 +56,6 @@ WMFReader::WMFReader(AbstractMediaDecoder* aDecoder)
     mAudioFrameOffset(0),
     mHasAudio(false),
     mHasVideo(false),
-    mCanSeek(false),
     mUseHwAccel(false),
     mMustRecaptureAudioPosition(true),
     mIsMP3Enabled(WMFDecoder::IsMP3Supported()),
@@ -80,37 +80,13 @@ WMFReader::~WMFReader()
   MOZ_COUNT_DTOR(WMFReader);
 }
 
-void
-WMFReader::OnDecodeThreadStart()
-{
-  NS_ASSERTION(mDecoder->OnDecodeThread(), "Should be on decode thread.");
-
-  // XXX WebAudio will call this on the main thread so CoInit will definitely
-  // fail. You cannot change the concurrency model once already set.
-  // The main thread will continue to be STA, which seems to work, but MSDN
-  // recommends that MTA be used.
-  mCOMInitialized = SUCCEEDED(CoInitializeEx(0, COINIT_MULTITHREADED));
-  NS_ENSURE_TRUE_VOID(mCOMInitialized);
-}
-
-void
-WMFReader::OnDecodeThreadFinish()
-{
-  NS_ASSERTION(mDecoder->OnDecodeThread(), "Should be on decode thread.");
-  if (mCOMInitialized) {
-    CoUninitialize();
-  }
-}
-
 bool
 WMFReader::InitializeDXVA()
 {
-  if (!Preferences::GetBool("media.windows-media-foundation.use-dxva", false) ||
-      WinUtils::GetWindowsVersion() == WinUtils::VISTA_VERSION) {
-    // Don't use DXVA on Vista until bug 901944's fix has time to bake on
-    // the release train.
+  if (!Preferences::GetBool("media.windows-media-foundation.use-dxva", false)) {
     return false;
   }
+  MOZ_ASSERT(mDecoder->GetImageContainer());
 
   // Extract the layer manager backend type so that we can determine
   // whether it's worthwhile using DXVA. If we're not running with a D3D
@@ -120,24 +96,23 @@ WMFReader::InitializeDXVA()
   MediaDecoderOwner* owner = mDecoder->GetOwner();
   NS_ENSURE_TRUE(owner, false);
 
-  HTMLMediaElement* element = owner->GetMediaElement();
+  dom::HTMLMediaElement* element = owner->GetMediaElement();
   NS_ENSURE_TRUE(element, false);
 
-  nsIDocument* doc = element->GetOwnerDocument();
-  NS_ENSURE_TRUE(doc, false);
-
-  nsRefPtr<LayerManager> layerManager = nsContentUtils::LayerManagerForDocument(doc);
+  nsRefPtr<LayerManager> layerManager =
+    nsContentUtils::LayerManagerForDocument(element->OwnerDoc());
   NS_ENSURE_TRUE(layerManager, false);
 
-  if (layerManager->GetBackendType() != LayersBackend::LAYERS_D3D9 &&
-      layerManager->GetBackendType() != LayersBackend::LAYERS_D3D10) {
+  LayersBackend backend = layerManager->GetCompositorBackendType();
+  if (backend != LayersBackend::LAYERS_D3D9 &&
+      backend != LayersBackend::LAYERS_D3D10 &&
+      backend != LayersBackend::LAYERS_D3D11) {
     return false;
   }
 
   mDXVA2Manager = DXVA2Manager::Create();
-  NS_ENSURE_TRUE(mDXVA2Manager, false);
 
-  return true;
+  return mDXVA2Manager != nullptr;
 }
 
 nsresult
@@ -160,7 +135,12 @@ WMFReader::Init(MediaDecoderReader* aCloneDonor)
   rv = mByteStream->Init();
   NS_ENSURE_SUCCESS(rv, rv);
 
-  mUseHwAccel = InitializeDXVA();
+  if (mDecoder->GetImageContainer() != nullptr &&
+      IsVideoContentType(mDecoder->GetResource()->GetContentType())) {
+    mUseHwAccel = InitializeDXVA();
+  } else {
+    mUseHwAccel = false;
+  }
 
   return NS_OK;
 }
@@ -213,7 +193,7 @@ ConfigureSourceReaderStream(IMFSourceReader *aReader,
   }
   if (!isSubTypeAllowed) {
     nsCString name = GetGUIDName(subType);
-    LOG("ConfigureSourceReaderStream subType=%s is not allowed to be decoded", name.get());
+    DECODER_LOG("ConfigureSourceReaderStream subType=%s is not allowed to be decoded", name.get());
     return E_FAIL;
   }
 
@@ -234,7 +214,7 @@ ConfigureSourceReaderStream(IMFSourceReader *aReader,
 
   // Set the uncompressed format. This can fail if the decoder can't produce
   // that type.
-  return aReader->SetCurrentMediaType(aStreamIndex, NULL, type);
+  return aReader->SetCurrentMediaType(aStreamIndex, nullptr, type);
 }
 
 // Returns the duration of the resource, in microseconds.
@@ -279,93 +259,6 @@ GetSourceReaderCanSeek(IMFSourceReader* aReader, bool& aOutCanSeek)
   return S_OK;
 }
 
-static HRESULT
-GetDefaultStride(IMFMediaType *aType, uint32_t* aOutStride)
-{
-  // Try to get the default stride from the media type.
-  HRESULT hr = aType->GetUINT32(MF_MT_DEFAULT_STRIDE, aOutStride);
-  if (SUCCEEDED(hr)) {
-    return S_OK;
-  }
-
-  // Stride attribute not set, calculate it.
-  GUID subtype = GUID_NULL;
-  uint32_t width = 0;
-  uint32_t height = 0;
-
-  hr = aType->GetGUID(MF_MT_SUBTYPE, &subtype);
-  NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
-
-  hr = MFGetAttributeSize(aType, MF_MT_FRAME_SIZE, &width, &height);
-  NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
-
-  hr = wmf::MFGetStrideForBitmapInfoHeader(subtype.Data1, width, (LONG*)(aOutStride));
-  NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
-
-  return hr;
-}
-
-static int32_t
-MFOffsetToInt32(const MFOffset& aOffset)
-{
-  return int32_t(aOffset.value + (aOffset.fract / 65536.0f));
-}
-
-// Gets the sub-region of the video frame that should be displayed.
-// See: http://msdn.microsoft.com/en-us/library/windows/desktop/bb530115(v=vs.85).aspx
-static HRESULT
-GetPictureRegion(IMFMediaType* aMediaType, nsIntRect& aOutPictureRegion)
-{
-  // Determine if "pan and scan" is enabled for this media. If it is, we
-  // only display a region of the video frame, not the entire frame.
-  BOOL panScan = MFGetAttributeUINT32(aMediaType, MF_MT_PAN_SCAN_ENABLED, FALSE);
-
-  // If pan and scan mode is enabled. Try to get the display region.
-  HRESULT hr = E_FAIL;
-  MFVideoArea videoArea;
-  memset(&videoArea, 0, sizeof(MFVideoArea));
-  if (panScan) {
-    hr = aMediaType->GetBlob(MF_MT_PAN_SCAN_APERTURE,
-                             (UINT8*)&videoArea,
-                             sizeof(MFVideoArea),
-                             NULL);
-  }
-
-  // If we're not in pan-and-scan mode, or the pan-and-scan region is not set,
-  // check for a minimimum display aperture.
-  if (!panScan || hr == MF_E_ATTRIBUTENOTFOUND) {
-    hr = aMediaType->GetBlob(MF_MT_MINIMUM_DISPLAY_APERTURE,
-                             (UINT8*)&videoArea,
-                             sizeof(MFVideoArea),
-                             NULL);
-  }
-
-  if (hr == MF_E_ATTRIBUTENOTFOUND) {
-    // Minimum display aperture is not set, for "backward compatibility with
-    // some components", check for a geometric aperture.
-    hr = aMediaType->GetBlob(MF_MT_GEOMETRIC_APERTURE,
-                             (UINT8*)&videoArea,
-                             sizeof(MFVideoArea),
-                             NULL);
-  }
-
-  if (SUCCEEDED(hr)) {
-    // The media specified a picture region, return it.
-    aOutPictureRegion = nsIntRect(MFOffsetToInt32(videoArea.OffsetX),
-                                  MFOffsetToInt32(videoArea.OffsetY),
-                                  videoArea.Area.cx,
-                                  videoArea.Area.cy);
-    return S_OK;
-  }
-
-  // No picture region defined, fall back to using the entire video area.
-  UINT32 width = 0, height = 0;
-  hr = MFGetAttributeSize(aMediaType, MF_MT_FRAME_SIZE, &width, &height);
-  NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
-  aOutPictureRegion = nsIntRect(0, 0, width, height);
-  return S_OK;
-}
-
 HRESULT
 WMFReader::ConfigureVideoFrameGeometry(IMFMediaType* aMediaType)
 {
@@ -403,24 +296,24 @@ WMFReader::ConfigureVideoFrameGeometry(IMFMediaType* aMediaType)
   nsIntSize frameSize = nsIntSize(width, height);
   nsIntSize displaySize = nsIntSize(pictureRegion.width, pictureRegion.height);
   ScaleDisplayByAspectRatio(displaySize, float(aspectNum) / float(aspectDenom));
-  if (!VideoInfo::ValidateVideoRegion(frameSize, pictureRegion, displaySize)) {
+  if (!IsValidVideoRegion(frameSize, pictureRegion, displaySize)) {
     // Video track's frame sizes will overflow. Ignore the video track.
     return E_FAIL;
   }
 
   // Success! Save state.
-  mInfo.mDisplay = displaySize;
+  mInfo.mVideo.mDisplay = displaySize;
   GetDefaultStride(aMediaType, &mVideoStride);
   mVideoWidth = width;
   mVideoHeight = height;
   mPictureRegion = pictureRegion;
 
-  LOG("WMFReader frame geometry frame=(%u,%u) stride=%u picture=(%d, %d, %d, %d) display=(%d,%d) PAR=%d:%d",
-      width, height,
-      mVideoStride,
-      mPictureRegion.x, mPictureRegion.y, mPictureRegion.width, mPictureRegion.height,
-      displaySize.width, displaySize.height,
-      aspectNum, aspectDenom);
+  DECODER_LOG("WMFReader frame geometry frame=(%u,%u) stride=%u picture=(%d, %d, %d, %d) display=(%d,%d) PAR=%d:%d",
+              width, height,
+              mVideoStride,
+              mPictureRegion.x, mPictureRegion.y, mPictureRegion.width, mPictureRegion.height,
+              displaySize.width, displaySize.height,
+              aspectNum, aspectDenom);
 
   return S_OK;
 }
@@ -437,6 +330,11 @@ WMFReader::ConfigureVideoDecoder()
     return S_OK;
   }
 
+  if (!mDecoder->GetImageContainer()) {
+    // We can't display the video, so don't bother to decode; disable the stream.
+    return mSourceReader->SetStreamSelection(MF_SOURCE_READER_FIRST_VIDEO_STREAM, FALSE);
+  }
+
   static const GUID MP4VideoTypes[] = {
     MFVideoFormat_H264
   };
@@ -444,9 +342,9 @@ WMFReader::ConfigureVideoDecoder()
                                            MF_SOURCE_READER_FIRST_VIDEO_STREAM,
                                            mUseHwAccel ? MFVideoFormat_NV12 : MFVideoFormat_YV12,
                                            MP4VideoTypes,
-                                           NS_ARRAY_LENGTH(MP4VideoTypes));
+                                           ArrayLength(MP4VideoTypes));
   if (FAILED(hr)) {
-    LOG("Failed to configured video output");
+    DECODER_LOG("Failed to configured video output");
     return hr;
   }
 
@@ -463,9 +361,9 @@ WMFReader::ConfigureVideoDecoder()
     return hr;
   }
 
-  LOG("Successfully configured video stream");
+  DECODER_LOG("Successfully configured video stream");
 
-  mHasVideo = mInfo.mHasVideo = true;
+  mHasVideo = mInfo.mVideo.mHasVideo = true;
 
   return S_OK;
 }
@@ -477,18 +375,21 @@ WMFReader::GetSupportedAudioCodecs(const GUID** aCodecs, uint32_t* aNumCodecs)
   MOZ_ASSERT(aNumCodecs);
 
   if (mIsMP3Enabled) {
+    GUID aacOrMp3 = MFMPEG4Format_Base;
+    aacOrMp3.Data1 = 0x6D703461;// FOURCC('m','p','4','a');
     static const GUID codecs[] = {
       MFAudioFormat_AAC,
-      MFAudioFormat_MP3
+      MFAudioFormat_MP3,
+      aacOrMp3
     };
     *aCodecs = codecs;
-    *aNumCodecs = NS_ARRAY_LENGTH(codecs);
+    *aNumCodecs = ArrayLength(codecs);
   } else {
     static const GUID codecs[] = {
       MFAudioFormat_AAC
     };
     *aCodecs = codecs;
-    *aNumCodecs = NS_ARRAY_LENGTH(codecs);
+    *aNumCodecs = ArrayLength(codecs);
   }
 }
 
@@ -529,51 +430,47 @@ WMFReader::ConfigureAudioDecoder()
   mAudioChannels = MFGetAttributeUINT32(mediaType, MF_MT_AUDIO_NUM_CHANNELS, 0);
   mAudioBytesPerSample = MFGetAttributeUINT32(mediaType, MF_MT_AUDIO_BITS_PER_SAMPLE, 16) / 8;
 
-  mInfo.mAudioChannels = mAudioChannels;
-  mInfo.mAudioRate = mAudioRate;
-  mHasAudio = mInfo.mHasAudio = true;
+  mInfo.mAudio.mChannels = mAudioChannels;
+  mInfo.mAudio.mRate = mAudioRate;
+  mHasAudio = mInfo.mAudio.mHasAudio = true;
 
-  LOG("Successfully configured audio stream. rate=%u channels=%u bitsPerSample=%u",
-      mAudioRate, mAudioChannels, mAudioBytesPerSample);
+  DECODER_LOG("Successfully configured audio stream. rate=%u channels=%u bitsPerSample=%u",
+              mAudioRate, mAudioChannels, mAudioBytesPerSample);
 
   return S_OK;
 }
 
-nsresult
-WMFReader::ReadMetadata(VideoInfo* aInfo,
-                        MetadataTags** aTags)
+HRESULT
+WMFReader::CreateSourceReader()
 {
-  NS_ASSERTION(mDecoder->OnDecodeThread(), "Should be on decode thread.");
-
-  LOG("WMFReader::ReadMetadata()");
   HRESULT hr;
 
   RefPtr<IMFAttributes> attr;
   hr = wmf::MFCreateAttributes(byRef(attr), 1);
-  NS_ENSURE_TRUE(SUCCEEDED(hr), NS_ERROR_FAILURE);
+  NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
 
   hr = attr->SetUnknown(MF_SOURCE_READER_ASYNC_CALLBACK, mSourceReaderCallback);
-  NS_ENSURE_TRUE(SUCCEEDED(hr), NS_ERROR_FAILURE);
+  NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
 
   if (mUseHwAccel) {
     hr = attr->SetUnknown(MF_SOURCE_READER_D3D_MANAGER,
                           mDXVA2Manager->GetDXVADeviceManager());
     if (FAILED(hr)) {
-      LOG("Failed to set DXVA2 D3D Device manager on source reader attributes");
+      DECODER_LOG("Failed to set DXVA2 D3D Device manager on source reader attributes");
       mUseHwAccel = false;
     }
   }
 
   hr = wmf::MFCreateSourceReaderFromByteStream(mByteStream, attr, byRef(mSourceReader));
-  NS_ENSURE_TRUE(SUCCEEDED(hr), NS_ERROR_FAILURE);
+  NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
 
   hr = ConfigureVideoDecoder();
-  NS_ENSURE_TRUE(SUCCEEDED(hr), NS_ERROR_FAILURE);
+  NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
 
   hr = ConfigureAudioDecoder();
-  NS_ENSURE_TRUE(SUCCEEDED(hr), NS_ERROR_FAILURE);
+  NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
 
-  if (mUseHwAccel && mInfo.mHasVideo) {
+  if (mUseHwAccel && mInfo.mVideo.mHasVideo) {
     RefPtr<IMFTransform> videoDecoder;
     hr = mSourceReader->GetServiceForStream(MF_SOURCE_READER_FIRST_VIDEO_STREAM,
                                             GUID_NULL,
@@ -584,28 +481,67 @@ WMFReader::ReadMetadata(VideoInfo* aInfo,
       ULONG_PTR manager = ULONG_PTR(mDXVA2Manager->GetDXVADeviceManager());
       hr = videoDecoder->ProcessMessage(MFT_MESSAGE_SET_D3D_MANAGER,
                                         manager);
+      if (hr == MF_E_TRANSFORM_TYPE_NOT_SET) {
+        // Ignore MF_E_TRANSFORM_TYPE_NOT_SET. Vista returns this here
+        // on some, perhaps all, video cards. This may be because activating
+        // DXVA changes the available output types. It seems to be safe to
+        // ignore this error.
+        hr = S_OK;
+      }
     }
     if (FAILED(hr)) {
-      LOG("Failed to set DXVA2 D3D Device manager on decoder");
+      DECODER_LOG("Failed to set DXVA2 D3D Device manager on decoder hr=0x%x", hr);
       mUseHwAccel = false;
     }
   }
-  if (mInfo.mHasVideo) {
-    LOG("Using DXVA: %s", (mUseHwAccel ? "Yes" : "No"));
+  return hr;
+}
+
+nsresult
+WMFReader::ReadMetadata(MediaInfo* aInfo,
+                        MetadataTags** aTags)
+{
+  NS_ASSERTION(mDecoder->OnDecodeThread(), "Should be on decode thread.");
+
+  DECODER_LOG("WMFReader::ReadMetadata()");
+  HRESULT hr;
+
+  const bool triedToInitDXVA = mUseHwAccel;
+  if (FAILED(CreateSourceReader())) {
+    mSourceReader = nullptr;
+    if (triedToInitDXVA && !mUseHwAccel) {
+      // We tried to initialize DXVA and failed. Try again to create the
+      // IMFSourceReader but this time we won't use DXVA. Note that we
+      // must recreate the IMFSourceReader from scratch, as on some systems
+      // (AMD Radeon 3000) we cannot successfully reconfigure an existing
+      // reader to not use DXVA after we've failed to configure DXVA.
+      // See bug 987127.
+      if (FAILED(CreateSourceReader())) {
+        mSourceReader = nullptr;
+      }
+    }
+  }
+
+  if (!mSourceReader) {
+    NS_WARNING("Failed to create IMFSourceReader");
+    return NS_ERROR_FAILURE;
+  }
+
+  if (mInfo.HasVideo()) {
+    DECODER_LOG("Using DXVA: %s", (mUseHwAccel ? "Yes" : "No"));
   }
 
   // Abort if both video and audio failed to initialize.
-  NS_ENSURE_TRUE(mInfo.mHasAudio || mInfo.mHasVideo, NS_ERROR_FAILURE);
+  NS_ENSURE_TRUE(mInfo.HasValidMedia(), NS_ERROR_FAILURE);
 
+  // Get the duration, and report it to the decoder if we have it.
   int64_t duration = 0;
-  if (SUCCEEDED(GetSourceReaderDuration(mSourceReader, duration))) {
+  hr = GetSourceReaderDuration(mSourceReader, duration);
+  if (SUCCEEDED(hr)) {
     ReentrantMonitorAutoEnter mon(mDecoder->GetReentrantMonitor());
-    mDecoder->SetMediaDuration(duration);
+    mDecoder->SetMediaEndTime(duration);
   }
 
-  hr = GetSourceReaderCanSeek(mSourceReader, mCanSeek);
-  NS_ASSERTION(SUCCEEDED(hr), "Can't determine if resource is seekable");
-
   *aInfo = mInfo;
   *aTags = nullptr;
   // aTags can be retrieved using techniques like used here:
@@ -614,37 +550,20 @@ WMFReader::ReadMetadata(VideoInfo* aInfo,
   return NS_OK;
 }
 
-static int64_t
-GetSampleDuration(IMFSample* aSample)
+bool
+WMFReader::IsMediaSeekable()
 {
+  // Get the duration
   int64_t duration = 0;
-  aSample->GetSampleDuration(&duration);
-  return HNsToUsecs(duration);
-}
-
-HRESULT
-HNsToFrames(int64_t aHNs, uint32_t aRate, int64_t* aOutFrames)
-{
-  MOZ_ASSERT(aOutFrames);
-  const int64_t HNS_PER_S = USECS_PER_S * 10;
-  CheckedInt<int64_t> i = aHNs;
-  i *= aRate;
-  i /= HNS_PER_S;
-  NS_ENSURE_TRUE(i.isValid(), E_FAIL);
-  *aOutFrames = i.value();
-  return S_OK;
-}
-
-HRESULT
-FramesToUsecs(int64_t aSamples, uint32_t aRate, int64_t* aOutUsecs)
-{
-  MOZ_ASSERT(aOutUsecs);
-  CheckedInt<int64_t> i = aSamples;
-  i *= USECS_PER_S;
-  i /= aRate;
-  NS_ENSURE_TRUE(i.isValid(), E_FAIL);
-  *aOutUsecs = i.value();
-  return S_OK;
+  HRESULT hr = GetSourceReaderDuration(mSourceReader, duration);
+  // We can seek if we get a duration *and* the reader reports that it's
+  // seekable.
+  bool canSeek = false;
+  if (FAILED(hr) || FAILED(GetSourceReaderCanSeek(mSourceReader, canSeek)) ||
+      !canSeek) {
+    return false;
+  }
+  return true;
 }
 
 bool
@@ -661,9 +580,8 @@ WMFReader::DecodeAudioData()
                                  nullptr);
 
   if (FAILED(hr)) {
-    LOG("WMFReader::DecodeAudioData() ReadSample failed with hr=0x%x", hr);
+    DECODER_LOG("WMFReader::DecodeAudioData() ReadSample failed with hr=0x%x", hr);
     // End the stream.
-    mAudioQueue.Finish();
     return false;
   }
 
@@ -675,10 +593,9 @@ WMFReader::DecodeAudioData()
       (flags & MF_SOURCE_READERF_ERROR) ||
       (flags & MF_SOURCE_READERF_ENDOFSTREAM) ||
       (flags & MF_SOURCE_READERF_CURRENTMEDIATYPECHANGED)) {
-    LOG("WMFReader::DecodeAudioData() ReadSample failed with hr=0x%x flags=0x%x",
-        hr, flags);
+    DECODER_LOG("WMFReader::DecodeAudioData() ReadSample failed with hr=0x%x flags=0x%x",
+                hr, flags);
     // End the stream.
-    mAudioQueue.Finish();
     return false;
   }
 
@@ -736,8 +653,8 @@ WMFReader::DecodeAudioData()
                                  mAudioChannels));
 
   #ifdef LOG_SAMPLE_DECODE
-  LOG("Decoded audio sample! timestamp=%lld duration=%lld currentLength=%u",
-      timestamp, duration, currentLength);
+  DECODER_LOG("Decoded audio sample! timestamp=%lld duration=%lld currentLength=%u",
+              timestamp, duration, currentLength);
   #endif
 
   return true;
@@ -773,7 +690,7 @@ WMFReader::CreateBasicVideoFrame(IMFSample* aSample,
     hr = twoDBuffer->Lock2D(&data, &stride);
     NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
   } else {
-    hr = buffer->Lock(&data, NULL, NULL);
+    hr = buffer->Lock(&data, nullptr, nullptr);
     NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
     stride = mVideoStride;
   }
@@ -818,15 +735,15 @@ WMFReader::CreateBasicVideoFrame(IMFSample* aSample,
   b.mPlanes[2].mOffset = 0;
   b.mPlanes[2].mSkip = 0;
 
-  VideoData *v = VideoData::Create(mInfo,
+  VideoData *v = VideoData::Create(mInfo.mVideo,
                                    mDecoder->GetImageContainer(),
                                    aOffsetBytes,
                                    aTimestampUsecs,
-                                   aTimestampUsecs + aDurationUsecs,
+                                   aDurationUsecs,
                                    b,
                                    false,
                                    -1,
-                                   mPictureRegion);
+                                   ToIntRect(mPictureRegion));
   if (twoDBuffer) {
     twoDBuffer->Unlock2D();
   } else {
@@ -861,15 +778,15 @@ WMFReader::CreateD3DVideoFrame(IMFSample* aSample,
   NS_ENSURE_TRUE(SUCCEEDED(hr), hr);
   NS_ENSURE_TRUE(image, E_FAIL);
 
-  VideoData *v = VideoData::CreateFromImage(mInfo,
+  VideoData *v = VideoData::CreateFromImage(mInfo.mVideo,
                                             mDecoder->GetImageContainer(),
                                             aOffsetBytes,
                                             aTimestampUsecs,
-                                            aTimestampUsecs + aDurationUsecs,
+                                            aDurationUsecs,
                                             image.forget(),
                                             false,
                                             -1,
-                                            mPictureRegion);
+                                            ToIntRect(mPictureRegion));
 
   NS_ENSURE_TRUE(v, E_FAIL);
   *aOutVideoData = v;
@@ -897,9 +814,7 @@ WMFReader::DecodeVideoFrame(bool &aKeyframeSkip,
                                  nullptr,
                                  nullptr);
   if (FAILED(hr)) {
-    LOG("WMFReader::DecodeVideoData() ReadSample failed with hr=0x%x", hr);
-    // End the stream.
-    mVideoQueue.Finish();
+    DECODER_LOG("WMFReader::DecodeVideoData() ReadSample failed with hr=0x%x", hr);
     return false;
   }
 
@@ -911,7 +826,6 @@ WMFReader::DecodeVideoFrame(bool &aKeyframeSkip,
   if (flags & MF_SOURCE_READERF_ERROR) {
     NS_WARNING("WMFReader: Catastrophic failure reading video sample");
     // Future ReadSample() calls will fail, so give up and report end of stream.
-    mVideoQueue.Finish();
     return false;
   }
 
@@ -922,24 +836,21 @@ WMFReader::DecodeVideoFrame(bool &aKeyframeSkip,
 
   if (!sample) {
     if ((flags & MF_SOURCE_READERF_ENDOFSTREAM)) {
-      LOG("WMFReader; Null sample after video decode, at end of stream");
-      // End the stream.
-      mVideoQueue.Finish();
+      DECODER_LOG("WMFReader; Null sample after video decode, at end of stream");
       return false;
     }
-    LOG("WMFReader; Null sample after video decode. Maybe insufficient data...");
+    DECODER_LOG("WMFReader; Null sample after video decode. Maybe insufficient data...");
     return true;
   }
 
   if ((flags & MF_SOURCE_READERF_CURRENTMEDIATYPECHANGED)) {
-    LOG("WMFReader: Video media type changed!");
+    DECODER_LOG("WMFReader: Video media type changed!");
     RefPtr<IMFMediaType> mediaType;
     hr = mSourceReader->GetCurrentMediaType(MF_SOURCE_READER_FIRST_VIDEO_STREAM,
                                             byRef(mediaType));
     if (FAILED(hr) ||
         FAILED(ConfigureVideoFrameGeometry(mediaType))) {
       NS_WARNING("Failed to reconfigure video media type");
-      mVideoQueue.Finish();
       return false;
     }
   }
@@ -964,14 +875,13 @@ WMFReader::DecodeVideoFrame(bool &aKeyframeSkip,
   mVideoQueue.Push(v);
 
   #ifdef LOG_SAMPLE_DECODE
-  LOG("Decoded video sample timestamp=%lld duration=%lld stride=%d height=%u flags=%u",
-      timestamp, duration, mVideoStride, mVideoHeight, flags);
+  DECODER_LOG("Decoded video sample timestamp=%lld duration=%lld stride=%d height=%u flags=%u",
+              timestamp, duration, mVideoStride, mVideoHeight, flags);
   #endif
 
   if ((flags & MF_SOURCE_READERF_ENDOFSTREAM)) {
     // End of stream.
-    mVideoQueue.Finish();
-    LOG("End of video stream");
+    DECODER_LOG("End of video stream");
     return false;
   }
 
@@ -984,12 +894,14 @@ WMFReader::Seek(int64_t aTargetUs,
                 int64_t aEndTime,
                 int64_t aCurrentTime)
 {
-  LOG("WMFReader::Seek() %lld", aTargetUs);
+  DECODER_LOG("WMFReader::Seek() %lld", aTargetUs);
 
   NS_ASSERTION(mDecoder->OnDecodeThread(), "Should be on decode thread.");
-  if (!mCanSeek) {
-    return NS_ERROR_FAILURE;
-  }
+#ifdef DEBUG
+  bool canSeek = false;
+  GetSourceReaderCanSeek(mSourceReader, canSeek);
+  NS_ASSERTION(canSeek, "WMFReader::Seek() should only be called if we can seek!");
+#endif
 
   nsresult rv = ResetDecode();
   NS_ENSURE_SUCCESS(rv, rv);
@@ -1007,19 +919,6 @@ WMFReader::Seek(int64_t aTargetUs,
   hr = mSourceReader->SetCurrentPosition(GUID_NULL, var);
   NS_ENSURE_TRUE(SUCCEEDED(hr), NS_ERROR_FAILURE);
 
-  return DecodeToTarget(aTargetUs);
-}
-
-nsresult
-WMFReader::GetBuffered(mozilla::dom::TimeRanges* aBuffered, int64_t aStartTime)
-{
-  MediaResource* stream = mDecoder->GetResource();
-  int64_t durationUs = 0;
-  {
-    ReentrantMonitorAutoEnter mon(mDecoder->GetReentrantMonitor());
-    durationUs = mDecoder->GetMediaDuration();
-  }
-  GetEstimatedBufferedTimeRanges(stream, durationUs, aBuffered);
   return NS_OK;
 }
 
