@@ -12,7 +12,6 @@
 #include "nsIObjectOutputStream.h"
 #include "nsIPrincipal.h"
 #include "nsJSPrincipals.h"
-#include "nsIScriptGlobalObject.h"
 #include "nsIScriptObjectPrincipal.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsIServiceManager.h"
@@ -27,84 +26,16 @@
 #include "nsDOMCID.h"
 #include "nsNodeInfoManager.h"
 #include "nsContentUtils.h"
-#include "nsCxPusher.h"
 #include "nsCCUncollectableMarker.h"
 #include "nsDOMJSUtils.h" // for GetScriptContextFromJSContext
 #include "xpcpublic.h"
 #include "mozilla/dom/BindingUtils.h"
 
 using mozilla::dom::DestroyProtoAndIfaceCache;
-using mozilla::AutoPushJSContext;
-using mozilla::AutoSafeJSContext;
 using mozilla::dom::XULDocument;
 
-class nsXULPDGlobalObject : public nsISupports
-{
-public:
-    nsXULPDGlobalObject(nsXULPrototypeDocument* owner);
-
-    // nsISupports interface
-    NS_DECL_CYCLE_COLLECTING_ISUPPORTS
-    NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS(nsXULPDGlobalObject)
-
-    JSObject* GetCompilationGlobal();
-    void UnmarkCompilationGlobal()
-    {
-        if (mJSObject) {
-            JS::ExposeObjectToActiveJS(mJSObject);
-        }
-    }
-    void Destroy();
-    nsIPrincipal* GetPrincipal();
-    void ClearGlobalObjectOwner();
-
-protected:
-    virtual ~nsXULPDGlobalObject();
-
-    nsCOMPtr<nsIPrincipal> mCachedPrincipal;
-    nsXULPrototypeDocument* mGlobalObjectOwner; // weak reference
-    JS::Heap<JSObject*> mJSObject;
-    bool mDestroyed; // Probably not necessary, but let's be safe.
-
-    static const JSClass gSharedGlobalClass;
-};
-
-nsIPrincipal* nsXULPrototypeDocument::gSystemPrincipal;
-nsXULPDGlobalObject* nsXULPrototypeDocument::gSystemGlobal;
 uint32_t nsXULPrototypeDocument::gRefCnt;
 
-
-void
-nsXULPDGlobalObject_finalize(JSFreeOp *fop, JSObject *obj)
-{
-    nsXULPDGlobalObject* nativeThis = static_cast<nsXULPDGlobalObject*>(JS_GetPrivate(obj));
-    nativeThis->Destroy();
-
-    // The addref was part of JSObject construction
-    nsContentUtils::DeferredFinalize(nativeThis);
-}
-
-
-bool
-nsXULPDGlobalObject_resolve(JSContext *cx, JS::Handle<JSObject*> obj, JS::Handle<jsid> id)
-{
-    bool did_resolve = false;
-
-    return JS_ResolveStandardClass(cx, obj, id, &did_resolve);
-}
-
-
-const JSClass nsXULPDGlobalObject::gSharedGlobalClass = {
-    "nsXULPrototypeScript compilation scope",
-    JSCLASS_HAS_PRIVATE | JSCLASS_PRIVATE_IS_NSISUPPORTS |
-    JSCLASS_IMPLEMENTS_BARRIERS | JSCLASS_GLOBAL_FLAGS_WITH_SLOTS(0),
-    JS_PropertyStub,  JS_DeletePropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
-    JS_EnumerateStub, nsXULPDGlobalObject_resolve,  JS_ConvertStub,
-    nsXULPDGlobalObject_finalize, nullptr, nullptr, nullptr, nullptr
-};
-
-
-
 //----------------------------------------------------------------------
 //
 // ctors, dtors, n' stuff
@@ -131,36 +62,22 @@ nsXULPrototypeDocument::Init()
 
 nsXULPrototypeDocument::~nsXULPrototypeDocument()
 {
-    if (mGlobalObject) {
-        // cleaup cycles etc.
-        mGlobalObject->ClearGlobalObjectOwner();
-    }
-
     if (mRoot)
         mRoot->ReleaseSubtree();
-
-    if (--gRefCnt == 0) {
-        NS_IF_RELEASE(gSystemPrincipal);
-        NS_IF_RELEASE(gSystemGlobal);
-    }
 }
 
 NS_IMPL_CYCLE_COLLECTION_CLASS(nsXULPrototypeDocument)
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsXULPrototypeDocument)
-    tmp->mPrototypeWaiters.Clear();
+    NS_IMPL_CYCLE_COLLECTION_UNLINK(mPrototypeWaiters)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsXULPrototypeDocument)
     if (nsCCUncollectableMarker::InGeneration(cb, tmp->mCCGeneration)) {
         return NS_SUCCESS_INTERRUPTED_TRAVERSE;
     }
     NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mRoot)
-    NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mGlobalObject)
     NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mNodeInfoManager)
-    for (uint32_t i = 0; i < tmp->mPrototypeWaiters.Length(); ++i) {
-        NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(cb, "mPrototypeWaiters[i]");
-        cb.NoteXPCOMChild(static_cast<nsINode*>(tmp->mPrototypeWaiters[i].get()));
-    }
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mPrototypeWaiters)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsXULPrototypeDocument)
@@ -190,35 +107,6 @@ NS_NewXULPrototypeDocument(nsXULPrototypeDocument** aResult)
     return rv;
 }
 
-// Helper method that shares a system global among all prototype documents
-// that have the system principal as their security principal.   Called by
-// nsXULPrototypeDocument::Read and nsXULPrototypeDocument::GetCompilationGlobal.
-// This method greatly reduces the number of nsXULPDGlobalObjects and their
-// nsIScriptContexts in apps that load many XUL documents via chrome: URLs.
-
-nsXULPDGlobalObject *
-nsXULPrototypeDocument::NewXULPDGlobalObject()
-{
-    // Now compare DocumentPrincipal() to gSystemPrincipal, in order to create
-    // gSystemGlobal if the two pointers are equal.  Thus, gSystemGlobal
-    // implies gSystemPrincipal.
-    nsXULPDGlobalObject *global;
-    if (DocumentPrincipal() == gSystemPrincipal) {
-        if (!gSystemGlobal) {
-            gSystemGlobal = new nsXULPDGlobalObject(nullptr);
-            if (! gSystemGlobal)
-                return nullptr;
-            NS_ADDREF(gSystemGlobal);
-        }
-        global = gSystemGlobal;
-    } else {
-        global = new nsXULPDGlobalObject(this); // does not refcount
-        if (! global)
-            return nullptr;
-    }
-    return global;
-}
-
 //----------------------------------------------------------------------
 //
 // nsISerializable methods
@@ -229,7 +117,9 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
 {
     nsresult rv;
 
-    rv = aStream->ReadObject(true, getter_AddRefs(mURI));
+    nsCOMPtr<nsISupports> supports;
+    rv = aStream->ReadObject(true, getter_AddRefs(supports));
+    mURI = do_QueryInterface(supports);
 
     uint32_t count, i;
     nsCOMPtr<nsIURI> styleOverlayURI;
@@ -243,35 +133,31 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
     }
 
     for (i = 0; i < count; ++i) {
-        tmp = aStream->ReadObject(true, getter_AddRefs(styleOverlayURI));
+        tmp = aStream->ReadObject(true, getter_AddRefs(supports));
         if (NS_FAILED(tmp)) {
           rv = tmp;
         }
+        styleOverlayURI = do_QueryInterface(supports);
         mStyleSheetReferences.AppendObject(styleOverlayURI);
     }
 
 
     // nsIPrincipal mNodeInfoManager->mPrincipal
     nsCOMPtr<nsIPrincipal> principal;
-    tmp = aStream->ReadObject(true, getter_AddRefs(principal));
+    tmp = aStream->ReadObject(true, getter_AddRefs(supports));
+    principal = do_QueryInterface(supports);
     if (NS_FAILED(tmp)) {
       rv = tmp;
     }
     // Better safe than sorry....
     mNodeInfoManager->SetDocumentPrincipal(principal);
 
-
-    // nsIScriptGlobalObject mGlobalObject
-    mGlobalObject = NewXULPDGlobalObject();
-    if (! mGlobalObject)
-        return NS_ERROR_OUT_OF_MEMORY;
-
     mRoot = new nsXULPrototypeElement();
     if (! mRoot)
        return NS_ERROR_OUT_OF_MEMORY;
 
-    // nsINodeInfo table
-    nsCOMArray<nsINodeInfo> nodeInfos;
+    // mozilla::dom::NodeInfo table
+    nsTArray<nsRefPtr<mozilla::dom::NodeInfo>> nodeInfos;
 
     tmp = aStream->Read32(&count);
     if (NS_FAILED(tmp)) {
@@ -303,7 +189,7 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
           rv = tmp;
         }
 
-        nsCOMPtr<nsINodeInfo> nodeInfo;
+        nsRefPtr<mozilla::dom::NodeInfo> nodeInfo;
         // Using UINT16_MAX here as we don't know which nodeinfos will be
         // used for attributes and which for elements. And that doesn't really
         // matter.
@@ -313,8 +199,7 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
         if (NS_FAILED(tmp)) {
           rv = tmp;
         }
-        if (!nodeInfos.AppendObject(nodeInfo))
-          rv = NS_ERROR_OUT_OF_MEMORY;
+        nodeInfos.AppendElement(nodeInfo);
     }
 
     // Document contents
@@ -362,19 +247,16 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
 
 static nsresult
 GetNodeInfos(nsXULPrototypeElement* aPrototype,
-             nsCOMArray<nsINodeInfo>& aArray)
+             nsTArray<nsRefPtr<mozilla::dom::NodeInfo>>& aArray)
 {
-    nsresult rv;
-    if (aArray.IndexOf(aPrototype->mNodeInfo) < 0) {
-        if (!aArray.AppendObject(aPrototype->mNodeInfo)) {
-            return NS_ERROR_OUT_OF_MEMORY;
-        }
+    if (aArray.IndexOf(aPrototype->mNodeInfo) == aArray.NoIndex) {
+        aArray.AppendElement(aPrototype->mNodeInfo);
     }
 
     // Search attributes
     uint32_t i;
     for (i = 0; i < aPrototype->mNumAttributes; ++i) {
-        nsCOMPtr<nsINodeInfo> ni;
+        nsRefPtr<mozilla::dom::NodeInfo> ni;
         nsAttrName* name = &aPrototype->mAttributes[i].mName;
         if (name->IsAtom()) {
             ni = aPrototype->mNodeInfo->NodeInfoManager()->
@@ -385,10 +267,8 @@ GetNodeInfos(nsXULPrototypeElement* aPrototype,
             ni = name->NodeInfo();
         }
 
-        if (aArray.IndexOf(ni) < 0) {
-            if (!aArray.AppendObject(ni)) {
-                return NS_ERROR_OUT_OF_MEMORY;
-            }
+        if (aArray.IndexOf(ni) == aArray.NoIndex) {
+            aArray.AppendElement(ni);
         }
     }
 
@@ -396,8 +276,8 @@ GetNodeInfos(nsXULPrototypeElement* aPrototype,
     for (i = 0; i < aPrototype->mChildren.Length(); ++i) {
         nsXULPrototypeNode* child = aPrototype->mChildren[i];
         if (child->mType == nsXULPrototypeNode::eType_Element) {
-            rv = GetNodeInfos(static_cast<nsXULPrototypeElement*>(child),
-                              aArray);
+            nsresult rv =
+              GetNodeInfos(static_cast<nsXULPrototypeElement*>(child), aArray);
             NS_ENSURE_SUCCESS(rv, rv);
         }
     }
@@ -443,8 +323,8 @@ nsXULPrototypeDocument::Write(nsIObjectOutputStream* aStream)
     }
 #endif
 
-    // nsINodeInfo table
-    nsCOMArray<nsINodeInfo> nodeInfos;
+    // mozilla::dom::NodeInfo table
+    nsTArray<nsRefPtr<mozilla::dom::NodeInfo>> nodeInfos;
     if (mRoot) {
       tmp = GetNodeInfos(mRoot, nodeInfos);
       if (NS_FAILED(tmp)) {
@@ -452,13 +332,13 @@ nsXULPrototypeDocument::Write(nsIObjectOutputStream* aStream)
       }
     }
 
-    uint32_t nodeInfoCount = nodeInfos.Count();
+    uint32_t nodeInfoCount = nodeInfos.Length();
     tmp = aStream->Write32(nodeInfoCount);
     if (NS_FAILED(tmp)) {
       rv = tmp;
     }
     for (i = 0; i < nodeInfoCount; ++i) {
-        nsINodeInfo *nodeInfo = nodeInfos[i];
+        mozilla::dom::NodeInfo *nodeInfo = nodeInfos[i];
         NS_ENSURE_TRUE(nodeInfo, NS_ERROR_FAILURE);
 
         nsAutoString namespaceURI;
@@ -609,22 +489,10 @@ nsXULPrototypeDocument::SetDocumentPrincipal(nsIPrincipal* aPrincipal)
     mNodeInfoManager->SetDocumentPrincipal(aPrincipal);
 }
 
-JSObject*
-nsXULPrototypeDocument::GetCompilationGlobal()
-{
-  if (!mGlobalObject) {
-      mGlobalObject = NewXULPDGlobalObject();
-  }
-  return mGlobalObject->GetCompilationGlobal();
-}
-
 void
 nsXULPrototypeDocument::MarkInCCGeneration(uint32_t aCCGeneration)
 {
     mCCGeneration = aCCGeneration;
-    if (mGlobalObject) {
-        mGlobalObject->UnmarkCompilationGlobal();
-    }
 }
 
 nsNodeInfoManager*
@@ -687,118 +555,3 @@ nsXULPrototypeDocument::TraceProtos(JSTracer* aTrc, uint32_t aGCNumber)
     mRoot->TraceAllScripts(aTrc);
   }
 }
-
-//----------------------------------------------------------------------
-//
-// nsXULPDGlobalObject
-//
-
-nsXULPDGlobalObject::nsXULPDGlobalObject(nsXULPrototypeDocument* owner)
-  : mGlobalObjectOwner(owner)
-  , mJSObject(nullptr)
-  , mDestroyed(false)
-{
-}
-
-
-nsXULPDGlobalObject::~nsXULPDGlobalObject()
-{
-  MOZ_ASSERT(!mJSObject);
-}
-
-NS_IMPL_CYCLE_COLLECTION_CLASS(nsXULPDGlobalObject)
-
-NS_IMPL_CYCLE_COLLECTION_TRACE_BEGIN(nsXULPDGlobalObject)
-  NS_IMPL_CYCLE_COLLECTION_TRACE_JS_MEMBER_CALLBACK(mJSObject)
-NS_IMPL_CYCLE_COLLECTION_TRACE_END
-
-NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsXULPDGlobalObject)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_SCRIPT_OBJECTS
-NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
-
-NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsXULPDGlobalObject)
-  tmp->Destroy();
-NS_IMPL_CYCLE_COLLECTION_UNLINK_END
-
-NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsXULPDGlobalObject)
-  NS_INTERFACE_MAP_ENTRY(nsISupports)
-NS_INTERFACE_MAP_END
-
-NS_IMPL_CYCLE_COLLECTING_ADDREF(nsXULPDGlobalObject)
-NS_IMPL_CYCLE_COLLECTING_RELEASE(nsXULPDGlobalObject)
-
-JSObject *
-nsXULPDGlobalObject::GetCompilationGlobal()
-{
-  if (mJSObject) {
-    // We've been initialized before. This is what we get.
-    JS::ExposeObjectToActiveJS(mJSObject);
-    return mJSObject;
-  }
-
-  if (mDestroyed) {
-    return nullptr;
-  }
-
-  AutoSafeJSContext cx;
-  JS::CompartmentOptions options;
-  options.setZone(JS::SystemZone)
-         .setInvisibleToDebugger(true);
-  mJSObject = JS_NewGlobalObject(cx, &gSharedGlobalClass,
-                                 nsJSPrincipals::get(GetPrincipal()),
-                                 JS::DontFireOnNewGlobalHook, options);
-  NS_ENSURE_TRUE(mJSObject, nullptr);
-
-  mozilla::HoldJSObjects(this);
-
-  // Add an owning reference from JS back to us. This'll be
-  // released when the JSObject is finalized.
-  JS_SetPrivate(mJSObject, this);
-  NS_ADDREF(this);
-
-  // Set the location information for the new global, so that tools like
-  // about:memory may use that information
-  nsIURI *ownerURI = mGlobalObjectOwner->GetURI();
-  xpc::SetLocationForGlobal(mJSObject, ownerURI);
-
-  return mJSObject;
-}
-
-void
-nsXULPDGlobalObject::ClearGlobalObjectOwner()
-{
-  NS_ASSERTION(!mCachedPrincipal, "This shouldn't ever be set until now!");
-
-  // Cache mGlobalObjectOwner's principal if possible.
-  if (this != nsXULPrototypeDocument::gSystemGlobal)
-    mCachedPrincipal = mGlobalObjectOwner->DocumentPrincipal();
-
-  mGlobalObjectOwner = nullptr;
-}
-
-void
-nsXULPDGlobalObject::Destroy()
-{
-  mDestroyed = true;
-  if (!mJSObject) {
-    return;
-  }
-  mJSObject = nullptr;
-  mozilla::DropJSObjects(this);
-}
-
-nsIPrincipal*
-nsXULPDGlobalObject::GetPrincipal()
-{
-    if (!mGlobalObjectOwner) {
-        // See nsXULPrototypeDocument::NewXULPDGlobalObject, the comment
-        // about gSystemGlobal implying gSystemPrincipal.
-        if (this == nsXULPrototypeDocument::gSystemGlobal) {
-            return nsXULPrototypeDocument::gSystemPrincipal;
-        }
-        // Return the cached principal if it exists.
-        return mCachedPrincipal;
-    }
-
-    return mGlobalObjectOwner->DocumentPrincipal();
-}
