@@ -28,6 +28,7 @@
 #ifndef nsRuleNetwork_h__
 #define nsRuleNetwork_h__
 
+#include "mozilla/Attributes.h"
 #include "nsCOMPtr.h"
 #include "nsCOMArray.h"
 #include "nsIAtom.h"
@@ -85,11 +86,13 @@ protected:
     public:
         List() { MOZ_COUNT_CTOR(MemoryElementSet::List); }
 
+    protected:
         ~List() {
             MOZ_COUNT_DTOR(MemoryElementSet::List);
             delete mElement;
             NS_IF_RELEASE(mNext); }
 
+    public:
         int32_t AddRef() { return ++mRefCnt; }
 
         int32_t Release() {
@@ -230,10 +233,13 @@ protected:
         List(const nsAssignment &aAssignment) : mAssignment(aAssignment) {
             MOZ_COUNT_CTOR(nsAssignmentSet::List); }
 
+    protected:
         ~List() {
             MOZ_COUNT_DTOR(nsAssignmentSet::List);
             NS_IF_RELEASE(mNext); }
 
+    public:
+
         int32_t AddRef() { return ++mRefCnt; }
 
         int32_t Release() {
@@ -797,7 +803,7 @@ public:
      * aTakenInstantiations will be set properly even if an error occurs.
      */
     virtual nsresult Propagate(InstantiationSet& aInstantiations,
-                               bool aIsUpdate, bool& aTakenInstantiations);
+                               bool aIsUpdate, bool& aTakenInstantiations) MOZ_OVERRIDE;
 
     /**
      * This is called by a child node on its parent to allow the
