@@ -4,13 +4,15 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#ifndef nsDocShellEnumerator_h___
+#define nsDocShellEnumerator_h___
+
 #include "nsISimpleEnumerator.h"
 #include "nsTArray.h"
 #include "nsIWeakReferenceUtils.h"
 
 class nsIDocShellTreeItem;
 
-
 /*
 // {13cbc281-35ae-11d5-be5b-bde0edece43c}
 #define NS_DOCSHELL_FORWARDS_ENUMERATOR_CID  \
@@ -35,11 +37,12 @@ protected:
     enumerateForwards,
     enumerateBackwards
   };
-  
+
+  virtual                     ~nsDocShellEnumerator();
+
 public:
 
                               nsDocShellEnumerator(int32_t inEnumerationDirection);
-  virtual                     ~nsDocShellEnumerator();
 
   // nsISupports
   NS_DECL_ISUPPORTS
@@ -105,5 +108,6 @@ public:
 protected:
 
   virtual nsresult            BuildArrayRecursive(nsIDocShellTreeItem* inItem, nsTArray<nsWeakPtr>& inItemArray);
-
 };
+
+#endif // nsDocShellEnumerator_h___
