@@ -6,11 +6,11 @@
 
 #include "IndexedDBChild.h"
 
-#include "nsIAtom.h"
 #include "nsIInputStream.h"
 
 #include "mozilla/Assertions.h"
 #include "mozilla/dom/ContentChild.h"
+#include "mozilla/dom/quota/Client.h"
 #include "mozilla/dom/quota/QuotaManager.h"
 
 #include "AsyncConnectionHelper.h"
@@ -24,6 +24,7 @@
 USING_INDEXEDDB_NAMESPACE
 
 using namespace mozilla::dom;
+using mozilla::dom::quota::Client;
 using mozilla::dom::quota::QuotaManager;
 
 namespace {
@@ -50,14 +51,14 @@ public:
   virtual nsresult
   OnSuccess() MOZ_OVERRIDE
   {
-    static_cast<IDBOpenDBRequest*>(mRequest.get())->SetTransaction(NULL);
+    static_cast<IDBOpenDBRequest*>(mRequest.get())->SetTransaction(nullptr);
     return AsyncConnectionHelper::OnSuccess();
   }
 
   virtual void
   OnError() MOZ_OVERRIDE
   {
-    static_cast<IDBOpenDBRequest*>(mRequest.get())->SetTransaction(NULL);
+    static_cast<IDBOpenDBRequest*>(mRequest.get())->SetTransaction(nullptr);
     AsyncConnectionHelper::OnError();
   }
 
@@ -103,7 +104,7 @@ class IPCDeleteDatabaseHelper : public AsyncConnectionHelper
 {
 public:
   IPCDeleteDatabaseHelper(IDBRequest* aRequest)
-  : AsyncConnectionHelper(static_cast<IDBDatabase*>(NULL), aRequest)
+  : AsyncConnectionHelper(static_cast<IDBDatabase*>(nullptr), aRequest)
   { }
 
   virtual nsresult UnpackResponseFromParentProcess(
@@ -140,7 +141,7 @@ public:
       return NS_OK;
     }
 
-    nsRefPtr<nsDOMEvent> event =
+    nsRefPtr<Event> event =
       IDBVersionChangeEvent::Create(mDatabase, mOldVersion, mNewVersion);
     MOZ_ASSERT(event);
 
@@ -158,12 +159,31 @@ public:
  * IndexedDBChild
  ******************************************************************************/
 
-IndexedDBChild::IndexedDBChild(const nsCString& aASCIIOrigin)
-: mFactory(nullptr), mASCIIOrigin(aASCIIOrigin)
+IndexedDBChild::IndexedDBChild(ContentChild* aContentChild,
+                               const nsCString& aASCIIOrigin)
+: mFactory(nullptr)
+, mManagerContent(aContentChild)
+, mManagerTab(nullptr)
+, mASCIIOrigin(aASCIIOrigin)
 #ifdef DEBUG
-  , mDisconnected(false)
+, mDisconnected(false)
 #endif
 {
+  MOZ_ASSERT(aContentChild);
+  MOZ_COUNT_CTOR(IndexedDBChild);
+}
+
+IndexedDBChild::IndexedDBChild(TabChild* aTabChild,
+                               const nsCString& aASCIIOrigin)
+: mFactory(nullptr)
+, mManagerContent(nullptr)
+, mManagerTab(aTabChild)
+, mASCIIOrigin(aASCIIOrigin)
+#ifdef DEBUG
+, mDisconnected(false)
+#endif
+{
+  MOZ_ASSERT(aTabChild);
   MOZ_COUNT_CTOR(IndexedDBChild);
 }
 
@@ -202,9 +222,9 @@ void
 IndexedDBChild::ActorDestroy(ActorDestroyReason aWhy)
 {
   if (mFactory) {
-    mFactory->SetActor(static_cast<IndexedDBChild*>(NULL));
+    mFactory->SetActor(static_cast<IndexedDBChild*>(nullptr));
 #ifdef DEBUG
-    mFactory = NULL;
+    mFactory = nullptr;
 #endif
   }
 }
@@ -247,7 +267,7 @@ IndexedDBChild::DeallocPIndexedDBDeleteDatabaseRequestChild(
 
 IndexedDBDatabaseChild::IndexedDBDatabaseChild(const nsString& aName,
                                                uint64_t aVersion)
-: mDatabase(NULL), mName(aName), mVersion(aVersion)
+: mDatabase(nullptr), mName(aName), mVersion(aVersion)
 {
   MOZ_COUNT_CTOR(IndexedDBDatabaseChild);
 }
@@ -284,15 +304,15 @@ IndexedDBDatabaseChild::EnsureDatabase(
                            const DatabaseInfoGuts& aDBInfo,
                            const InfallibleTArray<ObjectStoreInfoGuts>& aOSInfo)
 {
-  nsCOMPtr<nsIAtom> databaseId;
+  nsCString databaseId;
   if (mDatabase) {
     databaseId = mDatabase->Id();
   }
   else {
-    databaseId = QuotaManager::GetStorageId(aDBInfo.persistenceType,
-                                            aDBInfo.origin, aDBInfo.name);
+    QuotaManager::GetStorageId(aDBInfo.persistenceType, aDBInfo.origin,
+                               Client::IDB, aDBInfo.name, databaseId);
   }
-  NS_ENSURE_TRUE(databaseId, false);
+  MOZ_ASSERT(!databaseId.IsEmpty());
 
   nsRefPtr<DatabaseInfo> dbInfo;
   if (DatabaseInfo::Get(databaseId, getter_AddRefs(dbInfo))) {
@@ -326,7 +346,7 @@ IndexedDBDatabaseChild::EnsureDatabase(
   if (!mDatabase) {
     nsRefPtr<IDBDatabase> database =
       IDBDatabase::Create(aRequest, aRequest->Factory(), dbInfo.forget(),
-                          aDBInfo.origin, NULL, NULL);
+                          aDBInfo.origin, nullptr, nullptr);
     if (!database) {
       NS_WARNING("Failed to create database!");
       return false;
@@ -344,11 +364,10 @@ IndexedDBDatabaseChild::EnsureDatabase(
 void
 IndexedDBDatabaseChild::ActorDestroy(ActorDestroyReason aWhy)
 {
-  MOZ_ASSERT(!mStrongDatabase);
   if (mDatabase) {
-    mDatabase->SetActor(static_cast<IndexedDBDatabaseChild*>(NULL));
+    mDatabase->SetActor(static_cast<IndexedDBDatabaseChild*>(nullptr));
 #ifdef DEBUG
-    mDatabase = NULL;
+    mDatabase = nullptr;
 #endif
   }
 }
@@ -417,7 +436,7 @@ IndexedDBDatabaseChild::RecvError(const nsresult& aRv)
     request->Reset();
   }
   else {
-    openHelper = new IPCOpenDatabaseHelper(NULL, request);
+    openHelper = new IPCOpenDatabaseHelper(nullptr, request);
   }
 
   openHelper->SetError(aRv);
@@ -559,7 +578,7 @@ IndexedDBDatabaseChild::DeallocPIndexedDBTransactionChild(
  ******************************************************************************/
 
 IndexedDBTransactionChild::IndexedDBTransactionChild()
-: mTransaction(NULL)
+: mTransaction(nullptr)
 {
   MOZ_COUNT_CTOR(IndexedDBTransactionChild);
 }
@@ -626,9 +645,9 @@ IndexedDBTransactionChild::ActorDestroy(ActorDestroyReason aWhy)
   }
 
   if (mTransaction) {
-    mTransaction->SetActor(static_cast<IndexedDBTransactionChild*>(NULL));
+    mTransaction->SetActor(static_cast<IndexedDBTransactionChild*>(nullptr));
 #ifdef DEBUG
-    mTransaction = NULL;
+    mTransaction = nullptr;
 #endif
   }
 }
@@ -719,9 +738,9 @@ void
 IndexedDBObjectStoreChild::ActorDestroy(ActorDestroyReason aWhy)
 {
   if (mObjectStore) {
-    mObjectStore->SetActor(static_cast<IndexedDBObjectStoreChild*>(NULL));
+    mObjectStore->SetActor(static_cast<IndexedDBObjectStoreChild*>(nullptr));
 #ifdef DEBUG
-    mObjectStore = NULL;
+    mObjectStore = nullptr;
 #endif
   }
 }
@@ -742,17 +761,40 @@ IndexedDBObjectStoreChild::RecvPIndexedDBCursorConstructor(
 
   size_t direction = static_cast<size_t>(aParams.direction());
 
-  nsTArray<StructuredCloneFile> blobs;
-  IDBObjectStore::ConvertActorsToBlobs(aParams.blobsChild(), blobs);
-
   nsRefPtr<IDBCursor> cursor;
-  nsresult rv =
-    mObjectStore->OpenCursorFromChildProcess(request, direction, aParams.key(),
-                                             aParams.cloneInfo(), blobs,
-                                             getter_AddRefs(cursor));
-  NS_ENSURE_SUCCESS(rv, false);
+  nsresult rv;
 
-  MOZ_ASSERT(blobs.IsEmpty(), "Should have swapped blob elements!");
+  typedef ipc::OptionalStructuredCloneReadInfo CursorUnionType;
+
+  switch (aParams.optionalCloneInfo().type()) {
+    case CursorUnionType::TSerializedStructuredCloneReadInfo: {
+      nsTArray<StructuredCloneFile> blobs;
+      IDBObjectStore::ConvertActorsToBlobs(aParams.blobsChild(), blobs);
+
+      const SerializedStructuredCloneReadInfo& cloneInfo =
+        aParams.optionalCloneInfo().get_SerializedStructuredCloneReadInfo();
+
+      rv = mObjectStore->OpenCursorFromChildProcess(request, direction,
+                                                    aParams.key(), cloneInfo,
+                                                    blobs,
+                                                    getter_AddRefs(cursor));
+      NS_ENSURE_SUCCESS(rv, false);
+
+      MOZ_ASSERT(blobs.IsEmpty(), "Should have swapped blob elements!");
+    } break;
+
+    case CursorUnionType::Tvoid_t:
+      MOZ_ASSERT(aParams.blobsChild().IsEmpty());
+
+      rv = mObjectStore->OpenCursorFromChildProcess(request, direction,
+                                                    aParams.key(),
+                                                    getter_AddRefs(cursor));
+      NS_ENSURE_SUCCESS(rv, false);
+      break;
+
+    default:
+      MOZ_CRASH("Unknown union type!");
+  }
 
   actor->SetCursor(cursor);
   return true;
@@ -839,9 +881,9 @@ void
 IndexedDBIndexChild::ActorDestroy(ActorDestroyReason aWhy)
 {
   if (mIndex) {
-    mIndex->SetActor(static_cast<IndexedDBIndexChild*>(NULL));
+    mIndex->SetActor(static_cast<IndexedDBIndexChild*>(nullptr));
 #ifdef DEBUG
-    mIndex = NULL;
+    mIndex = nullptr;
 #endif
   }
 }
@@ -931,7 +973,7 @@ IndexedDBIndexChild::DeallocPIndexedDBCursorChild(PIndexedDBCursorChild* aActor)
  ******************************************************************************/
 
 IndexedDBCursorChild::IndexedDBCursorChild()
-: mCursor(NULL)
+: mCursor(nullptr)
 {
   MOZ_COUNT_CTOR(IndexedDBCursorChild);
 }
@@ -969,9 +1011,9 @@ void
 IndexedDBCursorChild::ActorDestroy(ActorDestroyReason aWhy)
 {
   if (mCursor) {
-    mCursor->SetActor(static_cast<IndexedDBCursorChild*>(NULL));
+    mCursor->SetActor(static_cast<IndexedDBCursorChild*>(nullptr));
 #ifdef DEBUG
-    mCursor = NULL;
+    mCursor = nullptr;
 #endif
   }
 }
@@ -1069,6 +1111,9 @@ IndexedDBObjectStoreRequestChild::Recv__delete__(const ResponseValue& aResponse)
     case ResponseValue::TGetAllResponse:
       MOZ_ASSERT(mRequestType == ParamsUnionType::TGetAllParams);
       break;
+    case ResponseValue::TGetAllKeysResponse:
+      MOZ_ASSERT(mRequestType == ParamsUnionType::TGetAllKeysParams);
+      break;
     case ResponseValue::TAddResponse:
       MOZ_ASSERT(mRequestType == ParamsUnionType::TAddParams);
       break;
@@ -1085,7 +1130,8 @@ IndexedDBObjectStoreRequestChild::Recv__delete__(const ResponseValue& aResponse)
       MOZ_ASSERT(mRequestType == ParamsUnionType::TCountParams);
       break;
     case ResponseValue::TOpenCursorResponse:
-      MOZ_ASSERT(mRequestType == ParamsUnionType::TOpenCursorParams);
+      MOZ_ASSERT(mRequestType == ParamsUnionType::TOpenCursorParams ||
+                 mRequestType == ParamsUnionType::TOpenKeyCursorParams);
       break;
 
     default:
@@ -1206,13 +1252,13 @@ IndexedDBCursorRequestChild::Recv__delete__(const ResponseValue& aResponse)
 IndexedDBDeleteDatabaseRequestChild::IndexedDBDeleteDatabaseRequestChild(
                                                  IDBFactory* aFactory,
                                                  IDBOpenDBRequest* aOpenRequest,
-                                                 nsIAtom* aDatabaseId)
+                                                 const nsACString& aDatabaseId)
 : mFactory(aFactory), mOpenRequest(aOpenRequest), mDatabaseId(aDatabaseId)
 {
   MOZ_COUNT_CTOR(IndexedDBDeleteDatabaseRequestChild);
   MOZ_ASSERT(aFactory);
   MOZ_ASSERT(aOpenRequest);
-  MOZ_ASSERT(aDatabaseId);
+  MOZ_ASSERT(!aDatabaseId.IsEmpty());
 }
 
 IndexedDBDeleteDatabaseRequestChild::~IndexedDBDeleteDatabaseRequestChild()
