@@ -10,22 +10,12 @@
 #include "npfunctions.h"
 #include "nsPluginHost.h"
 
-#include "jsapi.h"
+#include "nsCxPusher.h"
 
 #include "mozilla/PluginLibrary.h"
 
-/*
- * Use this macro before each exported function
- * (between the return address and the function
- * itself), to ensure that the function has the
- * right calling conventions on OS/2.
- */
-#define NP_CALLBACK NP_LOADDS
-
 #if defined(XP_WIN)
 #define NS_NPAPIPLUGIN_CALLBACK(_type, _name) _type (__stdcall * _name)
-#elif defined(XP_OS2)
-#define NS_NPAPIPLUGIN_CALLBACK(_type, _name) _type (_System * _name)
 #else
 #define NS_NPAPIPLUGIN_CALLBACK(_type, _name) _type (* _name)
 #endif
@@ -42,11 +32,10 @@ private:
 
 public:
   nsNPAPIPlugin();
-  virtual ~nsNPAPIPlugin();
 
   NS_DECL_ISUPPORTS
 
-  // Constructs and initializes an nsNPAPIPlugin object. A NULL file path
+  // Constructs and initializes an nsNPAPIPlugin object. A nullptr file path
   // will prevent this from calling NP_Initialize.
   static nsresult CreatePlugin(nsPluginTag *aPluginTag, nsNPAPIPlugin** aResult);
 
@@ -72,6 +61,8 @@ public:
   static nsresult RetainStream(NPStream *pstream, nsISupports **aRetainedPeer);
 
 protected:
+  virtual ~nsNPAPIPlugin();
+
   NPPluginFuncs mPluginFuncs;
   PluginLibrary* mLibrary;
 };
@@ -80,7 +71,8 @@ namespace mozilla {
 namespace plugins {
 namespace parent {
 
-JS_STATIC_ASSERT(sizeof(NPIdentifier) == sizeof(jsid));
+static_assert(sizeof(NPIdentifier) == sizeof(jsid),
+              "NPIdentifier must be binary compatible with jsid.");
 
 inline jsid
 NPIdentifierToJSId(NPIdentifier id)
@@ -137,113 +129,109 @@ JSContext* GetJSContext(NPP npp);
 inline bool
 NPStringIdentifierIsPermanent(NPP npp, NPIdentifier id)
 {
-  JSContext* cx = GetJSContext(npp);
-  if (!cx) // OOM?
-    return false;
-
-  JSAutoRequest ar(cx);
+  AutoSafeJSContext cx;
   return JS_StringHasBeenInterned(cx, NPIdentifierToString(id));
 }
 
 #define NPIdentifier_VOID (JSIdToNPIdentifier(JSID_VOID))
 
-NPObject* NP_CALLBACK
+NPObject*
 _getwindowobject(NPP npp);
 
-NPObject* NP_CALLBACK
+NPObject*
 _getpluginelement(NPP npp);
 
-NPIdentifier NP_CALLBACK
+NPIdentifier
 _getstringidentifier(const NPUTF8* name);
 
-void NP_CALLBACK
+void
 _getstringidentifiers(const NPUTF8** names, int32_t nameCount,
                       NPIdentifier *identifiers);
 
-bool NP_CALLBACK
+bool
 _identifierisstring(NPIdentifier identifiers);
 
-NPIdentifier NP_CALLBACK
+NPIdentifier
 _getintidentifier(int32_t intid);
 
-NPUTF8* NP_CALLBACK
+NPUTF8*
 _utf8fromidentifier(NPIdentifier identifier);
 
-int32_t NP_CALLBACK
+int32_t
 _intfromidentifier(NPIdentifier identifier);
 
-NPObject* NP_CALLBACK
+NPObject*
 _createobject(NPP npp, NPClass* aClass);
 
-NPObject* NP_CALLBACK
+NPObject*
 _retainobject(NPObject* npobj);
 
-void NP_CALLBACK
+void
 _releaseobject(NPObject* npobj);
 
-bool NP_CALLBACK
+bool
 _invoke(NPP npp, NPObject* npobj, NPIdentifier method, const NPVariant *args,
         uint32_t argCount, NPVariant *result);
 
-bool NP_CALLBACK
+bool
 _invokeDefault(NPP npp, NPObject* npobj, const NPVariant *args,
                uint32_t argCount, NPVariant *result);
 
-bool NP_CALLBACK
+bool
 _evaluate(NPP npp, NPObject* npobj, NPString *script, NPVariant *result);
 
-bool NP_CALLBACK
+bool
 _getproperty(NPP npp, NPObject* npobj, NPIdentifier property,
              NPVariant *result);
 
-bool NP_CALLBACK
+bool
 _setproperty(NPP npp, NPObject* npobj, NPIdentifier property,
              const NPVariant *value);
 
-bool NP_CALLBACK
+bool
 _removeproperty(NPP npp, NPObject* npobj, NPIdentifier property);
 
-bool NP_CALLBACK
+bool
 _hasproperty(NPP npp, NPObject* npobj, NPIdentifier propertyName);
 
-bool NP_CALLBACK
+bool
 _hasmethod(NPP npp, NPObject* npobj, NPIdentifier methodName);
 
-bool NP_CALLBACK
+bool
 _enumerate(NPP npp, NPObject *npobj, NPIdentifier **identifier,
            uint32_t *count);
 
-bool NP_CALLBACK
+bool
 _construct(NPP npp, NPObject* npobj, const NPVariant *args,
            uint32_t argCount, NPVariant *result);
 
-void NP_CALLBACK
+void
 _releasevariantvalue(NPVariant *variant);
 
-void NP_CALLBACK
+void
 _setexception(NPObject* npobj, const NPUTF8 *message);
 
-void NP_CALLBACK
+void
 _pushpopupsenabledstate(NPP npp, NPBool enabled);
 
-void NP_CALLBACK
+void
 _poppopupsenabledstate(NPP npp);
 
 typedef void(*PluginThreadCallback)(void *);
 
-void NP_CALLBACK
+void
 _pluginthreadasynccall(NPP instance, PluginThreadCallback func,
                        void *userData);
 
-NPError NP_CALLBACK
+NPError
 _getvalueforurl(NPP instance, NPNURLVariable variable, const char *url,
                 char **value, uint32_t *len);
 
-NPError NP_CALLBACK
+NPError
 _setvalueforurl(NPP instance, NPNURLVariable variable, const char *url,
                 const char *value, uint32_t len);
 
-NPError NP_CALLBACK
+NPError
 _getauthenticationinfo(NPP instance, const char *protocol, const char *host,
                        int32_t port, const char *scheme, const char *realm,
                        char **username, uint32_t *ulen, char **password,
@@ -251,95 +239,95 @@ _getauthenticationinfo(NPP instance, const char *protocol, const char *host,
 
 typedef void(*PluginTimerFunc)(NPP npp, uint32_t timerID);
 
-uint32_t NP_CALLBACK
+uint32_t
 _scheduletimer(NPP instance, uint32_t interval, NPBool repeat, PluginTimerFunc timerFunc);
 
-void NP_CALLBACK
+void
 _unscheduletimer(NPP instance, uint32_t timerID);
 
-NPError NP_CALLBACK
+NPError
 _popupcontextmenu(NPP instance, NPMenu* menu);
 
-NPError NP_CALLBACK
+NPError
 _initasyncsurface(NPP instance, NPSize *size, NPImageFormat format, void *initData, NPAsyncSurface *surface);
 
-NPError NP_CALLBACK
+NPError
 _finalizeasyncsurface(NPP instance, NPAsyncSurface *surface);
 
-void NP_CALLBACK
+void
 _setcurrentasyncsurface(NPP instance, NPAsyncSurface *surface, NPRect *changed);
 
-NPBool NP_CALLBACK
+NPBool
 _convertpoint(NPP instance, double sourceX, double sourceY, NPCoordinateSpace sourceSpace, double *destX, double *destY, NPCoordinateSpace destSpace);
 
-NPError NP_CALLBACK
+NPError
 _requestread(NPStream *pstream, NPByteRange *rangeList);
 
-NPError NP_CALLBACK
+NPError
 _geturlnotify(NPP npp, const char* relativeURL, const char* target,
               void* notifyData);
 
-NPError NP_CALLBACK
+NPError
 _getvalue(NPP npp, NPNVariable variable, void *r_value);
 
-NPError NP_CALLBACK
+NPError
 _setvalue(NPP npp, NPPVariable variable, void *r_value);
 
-NPError NP_CALLBACK
+NPError
 _geturl(NPP npp, const char* relativeURL, const char* target);
 
-NPError NP_CALLBACK
+NPError
 _posturlnotify(NPP npp, const char* relativeURL, const char *target,
                uint32_t len, const char *buf, NPBool file, void* notifyData);
 
-NPError NP_CALLBACK
+NPError
 _posturl(NPP npp, const char* relativeURL, const char *target, uint32_t len,
             const char *buf, NPBool file);
 
-NPError NP_CALLBACK
+NPError
 _newstream(NPP npp, NPMIMEType type, const char* window, NPStream** pstream);
 
-int32_t NP_CALLBACK
+int32_t
 _write(NPP npp, NPStream *pstream, int32_t len, void *buffer);
 
-NPError NP_CALLBACK
+NPError
 _destroystream(NPP npp, NPStream *pstream, NPError reason);
 
-void NP_CALLBACK
+void
 _status(NPP npp, const char *message);
 
-void NP_CALLBACK
+void
 _memfree (void *ptr);
 
-uint32_t NP_CALLBACK
+uint32_t
 _memflush(uint32_t size);
 
-void NP_CALLBACK
+void
 _reloadplugins(NPBool reloadPages);
 
-void NP_CALLBACK
+void
 _invalidaterect(NPP npp, NPRect *invalidRect);
 
-void NP_CALLBACK
+void
 _invalidateregion(NPP npp, NPRegion invalidRegion);
 
-void NP_CALLBACK
+void
 _forceredraw(NPP npp);
 
-const char* NP_CALLBACK
+const char*
 _useragent(NPP npp);
 
-void* NP_CALLBACK
+void*
 _memalloc (uint32_t size);
 
 // Deprecated entry points for the old Java plugin.
-void* NP_CALLBACK /* OJI type: JRIEnv* */
+void* /* OJI type: JRIEnv* */
 _getJavaEnv();
 
-void* NP_CALLBACK /* OJI type: jref */
+void* /* OJI type: jref */
 _getJavaPeer(NPP npp);
 
-void NP_CALLBACK
+void
 _urlredirectresponse(NPP instance, void* notifyData, NPBool allow);
 
 } /* namespace parent */
