@@ -4,6 +4,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "nsBindingManager.h"
+
 #include "nsCOMPtr.h"
 #include "nsXBLService.h"
 #include "nsIInputStream.h"
@@ -23,6 +25,7 @@
 #include "mozilla/dom/XMLDocument.h"
 #include "nsIStreamListener.h"
 #include "ChildIterator.h"
+#include "nsITimer.h"
 
 #include "nsXBLBinding.h"
 #include "nsXBLPrototypeBinding.h"
@@ -41,148 +44,16 @@
 #include "nsTHashtable.h"
 
 #include "nsIScriptContext.h"
-#include "nsBindingManager.h"
 #include "xpcpublic.h"
 #include "jswrapper.h"
-#include "nsCxPusher.h"
 
 #include "nsThreadUtils.h"
 #include "mozilla/dom/NodeListBinding.h"
+#include "mozilla/dom/ScriptSettings.h"
 
 using namespace mozilla;
 using namespace mozilla::dom;
 
-//
-// Generic pldhash table stuff for mapping one nsISupports to another
-//
-// These values are never null - a null value implies that this
-// whole key should be removed (See SetOrRemoveObject)
-class ObjectEntry : public PLDHashEntryHdr
-{
-public:
-
-  // note that these are allocated within the PLDHashTable, but we
-  // want to keep track of them anyway
-  ObjectEntry() { MOZ_COUNT_CTOR(ObjectEntry); }
-  ~ObjectEntry() { MOZ_COUNT_DTOR(ObjectEntry); }
-  
-  nsISupports* GetValue() { return mValue; }
-  nsISupports* GetKey() { return mKey; }
-  void SetValue(nsISupports* aValue) { mValue = aValue; }
-  void SetKey(nsISupports* aKey) { mKey = aKey; }
-  
-private:
-  nsCOMPtr<nsISupports> mKey;
-  nsCOMPtr<nsISupports> mValue;
-};
-
-static void
-ClearObjectEntry(PLDHashTable* table, PLDHashEntryHdr *entry)
-{
-  ObjectEntry* objEntry = static_cast<ObjectEntry*>(entry);
-  objEntry->~ObjectEntry();
-}
-
-static bool
-InitObjectEntry(PLDHashTable* table, PLDHashEntryHdr* entry, const void* key)
-{
-  new (entry) ObjectEntry;
-  return true;
-}
-  
-
-
-static const PLDHashTableOps ObjectTableOps = {
-  PL_DHashAllocTable,
-  PL_DHashFreeTable,
-  PL_DHashVoidPtrKeyStub,
-  PL_DHashMatchEntryStub,
-  PL_DHashMoveEntryStub,
-  ClearObjectEntry,
-  PL_DHashFinalizeStub,
-  InitObjectEntry
-};
-
-// helper routine for adding a new entry
-static nsresult
-AddObjectEntry(PLDHashTable& table, nsISupports* aKey, nsISupports* aValue)
-{
-  NS_ASSERTION(aKey, "key must be non-null");
-  if (!aKey) return NS_ERROR_INVALID_ARG;
-  
-  ObjectEntry *entry =
-    static_cast<ObjectEntry*>
-               (PL_DHashTableOperate(&table, aKey, PL_DHASH_ADD));
-
-  if (!entry)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  // only add the key if the entry is new
-  if (!entry->GetKey())
-    entry->SetKey(aKey);
-
-  // now attach the new entry - note that entry->mValue could possibly
-  // have a value already, this will release that.
-  entry->SetValue(aValue);
-  
-  return NS_OK;
-}
-
-// helper routine for looking up an existing entry. Note that the
-// return result is NOT addreffed
-static nsISupports*
-LookupObject(PLDHashTable& table, nsIContent* aKey)
-{
-  if (aKey && aKey->HasFlag(NODE_MAY_BE_IN_BINDING_MNGR)) {
-    ObjectEntry *entry =
-      static_cast<ObjectEntry*>
-                 (PL_DHashTableOperate(&table, aKey, PL_DHASH_LOOKUP));
-
-    if (PL_DHASH_ENTRY_IS_BUSY(entry))
-      return entry->GetValue();
-  }
-
-  return nullptr;
-}
-
-inline void
-RemoveObjectEntry(PLDHashTable& table, nsISupports* aKey)
-{
-  PL_DHashTableOperate(&table, aKey, PL_DHASH_REMOVE);
-}
-
-static nsresult
-SetOrRemoveObject(PLDHashTable& table, nsIContent* aKey, nsISupports* aValue)
-{
-  if (aValue) {
-    // lazily create the table, but only when adding elements
-    if (!table.ops) {
-      PL_DHashTableInit(&table, &ObjectTableOps, nullptr,
-                        sizeof(ObjectEntry), 16);
-    }
-    aKey->SetFlags(NODE_MAY_BE_IN_BINDING_MNGR);
-    return AddObjectEntry(table, aKey, aValue);
-  }
-
-  // no value, so remove the key from the table
-  if (table.ops) {
-    ObjectEntry* entry =
-      static_cast<ObjectEntry*>
-        (PL_DHashTableOperate(&table, aKey, PL_DHASH_LOOKUP));
-    if (entry && PL_DHASH_ENTRY_IS_BUSY(entry)) {
-      // Keep key and value alive while removing the entry.
-      nsCOMPtr<nsISupports> key = entry->GetKey();
-      nsCOMPtr<nsISupports> value = entry->GetValue();
-      RemoveObjectEntry(table, aKey);
-    }
-  }
-  return NS_OK;
-}
-
-// Implementation /////////////////////////////////////////////////////////////////
-
-// Static member variable initialization
-
 // Implement our nsISupports methods
 
 NS_IMPL_CYCLE_COLLECTION_CLASS(nsBindingManager)
@@ -199,9 +70,10 @@ NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsBindingManager)
   if (tmp->mLoadingDocTable)
     tmp->mLoadingDocTable->Clear();
 
-  if (tmp->mWrapperTable.ops)
-    PL_DHashTableFinish(&(tmp->mWrapperTable));
-  tmp->mWrapperTable.ops = nullptr;
+  if (tmp->mWrapperTable) {
+    tmp->mWrapperTable->Clear();
+    tmp->mWrapperTable = nullptr;
+  }
 
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mAttachedStack)
 
@@ -216,7 +88,7 @@ DocumentInfoHashtableTraverser(nsIURI* key,
                                nsXBLDocumentInfo* di,
                                void* userArg)
 {
-  nsCycleCollectionTraversalCallback *cb = 
+  nsCycleCollectionTraversalCallback *cb =
     static_cast<nsCycleCollectionTraversalCallback*>(userArg);
   NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(*cb, "mDocumentTable value");
   cb->NoteXPCOMChild(di);
@@ -228,7 +100,7 @@ LoadingDocHashtableTraverser(nsIURI* key,
                              nsIStreamListener* sl,
                              void* userArg)
 {
-  nsCycleCollectionTraversalCallback *cb = 
+  nsCycleCollectionTraversalCallback *cb =
     static_cast<nsCycleCollectionTraversalCallback*>(userArg);
   NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(*cb, "mLoadingDocTable value");
   cb->NoteXPCOMChild(sl);
@@ -261,15 +133,11 @@ nsBindingManager::nsBindingManager(nsIDocument* aDocument)
     mAttachedStackSizeOnOutermost(0),
     mDocument(aDocument)
 {
-  mWrapperTable.ops = nullptr;
 }
 
 nsBindingManager::~nsBindingManager(void)
 {
   mDestroyed = true;
-
-  if (mWrapperTable.ops)
-    PL_DHashTableFinish(&mWrapperTable);
 }
 
 nsXBLBinding*
@@ -301,12 +169,16 @@ nsBindingManager::RemoveBoundContent(nsIContent* aContent)
 
 nsIXPConnectWrappedJS*
 nsBindingManager::GetWrappedJS(nsIContent* aContent)
-{ 
-  if (mWrapperTable.ops) {
-    return static_cast<nsIXPConnectWrappedJS*>(LookupObject(mWrapperTable, aContent));
+{
+  if (!mWrapperTable) {
+    return nullptr;
   }
 
-  return nullptr;
+  if (!aContent || !aContent->HasFlag(NODE_MAY_BE_IN_BINDING_MNGR)) {
+    return nullptr;
+  }
+
+  return mWrapperTable->GetWeak(aContent);
 }
 
 nsresult
@@ -316,7 +188,27 @@ nsBindingManager::SetWrappedJS(nsIContent* aContent, nsIXPConnectWrappedJS* aWra
     return NS_OK;
   }
 
-  return SetOrRemoveObject(mWrapperTable, aContent, aWrappedJS);
+  if (aWrappedJS) {
+    // lazily create the table, but only when adding elements
+    if (!mWrapperTable) {
+      mWrapperTable = new WrapperHashtable();
+    }
+    aContent->SetFlags(NODE_MAY_BE_IN_BINDING_MNGR);
+
+    NS_ASSERTION(aContent, "key must be non-null");
+    if (!aContent) return NS_ERROR_INVALID_ARG;
+
+    mWrapperTable->Put(aContent, aWrappedJS);
+
+    return NS_OK;
+  }
+
+  // no value, so remove the key from the table
+  if (mWrapperTable) {
+    mWrapperTable->Remove(aContent);
+  }
+
+  return NS_OK;
 }
 
 void
@@ -325,17 +217,22 @@ nsBindingManager::RemovedFromDocumentInternal(nsIContent* aContent,
 {
   NS_PRECONDITION(aOldDocument != nullptr, "no old document");
 
-  if (mDestroyed)
-    return;
-
   nsRefPtr<nsXBLBinding> binding = aContent->GetXBLBinding();
   if (binding) {
-    binding->PrototypeBinding()->BindingDetached(binding->GetBoundElement());
-    binding->ChangeDocument(aOldDocument, nullptr);
+    // The binding manager may have been destroyed before a runnable
+    // has had a chance to reach this point. If so, we bail out on calling
+    // BindingDetached (which may invoke a XBL destructor) and
+    // ChangeDocument, but we still want to clear out the binding
+    // and insertion parent that may hold references.
+    if (!mDestroyed) {
+      binding->PrototypeBinding()->BindingDetached(binding->GetBoundElement());
+      binding->ChangeDocument(aOldDocument, nullptr);
+    }
+
     aContent->SetXBLBinding(nullptr, this);
   }
 
-  // Clear out insertion parents and content lists.
+  // Clear out insertion parent and content lists.
   aContent->SetXBLInsertionParent(nullptr);
 }
 
@@ -391,7 +288,7 @@ nsBindingManager::ClearBinding(nsIContent* aContent)
   // currentdoc too?  What's the one that should be passed to
   // ChangeDocument?
   nsCOMPtr<nsIDocument> doc = aContent->OwnerDoc();
-  
+
   // Finally remove the binding...
   // XXXbz this doesn't remove the implementation!  Should fix!  Until
   // then we need the explicit UnhookEventHandlers here.
@@ -399,7 +296,7 @@ nsBindingManager::ClearBinding(nsIContent* aContent)
   binding->ChangeDocument(doc, nullptr);
   aContent->SetXBLBinding(nullptr, this);
   binding->MarkForDeath();
-  
+
   // ...and recreate its frames. We need to do this since the frames may have
   // been removed and style may have changed due to the removal of the
   // anonymous children.
@@ -416,7 +313,7 @@ nsBindingManager::LoadBindingDocument(nsIDocument* aBoundDoc,
                                       nsIPrincipal* aOriginPrincipal)
 {
   NS_PRECONDITION(aURL, "Must have a URI to load!");
-  
+
   // First we need to load our binding.
   nsXBLService* xblService = nsXBLService::GetInstance();
   if (!xblService)
@@ -438,7 +335,7 @@ nsBindingManager::RemoveFromAttachedQueue(nsXBLBinding* aBinding)
 {
   // Don't remove items here as that could mess up an executing
   // ProcessAttachedQueue. Instead, null the entry in the queue.
-  uint32_t index = mAttachedStack.IndexOf(aBinding);
+  size_t index = mAttachedStack.IndexOf(aBinding);
   if (index != mAttachedStack.NoIndex) {
     mAttachedStack[index] = nullptr;
   }
@@ -474,6 +371,16 @@ nsBindingManager::PostProcessAttachedQueueEvent()
   }
 }
 
+// static
+void
+nsBindingManager::PostPAQEventCallback(nsITimer* aTimer, void* aClosure)
+{
+  nsRefPtr<nsBindingManager> mgr = 
+    already_AddRefed<nsBindingManager>(static_cast<nsBindingManager*>(aClosure));
+  mgr->PostProcessAttachedQueueEvent();
+  NS_RELEASE(aTimer);
+}
+
 void
 nsBindingManager::DoProcessAttachedQueue()
 {
@@ -482,13 +389,25 @@ nsBindingManager::DoProcessAttachedQueue()
 
     NS_ASSERTION(mAttachedStack.Length() == 0,
                "Shouldn't have pending bindings!");
-  
+
     mProcessAttachedQueueEvent = nullptr;
   } else {
     // Someone's doing event processing from inside a constructor.
     // They're evil, but we'll fight back!  Just poll on them being
     // done and repost the attached queue event.
-    PostProcessAttachedQueueEvent();
+    //
+    // But don't poll in a tight loop -- otherwise we keep the Gecko
+    // event loop non-empty and trigger bug 1021240 on OS X.
+    nsresult rv = NS_ERROR_FAILURE;
+    nsCOMPtr<nsITimer> timer = do_CreateInstance(NS_TIMER_CONTRACTID);
+    if (timer) {
+      rv = timer->InitWithFuncCallback(PostPAQEventCallback, this,
+                                       100, nsITimer::TYPE_ONE_SHOT);
+    }
+    if (NS_SUCCEEDED(rv)) {
+      NS_ADDREF_THIS();
+      NS_ADDREF(timer);
+    }
   }
 
   // No matter what, unblock onload for the event that's fired.
@@ -525,7 +444,7 @@ nsBindingManager::ProcessAttachedQueue(uint32_t aSkipSize)
   }
 
   NS_ASSERTION(mAttachedStack.Length() == aSkipSize, "How did we get here?");
-  
+
   mAttachedStack.Compact();
 }
 
@@ -569,7 +488,7 @@ nsresult
 nsBindingManager::PutXBLDocumentInfo(nsXBLDocumentInfo* aDocumentInfo)
 {
   NS_PRECONDITION(aDocumentInfo, "Must have a non-null documentinfo!");
-  
+
   if (!mDocumentTable) {
     mDocumentTable = new nsRefPtrHashtable<nsURIHashKey,nsXBLDocumentInfo>(16);
   }
@@ -600,7 +519,7 @@ nsresult
 nsBindingManager::PutLoadingDocListener(nsIURI* aURL, nsIStreamListener* aListener)
 {
   NS_PRECONDITION(aListener, "Must have a non-null listener!");
-  
+
   if (!mLoadingDocTable) {
     mLoadingDocTable = new nsInterfaceHashtable<nsURIHashKey,nsIStreamListener>(16);
   }
@@ -653,12 +572,12 @@ nsBindingManager::FlushSkinBindings()
 
 // Used below to protect from recurring in QI calls through XPConnect.
 struct AntiRecursionData {
-  nsIContent* element; 
-  REFNSIID iid; 
+  nsIContent* element;
+  REFNSIID iid;
   AntiRecursionData* next;
 
-  AntiRecursionData(nsIContent* aElement, 
-                    REFNSIID aIID, 
+  AntiRecursionData(nsIContent* aElement,
+                    REFNSIID aIID,
                     AntiRecursionData* aNext)
     : element(aElement), iid(aIID), next(aNext) {}
 };
@@ -676,19 +595,19 @@ nsBindingManager::GetBindingImplementation(nsIContent* aContent, REFNSIID aIID,
       nsCOMPtr<nsIXPConnectWrappedJS> wrappedJS = GetWrappedJS(aContent);
 
       if (wrappedJS) {
-        // Protect from recurring in QI calls through XPConnect. 
+        // Protect from recurring in QI calls through XPConnect.
         // This can happen when a second binding is being resolved.
         // At that point a wrappedJS exists, but it doesn't yet know about
-        // the iid we are asking for. So, without this protection, 
+        // the iid we are asking for. So, without this protection,
         // AggregatedQueryInterface would end up recurring back into itself
-        // through this code. 
+        // through this code.
         //
-        // With this protection, when we detect the recursion we return 
-        // NS_NOINTERFACE in the inner call. The outer call will then fall 
+        // With this protection, when we detect the recursion we return
+        // NS_NOINTERFACE in the inner call. The outer call will then fall
         // through (see below) and build a new chained wrappedJS for the iid.
         //
         // We're careful to not assume that only one direct nesting can occur
-        // because there is a call into JS in the middle and we can't assume 
+        // because there is a call into JS in the middle and we can't assume
         // that this code won't be reached by some more complex nesting path.
         //
         // NOTE: We *assume* this is single threaded, so we can use a
@@ -707,33 +626,21 @@ nsBindingManager::GetBindingImplementation(nsIContent* aContent, REFNSIID aIID,
         list = &item;
 
         nsresult rv = wrappedJS->AggregatedQueryInterface(aIID, aResult);
-        
+
         list = item.next;
-        
+
         if (*aResult)
           return rv;
-        
+
         // No result was found, so this must be another XBL interface.
         // Fall through to create a new wrapper.
       }
 
       // We have never made a wrapper for this implementation.
       // Create an XPC wrapper for the script object and hand it back.
-
-      nsIDocument* doc = aContent->OwnerDoc();
-
-      nsCOMPtr<nsIScriptGlobalObject> global =
-        do_QueryInterface(doc->GetWindow());
-      if (!global)
-        return NS_NOINTERFACE;
-
-      nsIScriptContext *context = global->GetContext();
-      if (!context)
-        return NS_NOINTERFACE;
-
-      AutoPushJSContext cx(context->GetNativeContext());
-      if (!cx)
-        return NS_NOINTERFACE;
+      AutoJSAPI jsapi;
+      jsapi.Init();
+      JSContext* cx = jsapi.cx();
 
       nsIXPConnect *xpConnect = nsContentUtils::XPConnect();
 
@@ -743,11 +650,13 @@ nsBindingManager::GetBindingImplementation(nsIContent* aContent, REFNSIID aIID,
       // If we're using an XBL scope, we need to use the Xray view to the bound
       // content in order to view the full array of methods defined in the
       // binding, some of which may not be exposed on the prototype of
-      // untrusted content.
+      // untrusted content. We don't need to consider add-on scopes here
+      // because they're chrome-only and no Xrays are involved.
       //
-      // If there's no separate XBL scope, we'll end up with the global of the
-      // reflector, and this will all be a no-op.
-      JS::Rooted<JSObject*> xblScope(cx, xpc::GetXBLScope(cx, jsobj));
+      // If there's no separate XBL scope, or if the reflector itself lives in
+      // the XBL scope, we'll end up with the global of the reflector.
+      JS::Rooted<JSObject*> xblScope(cx, xpc::GetXBLScopeOrGlobal(cx, jsobj));
+      NS_ENSURE_TRUE(xblScope, NS_ERROR_UNEXPECTED);
       JSAutoCompartment ac(cx, xblScope);
       bool ok = JS_WrapObject(cx, &jsobj);
       NS_ENSURE_TRUE(ok, NS_ERROR_OUT_OF_MEMORY);
@@ -768,7 +677,7 @@ nsBindingManager::GetBindingImplementation(nsIContent* aContent, REFNSIID aIID,
       return rv;
     }
   }
-  
+
   *aResult = nullptr;
   return NS_NOINTERFACE;
 }
@@ -779,13 +688,13 @@ nsBindingManager::WalkRules(nsIStyleRuleProcessor::EnumFunc aFunc,
                             bool* aCutOffInheritance)
 {
   *aCutOffInheritance = false;
-  
+
   NS_ASSERTION(aData->mElement, "How did that happen?");
 
   // Walk the binding scope chain, starting with the binding attached to our
   // content, up till we run out of scopes or we get cut off.
   nsIContent *content = aData->mElement;
-  
+
   do {
     nsXBLBinding *binding = content->GetXBLBinding();
     if (binding) {
@@ -848,7 +757,7 @@ static PLDHashOperator
 EnumWalkAllRules(nsPtrHashKey<nsIStyleRuleProcessor> *aKey, void* aClosure)
 {
   nsIStyleRuleProcessor *ruleProcessor = aKey->GetKey();
-    
+
   WalkAllRulesData *data = static_cast<WalkAllRulesData*>(aClosure);
 
   (*(data->mFunc))(ruleProcessor, data->mData);
@@ -882,7 +791,7 @@ static PLDHashOperator
 EnumMediumFeaturesChanged(nsPtrHashKey<nsIStyleRuleProcessor> *aKey, void* aClosure)
 {
   nsIStyleRuleProcessor *ruleProcessor = aKey->GetKey();
-    
+
   MediumFeaturesChangedData *data =
     static_cast<MediumFeaturesChangedData*>(aClosure);
 
@@ -916,23 +825,17 @@ static PLDHashOperator
 EnumAppendAllSheets(nsRefPtrHashKey<nsIContent> *aKey, void* aClosure)
 {
   nsIContent *boundContent = aKey->GetKey();
-  nsTArray<nsCSSStyleSheet*>* array =
-    static_cast<nsTArray<nsCSSStyleSheet*>*>(aClosure);
+  nsTArray<CSSStyleSheet*>* array =
+    static_cast<nsTArray<CSSStyleSheet*>*>(aClosure);
   for (nsXBLBinding *binding = boundContent->GetXBLBinding(); binding;
        binding = binding->GetBaseBinding()) {
-    nsXBLPrototypeResources::sheet_array_type* sheets =
-      binding->PrototypeBinding()->GetStyleSheets();
-    if (sheets) {
-      // Copy from nsTArray<nsRefPtr<nsCSSStyleSheet> > to
-      // nsTArray<nsCSSStyleSheet*>.
-      array->AppendElements(*sheets);
-    }
+    binding->PrototypeBinding()->AppendStyleSheetsTo(*array);
   }
   return PL_DHASH_NEXT;
 }
 
 void
-nsBindingManager::AppendAllSheets(nsTArray<nsCSSStyleSheet*>& aArray)
+nsBindingManager::AppendAllSheets(nsTArray<CSSStyleSheet*>& aArray)
 {
   if (mBoundContentSet) {
     mBoundContentSet->EnumerateEntries(EnumAppendAllSheets, &aArray);
@@ -943,12 +846,12 @@ static void
 InsertAppendedContent(XBLChildrenElement* aPoint,
                       nsIContent* aFirstNewContent)
 {
-  uint32_t insertionIndex;
+  int32_t insertionIndex;
   if (nsIContent* prevSibling = aFirstNewContent->GetPreviousSibling()) {
     // If we have a previous sibling, then it must already be in aPoint. Find
     // it and insert after it.
     insertionIndex = aPoint->IndexOfInsertedChild(prevSibling);
-    MOZ_ASSERT(insertionIndex != aPoint->NoIndex);
+    MOZ_ASSERT(insertionIndex != -1);
 
     // Our insertion index is one after our previous sibling's index.
     ++insertionIndex;
@@ -956,7 +859,7 @@ InsertAppendedContent(XBLChildrenElement* aPoint,
     // Otherwise, we append.
     // TODO This is wrong for nested insertion points. In that case, we need to
     // keep track of the right index to insert into.
-    insertionIndex = aPoint->mInsertedChildren.Length();
+    insertionIndex = aPoint->InsertedChildrenLength();
   }
 
   // Do the inserting.
@@ -1134,9 +1037,8 @@ nsBindingManager::Traverse(nsIContent *aContent,
     cb.NoteXPCOMChild(aContent);
   }
 
-  nsISupports *value;
-  if (mWrapperTable.ops &&
-      (value = LookupObject(mWrapperTable, aContent))) {
+  nsIXPConnectWrappedJS *value = GetWrappedJS(aContent);
+  if (value) {
     NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(cb, "[via binding manager] mWrapperTable key");
     cb.NoteXPCOMChild(aContent);
     NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(cb, "[via binding manager] mWrapperTable value");
@@ -1187,15 +1089,15 @@ nsBindingManager::HandleChildInsertion(nsIContent* aContainer,
     // TODO If there were multiple insertion points, this approximation can be
     // wrong. We need to re-run the distribution algorithm. In the meantime,
     // this should work well enough.
-    uint32_t index = aAppend ? point->mInsertedChildren.Length() : 0;
+    uint32_t index = aAppend ? point->InsertedChildrenLength() : 0;
     for (nsIContent* currentSibling = aChild->GetPreviousSibling();
          currentSibling;
          currentSibling = currentSibling->GetPreviousSibling()) {
       // If we find one of our previous siblings in the insertion point, the
       // index following it is the correct insertion point. Otherwise, we guess
       // based on whether we're appending or inserting.
-      uint32_t pointIndex = point->IndexOfInsertedChild(currentSibling);
-      if (pointIndex != point->NoIndex) {
+      int32_t pointIndex = point->IndexOfInsertedChild(currentSibling);
+      if (pointIndex != -1) {
         index = pointIndex + 1;
         break;
       }
@@ -1234,7 +1136,7 @@ nsBindingManager::FindNestedInsertionPoint(nsIContent* aContainer,
     if (!binding) {
       break;
     }
-  
+
     XBLChildrenElement* point = binding->FindInsertionPointFor(aChild);
     if (!point) {
       return nullptr;
