@@ -4,7 +4,10 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "SourceSurfaceRawData.h"
+
+#include "DataSurfaceHelpers.h"
 #include "Logging.h"
+#include "mozilla/Types.h" // for decltype
 
 namespace mozilla {
 namespace gfx {
@@ -27,12 +30,22 @@ SourceSurfaceRawData::InitWrappingData(uint8_t *aData,
 
 bool
 SourceSurfaceAlignedRawData::Init(const IntSize &aSize,
-                                  SurfaceFormat aFormat)
+                                  SurfaceFormat aFormat,
+                                  bool aZero)
 {
-  mStride = GetAlignedStride<16>(aSize.width * BytesPerPixel(aFormat));
-  mArray.Realloc(mStride * aSize.height);
-  mSize = aSize;
   mFormat = aFormat;
+  mStride = GetAlignedStride<16>(aSize.width * BytesPerPixel(aFormat));
+
+  size_t bufLen = BufferSizeFromStrideAndHeight(mStride, aSize.height);
+  if (bufLen > 0) {
+    static_assert(sizeof(decltype(mArray[0])) == 1,
+                  "mArray.Realloc() takes an object count, so its objects must be 1-byte sized if we use bufLen");
+    mArray.Realloc(/* actually an object count */ bufLen, aZero);
+    mSize = aSize;
+  } else {
+    mArray.Dealloc();
+    mSize.SizeTo(0, 0);
+  }
 
   return mArray != nullptr;
 }
@@ -40,12 +53,22 @@ SourceSurfaceAlignedRawData::Init(const IntSize &aSize,
 bool
 SourceSurfaceAlignedRawData::InitWithStride(const IntSize &aSize,
                                             SurfaceFormat aFormat,
-                                            int32_t aStride)
+                                            int32_t aStride,
+                                            bool aZero)
 {
-  mStride = aStride;
-  mArray.Realloc(mStride * aSize.height);
-  mSize = aSize;
   mFormat = aFormat;
+  mStride = aStride;
+
+  size_t bufLen = BufferSizeFromStrideAndHeight(mStride, aSize.height);
+  if (bufLen > 0) {
+    static_assert(sizeof(decltype(mArray[0])) == 1,
+                  "mArray.Realloc() takes an object count, so its objects must be 1-byte sized if we use bufLen");
+    mArray.Realloc(/* actually an object count */ bufLen, aZero);
+    mSize = aSize;
+  } else {
+    mArray.Dealloc();
+    mSize.SizeTo(0, 0);
+  }
 
   return mArray != nullptr;
 }
