@@ -8,7 +8,6 @@
 #include <string.h>                     // for memcpy, memset
 #include "SharedTextureImage.h"         // for SharedTextureImage
 #include "gfx2DGlue.h"
-#include "gfxImageSurface.h"            // for gfxImageSurface
 #include "gfxPlatform.h"                // for gfxPlatform
 #include "gfxUtils.h"                   // for gfxUtils
 #include "mozilla/RefPtr.h"             // for TemporaryRef
@@ -22,6 +21,7 @@
 #include "GrallocImages.h"
 #endif
 #include "gfx2DGlue.h"
+#include "mozilla/gfx/2D.h"
 
 #ifdef XP_MACOSX
 #include "mozilla/gfx/QuartzSupport.h"
@@ -36,27 +36,15 @@
 #include "D3D9SurfaceImage.h"
 #endif
 
-using namespace mozilla::ipc;
-using namespace android;
-using mozilla::gfx::DataSourceSurface;
-using mozilla::gfx::SourceSurface;
-
-
 namespace mozilla {
 namespace layers {
 
-class DataSourceSurface;
-class SourceSurface;
+using namespace mozilla::ipc;
+using namespace android;
+using namespace mozilla::gfx;
 
 Atomic<int32_t> Image::sSerialCounter(0);
 
-TemporaryRef<gfx::SourceSurface>
-Image::GetAsSourceSurface()
-{
-  nsRefPtr<gfxASurface> surface = DeprecatedGetAsSurface();
-  return gfxPlatform::GetPlatform()->GetSourceSurfaceForSurface(nullptr, surface);
-}
-
 already_AddRefed<Image>
 ImageFactory::CreateImage(ImageFormat aFormat,
                           const gfx::IntSize &,
@@ -141,11 +129,7 @@ ImageContainer::ImageContainer(int flag)
   if (flag == ENABLE_ASYNC && ImageBridgeChild::IsCreated()) {
     // the refcount of this ImageClient is 1. we don't use a RefPtr here because the refcount
     // of this class must be done on the ImageBridge thread.
-    if (gfxPlatform::GetPlatform()->UseDeprecatedTextures()) {
-      mImageClient = ImageBridgeChild::GetSingleton()->CreateImageClient(BUFFER_IMAGE_BUFFERED).drop();
-    } else {
-      mImageClient = ImageBridgeChild::GetSingleton()->CreateImageClient(BUFFER_IMAGE_SINGLE).drop();
-    }
+    mImageClient = ImageBridgeChild::GetSingleton()->CreateImageClient(CompositableType::BUFFER_IMAGE_SINGLE).drop();
     MOZ_ASSERT(mImageClient);
   }
 }
@@ -265,7 +249,7 @@ ImageContainer::HasCurrentImage()
 
   if (mRemoteData) {
     CrossProcessMutexAutoLock autoLock(*mRemoteDataMutex);
-    
+
     EnsureActiveImage();
 
     return !!mActiveImage.get();
@@ -278,7 +262,7 @@ already_AddRefed<Image>
 ImageContainer::LockCurrentImage()
 {
   ReentrantMonitorAutoEnter mon(mReentrantMonitor);
-  
+
   if (mRemoteData) {
     NS_ASSERTION(mRemoteDataMutex, "Should have remote data mutex when having remote data!");
     mRemoteDataMutex->Lock();
@@ -290,57 +274,6 @@ ImageContainer::LockCurrentImage()
   return retval.forget();
 }
 
-already_AddRefed<gfxASurface>
-ImageContainer::DeprecatedLockCurrentAsSurface(gfx::IntSize *aSize, Image** aCurrentImage)
-{
-  ReentrantMonitorAutoEnter mon(mReentrantMonitor);
-
-  if (mRemoteData) {
-    NS_ASSERTION(mRemoteDataMutex, "Should have remote data mutex when having remote data!");
-    mRemoteDataMutex->Lock();
-
-    EnsureActiveImage();
-
-    if (aCurrentImage) {
-      NS_IF_ADDREF(mActiveImage);
-      *aCurrentImage = mActiveImage.get();
-    }
-
-    if (!mActiveImage) {
-      return nullptr;
-    } 
-
-    if (mActiveImage->GetFormat() == ImageFormat::REMOTE_IMAGE_BITMAP) {
-      nsRefPtr<gfxImageSurface> newSurf =
-        new gfxImageSurface(mRemoteData->mBitmap.mData,
-                            ThebesIntSize(mRemoteData->mSize),
-                            mRemoteData->mBitmap.mStride,
-                            mRemoteData->mFormat == RemoteImageData::BGRX32 ?
-                                                   gfxImageFormat::ARGB32 :
-                                                   gfxImageFormat::RGB24);
-
-      *aSize = newSurf->GetSize().ToIntSize();
-    
-      return newSurf.forget();
-    }
-
-    *aSize = mActiveImage->GetSize();
-    return mActiveImage->DeprecatedGetAsSurface();
-  }
-
-  if (aCurrentImage) {
-    NS_IF_ADDREF(mActiveImage);
-    *aCurrentImage = mActiveImage.get();
-  }
-
-  if (!mActiveImage) {
-    return nullptr;
-  }
-
-  *aSize = mActiveImage->GetSize();
-  return mActiveImage->DeprecatedGetAsSurface();
-}
-
 TemporaryRef<gfx::SourceSurface>
 ImageContainer::LockCurrentAsSourceSurface(gfx::IntSize *aSize, Image** aCurrentImage)
 {
@@ -402,26 +335,6 @@ ImageContainer::UnlockCurrentImage()
   }
 }
 
-already_AddRefed<gfxASurface>
-ImageContainer::DeprecatedGetCurrentAsSurface(gfx::IntSize *aSize)
-{
-  ReentrantMonitorAutoEnter mon(mReentrantMonitor);
-
-  if (mRemoteData) {
-    CrossProcessMutexAutoLock autoLock(*mRemoteDataMutex);
-    EnsureActiveImage();
-
-    if (!mActiveImage)
-      return nullptr;
-    *aSize = mRemoteData->mSize;
-  } else {
-    if (!mActiveImage)
-      return nullptr;
-    *aSize = mActiveImage->GetSize();
-  }
-  return mActiveImage->DeprecatedGetAsSurface();
-}
-
 TemporaryRef<gfx::SourceSurface>
 ImageContainer::GetCurrentAsSourceSurface(gfx::IntSize *aSize)
 {
@@ -492,13 +405,13 @@ ImageContainer::EnsureActiveImage()
     if (mRemoteData->mType == RemoteImageData::RAW_BITMAP &&
         mRemoteData->mBitmap.mData && !mActiveImage) {
       nsRefPtr<RemoteBitmapImage> newImg = new RemoteBitmapImage();
-      
+
       newImg->mFormat = mRemoteData->mFormat;
       newImg->mData = mRemoteData->mBitmap.mData;
       newImg->mSize = mRemoteData->mSize;
       newImg->mStride = mRemoteData->mBitmap.mStride;
       mRemoteData->mWasUpdated = false;
-              
+
       mActiveImage = newImg;
     }
 #ifdef XP_WIN
@@ -532,10 +445,29 @@ PlanarYCbCrImage::~PlanarYCbCrImage()
   }
 }
 
-uint8_t* 
+size_t
+PlanarYCbCrImage::SizeOfExcludingThis(MallocSizeOf aMallocSizeOf) const
+{
+  // Ignoring:
+  // - mData - just wraps mBuffer
+  // - Surfaces should be reported under gfx-surfaces-*:
+  //   - mSourceSurface
+  // - Base class:
+  //   - mImplData is not used
+  // Not owned:
+  // - mRecycleBin
+  size_t size = mBuffer.SizeOfExcludingThis(aMallocSizeOf);
+
+  // Could add in the future:
+  // - mBackendData (from base class)
+
+  return size;
+}
+
+uint8_t*
 PlanarYCbCrImage::AllocateBuffer(uint32_t aSize)
 {
-  return mRecycleBin->GetBuffer(aSize); 
+  return mRecycleBin->GetBuffer(aSize);
 }
 
 static void
@@ -626,33 +558,6 @@ PlanarYCbCrImage::AllocateAndGetNewBuffer(uint32_t aSize)
   return mBuffer;
 }
 
-already_AddRefed<gfxASurface>
-PlanarYCbCrImage::DeprecatedGetAsSurface()
-{
-  if (mDeprecatedSurface) {
-    nsRefPtr<gfxASurface> result = mDeprecatedSurface.get();
-    return result.forget();
-  }
-
-  gfx::SurfaceFormat format = gfx::ImageFormatToSurfaceFormat(GetOffscreenFormat());
-  gfx::IntSize size(mSize);
-  gfx::GetYCbCrToRGBDestFormatAndSize(mData, format, size);
-  if (size.width > PlanarYCbCrImage::MAX_DIMENSION ||
-      size.height > PlanarYCbCrImage::MAX_DIMENSION) {
-    NS_ERROR("Illegal image dest width or height");
-    return nullptr;
-  }
-
-  nsRefPtr<gfxImageSurface> imageSurface =
-    new gfxImageSurface(gfx::ThebesIntSize(mSize), gfx::SurfaceFormatToImageFormat(format));
-
-  gfx::ConvertYCbCrToRGB(mData, format, mSize, imageSurface->Data(), imageSurface->Stride());
-
-  mDeprecatedSurface = imageSurface;
-
-  return imageSurface.forget();
-}
-
 TemporaryRef<gfx::SourceSurface>
 PlanarYCbCrImage::GetAsSourceSurface()
 {
@@ -670,6 +575,10 @@ PlanarYCbCrImage::GetAsSourceSurface()
   }
 
   RefPtr<gfx::DataSourceSurface> surface = gfx::Factory::CreateDataSourceSurface(size, format);
+  if (!surface) {
+    NS_WARNING("Failed to create SourceSurface.");
+    return nullptr;
+  }
 
   gfx::ConvertYCbCrToRGB(mData, format, size, surface->GetData(), surface->Stride());
 
@@ -678,22 +587,6 @@ PlanarYCbCrImage::GetAsSourceSurface()
   return surface.forget();
 }
 
-already_AddRefed<gfxASurface>
-RemoteBitmapImage::DeprecatedGetAsSurface()
-{
-  nsRefPtr<gfxImageSurface> newSurf =
-    new gfxImageSurface(ThebesIntSize(mSize),
-    mFormat == RemoteImageData::BGRX32 ? gfxImageFormat::RGB24 : gfxImageFormat::ARGB32);
-
-  for (int y = 0; y < mSize.height; y++) {
-    memcpy(newSurf->Data() + newSurf->Stride() * y,
-           mData + mStride * y,
-           mSize.width * 4);
-  }
-
-  return newSurf.forget();
-}
-
 TemporaryRef<gfx::SourceSurface>
 RemoteBitmapImage::GetAsSourceSurface()
 {
@@ -701,6 +594,10 @@ RemoteBitmapImage::GetAsSourceSurface()
                          ? gfx::SurfaceFormat::B8G8R8X8
                          : gfx::SurfaceFormat::B8G8R8A8;
   RefPtr<gfx::DataSourceSurface> newSurf = gfx::Factory::CreateDataSourceSurface(mSize, fmt);
+  if (!newSurf) {
+    NS_WARNING("Failed to create SourceSurface.");
+    return nullptr;
+  }
 
   for (int y = 0; y < mSize.height; y++) {
     memcpy(newSurf->GetData() + newSurf->Stride() * y,
@@ -711,5 +608,59 @@ RemoteBitmapImage::GetAsSourceSurface()
   return newSurf;
 }
 
+CairoImage::CairoImage()
+  : Image(nullptr, ImageFormat::CAIRO_SURFACE)
+{}
+
+CairoImage::~CairoImage()
+{
+}
+
+TextureClient*
+CairoImage::GetTextureClient(CompositableClient *aClient)
+{
+  if (!aClient) {
+    return nullptr;
+  }
+
+  CompositableForwarder* forwarder = aClient->GetForwarder();
+  RefPtr<TextureClient> textureClient = mTextureClients.Get(forwarder->GetSerial());
+  if (textureClient) {
+    return textureClient;
+  }
+
+  RefPtr<SourceSurface> surface = GetAsSourceSurface();
+  MOZ_ASSERT(surface);
+  if (!surface) {
+    return nullptr;
+  }
+
+  // gfx::BackendType::NONE means default to content backend
+  textureClient = aClient->CreateTextureClientForDrawing(surface->GetFormat(),
+                                                         surface->GetSize(),
+                                                         gfx::BackendType::NONE,
+                                                         TextureFlags::DEFAULT);
+  if (!textureClient) {
+    return nullptr;
+  }
+  MOZ_ASSERT(textureClient->CanExposeDrawTarget());
+  if (!textureClient->Lock(OpenMode::OPEN_WRITE_ONLY)) {
+    return nullptr;
+  }
+
+  TextureClientAutoUnlock autoUnolck(textureClient);
+  {
+    // We must not keep a reference to the DrawTarget after it has been unlocked.
+    DrawTarget* dt = textureClient->BorrowDrawTarget();
+    if (!dt) {
+      return nullptr;
+    }
+    dt->CopySurface(surface, IntRect(IntPoint(), surface->GetSize()), IntPoint());
+  }
+
+  mTextureClients.Put(forwarder->GetSerial(), textureClient);
+  return textureClient;
+}
+
 } // namespace
 } // namespace
