@@ -16,21 +16,26 @@
 #include "mozilla/gfx/Point.h"          // for IntSize, IntPoint
 #include "mozilla/gfx/Types.h"          // for SurfaceFormat, etc
 #include "mozilla/layers/CompositorTypes.h"  // for TextureFlags, etc
+#include "mozilla/layers/FenceUtils.h"  // for FenceHandle
 #include "mozilla/layers/LayersTypes.h"  // for LayerRenderState, etc
 #include "mozilla/mozalloc.h"           // for operator delete
 #include "nsCOMPtr.h"                   // for already_AddRefed
 #include "nsDebug.h"                    // for NS_RUNTIMEABORT
+#include "nsISupportsImpl.h"            // for MOZ_COUNT_CTOR, etc
 #include "nsRegion.h"                   // for nsIntRegion
 #include "nsTraceRefcnt.h"              // for MOZ_COUNT_CTOR, etc
 #include "nscore.h"                     // for nsACString
+#include "mozilla/layers/AtomicRefCountedWithFinalize.h"
 
-class gfxImageSurface;
 class gfxReusableSurfaceWrapper;
 struct nsIntPoint;
 struct nsIntSize;
 struct nsIntRect;
 
 namespace mozilla {
+namespace gl {
+class SurfaceStream;
+}
 namespace ipc {
 class Shmem;
 }
@@ -40,14 +45,18 @@ namespace layers {
 class Compositor;
 class CompositableHost;
 class CompositableBackendSpecificData;
+class CompositableParentManager;
 class SurfaceDescriptor;
+class SurfaceStreamDescriptor;
 class ISurfaceAllocator;
+class TextureHostOGL;
 class TextureSourceOGL;
 class TextureSourceD3D9;
 class TextureSourceD3D11;
 class TextureSourceBasic;
-class TextureParent;
 class DataTextureSource;
+class PTextureParent;
+class TextureParent;
 
 /**
  * A view on a TextureHost where the texture is internally represented as tiles
@@ -57,11 +66,11 @@ class DataTextureSource;
  * device texture, which forces us to split it in smaller parts.
  * Tiled Compositable is a different thing.
  */
-class TileIterator
+class BigImageIterator
 {
 public:
-  virtual void BeginTileIteration() = 0;
-  virtual void EndTileIteration() {};
+  virtual void BeginBigImageIteration() = 0;
+  virtual void EndBigImageIteration() {};
   virtual nsIntRect GetTileRect() = 0;
   virtual size_t GetTileCount() = 0;
   virtual bool NextTile() = 0;
@@ -76,12 +85,16 @@ public:
  *
  * This class is used on the compositor side.
  */
-class TextureSource : public RefCounted<TextureSource>
+class TextureSource
 {
-public:
-  TextureSource();
+protected:
   virtual ~TextureSource();
 
+public:
+  NS_INLINE_DECL_REFCOUNTING(TextureSource)
+
+  TextureSource();
+
   /**
    * Return the size of the texture in texels.
    * If this is a tile iterator, GetSize must return the size of the current tile.
@@ -91,7 +104,7 @@ public:
   /**
    * Return the pixel format of this texture
    */
-  virtual gfx::SurfaceFormat GetFormat() const { return gfx::FORMAT_UNKNOWN; }
+  virtual gfx::SurfaceFormat GetFormat() const { return gfx::SurfaceFormat::UNKNOWN; }
 
   /**
    * Cast to a TextureSource for for each backend..
@@ -116,7 +129,7 @@ public:
    * Overload this if the TextureSource supports big textures that don't fit in
    * one device texture and must be tiled internally.
    */
-  virtual TileIterator* AsTileIterator() { return nullptr; }
+  virtual BigImageIterator* AsBigImageIterator() { return nullptr; }
 
   virtual void SetCompositableBackendSpecificData(CompositableBackendSpecificData* aBackendData);
 
@@ -124,7 +137,6 @@ protected:
   RefPtr<CompositableBackendSpecificData> mCompositableBackendData;
 };
 
-
 /**
  * XXX - merge this class with TextureSource when deprecated texture classes
  * are completely removed.
@@ -136,17 +148,21 @@ public:
   {
     MOZ_COUNT_CTOR(NewTextureSource);
   }
+protected:
   virtual ~NewTextureSource()
   {
     MOZ_COUNT_DTOR(NewTextureSource);
   }
 
+public:
   /**
    * Should be overridden in order to deallocate the data that is associated
    * with the rendering backend, such as GL textures.
    */
   virtual void DeallocateDeviceData() = 0;
 
+  virtual void SetCompositor(Compositor* aCompositor) {}
+
   void SetNextSibling(NewTextureSource* aTexture)
   {
     mNextSibling = aTexture;
@@ -193,7 +209,6 @@ public:
    * the device texture it uploads to internally.
    */
   virtual bool Update(gfx::DataSourceSurface* aSurface,
-                      TextureFlags aFlags,
                       nsIntRegion* aDestRegion = nullptr,
                       gfx::IntPoint* aSrcOffset = nullptr) = 0;
 
@@ -257,22 +272,42 @@ private:
  * The Lock/Unlock mecanism here mirrors Lock/Unlock in TextureClient.
  *
  */
-class TextureHost : public RefCounted<TextureHost>
+class TextureHost
+  : public AtomicRefCountedWithFinalize<TextureHost>
 {
-public:
-  TextureHost(uint64_t aID,
-              TextureFlags aFlags);
+  /**
+   * Called once, just before the destructor.
+   *
+   * Here goes the shut-down code that uses virtual methods.
+   * Must only be called by Release().
+   */
+  void Finalize();
 
+  friend class AtomicRefCountedWithFinalize<TextureHost>;
+public:
+  TextureHost(TextureFlags aFlags);
+
+protected:
   virtual ~TextureHost();
 
+public:
   /**
    * Factory method.
    */
-  static TemporaryRef<TextureHost> Create(uint64_t aID,
-                                          const SurfaceDescriptor& aDesc,
+  static TemporaryRef<TextureHost> Create(const SurfaceDescriptor& aDesc,
                                           ISurfaceAllocator* aDeallocator,
                                           TextureFlags aFlags);
 
+  /**
+   * Tell to TextureChild that TextureHost is recycled.
+   * This function should be called from TextureHost's RecycleCallback.
+   * If SetRecycleCallback is set to TextureHost.
+   * TextureHost can be recycled by calling RecycleCallback
+   * when reference count becomes one.
+   * One reference count is always added by TextureChild.
+   */
+  void CompositorRecycle();
+
   /**
    * Lock the texture host for compositing.
    */
@@ -308,7 +343,7 @@ public:
    * @param aRegion The region that has been changed, if nil, it means that the
    * entire surface should be updated.
    */
-  virtual void Updated(const nsIntRegion* aRegion) {}
+  virtual void Updated(const nsIntRegion* aRegion = nullptr) {}
 
   /**
    * Sets this TextureHost's compositor.
@@ -332,32 +367,20 @@ public:
   virtual void DeallocateSharedData() {}
 
   /**
-   * An ID to differentiate TextureHosts of a given CompositableHost.
+   * Should be overridden in order to force the TextureHost to drop all references
+   * to it's shared data.
    *
-   * A TextureHost and its corresponding TextureClient always have the same ID.
-   * TextureHosts of a given CompositableHost always have different IDs.
-   * TextureHosts of different CompositableHosts, may have the same ID.
-   * Zero is always an invalid ID.
+   * This is important to ensure the correctness of the deallocation protocol.
    */
-  uint64_t GetID() const { return mID; }
+  virtual void ForgetSharedData() {}
 
   virtual gfx::IntSize GetSize() const = 0;
 
-  /**
-   * TextureHosts are kept as a linked list in their compositable
-   * XXX - This is just a poor man's PTexture. The purpose of this list is
-   * to keep TextureHost alive which should be independent from compositables.
-   * It will be removed when we add the PTetxure protocol (which will more
-   * gracefully handle the lifetime of textures). See bug 897452
-   */
-  TextureHost* GetNextSibling() const { return mNextTexture; }
-  void SetNextSibling(TextureHost* aNext) { mNextTexture = aNext; }
-
   /**
    * Debug facility.
    * XXX - cool kids use Moz2D. See bug 882113.
    */
-  virtual already_AddRefed<gfxImageSurface> GetAsSurface() = 0;
+  virtual TemporaryRef<gfx::DataSourceSurface> GetAsSurface() = 0;
 
   /**
    * XXX - Flags should only be set at creation time, this will be removed.
@@ -371,6 +394,41 @@ public:
 
   TextureFlags GetFlags() { return mFlags; }
 
+  /**
+   * Allocate and deallocate a TextureParent actor.
+   *
+   * TextureParent< is an implementation detail of TextureHost that is not
+   * exposed to the rest of the code base. CreateIPDLActor and DestroyIPDLActor
+   * are for use with the managing IPDL protocols only (so that they can
+   * implement AllocPTextureParent and DeallocPTextureParent).
+   */
+  static PTextureParent* CreateIPDLActor(CompositableParentManager* aManager,
+                                         const SurfaceDescriptor& aSharedData,
+                                         TextureFlags aFlags);
+  static bool DestroyIPDLActor(PTextureParent* actor);
+
+  /**
+   * Destroy the TextureChild/Parent pair.
+   */
+  static bool SendDeleteIPDLActor(PTextureParent* actor);
+
+  /**
+   * Get the TextureHost corresponding to the actor passed in parameter.
+   */
+  static TextureHost* AsTextureHost(PTextureParent* actor);
+
+  /**
+   * Return a pointer to the IPDLActor.
+   *
+   * This is to be used with IPDL messages only. Do not store the returned
+   * pointer.
+   */
+  PTextureParent* GetIPDLActor();
+
+  static void SendFenceHandleIfPresent(PTextureParent* actor);
+
+  FenceHandle GetAndResetReleaseFenceHandle();
+
   /**
    * Specific to B2G's Composer2D
    * XXX - more doc here
@@ -384,16 +442,34 @@ public:
 
   virtual void SetCompositableBackendSpecificData(CompositableBackendSpecificData* aBackendData);
 
-#ifdef MOZ_LAYERS_HAVE_LOG
+  // If a texture host holds a reference to shmem, it should override this method
+  // to forget about the shmem _without_ releasing it.
+  virtual void OnShutdown() {}
+
+  // Forget buffer actor. Used only for hacky fix for bug 966446.
+  virtual void ForgetBufferActor() {}
+
   virtual const char *Name() { return "TextureHost"; }
-  virtual void PrintInfo(nsACString& aTo, const char* aPrefix);
-#endif
+  virtual void PrintInfo(std::stringstream& aStream, const char* aPrefix);
+
+  /**
+   * Indicates whether the TextureHost implementation is backed by an
+   * in-memory buffer. The consequence of this is that locking the
+   * TextureHost does not contend with locking the texture on the client side.
+   */
+  virtual bool HasInternalBuffer() const { return false; }
+
+  /**
+   * Cast to a TextureHost for each backend.
+   */
+  virtual TextureHostOGL* AsHostOGL() { return nullptr; }
 
 protected:
-  uint64_t mID;
-  RefPtr<TextureHost> mNextTexture;
+  PTextureParent* mActor;
   TextureFlags mFlags;
   RefPtr<CompositableBackendSpecificData> mCompositableBackendData;
+
+  friend class TextureParent;
 };
 
 /**
@@ -412,15 +488,16 @@ protected:
 class BufferTextureHost : public TextureHost
 {
 public:
-  BufferTextureHost(uint64_t aID,
-                    gfx::SurfaceFormat aFormat,
+  BufferTextureHost(gfx::SurfaceFormat aFormat,
                     TextureFlags aFlags);
 
   ~BufferTextureHost();
 
   virtual uint8_t* GetBuffer() = 0;
 
-  virtual void Updated(const nsIntRegion* aRegion) MOZ_OVERRIDE;
+  virtual size_t GetBufferSize() = 0;
+
+  virtual void Updated(const nsIntRegion* aRegion = nullptr) MOZ_OVERRIDE;
 
   virtual bool Lock() MOZ_OVERRIDE;
 
@@ -437,13 +514,15 @@ public:
    * GetTextureSources.
    *
    * If the shared format is YCbCr and the compositor does not support it,
-   * GetFormat will be RGB32 (even though mFormat is FORMAT_YUV).
+   * GetFormat will be RGB32 (even though mFormat is SurfaceFormat::YUV).
    */
   virtual gfx::SurfaceFormat GetFormat() const MOZ_OVERRIDE;
 
   virtual gfx::IntSize GetSize() const MOZ_OVERRIDE { return mSize; }
 
-  virtual already_AddRefed<gfxImageSurface> GetAsSurface() MOZ_OVERRIDE;
+  virtual TemporaryRef<gfx::DataSourceSurface> GetAsSurface() MOZ_OVERRIDE;
+
+  virtual bool HasInternalBuffer() const MOZ_OVERRIDE { return true; }
 
 protected:
   bool Upload(nsIntRegion *aRegion = nullptr);
@@ -457,7 +536,7 @@ protected:
   gfx::SurfaceFormat mFormat;
   uint32_t mUpdateSerial;
   bool mLocked;
-  bool mPartialUpdate;
+  bool mNeedsFullUpdate;
 };
 
 /**
@@ -468,25 +547,30 @@ protected:
 class ShmemTextureHost : public BufferTextureHost
 {
 public:
-  ShmemTextureHost(uint64_t aID,
-                   const ipc::Shmem& aShmem,
+  ShmemTextureHost(const mozilla::ipc::Shmem& aShmem,
                    gfx::SurfaceFormat aFormat,
                    ISurfaceAllocator* aDeallocator,
                    TextureFlags aFlags);
 
+protected:
   ~ShmemTextureHost();
 
+public:
   virtual void DeallocateSharedData() MOZ_OVERRIDE;
 
+  virtual void ForgetSharedData() MOZ_OVERRIDE;
+
   virtual uint8_t* GetBuffer() MOZ_OVERRIDE;
 
-#ifdef MOZ_LAYERS_HAVE_LOG
+  virtual size_t GetBufferSize() MOZ_OVERRIDE;
+
   virtual const char *Name() MOZ_OVERRIDE { return "ShmemTextureHost"; }
-#endif
+
+  virtual void OnShutdown() MOZ_OVERRIDE;
 
 protected:
-  ipc::Shmem* mShmem;
-  ISurfaceAllocator* mDeallocator;
+  mozilla::ipc::Shmem* mShmem;
+  RefPtr<ISurfaceAllocator> mDeallocator;
 };
 
 /**
@@ -498,298 +582,93 @@ protected:
 class MemoryTextureHost : public BufferTextureHost
 {
 public:
-  MemoryTextureHost(uint64_t aID,
-                    uint8_t* aBuffer,
+  MemoryTextureHost(uint8_t* aBuffer,
                     gfx::SurfaceFormat aFormat,
                     TextureFlags aFlags);
 
+protected:
   ~MemoryTextureHost();
 
+public:
   virtual void DeallocateSharedData() MOZ_OVERRIDE;
 
+  virtual void ForgetSharedData() MOZ_OVERRIDE;
+
   virtual uint8_t* GetBuffer() MOZ_OVERRIDE;
 
-#ifdef MOZ_LAYERS_HAVE_LOG
+  virtual size_t GetBufferSize() MOZ_OVERRIDE;
+
   virtual const char *Name() MOZ_OVERRIDE { return "MemoryTextureHost"; }
-#endif
 
 protected:
   uint8_t* mBuffer;
 };
 
-
 /**
- * XXX - This class is deprectaed, will be removed soon.
- *
- * DeprecatedTextureHost is a thin abstraction over texture data that need to be shared
- * or transfered from the content process to the compositor process. It is the
- * compositor-side half of a DeprecatedTextureClient/DeprecatedTextureHost pair. A corresponding
- * DeprecatedTextureClient lives on the client-side.
- *
- * DeprecatedTextureHost only knows how to deserialize or synchronize generic image data
- * (SurfaceDescriptor) and provide access to one or more TextureSource objects
- * (these provide the necessary APIs for compositor backends to composite the
- * image).
- *
- * A DeprecatedTextureHost should mostly correspond to one or several SurfaceDescriptor
- * types. This means that for YCbCr planes, even though they are represented as
- * 3 textures internally, use 1 DeprecatedTextureHost and not 3, because the 3 planes
- * arrive in the same IPC message.
- *
- * The Lock/Unlock mechanism here mirrors Lock/Unlock in DeprecatedTextureClient. These two
- * methods don't always have to use blocking locks, unless a resource is shared
- * between the two sides (like shared texture handles). For instance, in some
- * cases the data received in Update(...) is a copy in shared memory of the data
- * owned by the content process, in which case no blocking lock is required.
- *
- * DeprecatedTextureHosts can be changed at any time, for example if we receive a
- * SurfaceDescriptor type that was not expected. This should be an incentive
- * to keep the ownership model simple (especially on the OpenGL case, where
- * we have additionnal constraints).
- *
- * There are two fundamental operations carried out on texture hosts - update
- * from the content thread and composition. Texture upload can occur in either
- * phase. Update happens in response to an IPDL message from content and
- * composition when the compositor 'ticks'. We may composite many times before
- * update.
- *
- * Update ends up at DeprecatedTextureHost::UpdateImpl. It always occurs in a layers
- * transacton. (TextureParent should call EnsureTexture before updating to
- * ensure the DeprecatedTextureHost exists and is of the correct type).
- *
- * CompositableHost::Composite does compositing. It should check the texture
- * host exists (and give up otherwise), then lock the texture host
- * (DeprecatedTextureHost::Lock). Then it passes the texture host to the Compositor in an
- * effect as a texture source, which does the actual composition. Finally the
- * compositable calls Unlock on the DeprecatedTextureHost.
- *
- * The class TextureImageDeprecatedTextureHostOGL is a good example of a DeprecatedTextureHost
- * implementation.
- *
- * This class is used only on the compositor side.
+ * A TextureHost for shared SurfaceStream
  */
-class DeprecatedTextureHost : public TextureSource
+class StreamTextureHost : public TextureHost
 {
 public:
-  /**
-   * Create a new texture host to handle surfaces of aDescriptorType
-   *
-   * @param aDescriptorType The SurfaceDescriptor type being passed
-   * @param aDeprecatedTextureHostFlags Modifier flags that specify changes in
-   * the usage of a aDescriptorType, see DeprecatedTextureHostFlags
-   * @param aTextureFlags Flags to pass to the new DeprecatedTextureHost
-   */
-  static TemporaryRef<DeprecatedTextureHost> CreateDeprecatedTextureHost(SurfaceDescriptorType aDescriptorType,
-                                                     uint32_t aDeprecatedTextureHostFlags,
-                                                     uint32_t aTextureFlags,
-                                                     CompositableHost* aCompositableHost);
+  StreamTextureHost(TextureFlags aFlags,
+                    const SurfaceStreamDescriptor& aDesc);
 
-  DeprecatedTextureHost();
-  virtual ~DeprecatedTextureHost();
+  virtual ~StreamTextureHost();
 
-  virtual gfx::SurfaceFormat GetFormat() const MOZ_OVERRIDE { return mFormat; }
+  virtual void DeallocateDeviceData() MOZ_OVERRIDE {};
 
-  virtual bool IsValid() const { return true; }
+  virtual void SetCompositor(Compositor* aCompositor) MOZ_OVERRIDE;
 
-  /**
-   * Update the texture host using the data from aSurfaceDescriptor.
-   *
-   * @param aImage Source image to update with.
-   * @param aRegion Region of the texture host to update.
-   * @param aOffset Offset in the source to update from
-   */
-  void Update(const SurfaceDescriptor& aImage,
-              nsIntRegion *aRegion = nullptr,
-              nsIntPoint* aOffset = nullptr);
+  virtual bool Lock() MOZ_OVERRIDE;
 
-  /**
-   * Change the current surface of the texture host to aImage. aResult will return
-   * the previous surface.
-   */
-  void SwapTextures(const SurfaceDescriptor& aImage,
-                    SurfaceDescriptor* aResult = nullptr,
-                    nsIntRegion *aRegion = nullptr);
+  virtual void Unlock() MOZ_OVERRIDE;
 
-  /**
-   * Update for tiled texture hosts could probably have a better signature, but we
-   * will replace it with PTexture stuff anyway, so nm.
-   */
-  virtual void Update(gfxReusableSurfaceWrapper* aReusableSurface,
-  	                  TextureFlags aFlags,
-  	                  const gfx::IntSize& aSize) {}
+  virtual gfx::SurfaceFormat GetFormat() const MOZ_OVERRIDE;
 
-  /**
-   * Lock the texture host for compositing, returns true if the DeprecatedTextureHost is
-   * valid for composition.
-   */
-  virtual bool Lock() { return IsValid(); }
-
-  /**
-   * Unlock the texture host after compositing.
-   * Should handle the case where Lock failed without crashing.
-   */
-  virtual void Unlock() {}
-
-  void SetFlags(TextureFlags aFlags) { mFlags = aFlags; }
-  void AddFlag(TextureFlags aFlag) { mFlags |= aFlag; }
-  TextureFlags GetFlags() { return mFlags; }
-
-  /**
-   * Sets ths DeprecatedTextureHost's compositor.
-   * A DeprecatedTextureHost can change compositor on certain occasions, in particular if
-   * it belongs to an async Compositable.
-   * aCompositor can be null, in which case the DeprecatedTextureHost must cleanup  all
-   * of it's device textures.
-   */
-  virtual void SetCompositor(Compositor* aCompositor) {}
-
-  ISurfaceAllocator* GetDeAllocator()
+  virtual NewTextureSource* GetTextureSources() MOZ_OVERRIDE
   {
-    return mDeAllocator;
+    return mTextureSource;
   }
 
-  bool operator== (const DeprecatedTextureHost& o) const
+  virtual TemporaryRef<gfx::DataSourceSurface> GetAsSurface() MOZ_OVERRIDE
   {
-    return GetIdentifier() == o.GetIdentifier();
-  }
-  bool operator!= (const DeprecatedTextureHost& o) const
-  {
-    return GetIdentifier() != o.GetIdentifier();
+    return nullptr; // XXX - implement this (for MOZ_DUMP_PAINTING)
   }
 
-  virtual LayerRenderState GetRenderState()
-  {
-    return LayerRenderState();
-  }
-
-  virtual already_AddRefed<gfxImageSurface> GetAsSurface() = 0;
+  virtual gfx::IntSize GetSize() const MOZ_OVERRIDE;
 
 #ifdef MOZ_LAYERS_HAVE_LOG
-  virtual const char *Name() = 0;
-  virtual void PrintInfo(nsACString& aTo, const char* aPrefix);
+  virtual const char* Name() { return "StreamTextureHost"; }
 #endif
 
-  /**
-   * TEMPORARY.
-   *
-   * Ensure that a buffer of the given size/type has been allocated so that
-   * we can update it using Update and/or CopyTo.
-   */
-  virtual void EnsureBuffer(const nsIntSize& aSize, gfxContentType aType)
-  {
-    NS_RUNTIMEABORT("DeprecatedTextureHost doesn't support EnsureBuffer");
-  }
-
-  /**
-   * Copy the contents of this DeprecatedTextureHost to aDest. aDest must already
-   * have a suitable buffer allocated using EnsureBuffer.
-   *
-   * @param aSourceRect Area of this texture host to copy.
-   * @param aDest Destination texture host.
-   * @param aDestRect Destination rect.
-   */
-  virtual void CopyTo(const nsIntRect& aSourceRect,
-                      DeprecatedTextureHost *aDest,
-                      const nsIntRect& aDestRect)
-  {
-    NS_RUNTIMEABORT("DeprecatedTextureHost doesn't support CopyTo");
-  }
-
-
-  SurfaceDescriptor* GetBuffer() const { return mBuffer; }
-  virtual SurfaceDescriptor* LockSurfaceDescriptor() const { return GetBuffer(); }
-
-  /**
-   * Set a SurfaceDescriptor for this texture host. By setting a buffer and
-   * allocator/de-allocator for the DeprecatedTextureHost, you cause the DeprecatedTextureHost to
-   * retain a SurfaceDescriptor.
-   * Ownership of the SurfaceDescriptor passes to this.
-   */
-  // only made virtual to allow overriding in GrallocDeprecatedTextureHostOGL, for hacky fix in gecko 23 for bug 862324.
-  // see bug 865908 about fixing this.
-  virtual void SetBuffer(SurfaceDescriptor* aBuffer, ISurfaceAllocator* aAllocator)
-  {
-    MOZ_ASSERT(!mBuffer || mBuffer == aBuffer, "Will leak the old mBuffer");
-    mBuffer = aBuffer;
-    mDeAllocator = aAllocator;
-  }
-
-  // used only for hacky fix in gecko 23 for bug 862324
-  // see bug 865908 about fixing this.
-  virtual void ForgetBuffer() {}
-
 protected:
-  /**
-   * Should be implemented by the backend-specific DeprecatedTextureHost classes
-   *
-   * It should not take a reference to aImage, unless it knows the data
-   * to be thread-safe.
-   */
-  virtual void UpdateImpl(const SurfaceDescriptor& aImage,
-                          nsIntRegion *aRegion,
-                          nsIntPoint *aOffset = nullptr)
-  {
-    NS_RUNTIMEABORT("Should not be reached");
-  }
-
-  /**
-   * Should be implemented by the backend-specific DeprecatedTextureHost classes.
-   *
-   * Doesn't need to do the actual surface descriptor swap, just
-   * any preparation work required to use the new descriptor.
-   *
-   * If the implementation doesn't define anything in particular
-   * for handling swaps, then we can just do an update instead.
-   */
-  virtual void SwapTexturesImpl(const SurfaceDescriptor& aImage,
-                                nsIntRegion *aRegion)
-  {
-    UpdateImpl(aImage, aRegion, nullptr);
-  }
-
-  // An internal identifier for this texture host. Two texture hosts
-  // should be considered equal iff their identifiers match. Should
-  // not be exposed publicly.
-  virtual uint64_t GetIdentifier() const
-  {
-    return reinterpret_cast<uint64_t>(this);
-  }
-
-  // Texture info
-  TextureFlags mFlags;
-  SurfaceDescriptor* mBuffer; // FIXME [bjacob] it's terrible to have a SurfaceDescriptor here,
-                              // because SurfaceDescriptor's may have raw pointers to IPDL actors,
-                              // which can go away under our feet at any time. This is the cause
-                              // of bug 862324 among others. Our current understanding is that
-                              // this will be gone in Gecko 24. See bug 858914.
-  ISurfaceAllocator* mDeAllocator;
-  gfx::SurfaceFormat mFormat;
+  Compositor* mCompositor;
+  gl::SurfaceStream* mStream;
+  RefPtr<NewTextureSource> mTextureSource;
+  RefPtr<DataTextureSource> mDataTextureSource;
 };
 
-class AutoLockDeprecatedTextureHost
+class MOZ_STACK_CLASS AutoLockTextureHost
 {
 public:
-  AutoLockDeprecatedTextureHost(DeprecatedTextureHost* aHost)
-    : mDeprecatedTextureHost(aHost)
-    , mIsValid(true)
+  AutoLockTextureHost(TextureHost* aTexture)
+    : mTexture(aTexture)
   {
-    if (mDeprecatedTextureHost) {
-      mIsValid = mDeprecatedTextureHost->Lock();
+    mLocked = mTexture ? mTexture->Lock() : false;
+  }
+
+  ~AutoLockTextureHost()
+  {
+    if (mTexture && mLocked) {
+      mTexture->Unlock();
     }
   }
 
-  ~AutoLockDeprecatedTextureHost()
-  {
-    if (mDeprecatedTextureHost && mIsValid) {
-      mDeprecatedTextureHost->Unlock();
-    }
-  }
-
-  bool IsValid() { return mIsValid; }
+  bool Failed() { return mTexture && !mLocked; }
 
 private:
-  DeprecatedTextureHost *mDeprecatedTextureHost;
-  bool mIsValid;
+  RefPtr<TextureHost> mTexture;
+  bool mLocked;
 };
 
 /**
@@ -799,11 +678,19 @@ private:
 class CompositingRenderTarget : public TextureSource
 {
 public:
+  CompositingRenderTarget(const gfx::IntPoint& aOrigin)
+    : mOrigin(aOrigin)
+  {}
   virtual ~CompositingRenderTarget() {}
 
 #ifdef MOZ_DUMP_PAINTING
-  virtual already_AddRefed<gfxImageSurface> Dump(Compositor* aCompositor) { return nullptr; }
+  virtual TemporaryRef<gfx::DataSourceSurface> Dump(Compositor* aCompositor) { return nullptr; }
 #endif
+
+  const gfx::IntPoint& GetOrigin() { return mOrigin; }
+
+private:
+  gfx::IntPoint mOrigin;
 };
 
 /**
@@ -811,8 +698,7 @@ public:
  * Not all SurfaceDescriptor types are supported
  */
 TemporaryRef<TextureHost>
-CreateBackendIndependentTextureHost(uint64_t aID,
-                                    const SurfaceDescriptor& aDesc,
+CreateBackendIndependentTextureHost(const SurfaceDescriptor& aDesc,
                                     ISurfaceAllocator* aDeallocator,
                                     TextureFlags aFlags);
 
