@@ -4,11 +4,19 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "ImageWrapper.h"
+#include "mozilla/gfx/2D.h"
+#include "mozilla/RefPtr.h"
+#include "Orientation.h"
 
-using mozilla::layers::LayerManager;
-using mozilla::layers::ImageContainer;
+#include "mozilla/MemoryReporting.h"
 
 namespace mozilla {
+
+using gfx::DataSourceSurface;
+using gfx::SourceSurface;
+using layers::LayerManager;
+using layers::ImageContainer;
+
 namespace image {
 
 // Inherited methods from Image.
@@ -19,7 +27,7 @@ ImageWrapper::Init(const char* aMimeType, uint32_t aFlags)
   return mInnerImage->Init(aMimeType, aFlags);
 }
 
-imgStatusTracker&
+already_AddRefed<imgStatusTracker>
 ImageWrapper::GetStatusTracker()
 {
   return mInnerImage->GetStatusTracker();
@@ -38,13 +46,13 @@ ImageWrapper::SizeOfData()
 }
 
 size_t
-ImageWrapper::HeapSizeOfSourceWithComputedFallback(nsMallocSizeOfFun aMallocSizeOf) const
+ImageWrapper::HeapSizeOfSourceWithComputedFallback(MallocSizeOf aMallocSizeOf) const
 {
   return mInnerImage->HeapSizeOfSourceWithComputedFallback(aMallocSizeOf);
 }
 
 size_t
-ImageWrapper::HeapSizeOfDecodedWithComputedFallback(nsMallocSizeOfFun aMallocSizeOf) const
+ImageWrapper::HeapSizeOfDecodedWithComputedFallback(MallocSizeOf aMallocSizeOf) const
 {
   return mInnerImage->HeapSizeOfDecodedWithComputedFallback(aMallocSizeOf);
 }
@@ -64,12 +72,16 @@ ImageWrapper::OutOfProcessSizeOfDecoded() const
 void
 ImageWrapper::IncrementAnimationConsumers()
 {
+  MOZ_ASSERT(NS_IsMainThread(), "Main thread only to encourage serialization "
+                                "with DecrementAnimationConsumers");
   mInnerImage->IncrementAnimationConsumers();
 }
 
 void
 ImageWrapper::DecrementAnimationConsumers()
 {
+  MOZ_ASSERT(NS_IsMainThread(), "Main thread only to encourage serialization "
+                                "with IncrementAnimationConsumers");
   mInnerImage->DecrementAnimationConsumers();
 }
 
@@ -131,7 +143,7 @@ ImageWrapper::SetHasError()
   mInnerImage->SetHasError();
 }
 
-nsIURI*
+ImageURL*
 ImageWrapper::GetURI()
 {
   return mInnerImage->GetURI();
@@ -139,7 +151,7 @@ ImageWrapper::GetURI()
 
 // Methods inherited from XPCOM interfaces.
 
-NS_IMPL_ISUPPORTS1(ImageWrapper, imgIContainer)
+NS_IMPL_ISUPPORTS(ImageWrapper, imgIContainer)
 
 NS_IMETHODIMP
 ImageWrapper::GetWidth(int32_t* aWidth)
@@ -165,6 +177,12 @@ ImageWrapper::GetIntrinsicRatio(nsSize* aSize)
   return mInnerImage->GetIntrinsicRatio(aSize);
 }
 
+NS_IMETHODIMP_(Orientation)
+ImageWrapper::GetOrientation()
+{
+  return mInnerImage->GetOrientation();
+}
+
 NS_IMETHODIMP
 ImageWrapper::GetType(uint16_t* aType)
 {
@@ -183,12 +201,11 @@ ImageWrapper::GetAnimated(bool* aAnimated)
   return mInnerImage->GetAnimated(aAnimated);
 }
 
-NS_IMETHODIMP
+NS_IMETHODIMP_(TemporaryRef<SourceSurface>)
 ImageWrapper::GetFrame(uint32_t aWhichFrame,
-                       uint32_t aFlags,
-                       gfxASurface** _retval)
+                       uint32_t aFlags)
 {
-  return mInnerImage->GetFrame(aWhichFrame, aFlags, _retval);
+  return mInnerImage->GetFrame(aWhichFrame, aFlags);
 }
 
 NS_IMETHODIMP_(bool)
@@ -205,7 +222,7 @@ ImageWrapper::GetImageContainer(LayerManager* aManager, ImageContainer** _retval
 
 NS_IMETHODIMP
 ImageWrapper::Draw(gfxContext* aContext,
-                   gfxPattern::GraphicsFilter aFilter,
+                   GraphicsFilter aFilter,
                    const gfxMatrix& aUserSpaceToImageSpace,
                    const gfxRect& aFill,
                    const nsIntRect& aSubimage,
@@ -231,15 +248,25 @@ ImageWrapper::StartDecoding()
   return mInnerImage->StartDecoding();
 }
 
+bool
+ImageWrapper::IsDecoded()
+{
+  return mInnerImage->IsDecoded();
+}
+
 NS_IMETHODIMP
 ImageWrapper::LockImage()
 {
+  MOZ_ASSERT(NS_IsMainThread(),
+             "Main thread to encourage serialization with UnlockImage");
   return mInnerImage->LockImage();
 }
 
 NS_IMETHODIMP
 ImageWrapper::UnlockImage()
 {
+  MOZ_ASSERT(NS_IsMainThread(),
+             "Main thread to encourage serialization with LockImage");
   return mInnerImage->UnlockImage();
 }
 
@@ -250,7 +277,7 @@ ImageWrapper::RequestDiscard()
 }
 
 NS_IMETHODIMP_(void)
-ImageWrapper::RequestRefresh(const mozilla::TimeStamp& aTime)
+ImageWrapper::RequestRefresh(const TimeStamp& aTime)
 {
   return mInnerImage->RequestRefresh(aTime);
 }
@@ -279,5 +306,29 @@ ImageWrapper::GetFrameIndex(uint32_t aWhichFrame)
   return mInnerImage->GetFrameIndex(aWhichFrame);
 }
 
+NS_IMETHODIMP_(int32_t)
+ImageWrapper::GetFirstFrameDelay()
+{
+  return mInnerImage->GetFirstFrameDelay();
+}
+
+NS_IMETHODIMP_(void)
+ImageWrapper::SetAnimationStartTime(const TimeStamp& aTime)
+{
+  mInnerImage->SetAnimationStartTime(aTime);
+}
+
+NS_IMETHODIMP_(nsIntRect)
+ImageWrapper::GetImageSpaceInvalidationRect(const nsIntRect& aRect)
+{
+  return mInnerImage->GetImageSpaceInvalidationRect(aRect);
+}
+
+already_AddRefed<imgIContainer>
+ImageWrapper::Unwrap()
+{
+  return mInnerImage->Unwrap();
+}
+
 } // namespace image
 } // namespace mozilla
