@@ -8,20 +8,20 @@
 
 #include "nsICollation.h"
 #include "nsCollation.h"
+#include "mozilla/Attributes.h"
 #include <Carbon/Carbon.h>
 
 // Maximum number of characters for a buffer to remember 
 // the generated collation key.
-const PRUint32 kCacheSize = 128;
+const uint32_t kCacheSize = 128;
 // According to the documentation, the length of the key should typically be
 // at least 5 * textLength, but 6* would be safer.
-const PRUint32 kCollationValueSizeFactor = 6;
+const uint32_t kCollationValueSizeFactor = 6;
 
-class nsCollationMacUC : public nsICollation {
+class nsCollationMacUC MOZ_FINAL : public nsICollation {
 
 public: 
   nsCollationMacUC();
-  ~nsCollationMacUC(); 
 
   // nsISupports interface
   NS_DECL_ISUPPORTS
@@ -30,19 +30,21 @@ public:
   NS_DECL_NSICOLLATION
 
 protected:
+  ~nsCollationMacUC(); 
+
   nsresult ConvertLocale(nsILocale* aNSLocale, LocaleRef* aMacLocale);
-  nsresult StrengthToOptions(const PRInt32 aStrength,
+  nsresult StrengthToOptions(const int32_t aStrength,
                              UCCollateOptions* aOptions);
-  nsresult EnsureCollator(const PRInt32 newStrength);
+  nsresult EnsureCollator(const int32_t newStrength);
 
 private:
   bool mInit;
   bool mHasCollator;
   LocaleRef mLocale;
-  PRInt32 mLastStrength;
+  int32_t mLastStrength;
   CollatorRef mCollator;
   void *mBuffer; // temporary buffer to generate collation keys
-  PRUint32 mBufferLen; // byte length of buffer
+  uint32_t mBufferLen; // byte length of buffer
 };
 
 #endif  /* nsCollationMacUC_h_ */
