@@ -327,10 +327,10 @@ Any type, naked or pointer, can be const (const T) or ref (T&).
                     T=copy.deepcopy(self.T, memo))
 Type.BOOL = Type('bool')
 Type.INT = Type('int')
-Type.INT32 = Type('int32')
+Type.INT32 = Type('int32_t')
 Type.INTPTR = Type('intptr_t')
-Type.UINT32 = Type('uint32')
-Type.UINT32PTR = Type('uint32', ptr=1)
+Type.UINT32 = Type('uint32_t')
+Type.UINT32PTR = Type('uint32_t', ptr=1)
 Type.SIZE = Type('size_t')
 Type.VOID = Type('void')
 Type.VOIDPTR = Type('void', ptr=1)
@@ -370,6 +370,14 @@ class Typedef(Node):
         self.fromtype = fromtype
         self.totypename = totypename
 
+    def __cmp__(self, o):
+        return cmp(self.totypename, o.totypename)
+    def __eq__(self, o):
+        return (self.__class__ == o.__class__
+                and 0 == cmp(self, o))
+    def __hash__(self):
+        return hash(self.totypename)
+
 class Using(Node):
     def __init__(self, type):
         Node.__init__(self)
@@ -439,7 +447,7 @@ class FriendClassDecl(Node):
 class MethodDecl(Node):
     def __init__(self, name, params=[ ], ret=Type('void'),
                  virtual=0, const=0, pure=0, static=0, warn_unused=0,
-                 inline=0, force_inline=0,
+                 inline=0, force_inline=0, never_inline=0,
                  typeop=None,
                  T=None):
         assert not (virtual and static)
@@ -450,6 +458,8 @@ class MethodDecl(Node):
         assert not isinstance(ret, list)
         for decl in params:  assert not isinstance(decl, str)
         assert not isinstance(T, int)
+        assert not (inline and never_inline)
+        assert not (force_inline and never_inline)
 
         if typeop is not None:
             ret = None
@@ -465,8 +475,10 @@ class MethodDecl(Node):
         self.warn_unused = warn_unused  # bool
         self.force_inline = (force_inline or T) # bool
         self.inline = inline            # bool
+        self.never_inline = never_inline # bool
         self.typeop = typeop            # Type or None
         self.T = T                      # Type or None
+        self.only_for_definition = False
 
     def __deepcopy__(self, memo):
         return MethodDecl(
@@ -480,6 +492,7 @@ class MethodDecl(Node):
             warn_unused=self.warn_unused,
             inline=self.inline,
             force_inline=self.force_inline,
+            never_inline=self.never_inline,
             typeop=copy.deepcopy(self.typeop, memo),
             T=copy.deepcopy(self.T, memo))
 
@@ -564,7 +577,7 @@ class ExprLiteral(Node):
         return ('%'+ self.type)% (self.value)
 ExprLiteral.ZERO = ExprLiteral.Int(0)
 ExprLiteral.ONE = ExprLiteral.Int(1)
-ExprLiteral.NULL = ExprLiteral.ZERO
+ExprLiteral.NULL = ExprVar('nullptr')
 ExprLiteral.TRUE = ExprVar('true')
 ExprLiteral.FALSE = ExprVar('false')
 
