@@ -25,6 +25,8 @@
 
 namespace js {
 
+namespace frontend { class TokenStream; }
+
 // These EcmaScript-defined coercions form the basis of the asm.js type system.
 enum AsmJSCoercion
 {
@@ -77,7 +79,7 @@ class AsmJSModule
                     } init;
                 } var;
                 uint32_t ffiIndex_;
-                ArrayBufferView::ViewType viewType_;
+                Scalar::Type viewType_;
                 AsmJSMathBuiltinFunction mathBuiltinFunc_;
                 struct {
                     ConstantKind kind_;
@@ -141,7 +143,7 @@ class AsmJSModule
             JS_ASSERT(pod.which_ == ArrayView);
             return name_;
         }
-        ArrayBufferView::ViewType viewType() const {
+        Scalar::Type viewType() const {
             JS_ASSERT(pod.which_ == ArrayView);
             return pod.u.viewType_;
         }
@@ -213,6 +215,15 @@ class AsmJSModule
     };
     typedef int32_t (*CodePtr)(uint64_t *args, uint8_t *global);
 
+    // An Exit holds bookkeeping information about an exit; the ExitDatum
+    // struct overlays the actual runtime data stored in the global data
+    // section.
+    struct ExitDatum
+    {
+        uint8_t *exit;
+        HeapPtrFunction fun;
+    };
+
     typedef Vector<AsmJSCoercion, 0, SystemAllocPolicy> ArgCoercionVector;
 
     enum ReturnType { Return_Int32, Return_Double, Return_Void };
@@ -264,14 +275,15 @@ class AsmJSModule
             argCoercions_ = mozilla::Move(rhs.argCoercions_);
             pod = rhs.pod;
         }
-        void updateCodeOffset(jit::MacroAssembler &masm) {
-            pod.codeOffset_ = masm.actualOffset(pod.codeOffset_);
-        }
 
         void initCodeOffset(unsigned off) {
             JS_ASSERT(pod.codeOffset_ == UINT32_MAX);
             pod.codeOffset_ = off;
         }
+        void updateCodeOffset(jit::MacroAssembler &masm) {
+            pod.codeOffset_ = masm.actualOffset(pod.codeOffset_);
+        }
+
 
         PropertyName *name() const {
             return name_;
@@ -301,6 +313,33 @@ class AsmJSModule
         bool clone(ExclusiveContext *cx, ExportedFunction *out) const;
     };
 
+    class CodeRange
+    {
+      public:
+        enum Kind { Entry, Function };
+
+      private:
+        Kind kind_;
+        uint32_t begin_;
+        uint32_t end_;
+        uint32_t functionNameIndex_;
+
+        friend class AsmJSModule;
+        CodeRange(Kind k, uint32_t begin, uint32_t end, uint32_t functionNameIndex)
+          : kind_(k), begin_(begin), end_(end), functionNameIndex_(functionNameIndex)
+        {}
+
+      public:
+        CodeRange() {}
+        Kind kind() const { return kind_; }
+        uint32_t begin() const { return begin_; }
+        uint32_t end() const { return end_; }
+        PropertyName *functionName(const AsmJSModule &module) const {
+            JS_ASSERT(kind_ == Function);
+            return module.functionNames_[functionNameIndex_].name();
+        }
+    };
+
     class Name
     {
         PropertyName *name_;
@@ -443,144 +482,155 @@ class AsmJSModule
     };
 
   private:
-    typedef Vector<Global, 0, SystemAllocPolicy> GlobalVector;
-    typedef Vector<Exit, 0, SystemAllocPolicy> ExitVector;
-    typedef Vector<ExportedFunction, 0, SystemAllocPolicy> ExportedFunctionVector;
-    typedef Vector<jit::CallSite, 0, SystemAllocPolicy> CallSiteVector;
-    typedef Vector<Name, 0, SystemAllocPolicy> FunctionNameVector;
-    typedef Vector<jit::AsmJSHeapAccess, 0, SystemAllocPolicy> HeapAccessVector;
-    typedef Vector<jit::IonScriptCounts *, 0, SystemAllocPolicy> FunctionCountsVector;
-#if defined(MOZ_VTUNE) || defined(JS_ION_PERF)
-    typedef Vector<ProfiledFunction, 0, SystemAllocPolicy> ProfiledFunctionVector;
-#endif
-#if defined(JS_ION_PERF)
-    typedef Vector<ProfiledBlocksFunction, 0, SystemAllocPolicy> ProfiledBlocksFunctionVector;
-#endif
-
-  private:
-    PropertyName *                        globalArgumentName_;
-    PropertyName *                        importArgumentName_;
-    PropertyName *                        bufferArgumentName_;
-
-    GlobalVector                          globals_;
-    ExitVector                            exits_;
-    ExportedFunctionVector                exports_;
-    CallSiteVector                        callSites_;
-    FunctionNameVector                    functionNames_;
-    HeapAccessVector                      heapAccesses_;
-#if defined(MOZ_VTUNE) || defined(JS_ION_PERF)
-    ProfiledFunctionVector                profiledFunctions_;
-#endif
-#if defined(JS_ION_PERF)
-    ProfiledBlocksFunctionVector          perfProfiledBlocksFunctions_;
-#endif
-
     struct Pod {
-        uint32_t                          funcLength_;
-        uint32_t                          funcLengthWithRightBrace_;
-        bool                              strict_;
-        uint32_t                          numGlobalVars_;
-        uint32_t                          numFFIs_;
         size_t                            funcPtrTableAndExitBytes_;
-        bool                              hasArrayView_;
         size_t                            functionBytes_; // just the function bodies, no stubs
         size_t                            codeBytes_;     // function bodies and stubs
         size_t                            totalBytes_;    // function bodies, stubs, and global data
         uint32_t                          minHeapLength_;
+        uint32_t                          numGlobalVars_;
+        uint32_t                          numFFIs_;
+        uint32_t                          srcLength_;
+        uint32_t                          srcLengthWithRightBrace_;
+        bool                              strict_;
+        bool                              hasArrayView_;
+        bool                              usesSignalHandlers_;
     } pod;
 
-    uint8_t *                             code_;
-    uint8_t *                             interruptExit_;
+    // These two fields need to be kept out pod as they depend on the position
+    // of the module within the ScriptSource and thus aren't invariant with
+    // respect to caching.
+    const uint32_t                        srcStart_;
+    const uint32_t                        srcBodyStart_;
 
-    StaticLinkData                        staticLinkData_;
-    bool                                  dynamicallyLinked_;
-    bool                                  loadedFromCache_;
-    HeapPtrArrayBufferObject              maybeHeap_;
-
-    // The next two fields need to be kept out of the Pod as they depend on the
-    // position of the module within the ScriptSource and thus aren't invariant
-    // with caching.
-    uint32_t                              funcStart_;
-    uint32_t                              offsetToEndOfUseAsm_;
+    Vector<Global,                 0, SystemAllocPolicy> globals_;
+    Vector<Exit,                   0, SystemAllocPolicy> exits_;
+    Vector<ExportedFunction,       0, SystemAllocPolicy> exports_;
+    Vector<jit::CallSite,          0, SystemAllocPolicy> callSites_;
+    Vector<CodeRange,              0, SystemAllocPolicy> codeRanges_;
+    Vector<Name,                   0, SystemAllocPolicy> functionNames_;
+    Vector<jit::AsmJSHeapAccess,   0, SystemAllocPolicy> heapAccesses_;
+    Vector<jit::IonScriptCounts*,  0, SystemAllocPolicy> functionCounts_;
+#if defined(MOZ_VTUNE) || defined(JS_ION_PERF)
+    Vector<ProfiledFunction,       0, SystemAllocPolicy> profiledFunctions_;
+#endif
+#if defined(JS_ION_PERF)
+    Vector<ProfiledBlocksFunction, 0, SystemAllocPolicy> perfProfiledBlocksFunctions_;
+#endif
 
     ScriptSource *                        scriptSource_;
-
-    FunctionCountsVector                  functionCounts_;
+    PropertyName *                        globalArgumentName_;
+    PropertyName *                        importArgumentName_;
+    PropertyName *                        bufferArgumentName_;
+    uint8_t *                             code_;
+    uint8_t *                             interruptExit_;
+    StaticLinkData                        staticLinkData_;
+    HeapPtrArrayBufferObject              maybeHeap_;
+    bool                                  dynamicallyLinked_;
+    bool                                  loadedFromCache_;
 
     // This field is accessed concurrently when requesting an interrupt.
     // Access must be synchronized via the runtime's interrupt lock.
     mutable bool                          codeIsProtected_;
 
   public:
-    explicit AsmJSModule(ScriptSource *scriptSource, uint32_t functStart,
-                         uint32_t offsetToEndOfUseAsm, bool strict);
+    explicit AsmJSModule(ScriptSource *scriptSource, uint32_t srcStart, uint32_t srcBodyStart,
+                         bool strict, bool canUseSignalHandlers);
+    void trace(JSTracer *trc);
     ~AsmJSModule();
 
-    void trace(JSTracer *trc) {
-        for (unsigned i = 0; i < globals_.length(); i++)
-            globals_[i].trace(trc);
-        for (unsigned i = 0; i < exports_.length(); i++)
-            exports_[i].trace(trc);
-        for (unsigned i = 0; i < exits_.length(); i++) {
-            if (exitIndexToGlobalDatum(i).fun)
-                MarkObject(trc, &exitIndexToGlobalDatum(i).fun, "asm.js imported function");
-        }
-        for (unsigned i = 0; i < functionNames_.length(); i++)
-            MarkStringUnbarriered(trc, &functionNames_[i].name(), "asm.js module function name");
-#if defined(MOZ_VTUNE) || defined(JS_ION_PERF)
-        for (unsigned i = 0; i < profiledFunctions_.length(); i++)
-            profiledFunctions_[i].trace(trc);
-#endif
-#if defined(JS_ION_PERF)
-        for (unsigned i = 0; i < perfProfiledBlocksFunctions_.length(); i++)
-            perfProfiledBlocksFunctions_[i].trace(trc);
-#endif
-        if (maybeHeap_)
-            gc::MarkObject(trc, &maybeHeap_, "asm.js heap");
+    // An AsmJSModule transitions monotonically through these states:
+    bool isFinishedWithModulePrologue() const { return pod.funcPtrTableAndExitBytes_ != SIZE_MAX; }
+    bool isFinishedWithFunctionBodies() const { return pod.functionBytes_ != UINT32_MAX; }
+    bool isFinished() const { return !!code_; }
+    bool isStaticallyLinked() const { return !!interruptExit_; }
+    bool isDynamicallyLinked() const { return dynamicallyLinked_; }
 
-        if (globalArgumentName_)
-            MarkStringUnbarriered(trc, &globalArgumentName_, "asm.js global argument name");
-        if (importArgumentName_)
-            MarkStringUnbarriered(trc, &importArgumentName_, "asm.js import argument name");
-        if (bufferArgumentName_)
-            MarkStringUnbarriered(trc, &bufferArgumentName_, "asm.js buffer argument name");
-    }
+    /*************************************************************************/
+    // These functions may be used as soon as the module is constructed:
 
     ScriptSource *scriptSource() const {
-        JS_ASSERT(scriptSource_ != nullptr);
+        JS_ASSERT(scriptSource_);
         return scriptSource_;
     }
-
-    /*
-     * funcStart() refers to the offset in the ScriptSource to the beginning
-     * of the function. If the function has been created with the Function
-     * constructor, this will be the first character in the function source.
-     * Otherwise, it will be the opening parenthesis of the arguments list.
-     */
-    uint32_t funcStart() const {
-        return funcStart_;
-    }
-    uint32_t offsetToEndOfUseAsm() const {
-        return offsetToEndOfUseAsm_;
-    }
-    void initFuncEnd(uint32_t endBeforeCurly, uint32_t endAfterCurly) {
-        JS_ASSERT(endBeforeCurly >= offsetToEndOfUseAsm_);
-        JS_ASSERT(endAfterCurly >= offsetToEndOfUseAsm_);
-        pod.funcLength_ = endBeforeCurly - funcStart_;
-        pod.funcLengthWithRightBrace_ = endAfterCurly - funcStart_;
-    }
-    uint32_t funcEndBeforeCurly() const {
-        return funcStart_ + pod.funcLength_;
-    }
-    uint32_t funcEndAfterCurly() const {
-        return funcStart_ + pod.funcLengthWithRightBrace_;
-    }
     bool strict() const {
         return pod.strict_;
     }
+    bool usesSignalHandlersForInterrupt() const {
+        return pod.usesSignalHandlers_;
+    }
+    bool usesSignalHandlersForOOB() const {
+#ifdef JS_CODEGEN_X64
+        return usesSignalHandlersForInterrupt();
+#else
+        return false;
+#endif
+    }
+    bool loadedFromCache() const {
+        return loadedFromCache_;
+    }
 
+    // srcStart() refers to the offset in the ScriptSource to the beginning of
+    // the asm.js module function. If the function has been created with the
+    // Function constructor, this will be the first character in the function
+    // source. Otherwise, it will be the opening parenthesis of the arguments
+    // list.
+    uint32_t srcStart() const {
+        return srcStart_;
+    }
+
+    // srcBodyStart() refers to the offset in the ScriptSource to the end
+    // of the 'use asm' string-literal token.
+    uint32_t srcBodyStart() const {
+        return srcBodyStart_;
+    }
+
+    // While these functions may be accessed at any time, their values will
+    // change as the module is compiled.
+    uint32_t minHeapLength() const {
+        return pod.minHeapLength_;
+    }
+    unsigned numFunctionCounts() const {
+        return functionCounts_.length();
+    }
+    jit::IonScriptCounts *functionCounts(unsigned i) {
+        return functionCounts_[i];
+    }
+
+    // about:memory reporting
+    void addSizeOfMisc(mozilla::MallocSizeOf mallocSizeOf, size_t *asmJSModuleCode,
+                       size_t *asmJSModuleData);
+
+    /*************************************************************************/
+    // These functions build the global scope of the module while parsing the
+    // module prologue (before the function bodies):
+
+    void initGlobalArgumentName(PropertyName *n) {
+        JS_ASSERT(!isFinishedWithModulePrologue());
+        JS_ASSERT_IF(n, n->isTenured());
+        globalArgumentName_ = n;
+    }
+    void initImportArgumentName(PropertyName *n) {
+        JS_ASSERT(!isFinishedWithModulePrologue());
+        JS_ASSERT_IF(n, n->isTenured());
+        importArgumentName_ = n;
+    }
+    void initBufferArgumentName(PropertyName *n) {
+        JS_ASSERT(!isFinishedWithModulePrologue());
+        JS_ASSERT_IF(n, n->isTenured());
+        bufferArgumentName_ = n;
+    }
+    PropertyName *globalArgumentName() const {
+        return globalArgumentName_;
+    }
+    PropertyName *importArgumentName() const {
+        return importArgumentName_;
+    }
+    PropertyName *bufferArgumentName() const {
+        return bufferArgumentName_;
+    }
     bool addGlobalVarInit(const Value &v, AsmJSCoercion coercion, uint32_t *globalIndex) {
-        JS_ASSERT(pod.funcPtrTableAndExitBytes_ == 0);
+        JS_ASSERT(!isFinishedWithModulePrologue());
         if (pod.numGlobalVars_ == UINT32_MAX)
             return false;
         Global g(Global::Variable, nullptr);
@@ -591,7 +641,7 @@ class AsmJSModule
         return globals_.append(g);
     }
     bool addGlobalVarImport(PropertyName *name, AsmJSCoercion coercion, uint32_t *globalIndex) {
-        JS_ASSERT(pod.funcPtrTableAndExitBytes_ == 0);
+        JS_ASSERT(!isFinishedWithModulePrologue());
         Global g(Global::Variable, name);
         g.pod.u.var.initKind_ = Global::InitImport;
         g.pod.u.var.coercion_ = coercion;
@@ -599,44 +649,78 @@ class AsmJSModule
         return globals_.append(g);
     }
     bool addFFI(PropertyName *field, uint32_t *ffiIndex) {
+        JS_ASSERT(!isFinishedWithModulePrologue());
         if (pod.numFFIs_ == UINT32_MAX)
             return false;
         Global g(Global::FFI, field);
         g.pod.u.ffiIndex_ = *ffiIndex = pod.numFFIs_++;
         return globals_.append(g);
     }
-    bool addArrayView(ArrayBufferView::ViewType vt, PropertyName *field) {
+    bool addArrayView(Scalar::Type vt, PropertyName *field) {
+        JS_ASSERT(!isFinishedWithModulePrologue());
         pod.hasArrayView_ = true;
         Global g(Global::ArrayView, field);
         g.pod.u.viewType_ = vt;
         return globals_.append(g);
     }
     bool addMathBuiltinFunction(AsmJSMathBuiltinFunction func, PropertyName *field) {
+        JS_ASSERT(!isFinishedWithModulePrologue());
         Global g(Global::MathBuiltinFunction, field);
         g.pod.u.mathBuiltinFunc_ = func;
         return globals_.append(g);
     }
     bool addMathBuiltinConstant(double value, PropertyName *field) {
+        JS_ASSERT(!isFinishedWithModulePrologue());
         Global g(Global::Constant, field);
         g.pod.u.constant.value_ = value;
         g.pod.u.constant.kind_ = Global::MathConstant;
         return globals_.append(g);
     }
     bool addGlobalConstant(double value, PropertyName *name) {
+        JS_ASSERT(!isFinishedWithModulePrologue());
         Global g(Global::Constant, name);
         g.pod.u.constant.value_ = value;
         g.pod.u.constant.kind_ = Global::GlobalConstant;
         return globals_.append(g);
     }
-    bool addFuncPtrTable(unsigned numElems, uint32_t *globalDataOffset) {
-        JS_ASSERT(IsPowerOfTwo(numElems));
-        if (SIZE_MAX - pod.funcPtrTableAndExitBytes_ < numElems * sizeof(void*))
+    unsigned numGlobals() const {
+        return globals_.length();
+    }
+    Global &global(unsigned i) {
+        return globals_[i];
+    }
+
+    /*************************************************************************/
+
+    void startFunctionBodies() {
+        JS_ASSERT(!isFinishedWithModulePrologue());
+        pod.funcPtrTableAndExitBytes_ = 0;
+        JS_ASSERT(isFinishedWithModulePrologue());
+    }
+
+    /*************************************************************************/
+    // These functions are called while parsing/compiling function bodies:
+
+    void requireHeapLengthToBeAtLeast(uint32_t len) {
+        JS_ASSERT(isFinishedWithModulePrologue() && !isFinishedWithFunctionBodies());
+        if (len > pod.minHeapLength_)
+            pod.minHeapLength_ = len;
+    }
+    bool addFunctionCodeRange(PropertyName *name, uint32_t begin, uint32_t end) {
+        JS_ASSERT(isFinishedWithModulePrologue() && !isFinishedWithFunctionBodies());
+        JS_ASSERT(name->isTenured());
+        if (functionNames_.length() >= UINT32_MAX)
             return false;
-        *globalDataOffset = globalDataBytes();
-        pod.funcPtrTableAndExitBytes_ += numElems * sizeof(void*);
-        return true;
+        CodeRange codeRange(CodeRange::Function, begin, end, functionNames_.length());
+        return functionNames_.append(name) && codeRanges_.append(codeRange);
+    }
+    bool addEntryCodeRange(unsigned exportIndex, uint32_t end) {
+        uint32_t begin = exports_[exportIndex].pod.codeOffset_;
+        CodeRange codeRange(CodeRange::Entry, begin, end, UINT32_MAX);
+        return codeRanges_.append(codeRange);
     }
     bool addExit(unsigned ffiIndex, unsigned *exitIndex) {
+        JS_ASSERT(isFinishedWithModulePrologue() && !isFinishedWithFunctionBodies());
         if (SIZE_MAX - pod.funcPtrTableAndExitBytes_ < sizeof(ExitDatum))
             return false;
         uint32_t globalDataOffset = globalDataBytes();
@@ -645,15 +729,89 @@ class AsmJSModule
         *exitIndex = unsigned(exits_.length());
         return exits_.append(Exit(ffiIndex, globalDataOffset));
     }
+    unsigned numExits() const {
+        JS_ASSERT(isFinishedWithModulePrologue());
+        return exits_.length();
+    }
+    Exit &exit(unsigned i) {
+        JS_ASSERT(isFinishedWithModulePrologue());
+        return exits_[i];
+    }
+    const Exit &exit(unsigned i) const {
+        JS_ASSERT(isFinishedWithModulePrologue());
+        return exits_[i];
+    }
+    bool addFuncPtrTable(unsigned numElems, uint32_t *globalDataOffset) {
+        JS_ASSERT(isFinishedWithModulePrologue() && !isFinished());
+        JS_ASSERT(IsPowerOfTwo(numElems));
+        if (SIZE_MAX - pod.funcPtrTableAndExitBytes_ < numElems * sizeof(void*))
+            return false;
+        *globalDataOffset = globalDataBytes();
+        pod.funcPtrTableAndExitBytes_ += numElems * sizeof(void*);
+        return true;
+    }
     bool addFunctionCounts(jit::IonScriptCounts *counts) {
+        JS_ASSERT(isFinishedWithModulePrologue() && !isFinishedWithFunctionBodies());
         return functionCounts_.append(counts);
     }
+#if defined(MOZ_VTUNE) || defined(JS_ION_PERF)
+    bool addProfiledFunction(PropertyName *name, unsigned codeStart, unsigned codeEnd,
+                             unsigned line, unsigned column)
+    {
+        JS_ASSERT(isFinishedWithModulePrologue() && !isFinishedWithFunctionBodies());
+        ProfiledFunction func(name, codeStart, codeEnd, line, column);
+        return profiledFunctions_.append(func);
+    }
+    unsigned numProfiledFunctions() const {
+        JS_ASSERT(isFinishedWithModulePrologue());
+        return profiledFunctions_.length();
+    }
+    ProfiledFunction &profiledFunction(unsigned i) {
+        JS_ASSERT(isFinishedWithModulePrologue());
+        return profiledFunctions_[i];
+    }
+#endif
+#ifdef JS_ION_PERF
+    bool addProfiledBlocks(PropertyName *name, unsigned codeBegin, unsigned inlineEnd,
+                           unsigned codeEnd, jit::BasicBlocksVector &basicBlocks)
+    {
+        JS_ASSERT(isFinishedWithModulePrologue() && !isFinishedWithFunctionBodies());
+        ProfiledBlocksFunction func(name, codeBegin, inlineEnd, codeEnd, basicBlocks);
+        return perfProfiledBlocksFunctions_.append(mozilla::Move(func));
+    }
+    unsigned numPerfBlocksFunctions() const {
+        JS_ASSERT(isFinishedWithModulePrologue());
+        return perfProfiledBlocksFunctions_.length();
+    }
+    ProfiledBlocksFunction &perfProfiledBlocksFunction(unsigned i) {
+        JS_ASSERT(isFinishedWithModulePrologue());
+        return perfProfiledBlocksFunctions_[i];
+    }
+#endif
 
-    bool addExportedFunction(PropertyName *name, uint32_t srcStart, uint32_t srcEnd,
+    /*************************************************************************/
+
+    // This function is called after compiling the function bodies (before
+    // compiling entries/exits) to record the extent of compiled function code.
+    void finishFunctionBodies(size_t functionBytes) {
+        JS_ASSERT(isFinishedWithModulePrologue() && !isFinishedWithFunctionBodies());
+        pod.functionBytes_ = functionBytes;
+        JS_ASSERT(isFinishedWithFunctionBodies());
+    }
+
+    /*************************************************************************/
+    // Exported functions are added after finishFunctionBodies() and before
+    // finish(). The list of exported functions can be accessed any time after
+    // the exported functions have been added.
+
+    bool addExportedFunction(PropertyName *name,
+                             uint32_t srcStart,
+                             uint32_t srcEnd,
                              PropertyName *maybeFieldName,
                              ArgCoercionVector &&argCoercions,
                              ReturnType returnType)
     {
+        JS_ASSERT(isFinishedWithFunctionBodies() && !isFinished());
         ExportedFunction func(name, srcStart, srcEnd, maybeFieldName,
                               mozilla::Move(argCoercions), returnType);
         if (exports_.length() >= UINT32_MAX)
@@ -661,109 +819,88 @@ class AsmJSModule
         return exports_.append(mozilla::Move(func));
     }
     unsigned numExportedFunctions() const {
+        JS_ASSERT(isFinishedWithFunctionBodies());
         return exports_.length();
     }
     const ExportedFunction &exportedFunction(unsigned i) const {
+        JS_ASSERT(isFinishedWithFunctionBodies());
         return exports_[i];
     }
     ExportedFunction &exportedFunction(unsigned i) {
+        JS_ASSERT(isFinishedWithFunctionBodies());
         return exports_[i];
     }
-    CodePtr entryTrampoline(const ExportedFunction &func) const {
-        JS_ASSERT(func.pod.codeOffset_ != UINT32_MAX);
-        return JS_DATA_TO_FUNC_PTR(CodePtr, code_ + func.pod.codeOffset_);
-    }
 
-    bool addFunctionName(PropertyName *name, uint32_t *nameIndex) {
-        JS_ASSERT(name->isTenured());
-        if (functionNames_.length() > jit::CallSiteDesc::FUNCTION_NAME_INDEX_MAX)
-            return false;
-        *nameIndex = functionNames_.length();
-        return functionNames_.append(name);
-    }
-    PropertyName *functionName(uint32_t i) const {
-        return functionNames_[i].name();
-    }
+    /*************************************************************************/
 
-#if defined(MOZ_VTUNE) || defined(JS_ION_PERF)
-    bool trackProfiledFunction(PropertyName *name, unsigned startCodeOffset, unsigned endCodeOffset,
-                               unsigned line, unsigned column)
-    {
-        ProfiledFunction func(name, startCodeOffset, endCodeOffset, line, column);
-        return profiledFunctions_.append(func);
-    }
-    unsigned numProfiledFunctions() const {
-        return profiledFunctions_.length();
-    }
-    ProfiledFunction &profiledFunction(unsigned i) {
-        return profiledFunctions_[i];
-    }
-#endif
+    // finish() is called once the entire module has been parsed (via
+    // tokenStream) and all function and entry/exit trampolines have been
+    // generated (via masm). After this function, the module must still be
+    // statically and dynamically linked before code can be run.
+    bool finish(ExclusiveContext *cx,
+                frontend::TokenStream &tokenStream,
+                jit::MacroAssembler &masm,
+                const jit::Label &interruptLabel);
 
-#ifdef JS_ION_PERF
-    bool trackPerfProfiledBlocks(PropertyName *name, unsigned startCodeOffset, unsigned endInlineCodeOffset,
-                                 unsigned endCodeOffset, jit::BasicBlocksVector &basicBlocks) {
-        ProfiledBlocksFunction func(name, startCodeOffset, endInlineCodeOffset, endCodeOffset, basicBlocks);
-        return perfProfiledBlocksFunctions_.append(mozilla::Move(func));
-    }
-    unsigned numPerfBlocksFunctions() const {
-        return perfProfiledBlocksFunctions_.length();
-    }
-    ProfiledBlocksFunction &perfProfiledBlocksFunction(unsigned i) {
-        return perfProfiledBlocksFunctions_[i];
-    }
-#endif
+    /*************************************************************************/
+    // These accessor functions can be used after finish():
 
     bool hasArrayView() const {
+        JS_ASSERT(isFinished());
         return pod.hasArrayView_;
     }
     unsigned numFFIs() const {
+        JS_ASSERT(isFinished());
         return pod.numFFIs_;
     }
-    unsigned numGlobalVars() const {
-        return pod.numGlobalVars_;
+    uint32_t srcEndBeforeCurly() const {
+        JS_ASSERT(isFinished());
+        return srcStart_ + pod.srcLength_;
     }
-    unsigned numGlobals() const {
-        return globals_.length();
+    uint32_t srcEndAfterCurly() const {
+        JS_ASSERT(isFinished());
+        return srcStart_ + pod.srcLengthWithRightBrace_;
     }
-    Global &global(unsigned i) {
-        return globals_[i];
+    uint8_t *codeBase() const {
+        JS_ASSERT(isFinished());
+        JS_ASSERT(uintptr_t(code_) % AsmJSPageSize == 0);
+        return code_;
     }
-    unsigned numExits() const {
-        return exits_.length();
+    size_t functionBytes() const {
+        JS_ASSERT(isFinished());
+        return pod.functionBytes_;
     }
-    Exit &exit(unsigned i) {
-        return exits_[i];
+    size_t codeBytes() const {
+        JS_ASSERT(isFinished());
+        return pod.codeBytes_;
     }
-    const Exit &exit(unsigned i) const {
-        return exits_[i];
+    bool containsPC(void *pc) const {
+        JS_ASSERT(isFinished());
+        return pc >= code_ && pc < (code_ + functionBytes());
     }
     uint8_t *interpExitTrampoline(const Exit &exit) const {
+        JS_ASSERT(isFinished());
         JS_ASSERT(exit.interpCodeOffset_);
         return code_ + exit.interpCodeOffset_;
     }
     uint8_t *ionExitTrampoline(const Exit &exit) const {
+        JS_ASSERT(isFinished());
         JS_ASSERT(exit.ionCodeOffset_);
         return code_ + exit.ionCodeOffset_;
     }
-    unsigned numFunctionCounts() const {
-        return functionCounts_.length();
-    }
-    jit::IonScriptCounts *functionCounts(unsigned i) {
-        return functionCounts_[i];
-    }
 
-    // An Exit holds bookkeeping information about an exit; the ExitDatum
-    // struct overlays the actual runtime data stored in the global data
-    // section.
-    struct ExitDatum
-    {
-        uint8_t *exit;
-        HeapPtrFunction fun;
-    };
+    // Lookup a callsite by the return pc (from the callee to the caller).
+    // Return null if no callsite was found.
+    const jit::CallSite *lookupCallSite(void *returnAddress) const;
+
+    // Lookup the name the code range containing the given pc. Return null if no
+    // code range was found.
+    const CodeRange *lookupCodeRange(void *pc) const;
+
+    // Lookup a heap access site by the pc which performs the access. Return
+    // null if no heap access was found.
+    const jit::AsmJSHeapAccess *lookupHeapAccess(void *pc) const;
 
-    // Global data section
-    //
     // The global data section is placed after the executable code (i.e., at
     // offset codeBytes_) in the module's linear allocation. The global data
     // are laid out in this order:
@@ -774,10 +911,11 @@ class AsmJSModule
     //      while type checking function bodies (as exits and uses of
     //      function-pointer tables are encountered).
     size_t offsetOfGlobalData() const {
-        JS_ASSERT(code_);
+        JS_ASSERT(isFinished());
         return pod.codeBytes_;
     }
     uint8_t *globalData() const {
+        JS_ASSERT(isFinished());
         return code_ + offsetOfGlobalData();
     }
     size_t globalDataBytes() const {
@@ -785,171 +923,108 @@ class AsmJSModule
                pod.numGlobalVars_ * sizeof(uint64_t) +
                pod.funcPtrTableAndExitBytes_;
     }
-    unsigned heapOffset() const {
+    static unsigned heapGlobalDataOffset() {
         return 0;
     }
     uint8_t *&heapDatum() const {
-        return *(uint8_t**)(globalData() + heapOffset());
+        JS_ASSERT(isFinished());
+        return *(uint8_t**)(globalData() + heapGlobalDataOffset());
     }
     unsigned globalVarIndexToGlobalDataOffset(unsigned i) const {
+        JS_ASSERT(isFinishedWithModulePrologue());
         JS_ASSERT(i < pod.numGlobalVars_);
         return sizeof(uint64_t) +
                i * sizeof(uint64_t);
     }
     void *globalVarIndexToGlobalDatum(unsigned i) const {
+        JS_ASSERT(isFinished());
         return (void *)(globalData() + globalVarIndexToGlobalDataOffset(i));
     }
     uint8_t **globalDataOffsetToFuncPtrTable(unsigned globalDataOffset) const {
+        JS_ASSERT(isFinished());
         JS_ASSERT(globalDataOffset < globalDataBytes());
         return (uint8_t **)(globalData() + globalDataOffset);
     }
     unsigned exitIndexToGlobalDataOffset(unsigned exitIndex) const {
+        JS_ASSERT(isFinishedWithModulePrologue());
         return exits_[exitIndex].globalDataOffset();
     }
     ExitDatum &exitIndexToGlobalDatum(unsigned exitIndex) const {
+        JS_ASSERT(isFinished());
         return *(ExitDatum *)(globalData() + exitIndexToGlobalDataOffset(exitIndex));
     }
-
-    void initFunctionBytes(size_t functionBytes) {
-        JS_ASSERT(pod.functionBytes_ == 0);
-        pod.functionBytes_ = functionBytes;
-    }
-    void updateFunctionBytes(jit::MacroAssembler &masm) {
-        pod.functionBytes_ = masm.actualOffset(pod.functionBytes_);
-        JS_ASSERT(pod.functionBytes_ % AsmJSPageSize == 0);
-    }
-    size_t functionBytes() const {
-        JS_ASSERT(pod.functionBytes_);
-        JS_ASSERT(pod.functionBytes_ % AsmJSPageSize == 0);
-        return pod.functionBytes_;
-    }
-    bool containsPC(void *pc) const {
-        return pc >= code_ && pc < (code_ + functionBytes());
-    }
-
-    void assignHeapAccesses(jit::AsmJSHeapAccessVector &&accesses) {
-        heapAccesses_ = Move(accesses);
-    }
-    unsigned numHeapAccesses() const {
-        return heapAccesses_.length();
-    }
-    const jit::AsmJSHeapAccess &heapAccess(unsigned i) const {
-        return heapAccesses_[i];
-    }
-    jit::AsmJSHeapAccess &heapAccess(unsigned i) {
-        return heapAccesses_[i];
-    }
-
-    void assignCallSites(jit::CallSiteVector &&callsites) {
-        callSites_ = Move(callsites);
-    }
-    unsigned numCallSites() const {
-        return callSites_.length();
-    }
-    const jit::CallSite &callSite(unsigned i) const {
-        return callSites_[i];
-    }
-    jit::CallSite &callSite(unsigned i) {
-        return callSites_[i];
-    }
-
-    void initHeap(Handle<ArrayBufferObject*> heap, JSContext *cx);
-
-    void requireHeapLengthToBeAtLeast(uint32_t len) {
-        if (len > pod.minHeapLength_)
-            pod.minHeapLength_ = len;
-    }
-    uint32_t minHeapLength() const {
-        return pod.minHeapLength_;
-    }
-
-    bool allocateAndCopyCode(ExclusiveContext *cx, jit::MacroAssembler &masm);
-
-    // StaticLinkData setters (called after finishing compilation, before
-    // staticLink).
-    bool addRelativeLink(RelativeLink link) {
-        return staticLinkData_.relativeLinks.append(link);
-    }
-    bool addAbsoluteLink(AbsoluteLink link) {
-        return staticLinkData_.absoluteLinks.append(link);
-    }
-    void setInterruptOffset(uint32_t offset) {
-        staticLinkData_.interruptExitOffset = offset;
-    }
-
-    void restoreToInitialState(ArrayBufferObject *maybePrevBuffer, ExclusiveContext *cx);
-    void setAutoFlushICacheRange();
-    void staticallyLink(ExclusiveContext *cx);
-
-    uint8_t *codeBase() const {
-        JS_ASSERT(code_);
-        JS_ASSERT(uintptr_t(code_) % AsmJSPageSize == 0);
-        return code_;
-    }
-
-    uint8_t *interruptExit() const {
-        return interruptExit_;
-    }
-
-    void setIsDynamicallyLinked() {
-        JS_ASSERT(!dynamicallyLinked_);
-        dynamicallyLinked_ = true;
-    }
-    bool isDynamicallyLinked() const {
-        return dynamicallyLinked_;
-    }
-    uint8_t *maybeHeap() const {
-        JS_ASSERT(dynamicallyLinked_);
-        return heapDatum();
-    }
-    ArrayBufferObject *maybeHeapBufferObject() const {
-        JS_ASSERT(dynamicallyLinked_);
-        return maybeHeap_;
-    }
-    size_t heapLength() const {
-        JS_ASSERT(dynamicallyLinked_);
-        return maybeHeap_ ? maybeHeap_->byteLength() : 0;
-    }
-
-    void initGlobalArgumentName(PropertyName *n) {
-        JS_ASSERT_IF(n, n->isTenured());
-        globalArgumentName_ = n;
-    }
-    void initImportArgumentName(PropertyName *n) {
-        JS_ASSERT_IF(n, n->isTenured());
-        importArgumentName_ = n;
-    }
-    void initBufferArgumentName(PropertyName *n) {
-        JS_ASSERT_IF(n, n->isTenured());
-        bufferArgumentName_ = n;
-    }
-
-    PropertyName *globalArgumentName() const {
-        return globalArgumentName_;
-    }
-    PropertyName *importArgumentName() const {
-        return importArgumentName_;
-    }
-    PropertyName *bufferArgumentName() const {
-        return bufferArgumentName_;
-    }
-
     void detachIonCompilation(size_t exitIndex) const {
+        JS_ASSERT(isFinished());
         exitIndexToGlobalDatum(exitIndex).exit = interpExitTrampoline(exit(exitIndex));
     }
 
-    void addSizeOfMisc(mozilla::MallocSizeOf mallocSizeOf, size_t *asmJSModuleCode,
-                       size_t *asmJSModuleData);
+    /*************************************************************************/
+    // These functions are called after finish() but before staticallyLink():
 
+    bool addRelativeLink(RelativeLink link) {
+        JS_ASSERT(isFinished() && !isStaticallyLinked());
+        return staticLinkData_.relativeLinks.append(link);
+    }
+
+    // A module is serialized after it is finished but before it is statically
+    // linked. (Technically, it could be serialized after static linking, but it
+    // would still need to be statically linked on deserialization.)
     size_t serializedSize() const;
     uint8_t *serialize(uint8_t *cursor) const;
     const uint8_t *deserialize(ExclusiveContext *cx, const uint8_t *cursor);
-    bool loadedFromCache() const { return loadedFromCache_; }
 
+    // Additionally, this function is called to flush the i-cache after
+    // deserialization and cloning (but still before static linking, to prevent
+    // a bunch of expensive micro-flushes).
+    void setAutoFlushICacheRange();
+
+    /*************************************************************************/
+
+    // After a module isFinished compiling or deserializing, it is "statically
+    // linked" which specializes the code to its current address (this allows
+    // code to be relocated between serialization and deserialization).
+    void staticallyLink(ExclusiveContext *cx);
+
+    // After a module is statically linked, it is "dynamically linked" which
+    // specializes it to a particular set of arguments. In particular, this
+    // binds the code to a particular heap (via initHeap) and set of global
+    // variables. A given asm.js module cannot be dynamically linked more than
+    // once so, if JS tries, the module is cloned.
+    void setIsDynamicallyLinked() {
+        JS_ASSERT(!isDynamicallyLinked());
+        dynamicallyLinked_ = true;
+        JS_ASSERT(isDynamicallyLinked());
+    }
+    void initHeap(Handle<ArrayBufferObject*> heap, JSContext *cx);
     bool clone(JSContext *cx, ScopedJSDeletePtr<AsmJSModule> *moduleOut) const;
+    void restoreToInitialState(ArrayBufferObject *maybePrevBuffer, ExclusiveContext *cx);
 
-    // These methods may only be called while holding the Runtime's interrupt
-    // lock.
+    /*************************************************************************/
+    // Functions that can be called after dynamic linking succeeds:
+
+    CodePtr entryTrampoline(const ExportedFunction &func) const {
+        JS_ASSERT(isDynamicallyLinked());
+        return JS_DATA_TO_FUNC_PTR(CodePtr, code_ + func.pod.codeOffset_);
+    }
+    uint8_t *interruptExit() const {
+        JS_ASSERT(isDynamicallyLinked());
+        return interruptExit_;
+    }
+    uint8_t *maybeHeap() const {
+        JS_ASSERT(isDynamicallyLinked());
+        return heapDatum();
+    }
+    ArrayBufferObject *maybeHeapBufferObject() const {
+        JS_ASSERT(isDynamicallyLinked());
+        return maybeHeap_;
+    }
+    size_t heapLength() const {
+        JS_ASSERT(isDynamicallyLinked());
+        return maybeHeap_ ? maybeHeap_->byteLength() : 0;
+    }
+
+    // Additionally, these functions may only be called while holding the
+    // runtime's interrupt lock.
     void protectCode(JSRuntime *rt) const;
     void unprotectCode(JSRuntime *rt) const;
     bool codeIsProtected(JSRuntime *rt) const;
