@@ -4,13 +4,13 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#ifndef jspropertytree_h___
-#define jspropertytree_h___
+#ifndef jspropertytree_h
+#define jspropertytree_h
 
 #include "jsalloc.h"
+#include "jspubtd.h"
 
 #include "js/HashTable.h"
-#include "js/RootingAPI.h"
 
 namespace js {
 
@@ -72,9 +72,9 @@ class PropertyTree
 {
     friend class ::JSFunction;
 
-    JSCompartment *compartment;
+    JSCompartment *compartment_;
 
-    bool insertChild(JSContext *cx, Shape *parent, Shape *child);
+    bool insertChild(ExclusiveContext *cx, Shape *parent, Shape *child);
 
     PropertyTree();
 
@@ -89,19 +89,18 @@ class PropertyTree
         MAX_HEIGHT_WITH_ELEMENTS_ACCESS = 128
     };
 
-    PropertyTree(JSCompartment *comp)
-        : compartment(comp)
+    explicit PropertyTree(JSCompartment *comp)
+        : compartment_(comp)
     {
     }
 
-    Shape *newShape(JSContext *cx);
-    Shape *getChild(JSContext *cx, Shape *parent, uint32_t nfixed, const StackShape &child);
+    JSCompartment *compartment() { return compartment_; }
 
-#ifdef DEBUG
-    static void dumpShapes(JSRuntime *rt);
-#endif
+    Shape *newShape(ExclusiveContext *cx);
+    Shape *getChild(ExclusiveContext *cx, Shape *parent, StackShape &child);
+    Shape *lookupChild(ThreadSafeContext *cx, Shape *parent, const StackShape &child);
 };
 
 } /* namespace js */
 
-#endif /* jspropertytree_h___ */
+#endif /* jspropertytree_h */
