@@ -7,8 +7,8 @@
 #ifndef prmjtime_h
 #define prmjtime_h
 
+#include <stddef.h>
 #include <stdint.h>
-#include <time.h>
 
 /*
  * Broken down form of 64 bit time value.
@@ -32,14 +32,24 @@ struct PRMJTime {
 
 /* Return the current local time in micro-seconds */
 extern int64_t
-PRMJ_Now(void);
+PRMJ_Now();
+
+/* Initialize the resources associated with PRMJ_Now. */
+#if defined(XP_WIN)
+extern void
+PRMJ_NowInit();
+#else
+inline void
+PRMJ_NowInit() {}
+#endif
 
 /* Release the resources associated with PRMJ_Now; don't call PRMJ_Now again */
 #if defined(JS_THREADSAFE) && defined(XP_WIN)
 extern void
-PRMJ_NowShutdown(void);
+PRMJ_NowShutdown();
 #else
-#define PRMJ_NowShutdown()
+inline void
+PRMJ_NowShutdown() {}
 #endif
 
 /* Format a time value into a buffer. Same semantics as strftime() */
