@@ -4,7 +4,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "jsoptparse.h"
+#include "shell/jsoptparse.h"
 
 #include <ctype.h>
 #include <stdarg.h>
@@ -59,6 +59,14 @@ OptionParser::setArgTerminatesOptions(const char *name, bool enabled)
     findArgument(name)->setTerminatesOptions(enabled);
 }
 
+void
+OptionParser::setArgCapturesRest(const char *name)
+{
+    MOZ_ASSERT(restArgument == -1, "only one argument may be set to capture the rest");
+    restArgument = findArgumentIndex(name);
+    MOZ_ASSERT(restArgument != -1, "unknown argument name passed to setArgCapturesRest");
+}
+
 OptionParser::Result
 OptionParser::error(const char *fmt, ...)
 {
@@ -127,7 +135,7 @@ PrintParagraph(const char *text, unsigned startColno, const unsigned limitColno,
             }
             break;
           default:
-            JS_NOT_REACHED("unhandled token splitting character in text");
+            MOZ_CRASH("unhandled token splitting character in text");
         }
     }
 }
@@ -135,10 +143,10 @@ PrintParagraph(const char *text, unsigned startColno, const unsigned limitColno,
 static const char *
 OptionFlagsToFormatInfo(char shortflag, bool isValued, size_t *length)
 {
-    static const char *fmt[4] = { "  -%c --%s ",
-                                  "  --%s ",
-                                  "  -%c --%s=%s ",
-                                  "  --%s=%s " };
+    static const char * const fmt[4] = { "  -%c --%s ",
+                                         "  --%s ",
+                                         "  -%c --%s=%s ",
+                                         "  --%s=%s " };
 
     /* How mny chars w/o longflag? */
     size_t lengths[4] = { strlen(fmt[0]) - 3,
@@ -245,7 +253,7 @@ OptionParser::extractValue(size_t argc, char **argv, size_t *i, char **value)
     char *eq = strchr(argv[*i], '=');
     if (eq) {
         *value = eq + 1;
-        if (value[0] == '\0')
+        if (*value[0] == '\0')
             return error("A value is required for option %.*s", eq - argv[*i], argv[*i]);
         return Okay;
     }
@@ -278,7 +286,7 @@ OptionParser::handleOption(Option *opt, size_t argc, char **argv, size_t *i, boo
        */
       case OptionKindString:
       {
-        char *value = NULL;
+        char *value = nullptr;
         if (Result r = extractValue(argc, argv, i, &value))
             return r;
         opt->asStringOption()->value = value;
@@ -286,7 +294,7 @@ OptionParser::handleOption(Option *opt, size_t argc, char **argv, size_t *i, boo
       }
       case OptionKindInt:
       {
-        char *value = NULL;
+        char *value = nullptr;
         if (Result r = extractValue(argc, argv, i, &value))
             return r;
         opt->asIntOption()->value = atoi(value);
@@ -294,15 +302,14 @@ OptionParser::handleOption(Option *opt, size_t argc, char **argv, size_t *i, boo
       }
       case OptionKindMultiString:
       {
-        char *value = NULL;
+        char *value = nullptr;
         if (Result r = extractValue(argc, argv, i, &value))
             return r;
         StringArg arg(value, *i);
         return opt->asMultiStringOption()->strings.append(arg) ? Okay : Fail;
       }
       default:
-        JS_NOT_REACHED("unhandled option kind");
-        return Fail;
+        MOZ_CRASH("unhandled option kind");
     }
 }
 
@@ -329,8 +336,7 @@ OptionParser::handleArg(size_t argc, char **argv, size_t *i, bool *optionsAllowe
         return arg->asMultiStringOption()->strings.append(value) ? Okay : Fail;
       }
       default:
-        JS_NOT_REACHED("unhandled argument kind");
-        return Fail;
+        MOZ_CRASH("unhandled argument kind");
     }
 }
 
@@ -350,10 +356,17 @@ OptionParser::parseArgs(int inputArgc, char **argv)
             /* Option. */
             Option *opt;
             if (arg[1] == '-') {
-                /* Long option. */
-                opt = findOption(arg + 2);
-                if (!opt)
-                    return error("Invalid long option: %s", arg);
+                if (arg[2] == '\0') {
+                    /* End of options */
+                    optionsAllowed = false;
+                    nextArgument = restArgument;
+                    continue;
+                } else {
+                    /* Long option. */
+                    opt = findOption(arg + 2);
+                    if (!opt)
+                        return error("Invalid long option: %s", arg);
+                }
             } else {
                 /* Short option */
                 if (arg[2] != '\0')
@@ -369,12 +382,8 @@ OptionParser::parseArgs(int inputArgc, char **argv)
             r = handleArg(argc, argv, &i, &optionsAllowed);
         }
 
-        switch (r) {
-          case Okay:
-            break;
-          default:
+        if (r != Okay)
             return r;
-        }
     }
     return Okay;
 }
@@ -457,7 +466,7 @@ OptionParser::findOption(char shortflag)
             return *it;
     }
 
-    return helpOption.shortflag == shortflag ? &helpOption : NULL;
+    return helpOption.shortflag == shortflag ? &helpOption : nullptr;
 }
 
 const Option *
@@ -487,7 +496,7 @@ OptionParser::findOption(const char *longflag)
   no_match:;
     }
 
-    return strcmp(helpOption.longflag, longflag) ? NULL : &helpOption;
+    return strcmp(helpOption.longflag, longflag) ? nullptr : &helpOption;
 }
 
 const Option *
@@ -498,21 +507,29 @@ OptionParser::findOption(const char *longflag) const
 
 /* Argument accessors */
 
+int
+OptionParser::findArgumentIndex(const char *name) const
+{
+    for (Option * const *it = arguments.begin(); it != arguments.end(); ++it) {
+        const char *target = (*it)->longflag;
+        if (strcmp(target, name) == 0)
+            return it - arguments.begin();
+    }
+    return -1;
+}
+
 Option *
 OptionParser::findArgument(const char *name)
 {
-    for (Option **it = arguments.begin(), **end = arguments.end(); it != end; ++it) {
-        const char *target = (*it)->longflag;
-        if (strcmp(target, name) == 0)
-            return *it;
-    }
-    return NULL;
+    int index = findArgumentIndex(name);
+    return (index == -1) ? nullptr : arguments[index];
 }
 
 const Option *
 OptionParser::findArgument(const char *name) const
 {
-    return const_cast<OptionParser *>(this)->findArgument(name);
+    int index = findArgumentIndex(name);
+    return (index == -1) ? nullptr : arguments[index];
 }
 
 const char *
@@ -588,7 +605,7 @@ OptionParser::addOptionalStringArg(const char *name, const char *help)
 {
     if (!arguments.reserve(arguments.length() + 1))
         return false;
-    StringOption *so = js_new<StringOption>(1, name, help, (const char *) NULL);
+    StringOption *so = js_new<StringOption>(1, name, help, (const char *) nullptr);
     if (!so)
         return false;
     arguments.infallibleAppend(so);
@@ -601,7 +618,7 @@ OptionParser::addOptionalMultiStringArg(const char *name, const char *help)
     JS_ASSERT_IF(!arguments.empty(), !arguments.back()->isVariadic());
     if (!arguments.reserve(arguments.length() + 1))
         return false;
-    MultiStringOption *mso = js_new<MultiStringOption>(1, name, help, (const char *) NULL);
+    MultiStringOption *mso = js_new<MultiStringOption>(1, name, help, (const char *) nullptr);
     if (!mso)
         return false;
     arguments.infallibleAppend(mso);
