@@ -4,27 +4,20 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch11/11.13/11.13.1/11.13.1-1-s.js
- * @description Strict Mode - TypeError is thrown if The LeftHandSide is a reference to a data property with the attribute value {[[Writable]]:false} under strict mode
+ * @path ch11/11.1/11.1.5/11.1.5-1-s.js
+ * @description Strict Mode - SyntaxError is thrown when 'eval' occurs as the Identifier in a PropertySetParameterList of a PropertyAssignment that is contained in strict code
  * @onlyStrict
  */
 
 
 function testcase() {
         "use strict";
-        var obj = {};
-        Object.defineProperty(obj, "prop", {
-            value: 10,
-            writable: false,
-            enumerable: true,
-            configurable: true
-        });
 
         try {
-            obj.prop = 20;
+            eval("var obj = {set _11_1_5_1_fun(eval) {}};");
             return false;
         } catch (e) {
-            return e instanceof TypeError && obj.prop === 10;
+            return (e instanceof SyntaxError);
         }
     }
 runTestCase(testcase);
