@@ -269,11 +269,11 @@ nsMathMLmpaddedFrame::UpdateValue(int32_t                  aSign,
              break;
 
         case NS_MATHML_PSEUDO_UNIT_HEIGHT:
-             scaler = aDesiredSize.TopAscent();
+             scaler = aDesiredSize.BlockStartAscent();
              break;
 
         case NS_MATHML_PSEUDO_UNIT_DEPTH:
-             scaler = aDesiredSize.Height() - aDesiredSize.TopAscent();
+             scaler = aDesiredSize.Height() - aDesiredSize.BlockStartAscent();
              break;
 
         default:
@@ -327,8 +327,8 @@ nsMathMLmpaddedFrame::Place(nsRenderingContext& aRenderingContext,
     return rv;
   }
 
-  nscoord height = aDesiredSize.TopAscent();
-  nscoord depth  = aDesiredSize.Height() - aDesiredSize.TopAscent();
+  nscoord height = aDesiredSize.BlockStartAscent();
+  nscoord depth  = aDesiredSize.Height() - aDesiredSize.BlockStartAscent();
   // The REC says:
   //
   // "The lspace attribute ('leading' space) specifies the horizontal location
@@ -418,19 +418,19 @@ nsMathMLmpaddedFrame::Place(nsRenderingContext& aRenderingContext,
   nscoord dx = (StyleVisibility()->mDirection ?
                 width - initialWidth - lspace : lspace);
     
-  aDesiredSize.SetTopAscent(height);
+  aDesiredSize.SetBlockStartAscent(height);
   aDesiredSize.Width() = mBoundingMetrics.width;
-  aDesiredSize.Height() = depth + aDesiredSize.TopAscent();
+  aDesiredSize.Height() = depth + aDesiredSize.BlockStartAscent();
   mBoundingMetrics.ascent = height;
   mBoundingMetrics.descent = depth;
   aDesiredSize.mBoundingMetrics = mBoundingMetrics;
 
   mReference.x = 0;
-  mReference.y = aDesiredSize.TopAscent();
+  mReference.y = aDesiredSize.BlockStartAscent();
 
   if (aPlaceOrigin) {
     // Finish reflowing child frames, positioning their origins.
-    PositionRowChildFrames(dx, aDesiredSize.TopAscent() - voffset);
+    PositionRowChildFrames(dx, aDesiredSize.BlockStartAscent() - voffset);
   }
 
   return NS_OK;
