@@ -44,6 +44,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #ifndef nricemediastream_h__
 #define nricemediastream_h__
 
+#include <string>
 #include <vector>
 
 #include "sigslot.h"
@@ -63,6 +64,12 @@ typedef struct nr_ice_media_stream_ nr_ice_media_stream;
 
 class NrIceCtx;
 
+struct NrIceAddr {
+  std::string host;
+  uint16_t port;
+  std::string transport;
+};
+
 /* A summary of a candidate, for use in asking which candidate
    pair is active */
 struct NrIceCandidate {
@@ -73,9 +80,44 @@ struct NrIceCandidate {
     ICE_RELAYED
   };
 
-  std::string host;
-  uint16_t port;
+  NrIceAddr cand_addr;
+  NrIceAddr local_addr;
   Type type;
+  std::string codeword;
+};
+
+struct NrIceCandidatePair {
+
+  enum State {
+    STATE_FROZEN,
+    STATE_WAITING,
+    STATE_IN_PROGRESS,
+    STATE_FAILED,
+    STATE_SUCCEEDED,
+    STATE_CANCELLED
+  };
+
+  State state;
+  uint64_t priority;
+  // Set regardless of who nominated it. Does not necessarily mean that it is
+  // ready to be selected (ie; nominated by peer, but our check has not
+  // succeeded yet.) Note: since this implementation uses aggressive nomination,
+  // when we are the controlling agent, this will always be set if the pair is
+  // in STATE_SUCCEEDED.
+  bool nominated;
+  // Set if this candidate pair has been selected. Note: Since we are using
+  // aggressive nomination, this could change frequently as ICE runs.
+  bool selected;
+  NrIceCandidate local;
+  NrIceCandidate remote;
+  // TODO(bcampen@mozilla.com): Is it important to put the foundation in here?
+  std::string codeword;
+};
+
+// Abstract base class for opaque values.
+class NrIceOpaque {
+ public:
+  virtual ~NrIceOpaque() {}
 };
 
 class NrIceMediaStream {
@@ -83,8 +125,6 @@ class NrIceMediaStream {
   static RefPtr<NrIceMediaStream> Create(NrIceCtx *ctx,
                                          const std::string& name,
                                          int components);
-  ~NrIceMediaStream();
-
   enum State { ICE_CONNECTING, ICE_OPEN, ICE_CLOSED};
 
   State state() const { return state_; }
@@ -95,6 +135,13 @@ class NrIceMediaStream {
   // Get all the candidates
   std::vector<std::string> GetCandidates() const;
 
+  nsresult GetLocalCandidates(std::vector<NrIceCandidate>* candidates) const;
+  nsresult GetRemoteCandidates(std::vector<NrIceCandidate>* candidates) const;
+
+  // Get all candidate pairs, whether in the check list or triggered check
+  // queue, in priority order. |out_pairs| is cleared before being filled.
+  nsresult GetCandidatePairs(std::vector<NrIceCandidatePair>* out_pairs) const;
+
   // Get the default candidate as host and port
   nsresult GetDefaultCandidate(int component, std::string *host, int *port);
 
@@ -132,6 +179,12 @@ class NrIceMediaStream {
   // the context has been destroyed.
   void Close();
 
+  // Set an opaque value. Owned by the media stream.
+  void SetOpaque(NrIceOpaque *opaque) { opaque_ = opaque; }
+
+  // Get the opaque
+  NrIceOpaque* opaque() const { return opaque_; }
+
   sigslot::signal2<NrIceMediaStream *, const std::string& >
   SignalCandidate;  // A new ICE candidate:
   sigslot::signal1<NrIceMediaStream *> SignalReady;  // Candidate pair ready.
@@ -139,10 +192,6 @@ class NrIceMediaStream {
   sigslot::signal4<NrIceMediaStream *, int, const unsigned char *, int>
   SignalPacketReceived;  // Incoming packet
 
-  // Emit all the ICE candidates. Note that this doesn't
-  // work for trickle ICE yet--called internally
-  void EmitAllCandidates();
-
   NS_INLINE_DECL_THREADSAFE_REFCOUNTING(NrIceMediaStream)
 
  private:
@@ -152,7 +201,10 @@ class NrIceMediaStream {
       ctx_(ctx),
       name_(name),
       components_(components),
-      stream_(nullptr) {}
+      stream_(nullptr),
+      opaque_(nullptr) {}
+
+  ~NrIceMediaStream();
 
   DISALLOW_COPY_ASSIGN(NrIceMediaStream);
 
@@ -161,6 +213,7 @@ class NrIceMediaStream {
   const std::string name_;
   const int components_;
   nr_ice_media_stream *stream_;
+  ScopedDeletePtr<NrIceOpaque> opaque_;
 };
 
 
