@@ -9,7 +9,6 @@
  */
 
 #include "webrtc/modules/video_coding/main/source/session_info.h"
-
 #include "webrtc/modules/video_coding/main/source/packet.h"
 #include "webrtc/modules/rtp_rtcp/source/rtp_format_h264.h"
 
@@ -116,38 +115,104 @@ int VCMSessionInfo::NumPackets() const {
   return packets_.size();
 }
 
+void VCMSessionInfo::CopyPacket(uint8_t* dst, const uint8_t* src, size_t len) {
+  memcpy(dst, src, len);
+}
+
+void VCMSessionInfo::CopyWithStartCode(uint8_t* dst, const uint8_t* src, size_t len) {
+  // H.264 Start Code is 2 or more bytes of 0 followed by 1 byte of 1.
+  memset(dst, 0, RtpFormatH264::kStartCodeSize-1);
+  dst[RtpFormatH264::kStartCodeSize-1] = 1;
+  CopyPacket(dst + RtpFormatH264::kStartCodeSize, src, len);
+}
+
 int VCMSessionInfo::InsertBuffer(uint8_t* frame_buffer,
                                  PacketIterator packet_it) {
   VCMPacket& packet = *packet_it;
-  PacketIterator it;
 
-  int packet_size = packet.sizeBytes;
-  packet_size += (packet.insertStartCode ? kH264StartCodeLengthBytes : 0);
+  // Advance to the offset into the frame buffer for this packet.
+  for (PacketIterator it = packets_.begin(); it != packet_it; ++it)
+    frame_buffer += (*it).sizeBytes;
 
-  // Calculate the offset into the frame buffer for this packet.
-  int offset = 0;
-  for (it = packets_.begin(); it != packet_it; ++it)
-    offset += (*it).sizeBytes;
-
-  // Set the data pointer to pointing to the start of this packet in the
-  // frame buffer.
-  const uint8_t* data = packet.dataPtr;
-  packet.dataPtr = frame_buffer + offset;
-  packet.sizeBytes = packet_size;
-
-  ShiftSubsequentPackets(packet_it, packet_size);
-
-  const uint8_t startCode[] = {0, 0, 0, 1};
-  if (packet.insertStartCode) {
-    memcpy(const_cast<uint8_t*>(packet.dataPtr), startCode,
-           kH264StartCodeLengthBytes);
+  if (packet.codec == kVideoCodecH264) {
+    // Calculate extra packet size needed for adding start codes,
+    // and removing fragmentation and aggregation unit headers.
+    size_t nalu_size;
+    size_t all_nalu_size = 0;
+    const uint8_t* nalu_ptr = packet.dataPtr;
+    uint8_t nal_header = *nalu_ptr;
+    uint8_t fu_header;
+    switch (nal_header & RtpFormatH264::kTypeMask) {
+      case RtpFormatH264::kFuA:
+        fu_header = nalu_ptr[RtpFormatH264::kFuAHeaderOffset];
+        if (fu_header & RtpFormatH264::kFragStartBit) {
+          nal_header &= ~RtpFormatH264::kTypeMask; // Keep F/NRI bits
+          nal_header |= fu_header & RtpFormatH264::kTypeMask; // Keep NAL type
+          packet.sizeBytes -= RtpFormatH264::kFuAHeaderOffset;
+          packet.dataPtr += RtpFormatH264::kFuAHeaderOffset;
+          ShiftSubsequentPackets(packet_it, packet.sizeBytes +
+                                 RtpFormatH264::kStartCodeSize);
+          CopyWithStartCode(frame_buffer, packet.dataPtr, packet.sizeBytes);
+          frame_buffer[RtpFormatH264::kStartCodeSize] = nal_header;
+          packet.sizeBytes += RtpFormatH264::kStartCodeSize;
+          packet.dataPtr = frame_buffer;
+          packet.completeNALU = kNaluStart;
+        } else {
+          packet.sizeBytes -= RtpFormatH264::kFuAHeaderOffset +
+                              RtpFormatH264::kFuAHeaderSize;
+          packet.dataPtr += RtpFormatH264::kFuAHeaderOffset +
+                            RtpFormatH264::kFuAHeaderSize;
+          ShiftSubsequentPackets(packet_it, packet.sizeBytes);
+          CopyPacket(frame_buffer, packet.dataPtr, packet.sizeBytes);
+          packet.dataPtr = frame_buffer;
+          if (fu_header & RtpFormatH264::kFragEndBit) {
+            packet.completeNALU = kNaluEnd;
+          } else {
+            packet.completeNALU = kNaluIncomplete;
+          }
+        }
+        break;
+      case RtpFormatH264::kStapA:
+        packet.sizeBytes -= RtpFormatH264::kStapAHeaderOffset;
+        packet.dataPtr += RtpFormatH264::kStapAHeaderOffset;
+        for (nalu_ptr = packet.dataPtr;
+             nalu_ptr < packet.dataPtr + packet.sizeBytes;
+             nalu_ptr += nalu_size + RtpFormatH264::kAggUnitLengthSize) {
+          nalu_size = (nalu_ptr[0] << 8) + nalu_ptr[1];
+          all_nalu_size += nalu_size + RtpFormatH264::kStartCodeSize;
+        }
+        if (nalu_ptr > packet.dataPtr + packet.sizeBytes) {
+          // malformed packet
+          packet.completeNALU = kNaluIncomplete;
+          return -1;
+        }
+        ShiftSubsequentPackets(packet_it, all_nalu_size);
+        for (nalu_ptr = packet.dataPtr;
+             nalu_ptr < packet.dataPtr + packet.sizeBytes;
+             nalu_ptr += nalu_size + RtpFormatH264::kAggUnitLengthSize) {
+          nalu_size = (nalu_ptr[0] << 8) + nalu_ptr[1];
+          CopyWithStartCode(frame_buffer, nalu_ptr+2, nalu_size);
+          frame_buffer += nalu_size + RtpFormatH264::kStartCodeSize;
+        }
+        packet.sizeBytes = all_nalu_size;
+        packet.dataPtr = frame_buffer - all_nalu_size;
+        packet.completeNALU = kNaluComplete;
+        break;
+      default:
+        ShiftSubsequentPackets(packet_it, packet.sizeBytes +
+                               RtpFormatH264::kStartCodeSize);
+        CopyWithStartCode(frame_buffer, packet.dataPtr, packet.sizeBytes);
+        packet.sizeBytes += RtpFormatH264::kStartCodeSize;
+        packet.dataPtr = frame_buffer;
+        packet.completeNALU = kNaluComplete;
+        break;
+    } // switch nal_type
+  } else { // not H.264
+    ShiftSubsequentPackets(packet_it, packet.sizeBytes);
+    CopyPacket(frame_buffer, packet.dataPtr, packet.sizeBytes);
+    packet.dataPtr = frame_buffer;
   }
-  memcpy(const_cast<uint8_t*>(packet.dataPtr
-      + (packet.insertStartCode ? kH264StartCodeLengthBytes : 0)),
-      data,
-      packet.sizeBytes);
-
-  return packet_size;
+  return packet.sizeBytes;
 }
 
 void VCMSessionInfo::ShiftSubsequentPackets(PacketIterator it,
@@ -411,45 +476,41 @@ int VCMSessionInfo::InsertPacket(const VCMPacket& packet,
   // order and insert it. Loop over the list in reverse order.
   ReversePacketIterator rit = packets_.rbegin();
   for (; rit != packets_.rend(); ++rit)
-    if (LatestSequenceNumber(packet.seqNum, (*rit).seqNum) == packet.seqNum)
+    if (LatestSequenceNumber(packet.seqNum, (*rit).seqNum) == packet.seqNum) {
       break;
+    }
 
   // Check for duplicate packets.
   if (rit != packets_.rend() &&
-      (*rit).seqNum == packet.seqNum && (*rit).sizeBytes > 0)
+      (*rit).seqNum == packet.seqNum && (*rit).sizeBytes > 0) {
     return -2;
+  }
 
   PacketIterator packet_list_it;
+
   if (packet.codec == kVideoCodecH264) {
-    RTPVideoHeaderH264 h264 = packet.codecSpecificHeader.codecHeader.H264;
-    uint8_t nal_type = h264.nalu_header & RtpFormatH264::kH264NAL_TypeMask;
-
-    if (packet.isFirstPacket) {
-      if (HaveFirstPacket() == false ||
-          IsNewerSequenceNumber(first_packet_seq_num_, packet.seqNum)) {
-        first_packet_seq_num_ = packet.seqNum;
-        frame_type_ = packet.frameType;
-      }
+    // H.264 can have leading or trailing non-VCL (Video Coding Layer)
+    // NALUs, such as SPS/PPS/SEI and others.  Also, the RTP marker bit is
+    // not reliable for the last packet of a frame (RFC 6184 5.1 - "Decoders
+    // [] MUST NOT rely on this property"), so allow out-of-order packets to
+    // update the first and last seq# range.  Also mark as a key frame if
+    // any packet is of that type.
+    if (frame_type_ != kVideoFrameKey) {
+      frame_type_ = packet.frameType;
     }
-
-    // TODO(jesup) Handle STAP-A's here, since they must share a timestamp.  Break
-    // into individual packets at this point, then handle like kNaluCompletes
-
-    // Ignore Marker bit for reassembly, since it's not 100% guaranteed to be correct
-    // Look at kNaluComplete (single_nal), or an unbroken sequence of
-    // isFirstPacket/kNaluStart (FU-A with S bit), FU-A's, FU-A with E bit (kNaluEnd)
-    if ((packet.completeNALU == kNaluComplete || packet.completeNALU == kNaluEnd) &&
-        last_packet_seq_num_ == -1) {
-      last_packet_seq_num_ = static_cast<int>(packet.seqNum);
-    } else if (last_packet_seq_num_ != -1 &&
-      IsNewerSequenceNumber(packet.seqNum, last_packet_seq_num_)) {
-      //LOG(LS_WARNING) << "Received packet with a sequence number which is out "
-      //                 " of frame boundaries";
-      return -3;
+    if ((!HaveFirstPacket() ||
+        IsNewerSequenceNumber(first_packet_seq_num_, packet.seqNum)) &&
+        packet.isFirstPacket) {
+      first_packet_seq_num_ = packet.seqNum;
+    }
+    // Note: the code does *not* currently handle the Marker bit being totally
+    // absent from a frame.  It does not, however, depend on it being on the last
+    // packet of the 'frame'/'session'.
+    if ((!HaveLastPacket() && packet.markerBit) ||
+        (HaveLastPacket() &&
+        IsNewerSequenceNumber(packet.seqNum, last_packet_seq_num_))) {
+      last_packet_seq_num_ = packet.seqNum;
     }
-
-    // The insert operation invalidates the iterator |rit|.
-    packet_list_it = packets_.insert(rit.base(), packet);
   } else {
     // Only insert media packets between first and last packets (when available).
     // Placing check here, as to properly account for duplicate packets.
@@ -461,7 +522,7 @@ int VCMSessionInfo::InsertPacket(const VCMPacket& packet,
       // Store the sequence number for the first packet.
       first_packet_seq_num_ = static_cast<int>(packet.seqNum);
     } else if (first_packet_seq_num_ != -1 &&
-      !IsNewerSequenceNumber(packet.seqNum, first_packet_seq_num_)) {
+               !IsNewerSequenceNumber(packet.seqNum, first_packet_seq_num_)) {
       //LOG(LS_WARNING) << "Received packet with a sequence number which is out "
       //                 "of frame boundaries";
       return -3;
@@ -475,16 +536,16 @@ int VCMSessionInfo::InsertPacket(const VCMPacket& packet,
     if (packet.markerBit && last_packet_seq_num_ == -1) {
       last_packet_seq_num_ = static_cast<int>(packet.seqNum);
     } else if (last_packet_seq_num_ != -1 &&
-        IsNewerSequenceNumber(packet.seqNum, last_packet_seq_num_)) {
+               IsNewerSequenceNumber(packet.seqNum, last_packet_seq_num_)) {
       //LOG(LS_WARNING) << "Received packet with a sequence number which is out "
       //                 "of frame boundaries";
       return -3;
     }
-
-    // The insert operation invalidates the iterator |rit|.
-    packet_list_it = packets_.insert(rit.base(), packet);
   }
 
+  // The insert operation invalidates the iterator |rit|.
+  packet_list_it = packets_.insert(rit.base(), packet);
+
   int returnLength = InsertBuffer(frame_buffer, packet_list_it);
   UpdateCompleteSession();
   if (decode_error_mode == kWithErrors)
