@@ -124,6 +124,8 @@ public class GeckoApplication extends Application
         GeckoLoader.loadMozGlue(context);
         HomePanelsManager.getInstance().init(context);
 
+        // This getInstance call will force initialization of the NotificationHelper, but does nothing with the result
+        NotificationHelper.getInstance(context).init();
         super.onCreate();
     }
 
