@@ -5,9 +5,7 @@
 
 package org.mozilla.gecko;
 
-import org.mozilla.gecko.util.EventDispatcher;
 import org.mozilla.gecko.util.GeckoEventListener;
-import org.mozilla.gecko.util.GeckoEventResponder;
 
 import org.json.JSONException;
 import org.json.JSONObject;
@@ -67,7 +65,7 @@ class JavaAddonManager implements GeckoEventListener {
     }
 
     private JavaAddonManager() {
-        mDispatcher = GeckoAppShell.getEventDispatcher();
+        mDispatcher = EventDispatcher.getInstance();
         mAddonCallbacks = new HashMap<String, Map<String, GeckoEventListener>>();
     }
 
@@ -77,8 +75,9 @@ class JavaAddonManager implements GeckoEventListener {
             return;
         }
         mApplicationContext = applicationContext;
-        mDispatcher.registerEventListener("Dex:Load", this);
-        mDispatcher.registerEventListener("Dex:Unload", this);
+        mDispatcher.registerGeckoThreadListener(this,
+            "Dex:Load",
+            "Dex:Unload");
     }
 
     @Override
@@ -90,7 +89,7 @@ class JavaAddonManager implements GeckoEventListener {
                 Log.d(LOGTAG, "Attempting to load classes.dex file from " + zipFile + " and instantiate " + implClass);
                 try {
                     File tmpDir = mApplicationContext.getDir("dex", 0);
-                    DexClassLoader loader = new DexClassLoader(zipFile, tmpDir.getAbsolutePath(), null, ClassLoader.getSystemClassLoader());
+                    DexClassLoader loader = new DexClassLoader(zipFile, tmpDir.getAbsolutePath(), null, mApplicationContext.getClassLoader());
                     Class<?> c = loader.loadClass(implClass);
                     try {
                         Constructor<?> constructor = c.getDeclaredConstructor(Map.class);
@@ -123,7 +122,7 @@ class JavaAddonManager implements GeckoEventListener {
         addonCallbacks = new HashMap<String, GeckoEventListener>();
         for (String event : callbacks.keySet()) {
             CallbackWrapper wrapper = new CallbackWrapper(callbacks.get(event));
-            mDispatcher.registerEventListener(event, wrapper);
+            mDispatcher.registerGeckoThreadListener(wrapper, event);
             addonCallbacks.put(event, wrapper);
         }
         mAddonCallbacks.put(zipFile, addonCallbacks);
@@ -136,11 +135,11 @@ class JavaAddonManager implements GeckoEventListener {
             return;
         }
         for (String event : callbacks.keySet()) {
-            mDispatcher.unregisterEventListener(event, callbacks.get(event));
+            mDispatcher.unregisterGeckoThreadListener(callbacks.get(event), event);
         }
     }
 
-    private static class CallbackWrapper implements GeckoEventResponder {
+    private static class CallbackWrapper implements GeckoEventListener {
         private final Handler.Callback mDelegate;
         private Bundle mBundle;
 
@@ -184,16 +183,14 @@ class JavaAddonManager implements GeckoEventListener {
                 Message msg = new Message();
                 msg.setData(mBundle);
                 mDelegate.handleMessage(msg);
+
+                JSONObject obj = new JSONObject();
+                obj.put("response", mBundle.getString("response"));
+                EventDispatcher.sendResponse(json, obj);
+                mBundle = null;
             } catch (Exception e) {
                 Log.e(LOGTAG, "Caught exception thrown from wrapped addon message handler", e);
             }
         }
-
-        @Override
-        public String getResponse() {
-            String response = mBundle.getString("response");
-            mBundle = null;
-            return response;
-        }
     }
 }
