@@ -5,18 +5,24 @@
 
 package org.mozilla.gecko.db;
 
-import org.mozilla.gecko.db.BrowserContract.Bookmarks;
+import java.util.ArrayList;
+import java.util.EnumSet;
+import java.util.List;
+
 import org.mozilla.gecko.db.BrowserContract.ExpirePriority;
+import org.mozilla.gecko.db.SuggestedSites;
+import org.mozilla.gecko.distribution.Distribution;
+import org.mozilla.gecko.favicons.decoders.LoadFaviconResult;
+import org.mozilla.gecko.mozglue.RobocopTarget;
+import org.mozilla.gecko.util.StringUtils;
 
 import android.content.ContentResolver;
+import android.content.ContentValues;
+import android.content.Context;
 import android.database.ContentObserver;
 import android.database.Cursor;
-import android.database.CursorWrapper;
-import android.graphics.Bitmap;
 import android.graphics.drawable.BitmapDrawable;
-import android.util.SparseArray;
-
-import java.util.List;
+import android.graphics.Color;
 
 public class BrowserDB {
     private static boolean sAreContentProvidersEnabled = true;
@@ -31,15 +37,22 @@ public class BrowserDB {
         public static String KEYWORD = "keyword";
     }
 
+    public static enum FilterFlags {
+        EXCLUDE_PINNED_SITES
+    }
+
     private static BrowserDBIface sDb = null;
+    private static SuggestedSites sSuggestedSites;
 
     public interface BrowserDBIface {
         public void invalidateCachedState();
 
-        public Cursor filter(ContentResolver cr, CharSequence constraint, int limit);
+        @RobocopTarget
+        public Cursor filter(ContentResolver cr, CharSequence constraint, int limit,
+                             EnumSet<FilterFlags> flags);
 
-        // This should onlyl return frecent sites, BrowserDB.getTopSites will do the
-        // work to combine that list with the pinned sites list
+        // This should only return frecent sites. BrowserDB.getTopSites will do the
+        // work to combine that list with the pinned sites list.
         public Cursor getTopSites(ContentResolver cr, int limit);
 
         public void updateVisitedHistory(ContentResolver cr, String uri);
@@ -49,6 +62,7 @@ public class BrowserDB {
         public void updateHistoryEntry(ContentResolver cr, String uri, String title,
                                        long date, int visits);
 
+        @RobocopTarget
         public Cursor getAllVisitedHistory(ContentResolver cr);
 
         public Cursor getRecentHistory(ContentResolver cr, int limit);
@@ -57,46 +71,64 @@ public class BrowserDB {
 
         public void removeHistoryEntry(ContentResolver cr, int id);
 
+        @RobocopTarget
         public void removeHistoryEntry(ContentResolver cr, String url);
 
         public void clearHistory(ContentResolver cr);
 
+        @RobocopTarget
         public Cursor getBookmarksInFolder(ContentResolver cr, long folderId);
 
+        public Cursor getReadingList(ContentResolver cr);
+
         public boolean isVisited(ContentResolver cr, String uri);
 
         public int getReadingListCount(ContentResolver cr);
 
+        @RobocopTarget
         public boolean isBookmark(ContentResolver cr, String uri);
 
         public boolean isReadingListItem(ContentResolver cr, String uri);
 
+        /**
+         * Return a combination of fields about the provided URI
+         * in a single hit on the DB.
+         */
+        public int getItemFlags(ContentResolver cr, String uri);
+
         public String getUrlForKeyword(ContentResolver cr, String keyword);
 
+        @RobocopTarget
         public void addBookmark(ContentResolver cr, String title, String uri);
 
         public void removeBookmark(ContentResolver cr, int id);
 
+        @RobocopTarget
         public void removeBookmarksWithURL(ContentResolver cr, String uri);
 
+        @RobocopTarget
         public void updateBookmark(ContentResolver cr, int id, String uri, String title, String keyword);
 
-        public void addReadingListItem(ContentResolver cr, String title, String uri);
+        public void addReadingListItem(ContentResolver cr, ContentValues values);
 
         public void removeReadingListItemWithURL(ContentResolver cr, String uri);
 
-        public Bitmap getFaviconForUrl(ContentResolver cr, String uri);
+        public void removeReadingListItem(ContentResolver cr, int id);
+
+        public LoadFaviconResult getFaviconForUrl(ContentResolver cr, String uri);
 
         public String getFaviconUrlForHistoryUrl(ContentResolver cr, String url);
 
-        public void updateFaviconForUrl(ContentResolver cr, String pageUri, Bitmap favicon, String faviconUri);
+        public void updateFaviconForUrl(ContentResolver cr, String pageUri, byte[] encodedFavicon, String faviconUri);
 
         public void updateThumbnailForUrl(ContentResolver cr, String uri, BitmapDrawable thumbnail);
 
+        @RobocopTarget
         public byte[] getThumbnailForUrl(ContentResolver cr, String uri);
 
         public Cursor getThumbnailsForUrls(ContentResolver cr, List<String> urls);
 
+        @RobocopTarget
         public void removeThumbnails(ContentResolver cr);
 
         public void registerBookmarkObserver(ContentResolver cr, ContentObserver observer);
@@ -113,7 +145,11 @@ public class BrowserDB {
 
         public Cursor getPinnedSites(ContentResolver cr, int limit);
 
+        @RobocopTarget
         public Cursor getBookmarkForUrl(ContentResolver cr, String url);
+
+        public int addDefaultBookmarks(Context context, ContentResolver cr, int offset);
+        public int addDistributionBookmarks(ContentResolver cr, Distribution distribution, int offset);
     }
 
     static {
@@ -125,20 +161,74 @@ public class BrowserDB {
         sDb = new LocalBrowserDB(profile);
     }
 
+    public static int addDefaultBookmarks(Context context, ContentResolver cr, final int offset) {
+        return sDb.addDefaultBookmarks(context, cr, offset);
+    }
+
+    public static int addDistributionBookmarks(ContentResolver cr, Distribution distribution, int offset) {
+        return sDb.addDistributionBookmarks(cr, distribution, offset);
+    }
+
+    public static void setSuggestedSites(SuggestedSites suggestedSites) {
+        sSuggestedSites = suggestedSites;
+    }
+
+    public static boolean hideSuggestedSite(String url) {
+        return sSuggestedSites.hideSite(url);
+    }
+
     public static void invalidateCachedState() {
         sDb.invalidateCachedState();
     }
 
+    @RobocopTarget
     public static Cursor filter(ContentResolver cr, CharSequence constraint, int limit) {
-        return sDb.filter(cr, constraint, limit);
+        return filter(cr, constraint, limit, EnumSet.noneOf(FilterFlags.class));
+    }
+
+    @RobocopTarget
+    public static Cursor filter(ContentResolver cr, CharSequence constraint, int limit,
+                                EnumSet<FilterFlags> flags) {
+        return sDb.filter(cr, constraint, limit, flags);
+    }
+
+    private static void appendUrlsFromCursor(List<String> urls, Cursor c) {
+        c.moveToPosition(-1);
+        while (c.moveToNext()) {
+            String url = c.getString(c.getColumnIndex(URLColumns.URL));
+
+            // Do a simpler check before decoding to avoid parsing
+            // all URLs unnecessarily.
+            if (StringUtils.isUserEnteredUrl(url)) {
+                url = StringUtils.decodeUserEnteredUrl(url);
+            }
+
+            urls.add(url);
+        };
     }
 
     public static Cursor getTopSites(ContentResolver cr, int minLimit, int maxLimit) {
         // Note this is not a single query anymore, but actually returns a mixture
         // of two queries, one for topSites and one for pinned sites.
         Cursor pinnedSites = sDb.getPinnedSites(cr, minLimit);
-        Cursor topSites = sDb.getTopSites(cr, maxLimit - pinnedSites.getCount());
-        return new TopSitesCursorWrapper(pinnedSites, topSites, minLimit);
+
+        int pinnedCount = pinnedSites.getCount();
+        Cursor topSites = sDb.getTopSites(cr, maxLimit - pinnedCount);
+        int topCount = topSites.getCount();
+
+        Cursor suggestedSites = null;
+        if (sSuggestedSites != null) {
+            final int count = minLimit - pinnedCount - topCount;
+            if (count > 0) {
+                final List<String> excludeUrls = new ArrayList<String>(pinnedCount + topCount);
+                appendUrlsFromCursor(excludeUrls, pinnedSites);
+                appendUrlsFromCursor(excludeUrls, topSites);
+
+                suggestedSites = sSuggestedSites.get(count, excludeUrls);
+            }
+        }
+
+        return new TopSitesCursorWrapper(pinnedSites, topSites, suggestedSites, minLimit);
     }
 
     public static void updateVisitedHistory(ContentResolver cr, String uri) {
@@ -160,6 +250,7 @@ public class BrowserDB {
         }
     }
 
+    @RobocopTarget
     public static Cursor getAllVisitedHistory(ContentResolver cr) {
         return (sAreContentProvidersEnabled ? sDb.getAllVisitedHistory(cr) : null);
     }
@@ -181,18 +272,26 @@ public class BrowserDB {
         sDb.removeHistoryEntry(cr, id);
     }
 
+    @RobocopTarget
     public static void removeHistoryEntry(ContentResolver cr, String url) {
         sDb.removeHistoryEntry(cr, url);
     }
 
+    @RobocopTarget
     public static void clearHistory(ContentResolver cr) {
         sDb.clearHistory(cr);
     }
 
+    @RobocopTarget
     public static Cursor getBookmarksInFolder(ContentResolver cr, long folderId) {
         return sDb.getBookmarksInFolder(cr, folderId);
     }
 
+    @RobocopTarget
+    public static Cursor getReadingList(ContentResolver cr) {
+        return sDb.getReadingList(cr);
+    }
+
     public static String getUrlForKeyword(ContentResolver cr, String keyword) {
         return sDb.getUrlForKeyword(cr, keyword);
     }
@@ -205,6 +304,7 @@ public class BrowserDB {
         return sDb.getReadingListCount(cr);
     }
 
+    @RobocopTarget
     public static boolean isBookmark(ContentResolver cr, String uri) {
         return (sAreContentProvidersEnabled && sDb.isBookmark(cr, uri));
     }
@@ -213,6 +313,13 @@ public class BrowserDB {
         return (sAreContentProvidersEnabled && sDb.isReadingListItem(cr, uri));
     }
 
+    public static int getItemFlags(ContentResolver cr, String uri) {
+        if (!sAreContentProvidersEnabled) {
+            return 0;
+        }
+        return sDb.getItemFlags(cr, uri);
+    }
+
     public static void addBookmark(ContentResolver cr, String title, String uri) {
         sDb.addBookmark(cr, title, uri);
     }
@@ -221,23 +328,29 @@ public class BrowserDB {
         sDb.removeBookmark(cr, id);
     }
 
+    @RobocopTarget
     public static void removeBookmarksWithURL(ContentResolver cr, String uri) {
         sDb.removeBookmarksWithURL(cr, uri);
     }
 
+    @RobocopTarget
     public static void updateBookmark(ContentResolver cr, int id, String uri, String title, String keyword) {
         sDb.updateBookmark(cr, id, uri, title, keyword);
     }
 
-    public static void addReadingListItem(ContentResolver cr, String title, String uri) {
-        sDb.addReadingListItem(cr, title, uri);
+    public static void addReadingListItem(ContentResolver cr, ContentValues values) {
+        sDb.addReadingListItem(cr, values);
     }
 
     public static void removeReadingListItemWithURL(ContentResolver cr, String uri) {
         sDb.removeReadingListItemWithURL(cr, uri);
     }
 
-    public static Bitmap getFaviconForFaviconUrl(ContentResolver cr, String faviconURL) {
+    public static void removeReadingListItem(ContentResolver cr, int id) {
+        sDb.removeReadingListItem(cr, id);
+    }
+
+    public static LoadFaviconResult getFaviconForFaviconUrl(ContentResolver cr, String faviconURL) {
         return sDb.getFaviconForUrl(cr, faviconURL);
     }
 
@@ -245,14 +358,15 @@ public class BrowserDB {
         return sDb.getFaviconUrlForHistoryUrl(cr, url);
     }
 
-    public static void updateFaviconForUrl(ContentResolver cr, String pageUri, Bitmap favicon, String faviconUri) {
-        sDb.updateFaviconForUrl(cr, pageUri, favicon, faviconUri);
+    public static void updateFaviconForUrl(ContentResolver cr, String pageUri, byte[] encodedFavicon, String faviconUri) {
+        sDb.updateFaviconForUrl(cr, pageUri, encodedFavicon, faviconUri);
     }
 
     public static void updateThumbnailForUrl(ContentResolver cr, String uri, BitmapDrawable thumbnail) {
         sDb.updateThumbnailForUrl(cr, uri, thumbnail);
     }
 
+    @RobocopTarget
     public static byte[] getThumbnailForUrl(ContentResolver cr, String uri) {
         return sDb.getThumbnailForUrl(cr, uri);
     }
@@ -261,6 +375,7 @@ public class BrowserDB {
         return sDb.getThumbnailsForUrls(cr, urls);
     }
 
+    @RobocopTarget
     public static void removeThumbnails(ContentResolver cr) {
         sDb.removeThumbnails(cr);
     }
@@ -297,6 +412,7 @@ public class BrowserDB {
         return sDb.getPinnedSites(cr, limit);
     }
 
+    @RobocopTarget
     public static Cursor getBookmarkForUrl(ContentResolver cr, String url) {
         return sDb.getBookmarkForUrl(cr, url);
     }
@@ -309,182 +425,20 @@ public class BrowserDB {
         sAreContentProvidersEnabled = enableContentProviders;
     }
 
-    public static class PinnedSite {
-        public String title = "";
-        public String url = "";
-
-        public PinnedSite(String aTitle, String aUrl) {
-            title = aTitle;
-            url = aUrl;
-        }
+    public static boolean hasSuggestedImageUrl(String url) {
+        return sSuggestedSites.contains(url);
     }
 
-    /* Cursor wrapper that forces top sites to contain at least
-     * mNumberOfTopSites entries. For rows outside the wrapped cursor
-     * will return empty strings and zero.
-     */
-    public static class TopSitesCursorWrapper extends CursorWrapper {
-        int mIndex = -1; // Current position of the cursor
-        Cursor mCursor = null;
-        int mSize = 0;
-        private SparseArray<PinnedSite> mPinnedSites = null;
+    public static String getSuggestedImageUrlForUrl(String url) {
+        return sSuggestedSites.getImageUrlForUrl(url);
+    }
 
-        public TopSitesCursorWrapper(Cursor pinnedCursor, Cursor normalCursor, int minSize) {
-            super(normalCursor);
-
-            setPinnedSites(pinnedCursor);
-            mCursor = normalCursor;
-            mSize = Math.max(minSize, mPinnedSites.size() + mCursor.getCount());
+    public static int getSuggestedBackgroundColorForUrl(String url) {
+        final String bgColor = sSuggestedSites.getBackgroundColorForUrl(url);
+        if (bgColor != null) {
+            return Color.parseColor(bgColor);
         }
 
-        public void setPinnedSites(Cursor c) {
-            mPinnedSites = new SparseArray<PinnedSite>();
-
-            if (c == null) {
-                return;
-            }
-
-            try {
-                if (c.getCount() <= 0) {
-                    return;
-                }
-                c.moveToPosition(0);
-                do {
-                    int pos = c.getInt(c.getColumnIndex(Bookmarks.POSITION));
-                    String url = c.getString(c.getColumnIndex(URLColumns.URL));
-                    String title = c.getString(c.getColumnIndex(URLColumns.TITLE));
-                    mPinnedSites.put(pos, new PinnedSite(title, url));
-                } while (c.moveToNext());
-            } finally {
-                c.close();
-            }
-        }
-
-        public boolean hasPinnedSites() {
-            return mPinnedSites != null && mPinnedSites.size() > 0;
-        }
-
-        public PinnedSite getPinnedSite(int position) {
-            if (!hasPinnedSites()) {
-                return null;
-            }
-            return mPinnedSites.get(position);
-        }
-
-        public boolean isPinned() {
-            return mPinnedSites.get(mIndex) != null;
-        }
-
-        private int getPinnedBefore(int position) {
-            int numFound = 0;
-            if (!hasPinnedSites()) {
-                return numFound;
-            }
-
-            for (int i = 0; i < position; i++) {
-                if (mPinnedSites.get(i) != null) {
-                    numFound++;
-                }
-            }
-
-            return numFound;
-        }
-
-        @Override
-        public int getPosition() { return mIndex; }
-        @Override
-        public int getCount() { return mSize; }
-        @Override
-        public boolean isAfterLast() { return mIndex >= mSize; }
-        @Override
-        public boolean isBeforeFirst() { return mIndex < 0; }
-        @Override
-        public boolean isLast() { return mIndex == mSize - 1; }
-        @Override
-        public boolean moveToNext() { return moveToPosition(mIndex + 1); }
-        @Override
-        public boolean moveToPrevious() { return moveToPosition(mIndex - 1); }
-
-        @Override
-        public boolean moveToPosition(int position) {
-            mIndex = position;
-
-            // Move the real cursor as if we were stepping through it to this position.
-            // Account for pinned sites, and be careful to update its position to the
-            // minimum or maximum position, even if we're moving beyond its bounds.
-            int before = getPinnedBefore(position);
-            int p2 = position - before;
-            if (p2 <= -1) {
-                super.moveToPosition(-1);
-            } else if (p2 >= mCursor.getCount()) {
-                super.moveToPosition(mCursor.getCount());
-            } else {
-                super.moveToPosition(p2);
-            }
-
-            return !(isBeforeFirst() || isAfterLast());
-        }
-
-        @Override
-        public long getLong(int columnIndex) {
-            if (hasPinnedSites()) {
-                PinnedSite site = getPinnedSite(mIndex);
-                if (site != null) {
-                    return 0;
-                }
-            }
-
-            if (!super.isBeforeFirst() && !super.isAfterLast())
-                return super.getLong(columnIndex);
-            return 0;
-        }
-
-        @Override
-        public int getInt(int columnIndex) {
-            if (hasPinnedSites()) {
-                PinnedSite site = getPinnedSite(mIndex);
-                if (site != null) {
-                    return 0;
-                }
-            }
-
-            if (!super.isBeforeFirst() && !super.isAfterLast())
-                return super.getInt(columnIndex);
-            return 0;
-        }
-
-        @Override
-        public String getString(int columnIndex) {
-            if (hasPinnedSites()) {
-                PinnedSite site = getPinnedSite(mIndex);
-                if (site != null) {
-                    if (columnIndex == mCursor.getColumnIndex(URLColumns.URL)) {
-                        return site.url;
-                    } else if (columnIndex == mCursor.getColumnIndex(URLColumns.TITLE)) {
-                        return site.title;
-                    }
-                    return "";
-                }
-            }
-
-            if (!super.isBeforeFirst() && !super.isAfterLast())
-                return super.getString(columnIndex);
-            return "";
-        }
-
-        @Override
-        public boolean move(int offset) {
-            return moveToPosition(mIndex + offset);
-        }
-
-        @Override
-        public boolean moveToFirst() {
-            return moveToPosition(0);
-        }
-
-        @Override
-        public boolean moveToLast() {
-            return moveToPosition(mSize-1);
-        }
+        return 0;
     }
 }
