@@ -6,15 +6,24 @@
 package org.mozilla.gecko.home;
 
 import com.squareup.picasso.Picasso;
+import com.squareup.picasso.Callback;
 
 import org.mozilla.gecko.db.BrowserContract.TopSites;
+import org.mozilla.gecko.db.URLMetadata;
 import org.mozilla.gecko.favicons.Favicons;
+import org.mozilla.gecko.gfx.BitmapUtils;
 import org.mozilla.gecko.R;
+import org.mozilla.gecko.util.ThreadUtils;
+import org.mozilla.gecko.util.UiAsyncTask;
 
 import android.content.Context;
+import android.content.ContentResolver;
 import android.graphics.Bitmap;
+import android.graphics.drawable.BitmapDrawable;
+import android.support.v4.content.AsyncTaskLoader;
 import android.text.TextUtils;
 import android.util.AttributeSet;
+import android.util.Log;
 import android.view.LayoutInflater;
 import android.widget.ImageView;
 import android.widget.ImageView.ScaleType;
@@ -36,6 +45,7 @@ public class TopSitesGridItemView extends RelativeLayout {
     private static final ScaleType SCALE_TYPE_FAVICON   = ScaleType.CENTER;
     private static final ScaleType SCALE_TYPE_RESOURCE  = ScaleType.CENTER;
     private static final ScaleType SCALE_TYPE_THUMBNAIL = ScaleType.CENTER_CROP;
+    private static final ScaleType SCALE_TYPE_URL       = ScaleType.CENTER_INSIDE;
 
     // Child views.
     private final TextView mTitleView;
@@ -139,7 +149,7 @@ public class TopSitesGridItemView extends RelativeLayout {
         updateType(TopSites.TYPE_BLANK);
         updateTitleView();
         setLoadId(Favicons.NOT_LOADING);
-        Picasso.with(getContext()).cancelRequest(mThumbnailView);
+        ImageLoader.with(getContext()).cancelRequest(mThumbnailView);
         displayThumbnail(R.drawable.top_site_add);
 
     }
@@ -155,7 +165,7 @@ public class TopSitesGridItemView extends RelativeLayout {
      *
      * Returns true if any fields changed.
      */
-    public boolean updateState(final String title, final String url, final int type, final Bitmap thumbnail) {
+    public boolean updateState(final String title, final String url, final int type, final TopSitesPanel.ThumbnailInfo thumbnail) {
         boolean changed = false;
         if (mUrl == null || !mUrl.equals(url)) {
             mUrl = url;
@@ -168,7 +178,11 @@ public class TopSitesGridItemView extends RelativeLayout {
         }
 
         if (thumbnail != null) {
-            displayThumbnail(thumbnail);
+            if (thumbnail.imageUrl != null) {
+                displayThumbnail(thumbnail.imageUrl, thumbnail.bgColor);
+            } else if (thumbnail.bitmap != null) {
+                displayThumbnail(thumbnail.bitmap);
+            }
         } else if (changed) {
             // Because we'll have a new favicon or thumbnail arriving shortly, and
             // we need to not reject it because we already had a thumbnail.
@@ -178,7 +192,7 @@ public class TopSitesGridItemView extends RelativeLayout {
         if (changed) {
             updateTitleView();
             setLoadId(Favicons.NOT_LOADING);
-            Picasso.with(getContext()).cancelRequest(mThumbnailView);
+            ImageLoader.with(getContext()).cancelRequest(mThumbnailView);
         }
 
         if (updateType(type)) {
@@ -219,7 +233,7 @@ public class TopSitesGridItemView extends RelativeLayout {
         }
         mThumbnailSet = true;
         Favicons.cancelFaviconLoad(mLoadId);
-        Picasso.with(getContext()).cancelRequest(mThumbnailView);
+        ImageLoader.with(getContext()).cancelRequest(mThumbnailView);
 
         mThumbnailView.setScaleType(SCALE_TYPE_THUMBNAIL);
         mThumbnailView.setImageBitmap(thumbnail);
@@ -232,16 +246,16 @@ public class TopSitesGridItemView extends RelativeLayout {
      * @param imageUrl URL of the image to show.
      * @param bgColor background color to use in the view.
      */
-    public void displayThumbnail(String imageUrl, int bgColor) {
-        mThumbnailView.setScaleType(SCALE_TYPE_RESOURCE);
+    public void displayThumbnail(final String imageUrl, final int bgColor) {
+        mThumbnailView.setScaleType(SCALE_TYPE_URL);
         mThumbnailView.setBackgroundColor(bgColor);
         mThumbnailSet = true;
 
-        Picasso.with(getContext())
-               .load(imageUrl)
-               .noFade()
-               .error(R.drawable.favicon)
-               .into(mThumbnailView);
+        ImageLoader.with(getContext())
+                   .load(imageUrl)
+                   .noFade()
+                   .error(R.drawable.favicon)
+                   .into(mThumbnailView);
     }
 
     public void displayFavicon(Bitmap favicon, String faviconURL, int expectedLoadId) {
