@@ -30,7 +30,7 @@ import android.widget.TextView;
 public class TwoLinePageRow extends LinearLayout
                             implements Tabs.OnTabsChangedListener {
 
-    private static final int NO_ICON = 0;
+    protected static final int NO_ICON = 0;
 
     private final TextView mTitle;
     private final TextView mUrl;
@@ -122,15 +122,19 @@ public class TwoLinePageRow extends LinearLayout
         mTitle.setText(text);
     }
 
-    private void setUrl(String text) {
+    protected void setUrl(String text) {
         mUrl.setText(text);
     }
 
-    private void setUrl(int stringId) {
+    protected void setUrl(int stringId) {
         mUrl.setText(stringId);
     }
 
-    private void setSwitchToTabIcon(int iconId) {
+    protected String getUrl() {
+        return mPageUrl;
+    }
+
+    protected void setSwitchToTabIcon(int iconId) {
         if (mSwitchToTabIconId == iconId) {
             return;
         }
@@ -159,10 +163,10 @@ public class TwoLinePageRow extends LinearLayout
 
     /**
      * Replaces the page URL with "Switch to tab" if there is already a tab open with that URL.
-     * Only looks for tabs that are either private or non-private, depending on the current 
+     * Only looks for tabs that are either private or non-private, depending on the current
      * selected tab.
      */
-    private void updateDisplayedUrl() {
+    protected void updateDisplayedUrl() {
         boolean isPrivate = Tabs.getInstance().getSelectedTab().isPrivate();
         Tab tab = Tabs.getInstance().getFirstTabForUrl(mPageUrl, isPrivate);
         if (!mShowIcons || tab == null) {
@@ -193,21 +197,11 @@ public class TwoLinePageRow extends LinearLayout
             final int bookmarkIdIndex = cursor.getColumnIndex(Combined.BOOKMARK_ID);
             if (bookmarkIdIndex != -1) {
                 final long bookmarkId = cursor.getLong(bookmarkIdIndex);
-                final int displayIndex = cursor.getColumnIndex(Combined.DISPLAY);
-
-                final int display;
-                if (displayIndex != -1) {
-                    display = cursor.getInt(displayIndex);
-                } else {
-                    display = Combined.DISPLAY_NORMAL;
-                }
 
                 // The bookmark id will be 0 (null in database) when the url
                 // is not a bookmark.
                 if (bookmarkId == 0) {
                     setPageTypeIcon(NO_ICON);
-                } else if (display == Combined.DISPLAY_READER) {
-                    setPageTypeIcon(R.drawable.ic_url_bar_reader);
                 } else {
                     setPageTypeIcon(R.drawable.ic_url_bar_star);
                 }
