@@ -1,8 +1,7 @@
 package org.mozilla.gecko.tests;
 
-import static org.mozilla.gecko.tests.helpers.AssertionHelper.*;
-
-import org.mozilla.gecko.tests.helpers.*;
+import org.mozilla.gecko.tests.helpers.GeckoHelper;
+import org.mozilla.gecko.tests.helpers.NavigationHelper;
 
 /**
  * Tests that navigating through session history (ex: forward, back) sets the correct UI state.
@@ -29,10 +28,7 @@ public class testSessionHistory extends UITest {
         NavigationHelper.goForward();
         mToolbar.assertTitle(StringHelper.ROBOCOP_BLANK_PAGE_02_TITLE);
 
-        // TODO: Implement this functionality and uncomment.
-        /*
         NavigationHelper.reload();
         mToolbar.assertTitle(StringHelper.ROBOCOP_BLANK_PAGE_02_TITLE);
-        */
     }
 }
