@@ -3,35 +3,33 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-
-#ifndef nsJAR_h__
-#define nsJAR_h__
+#ifndef nsJAR_h_
+#define nsJAR_h_
 
 #include "nscore.h"
-#include "pratom.h"
-#include "prmem.h"
 #include "prio.h"
 #include "plstr.h"
 #include "prlog.h"
-#include "prtypes.h"
 #include "prinrval.h"
 
 #include "mozilla/Mutex.h"
 #include "nsIComponentManager.h"
 #include "nsCOMPtr.h"
+#include "nsClassHashtable.h"
 #include "nsString.h"
 #include "nsIFile.h"
 #include "nsStringEnumerator.h"
-#include "nsHashtable.h"
+#include "nsHashKeys.h"
+#include "nsRefPtrHashtable.h"
+#include "nsTHashtable.h"
 #include "nsIZipReader.h"
 #include "nsZipArchive.h"
-#include "nsICertificatePrincipal.h"
-#include "nsISignatureVerifier.h"
 #include "nsIObserverService.h"
 #include "nsWeakReference.h"
 #include "nsIObserver.h"
 #include "mozilla/Attributes.h"
 
+class nsICertificatePrincipal;
 class nsIInputStream;
 class nsJARManifestItem;
 class nsZipReaderCache;
@@ -50,9 +48,9 @@ typedef enum
 } JARManifestStatusType;
 
 /*-------------------------------------------------------------------------
- * Class nsJAR declaration. 
- * nsJAR serves as an XPCOM wrapper for nsZipArchive with the addition of 
- * JAR manifest file parsing. 
+ * Class nsJAR declaration.
+ * nsJAR serves as an XPCOM wrapper for nsZipArchive with the addition of
+ * JAR manifest file parsing.
  *------------------------------------------------------------------------*/
 class nsJAR : public nsIZipReader
 {
@@ -61,14 +59,17 @@ class nsJAR : public nsIZipReader
   // Allows nsZipReaderCache to access mOuterZipEntry
   friend class nsZipReaderCache;
 
+  private:
+
+    virtual ~nsJAR();
+
   public:
 
     nsJAR();
-    virtual ~nsJAR();
-    
+
     NS_DEFINE_STATIC_CID_ACCESSOR( NS_ZIPREADER_CID )
-  
-    NS_DECL_ISUPPORTS
+
+    NS_DECL_THREADSAFE_ISUPPORTS
 
     NS_DECL_NSIZIPREADER
 
@@ -77,7 +78,7 @@ class nsJAR : public nsIZipReader
     PRIntervalTime GetReleaseTime() {
         return mReleaseTime;
     }
-    
+
     bool IsReleased() {
         return mReleaseTime != PR_INTERVAL_NO_TIMEOUT;
     }
@@ -85,38 +86,42 @@ class nsJAR : public nsIZipReader
     void SetReleaseTime() {
       mReleaseTime = PR_IntervalNow();
     }
-    
+
     void ClearReleaseTime() {
       mReleaseTime = PR_INTERVAL_NO_TIMEOUT;
     }
-    
+
     void SetZipReaderCache(nsZipReaderCache* cache) {
       mCache = cache;
     }
 
+    nsresult GetNSPRFileDesc(PRFileDesc** aNSPRFileDesc);
+
   protected:
+    typedef nsClassHashtable<nsCStringHashKey, nsJARManifestItem> ManifestDataHashtable;
+
     //-- Private data members
     nsCOMPtr<nsIFile>        mZipFile;        // The zip/jar file on disk
     nsCString                mOuterZipEntry;  // The entry in the zip this zip is reading from
     nsRefPtr<nsZipArchive>   mZip;            // The underlying zip archive
-    nsObjectHashtable        mManifestData;   // Stores metadata for each entry
+    ManifestDataHashtable    mManifestData;   // Stores metadata for each entry
     bool                     mParsedManifest; // True if manifest has been parsed
     nsCOMPtr<nsICertificatePrincipal> mPrincipal; // The entity which signed this file
     int16_t                  mGlobalStatus;   // Global signature verification status
     PRIntervalTime           mReleaseTime;    // used by nsZipReaderCache for flushing entries
     nsZipReaderCache*        mCache;          // if cached, this points to the cache it's contained in
-    mozilla::Mutex           mLock;	
+    mozilla::Mutex           mLock;
     int64_t                  mMtime;
     int32_t                  mTotalItemsInManifest;
     bool                     mOpened;
 
     nsresult ParseManifest();
     void     ReportError(const nsACString &aFilename, int16_t errorCode);
-    nsresult LoadEntry(const nsACString &aFilename, char** aBuf, 
+    nsresult LoadEntry(const nsACString &aFilename, char** aBuf,
                        uint32_t* aBufLen = nullptr);
-    int32_t  ReadLine(const char** src); 
+    int32_t  ReadLine(const char** src);
     nsresult ParseOneFile(const char* filebuf, int16_t aFileType);
-    nsresult VerifyEntry(nsJARManifestItem* aEntry, const char* aEntryData, 
+    nsresult VerifyEntry(nsJARManifestItem* aEntry, const char* aEntryData,
                          uint32_t aLen);
 
     nsresult CalculateDigest(const char* aInBuf, uint32_t aInBufLen,
@@ -126,41 +131,43 @@ class nsJAR : public nsIZipReader
 /**
  * nsJARItem
  *
- * An individual JAR entry. A set of nsJARItems macthing a
+ * An individual JAR entry. A set of nsJARItems matching a
  * supplied pattern are returned in a nsJAREnumerator.
  */
 class nsJARItem : public nsIZipEntry
 {
 public:
-    NS_DECL_ISUPPORTS
+    NS_DECL_THREADSAFE_ISUPPORTS
     NS_DECL_NSIZIPENTRY
-    
+
     nsJARItem(nsZipItem* aZipItem);
-    virtual ~nsJARItem() {}
 
 private:
+    virtual ~nsJARItem() {}
+
     uint32_t     mSize;             /* size in original file */
     uint32_t     mRealsize;         /* inflated size */
     uint32_t     mCrc32;
     PRTime       mLastModTime;
     uint16_t     mCompression;
-    bool mIsDirectory; 
+    uint32_t     mPermissions;
+    bool mIsDirectory;
     bool mIsSynthetic;
 };
 
 /**
  * nsJAREnumerator
  *
- * Enumerates a list of files in a zip archive 
+ * Enumerates a list of files in a zip archive
  * (based on a pattern match in its member nsZipFind).
  */
 class nsJAREnumerator MOZ_FINAL : public nsIUTF8StringEnumerator
 {
 public:
-    NS_DECL_ISUPPORTS
+    NS_DECL_THREADSAFE_ISUPPORTS
     NS_DECL_NSIUTF8STRINGENUMERATOR
 
-    nsJAREnumerator(nsZipFind *aFind) : mFind(aFind), mName(nullptr) { 
+    nsJAREnumerator(nsZipFind *aFind) : mFind(aFind), mName(nullptr) {
       NS_ASSERTION(mFind, "nsJAREnumerator: Missing zipFind.");
     }
 
@@ -182,19 +189,28 @@ class nsZipReaderCache : public nsIZipReaderCache, public nsIObserver,
                          public nsSupportsWeakReference
 {
 public:
-  NS_DECL_ISUPPORTS
+  NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSIZIPREADERCACHE
   NS_DECL_NSIOBSERVER
 
   nsZipReaderCache();
-  virtual ~nsZipReaderCache();
 
   nsresult ReleaseZip(nsJAR* reader);
 
+  bool IsMustCacheFdEnabled() {
+    return mMustCacheFd;
+  }
+
+  typedef nsRefPtrHashtable<nsCStringHashKey, nsJAR> ZipsHashtable;
+
 protected:
+
+  virtual ~nsZipReaderCache();
+
   mozilla::Mutex        mLock;
-  int32_t               mCacheSize;
-  nsSupportsHashtable   mZips;
+  uint32_t              mCacheSize;
+  ZipsHashtable         mZips;
+  bool                  mMustCacheFd;
 
 #ifdef ZIP_CACHE_HIT_RATE
   uint32_t              mZipCacheLookups;
@@ -207,4 +223,4 @@ protected:
 
 ////////////////////////////////////////////////////////////////////////////////
 
-#endif /* nsJAR_h__ */
+#endif /* nsJAR_h_ */
