@@ -24,6 +24,7 @@
 class nsPACMan;
 class nsISystemProxySettings;
 class nsIThread;
+class WaitForThreadShutdown;
 
 /**
  * This class defines a callback interface used by AsyncGetProxyForURI.
@@ -164,6 +165,7 @@ private:
   friend class PendingPACQuery;
   friend class PACLoadComplete;
   friend class ExecutePACThreadAction;
+  friend class WaitForThreadShutdown;
 
   ~nsPACMan();
 
