@@ -43,6 +43,7 @@ HTML5_ATOM(nobr, "nobr")
 HTML5_ATOM(input, "input")
 HTML5_ATOM(option, "option")
 HTML5_ATOM(ruby, "ruby")
+HTML5_ATOM(rtc, "rtc")
 HTML5_ATOM(select, "select")
 HTML5_ATOM(optgroup, "optgroup")
 HTML5_ATOM(tbody, "tbody")
@@ -181,6 +182,7 @@ HTML5_ATOM(scale, "scale")
 HTML5_ATOM(speed, "speed")
 HTML5_ATOM(rules, "rules")
 HTML5_ATOM(stemh, "stemh")
+HTML5_ATOM(sizes, "sizes")
 HTML5_ATOM(stemv, "stemv")
 HTML5_ATOM(start, "start")
 HTML5_ATOM(accept, "accept")
@@ -234,6 +236,7 @@ HTML5_ATOM(stroke, "stroke")
 HTML5_ATOM(radius, "radius")
 HTML5_ATOM(result, "result")
 HTML5_ATOM(repeat, "repeat")
+HTML5_ATOM(srcset, "srcset")
 HTML5_ATOM(rspace, "rspace")
 HTML5_ATOM(rotate, "rotate")
 HTML5_ATOM(rquote, "rquote")
@@ -363,6 +366,7 @@ HTML5_ATOM(filterRes, "filterRes")
 HTML5_ATOM(fill_rule, "fill-rule")
 HTML5_ATOM(fontstyle, "fontstyle")
 HTML5_ATOM(font_size, "font-size")
+HTML5_ATOM(keysystem, "keysystem")
 HTML5_ATOM(keypoints, "keypoints")
 HTML5_ATOM(keyPoints, "keyPoints")
 HTML5_ATOM(hidefocus, "hidefocus")
@@ -738,6 +742,7 @@ HTML5_ATOM(mo, "mo")
 HTML5_ATOM(ms, "ms")
 HTML5_ATOM(or_, "or")
 HTML5_ATOM(pi, "pi")
+HTML5_ATOM(rb, "rb")
 HTML5_ATOM(rp, "rp")
 HTML5_ATOM(tt, "tt")
 HTML5_ATOM(and_, "and")
@@ -907,7 +912,6 @@ HTML5_ATOM(arccosh, "arccosh")
 HTML5_ATOM(arccoth, "arccoth")
 HTML5_ATOM(acronym, "acronym")
 HTML5_ATOM(bgsound, "bgsound")
-HTML5_ATOM(command, "command")
 HTML5_ATOM(compose, "compose")
 HTML5_ATOM(ceiling, "ceiling")
 HTML5_ATOM(csymbol, "csymbol")
@@ -942,6 +946,7 @@ HTML5_ATOM(marquee, "marquee")
 HTML5_ATOM(maction, "maction")
 HTML5_ATOM(msubsup, "msubsup")
 HTML5_ATOM(polygon, "polygon")
+HTML5_ATOM(picture, "picture")
 HTML5_ATOM(product, "product")
 HTML5_ATOM(setdiff, "setdiff")
 HTML5_ATOM(section, "section")
@@ -1028,6 +1033,8 @@ HTML5_ATOM(feturbulence, "feturbulence")
 HTML5_ATOM(feTurbulence, "feTurbulence")
 HTML5_ATOM(fepointlight, "fepointlight")
 HTML5_ATOM(fePointLight, "fePointLight")
+HTML5_ATOM(fedropshadow, "fedropshadow")
+HTML5_ATOM(feDropShadow, "feDropShadow")
 HTML5_ATOM(femorphology, "femorphology")
 HTML5_ATOM(feMorphology, "feMorphology")
 HTML5_ATOM(outerproduct, "outerproduct")
