@@ -168,12 +168,13 @@ typedef struct nsKeyPairInfoStr {
 class nsCryptoRunArgs : public nsISupports {
 public:
   nsCryptoRunArgs(JSContext *aCx);
-  virtual ~nsCryptoRunArgs();
   nsCOMPtr<nsISupports> m_kungFuDeathGrip;
   JSContext *m_cx;
   JS::PersistentRooted<JSObject*> m_scope;
   nsXPIDLCString m_jsCallback;
   NS_DECL_ISUPPORTS
+protected:
+  virtual ~nsCryptoRunArgs();
 };
 
 //This class is used to run the callback code
@@ -191,6 +192,13 @@ private:
   nsCryptoRunArgs *m_args;
 };
 
+namespace mozilla {
+template<>
+struct HasDangerousPublicDestructor<nsCryptoRunnable>
+{
+  static const bool value = true;
+};
+}
 
 //We're going to inherit the memory passed
 //into us.
@@ -199,10 +207,11 @@ private:
 class nsP12Runnable : public nsIRunnable {
 public:
   nsP12Runnable(nsIX509Cert **certArr, int32_t numCerts, nsIPK11Token *token);
-  virtual ~nsP12Runnable();
 
   NS_IMETHOD Run();
   NS_DECL_ISUPPORTS
+protected:
+  virtual ~nsP12Runnable();
 private:
   nsCOMPtr<nsIPK11Token> mToken;
   nsIX509Cert **mCertArr;
@@ -963,13 +972,13 @@ cryptojs_ReadArgsAndGenerateKey(JSContext *cx,
   jsString = JS::ToString(cx, v);
   NS_ENSURE_TRUE(jsString, NS_ERROR_OUT_OF_MEMORY);
   argv[2] = STRING_TO_JSVAL(jsString);
-  nsDependentJSString dependentKeyGenAlg;
-  NS_ENSURE_TRUE(dependentKeyGenAlg.init(cx, jsString), NS_ERROR_UNEXPECTED);
-  nsAutoString keyGenAlg(dependentKeyGenAlg);
+  nsAutoJSString autoJSKeyGenAlg;
+  NS_ENSURE_TRUE(autoJSKeyGenAlg.init(cx, jsString), NS_ERROR_UNEXPECTED);
+  nsAutoString keyGenAlg(autoJSKeyGenAlg);
   keyGenAlg.Trim("\r\n\t ");
   keyGenType->keyGenType = cryptojs_interpret_key_gen_type(keyGenAlg);
   if (keyGenType->keyGenType == invalidKeyGen) {
-    NS_LossyConvertUTF16toASCII keyGenAlgNarrow(dependentKeyGenAlg);
+    NS_LossyConvertUTF16toASCII keyGenAlgNarrow(autoJSKeyGenAlg);
     JS_ReportError(cx, "%s%s%s", JS_ERROR,
                    "invalid key generation argument:",
                    keyGenAlgNarrow.get());
@@ -985,7 +994,7 @@ cryptojs_ReadArgsAndGenerateKey(JSContext *cx,
                                    *slot,willEscrow);
 
   if (rv != NS_OK) {
-    NS_LossyConvertUTF16toASCII keyGenAlgNarrow(dependentKeyGenAlg);
+    NS_LossyConvertUTF16toASCII keyGenAlgNarrow(autoJSKeyGenAlg);
     JS_ReportError(cx,"%s%s%s", JS_ERROR,
                    "could not generate the key for algorithm ",
                    keyGenAlgNarrow.get());
@@ -1041,7 +1050,7 @@ nsSetEscrowAuthority(CRMFCertRequest *certReq, nsKeyPairInfo *keyInfo,
       CRMF_CertRequestIsControlPresent(certReq, crmfPKIArchiveOptionsControl)){
     return NS_ERROR_FAILURE;
   }
-  mozilla::pkix::ScopedCERTCertificate cert(wrappingCert->GetCert());
+  ScopedCERTCertificate cert(wrappingCert->GetCert());
   if (!cert)
     return NS_ERROR_FAILURE;
 
@@ -1893,7 +1902,7 @@ nsCrypto::GenerateCRMFRequest(JSContext* aContext,
   }
 
   nsCOMPtr<nsIContentSecurityPolicy> csp;
-  if (!nsContentUtils::GetContentSecurityPolicy(aContext, getter_AddRefs(csp))) {
+  if (!nsContentUtils::GetContentSecurityPolicy(getter_AddRefs(csp))) {
     NS_ERROR("Error: failed to get CSP");
     aRv.Throw(NS_ERROR_FAILURE);
     return nullptr;
@@ -1941,7 +1950,7 @@ nsCrypto::GenerateCRMFRequest(JSContext* aContext,
       aRv.Throw(NS_ERROR_FAILURE);
       return nullptr;
     }
-    mozilla::pkix::ScopedCERTCertificate cert(
+    ScopedCERTCertificate cert(
       CERT_NewTempCertificate(CERT_GetDefaultCertDB(),
                               &certDer, nullptr, false, true));
     if (!cert) {
@@ -2013,9 +2022,6 @@ nsCrypto::GenerateCRMFRequest(JSContext* aContext,
                                                  const_cast<char*>(aRegToken.get()),
                                                  const_cast<char*>(aAuthenticator.get()),
                                                  escrowCert);
-#ifdef DEBUG_javi
-  printf ("Created the folloing CRMF request:\n%s\n", encodedRequest);
-#endif
   if (!encodedRequest) {
     nsFreeKeyPairInfo(keyids, numRequests);
     aRv.Throw(NS_ERROR_FAILURE);
@@ -2193,8 +2199,7 @@ nsCertAlreadyExists(SECItem *derCert)
   CERTCertDBHandle *handle = CERT_GetDefaultCertDB();
   bool retVal = false;
 
-  mozilla::pkix::ScopedCERTCertificate cert(
-    CERT_FindCertByDERCert(handle, derCert));
+  ScopedCERTCertificate cert(CERT_FindCertByDERCert(handle, derCert));
   if (cert) {
     if (cert->isperm && !cert->nickname && !cert->emailAddr) {
       //If the cert doesn't have a nickname or email addr, it is
@@ -2355,8 +2360,7 @@ nsCrypto::ImportUserCertificates(const nsAString& aNickname,
 
   //Import the root chain into the cert db.
  {
-  mozilla::pkix::ScopedCERTCertList
-    caPubs(CMMF_CertRepContentGetCAPubs(certRepContent));
+  ScopedCERTCertList caPubs(CMMF_CertRepContentGetCAPubs(certRepContent));
   if (caPubs) {
     int32_t numCAs = nsCertListCount(caPubs.get());
     
