@@ -6,9 +6,12 @@
 #define _NS_DATASIGNATUREVERIFIER_H_
 
 #include "nsIDataSignatureVerifier.h"
+#include "mozilla/Attributes.h"
 
 #include "keythi.h"
 
+typedef struct CERTCertificateStr CERTCertificate;
+
 // 296d76aa-275b-4f3c-af8a-30a4026c18fc
 #define NS_DATASIGNATUREVERIFIER_CID \
     { 0x296d76aa, 0x275b, 0x4f3c, \
@@ -16,7 +19,7 @@
 #define NS_DATASIGNATUREVERIFIER_CONTRACTID \
     "@mozilla.org/security/datasignatureverifier;1"
 
-class nsDataSignatureVerifier : public nsIDataSignatureVerifier
+class nsDataSignatureVerifier MOZ_FINAL : public nsIDataSignatureVerifier
 {
 public:
   NS_DECL_ISUPPORTS
@@ -32,4 +35,14 @@ private:
   }
 };
 
+namespace mozilla {
+
+nsresult VerifyCMSDetachedSignatureIncludingCertificate(
+  const SECItem& buffer, const SECItem& detachedDigest,
+  nsresult (*verifyCertificate)(CERTCertificate* cert, void* context,
+                                void* pinArg),
+  void* verifyCertificateContext, void* pinArg);
+
+} // namespace mozilla
+
 #endif // _NS_DATASIGNATUREVERIFIER_H_
