@@ -2,14 +2,14 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
-import subprocess
 import re
 import os
 import shutil
 import tempfile
 import time
 
-from devicemanager import DeviceManager, DMError, _pop_last_line
+from devicemanager import DeviceManager, DMError
+from mozprocess import ProcessHandler
 import mozfile
 import mozlog
 
@@ -29,16 +29,17 @@ class DeviceManagerADB(DeviceManager):
     _pollingInterval = 0.01
     _packageName = None
     _tempDir = None
+    connected = False
     default_timeout = 300
 
     def __init__(self, host=None, port=5555, retryLimit=5, packageName='fennec',
                  adbPath='adb', deviceSerial=None, deviceRoot=None,
-                 logLevel=mozlog.ERROR, **kwargs):
-        DeviceManager.__init__(self, logLevel)
+                 logLevel=mozlog.ERROR, autoconnect=True, **kwargs):
+        DeviceManager.__init__(self, logLevel=logLevel,
+                               deviceRoot=deviceRoot)
         self.host = host
         self.port = port
         self.retryLimit = retryLimit
-        self.deviceRoot = deviceRoot
 
         # the path to adb, or 'adb' to assume that it's on the PATH
         self._adbPath = adbPath
@@ -58,28 +59,30 @@ class DeviceManagerADB(DeviceManager):
         # verify that we can run the adb command. can't continue otherwise
         self._verifyADB()
 
-        # try to connect to the device over tcp/ip if we have a hostname
-        if self.host:
-            self._connectRemoteADB()
+        if autoconnect:
+            self.connect()
 
-        # verify that we can connect to the device. can't continue
-        self._verifyDevice()
+    def connect(self):
+        if not self.connected:
+            # try to connect to the device over tcp/ip if we have a hostname
+            if self.host:
+                self._connectRemoteADB()
 
-        # set up device root
-        self._setupDeviceRoot()
+            # verify that we can connect to the device. can't continue
+            self._verifyDevice()
 
-        # Some commands require root to work properly, even with ADB (e.g.
-        # grabbing APKs out of /data). For these cases, we check whether
-        # we're running as root. If that isn't true, check for the
-        # existence of an su binary
-        self._checkForRoot()
+            # Some commands require root to work properly, even with ADB (e.g.
+            # grabbing APKs out of /data). For these cases, we check whether
+            # we're running as root. If that isn't true, check for the
+            # existence of an su binary
+            self._checkForRoot()
 
-        # can we use zip to speed up some file operations? (currently not
-        # required)
-        try:
-            self._verifyZip()
-        except DMError:
-            pass
+            # can we use zip to speed up some file operations? (currently not
+            # required)
+            try:
+                self._verifyZip()
+            except DMError:
+                pass
 
     def __del__(self):
         if self.host:
@@ -118,37 +121,32 @@ class DeviceManagerADB(DeviceManager):
             args.extend(['-s', self._deviceSerial])
         args.extend(["shell", cmdline])
 
-        procOut = tempfile.SpooledTemporaryFile()
-        procErr = tempfile.SpooledTemporaryFile()
-        proc = subprocess.Popen(args, stdout=procOut, stderr=procErr)
+        def _raise():
+            raise DMError("Timeout exceeded for shell call")
+
+        self._logger.debug("shell - command: %s" % ' '.join(args))
+        proc = ProcessHandler(args, processOutputLine=self._log, onTimeout=_raise)
 
         if not timeout:
             # We are asserting that all commands will complete in this time unless otherwise specified
             timeout = self.default_timeout
 
         timeout = int(timeout)
-        start_time = time.time()
-        ret_code = proc.poll()
-        while ((time.time() - start_time) <= timeout) and ret_code == None:
-            time.sleep(self._pollingInterval)
-            ret_code = proc.poll()
-        if ret_code == None:
-            proc.kill()
-            raise DMError("Timeout exceeded for shell call")
+        proc.run(timeout)
+        proc.wait()
+        output = proc.output
 
-        procOut.seek(0)
-        outputfile.write(procOut.read().rstrip('\n'))
-        procOut.close()
-        procErr.close()
-
-        lastline = _pop_last_line(outputfile)
-        if lastline:
-            m = re.search('([0-9]+)', lastline)
-            if m:
-                return_code = m.group(1)
-                outputfile.seek(-2, 2)
-                outputfile.truncate() # truncate off the return code
-                return int(return_code)
+        if output:
+            lastline = output[-1]
+            if lastline:
+                m = re.search('([0-9]+)', lastline)
+                if m:
+                    return_code = m.group(1)
+                    for line in output:
+                        outputfile.write(line + '\n')
+                    outputfile.seek(-2, 2)
+                    outputfile.truncate() # truncate off the return code
+                    return int(return_code)
 
         return None
 
@@ -173,9 +171,9 @@ class DeviceManagerADB(DeviceManager):
     def devices(self):
         "Return a list of connected devices as (serial, status) tuples."
         proc = self._runCmd(['devices'])
-        proc.stdout.readline() # ignore first line of output
+        proc.output.pop(0) # ignore first line of output
         devices = []
-        for line in iter(proc.stdout.readline, ''):
+        for line in proc.output:
             result = re.match('(.*?)\t(.*)', line)
             if result:
                 devices.append((result.group(1), result.group(2)))
@@ -203,8 +201,8 @@ class DeviceManagerADB(DeviceManager):
                        retryLimit=retryLimit)
 
     def mkDir(self, name):
-        result = self._runCmd(["shell", "mkdir", name]).stdout.read()
-        if 'read-only file system' in result.lower():
+        result = self._runCmd(["shell", "mkdir", name]).output
+        if len(result) and 'read-only file system' in result[0].lower():
             raise DMError("Error creating directory: read only file system")
 
     def pushDir(self, localDir, remoteDir, retryLimit=None, timeout=None):
@@ -219,12 +217,12 @@ class DeviceManagerADB(DeviceManager):
             try:
                 localZip = tempfile.mktemp() + ".zip"
                 remoteZip = remoteDir + "/adbdmtmp.zip"
-                subprocess.Popen(["zip", "-r", localZip, '.'], cwd=localDir,
-                                 stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()
+                ProcessHandler(["zip", "-r", localZip, '.'], cwd=localDir,
+                        processOutputLine=self._log).run().wait()
                 self.pushFile(localZip, remoteZip, retryLimit=retryLimit, createDir=False)
                 mozfile.remove(localZip)
                 data = self._runCmd(["shell", "unzip", "-o", remoteZip,
-                                     "-d", remoteDir]).stdout.read()
+                                     "-d", remoteDir]).output[0]
                 self._checkCmd(["shell", "rm", remoteZip],
                                retryLimit=retryLimit, timeout=timeout)
                 if re.search("unzip: exiting", data) or re.search("Operation not permitted", data):
@@ -242,9 +240,8 @@ class DeviceManagerADB(DeviceManager):
             mozfile.remove(tmpDir)
 
     def dirExists(self, remotePath):
-        p = self._runCmd(["shell", "ls", "-a", remotePath + '/'])
+        data = self._runCmd(["shell", "ls", "-a", remotePath + '/']).output
 
-        data = p.stdout.readlines()
         if len(data) == 1:
             res = data[0]
             if "Not a directory" in res or "No such file or directory" in res:
@@ -252,8 +249,7 @@ class DeviceManagerADB(DeviceManager):
         return True
 
     def fileExists(self, filepath):
-        p = self._runCmd(["shell", "ls", "-a", filepath])
-        data = p.stdout.readlines()
+        data = self._runCmd(["shell", "ls", "-a", filepath]).output
         if len(data) == 1:
             foundpath = data[0].decode('utf-8').rstrip()
             if foundpath == filepath:
@@ -277,8 +273,7 @@ class DeviceManagerADB(DeviceManager):
         self._checkCmd(["shell", "dd", "if=%s" % source, "of=%s" % destination])
 
     def listFiles(self, rootdir):
-        p = self._runCmd(["shell", "ls", "-a", rootdir])
-        data = p.stdout.readlines()
+        data = self._runCmd(["shell", "ls", "-a", rootdir]).output
         data[:] = [item.rstrip('\r\n') for item in data]
         if (len(data) == 1):
             if (data[0] == rootdir):
@@ -295,18 +290,17 @@ class DeviceManagerADB(DeviceManager):
 
     def getProcessList(self):
         p = self._runCmd(["shell", "ps"])
-            # first line is the headers
-        p.stdout.readline()
-        proc = p.stdout.readline()
+        # first line is the headers
+        p.output.pop(0)
         ret = []
-        while (proc):
+        for proc in p.output:
             els = proc.split()
-            # we need to figure out if this is "user pid name" or "pid user vsz stat command"
+            # We need to figure out if this is "user pid name" or
+            # "pid user vsz stat command"
             if els[1].isdigit():
                 ret.append(list([int(els[1]), els[len(els) - 1], els[0]]))
             else:
                 ret.append(list([int(els[0]), els[len(els) - 1], els[1]]))
-            proc =  p.stdout.readline()
         return ret
 
     def fireProcess(self, appname, failIfRunning=False):
@@ -379,17 +373,16 @@ class DeviceManagerADB(DeviceManager):
                     args.append("-%d" % sig)
                 args.append(str(pid))
                 p = self._runCmd(args)
-                p.communicate()
                 if p.returncode != 0:
                     raise DMError("Error killing process "
-                                  "'%s': %s" % (appname, p.stdout.read()))
+                                  "'%s': %s" % (appname, p.output))
 
     def _runPull(self, remoteFile, localFile):
         """
         Pulls remoteFile from device to host
         """
         try:
-            self._runCmd(["pull",  remoteFile, localFile]).communicate()
+            self._runCmd(["pull",  remoteFile, localFile])
         except (OSError, ValueError):
             raise DMError("Error pulling remote file '%s' to '%s'" % (remoteFile, localFile))
 
@@ -443,20 +436,13 @@ class DeviceManagerADB(DeviceManager):
 
         return md5
 
-    def _setupDeviceRoot(self):
-        """
-        setup the device root and cache its value
-        """
-        # if self.deviceRoot is already set, create it if necessary, and use it
-        if self.deviceRoot:
-            if not self.dirExists(self.deviceRoot):
-                try:
-                    self.mkDir(self.deviceRoot)
-                except:
-                    self._logger.error("Unable to create device root %s" % self.deviceRoot)
-                    raise
-            return
+    def _setupDeviceRoot(self, deviceRoot):
+        # user-specified device root, create it and return it
+        if deviceRoot:
+            self.mkDir(deviceRoot)
+            return deviceRoot
 
+        # we must determine the device root ourselves
         paths = [('/storage/sdcard0', 'tests'),
                  ('/storage/sdcard1', 'tests'),
                  ('/sdcard', 'tests'),
@@ -464,53 +450,35 @@ class DeviceManagerADB(DeviceManager):
                  ('/data/local', 'tests')]
         for (basePath, subPath) in paths:
             if self.dirExists(basePath):
-                testRoot = os.path.join(basePath, subPath)
+                root = os.path.join(basePath, subPath)
                 try:
-                    self.mkDir(testRoot)
-                    self.deviceRoot = testRoot
-                    return
+                    self.mkDir(root)
+                    return root
                 except:
                     pass
 
         raise DMError("Unable to set up device root using paths: [%s]"
                         % ", ".join(["'%s'" % os.path.join(b, s) for b, s in paths]))
 
-    def getDeviceRoot(self):
-        return self.deviceRoot
-
     def getTempDir(self):
         # Cache result to speed up operations depending
         # on the temporary directory.
         if not self._tempDir:
-            self._tempDir = self.getDeviceRoot() + "/tmp"
+            self._tempDir = "%s/tmp" % self.deviceRoot
             self.mkDir(self._tempDir)
 
         return self._tempDir
 
-    def getAppRoot(self, packageName):
-        devroot = self.getDeviceRoot()
-        if (devroot == None):
-            return None
-
-        if (packageName and self.dirExists('/data/data/' + packageName)):
-            self._packageName = packageName
-            return '/data/data/' + packageName
-        elif (self._packageName and self.dirExists('/data/data/' + self._packageName)):
-            return '/data/data/' + self._packageName
-
-        # Failure (either not installed or not a recognized platform)
-        raise DMError("Failed to get application root for: %s" % packageName)
-
     def reboot(self, wait = False, **kwargs):
         self._checkCmd(["reboot"])
         if wait:
             self._checkCmd(["wait-for-device", "shell", "ls", "/sbin"])
 
     def updateApp(self, appBundlePath, **kwargs):
-        return self._runCmd(["install", "-r", appBundlePath]).stdout.read()
+        return self._runCmd(["install", "-r", appBundlePath]).output
 
     def getCurrentTime(self):
-        timestr = self._runCmd(["shell", "date", "+%s"]).stdout.read().strip()
+        timestr = str(self._runCmd(["shell", "date", "+%s"]).output[0])
         if (not timestr or not timestr.isdigit()):
             raise DMError("Unable to get current time using date (got: '%s')" % timestr)
         return int(timestr)*1000
@@ -518,11 +486,11 @@ class DeviceManagerADB(DeviceManager):
     def getInfo(self, directive=None):
         ret = {}
         if (directive == "id" or directive == "all"):
-            ret["id"] = self._runCmd(["get-serialno"]).stdout.read()
+            ret["id"] = self._runCmd(["get-serialno"]).output[0]
         if (directive == "os" or directive == "all"):
-            ret["os"] = self._runCmd(["shell", "getprop", "ro.build.display.id"]).stdout.read()
+            ret["os"] = self._runCmd(["shell", "getprop", "ro.build.display.id"]).output[0]
         if (directive == "uptime" or directive == "all"):
-            utime = self._runCmd(["shell", "uptime"]).stdout.read()
+            utime = self._runCmd(["shell", "uptime"]).output[0]
             if (not utime):
                 raise DMError("error getting uptime")
             utime = utime[9:]
@@ -533,15 +501,14 @@ class DeviceManagerADB(DeviceManager):
             seconds = utime[0:utime.find(",")]
             ret["uptime"] = ["0 days " + hours + " hours " + minutes + " minutes " + seconds + " seconds"]
         if (directive == "process" or directive == "all"):
-            ret["process"] = self._runCmd(["shell", "ps"]).stdout.read()
+            ret["process"] = self._runCmd(["shell", "ps"]).output
         if (directive == "systime" or directive == "all"):
-            ret["systime"] = self._runCmd(["shell", "date"]).stdout.read()
+            ret["systime"] = self._runCmd(["shell", "date"]).output[0]
         self._logger.info(ret)
         return ret
 
     def uninstallApp(self, appName, installPath=None):
-        data = self._runCmd(["uninstall", appName]).stdout.read().strip()
-        status = data.split('\n')[0].strip()
+        status = self._runCmd(["uninstall", appName]).output[0].strip()
         if status != 'Success':
             raise DMError("uninstall failed for %s. Got: %s" % (appName, status))
 
@@ -553,14 +520,18 @@ class DeviceManagerADB(DeviceManager):
         """
         Runs a command using adb
 
-        returns: returncode from subprocess.Popen
+        returns: instance of ProcessHandler
         """
         finalArgs = [self._adbPath]
         if self._deviceSerial:
             finalArgs.extend(['-s', self._deviceSerial])
         finalArgs.extend(args)
-        return subprocess.Popen(finalArgs, stdout=subprocess.PIPE,
-                                stderr=subprocess.STDOUT)
+        self._logger.debug("_runCmd - command: %s" % ' '.join(finalArgs))
+        proc = ProcessHandler(finalArgs, storeOutput=True,
+                processOutputLine=self._log)
+        proc.run()
+        proc.returncode = proc.wait()
+        return proc
 
     # timeout is specified in seconds, and if no timeout is given,
     # we will run until we hit the default_timeout specified in the __init__
@@ -569,32 +540,31 @@ class DeviceManagerADB(DeviceManager):
         Runs a command using adb and waits for the command to finish.
         If timeout is specified, the process is killed after <timeout> seconds.
 
-        returns: returncode from subprocess.Popen
+        returns: returncode from process
         """
         retryLimit = retryLimit or self.retryLimit
         finalArgs = [self._adbPath]
         if self._deviceSerial:
             finalArgs.extend(['-s', self._deviceSerial])
         finalArgs.extend(args)
+        self._logger.debug("_checkCmd - command: %s" % ' '.join(finalArgs))
         if not timeout:
-            # We are asserting that all commands will complete in this time unless otherwise specified
+            # We are asserting that all commands will complete in this
+            # time unless otherwise specified
             timeout = self.default_timeout
 
         timeout = int(timeout)
         retries = 0
-        with tempfile.SpooledTemporaryFile() as procOut:
-            while retries < retryLimit:
-                proc = subprocess.Popen(finalArgs, stdout=procOut, stderr=subprocess.STDOUT)
-                start_time = time.time()
-                ret_code = proc.poll()
-                while ((time.time() - start_time) <= timeout) and ret_code == None:
-                    time.sleep(self._pollingInterval)
-                    ret_code = proc.poll()
-                if ret_code == None:
-                    proc.kill()
-                    retries += 1
-                    continue
+        while retries < retryLimit:
+            proc = ProcessHandler(finalArgs, processOutputLine=self._log)
+            proc.run(timeout=timeout)
+            ret_code = proc.wait()
+            if ret_code == None:
+                proc.kill()
+                retries += 1
+            else:
                 return ret_code
+
         raise DMError("Timeout exceeded for _checkCmd call after %d retries." % retries)
 
     def chmodDir(self, remoteDir, mask="777"):
@@ -625,17 +595,12 @@ class DeviceManagerADB(DeviceManager):
             self._checkCmd(["version"])
         except os.error, err:
             raise DMError("unable to execute ADB (%s): ensure Android SDK is installed and adb is in your $PATH" % err)
-        except subprocess.CalledProcessError:
-            raise DMError("unable to execute ADB: ensure Android SDK is installed and adb is in your $PATH")
 
     def _verifyDevice(self):
         # If there is a device serial number, see if adb is connected to it
         if self._deviceSerial:
             deviceStatus = None
-            proc = subprocess.Popen([self._adbPath, "devices"],
-                                    stdout=subprocess.PIPE,
-                                    stderr=subprocess.STDOUT)
-            for line in proc.stdout:
+            for line in self._runCmd(["devices"]).output:
                 m = re.match('(.+)?\s+(.+)$', line)
                 if m:
                     if self._deviceSerial == m.group(1):
@@ -646,20 +611,14 @@ class DeviceManagerADB(DeviceManager):
                 raise DMError("bad status for device %s: %s" % (self._deviceSerial, deviceStatus))
 
         # Check to see if we can connect to device and run a simple command
-        ret = None
-        try:
-            ret = self._checkCmd(["shell", "echo"])
-        except subprocess.CalledProcessError:
-            raise DMError("unable to connect to device: is it plugged in?")
-        if ret:
+        if self._checkCmd(["shell", "echo"]) is None:
             raise DMError("unable to connect to device")
 
     def _checkForRoot(self):
         # Check whether we _are_ root by default (some development boards work
         # this way, this is also the result of some relatively rare rooting
         # techniques)
-        proc = self._runCmd(["shell", "id"])
-        data = proc.stdout.read()
+        data = self._runCmd(["shell", "id"]).output[0]
         if data.find('uid=0(root)') >= 0:
             self._haveRootShell = True
             # if this returns true, we don't care about su
@@ -678,20 +637,20 @@ class DeviceManagerADB(DeviceManager):
         if retcode is None: # still not terminated, kill
             proc.kill()
 
-        data = proc.stdout.read()
+        data = proc.output[0]
         if data.find('uid=0(root)') >= 0:
             self._haveSu = True
 
     def _isUnzipAvailable(self):
-        data = self._runCmd(["shell", "unzip"]).stdout.read()
-        if (re.search('Usage', data)):
-            return True
-        else:
-            return False
+        data = self._runCmd(["shell", "unzip"]).output
+        for line in data:
+            if (re.search('Usage', line)):
+                return True
+        return False
 
     def _isLocalZipAvailable(self):
         try:
-            subprocess.check_call(["zip", "-?"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            self._checkCmd(["zip", "-?"])
         except:
             return False
         return True
