@@ -5,17 +5,20 @@
 
 const Cc = Components.classes;
 const Ci = Components.interfaces;
+const Cu = Components.utils;
 
-Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
-Components.utils.import("resource://gre/modules/Services.jsm");
-Components.utils.import("resource://gre/modules/PrivateBrowsingUtils.jsm");
+Cu.import("resource://gre/modules/XPCOMUtils.jsm");
+Cu.import("resource://gre/modules/Services.jsm");
+Cu.import("resource://gre/modules/Timer.jsm");
+Cu.import("resource://gre/modules/PrivateBrowsingUtils.jsm");
+Cu.import("resource://gre/modules/LoginManagerContent.jsm");
 
-XPCOMUtils.defineLazyModuleGetter(this, "LoginManagerContent",
-                                  "resource://gre/modules/LoginManagerContent.jsm");
 XPCOMUtils.defineLazyModuleGetter(this, "Promise",
                                   "resource://gre/modules/Promise.jsm");
 XPCOMUtils.defineLazyModuleGetter(this, "Task",
                                   "resource://gre/modules/Task.jsm");
+XPCOMUtils.defineLazyModuleGetter(this, "BrowserUtils",
+                                  "resource://gre/modules/BrowserUtils.jsm");
 
 var debug = false;
 function log(...pieces) {
@@ -116,11 +119,17 @@ LoginManager.prototype = {
 
         // Form submit observer checks forms for new logins and pw changes.
         Services.obs.addObserver(this._observer, "xpcom-shutdown", false);
-        Services.obs.addObserver(this._observer, "passwordmgr-storage-replace",
-                                 false);
 
-        // Initialize storage so that asynchronous data loading can start.
-        this._initStorage();
+        // TODO: Make this class useful in the child process (in addition to
+        // autoCompleteSearchAsync and fillForm).
+        if (Services.appinfo.processType ===
+            Services.appinfo.PROCESS_TYPE_DEFAULT) {
+            Services.obs.addObserver(this._observer, "passwordmgr-storage-replace",
+                                     false);
+
+            // Initialize storage so that asynchronous data loading can start.
+            this._initStorage();
+        }
     },
 
 
@@ -408,91 +417,53 @@ LoginManager.prototype = {
         return this._storage.setLoginSavingEnabled(hostname, enabled);
     },
 
-
     /*
-     * autoCompleteSearch
+     * autoCompleteSearchAsync
      *
      * Yuck. This is called directly by satchel:
      * nsFormFillController::StartSearch()
-     * [toolkit/components/satchel/src/nsFormFillController.cpp]
+     * [toolkit/components/satchel/nsFormFillController.cpp]
      *
      * We really ought to have a simple way for code to register an
      * auto-complete provider, and not have satchel calling pwmgr directly.
      */
-    autoCompleteSearch : function (aSearchString, aPreviousResult, aElement) {
-        // aPreviousResult & aResult are nsIAutoCompleteResult,
-        // aElement is nsIDOMHTMLInputElement
+    autoCompleteSearchAsync : function (aSearchString, aPreviousResult,
+                                        aElement, aCallback) {
+        // aPreviousResult is an nsIAutoCompleteResult, aElement is
+        // nsIDOMHTMLInputElement
 
-        if (!this._remember)
-            return null;
+        if (!this._remember) {
+            setTimeout(function() {
+                aCallback.onSearchCompletion(new UserAutoCompleteResult(aSearchString, []));
+            }, 0);
+            return;
+        }
 
         log("AutoCompleteSearch invoked. Search is:", aSearchString);
 
-        var result = null;
-
-        if (aPreviousResult &&
-                aSearchString.substr(0, aPreviousResult.searchString.length) == aPreviousResult.searchString) {
-            log("Using previous autocomplete result");
-            result = aPreviousResult;
-            result.wrappedJSObject.searchString = aSearchString;
-
-            // We have a list of results for a shorter search string, so just
-            // filter them further based on the new search string.
-            // Count backwards, because result.matchCount is decremented
-            // when we remove an entry.
-            for (var i = result.matchCount - 1; i >= 0; i--) {
-                var match = result.getValueAt(i);
-
-                // Remove results that are too short, or have different prefix.
-                if (aSearchString.length > match.length ||
-                    aSearchString.toLowerCase() !=
-                        match.substr(0, aSearchString.length).toLowerCase())
-                {
-                    log("Removing autocomplete entry:", match);
-                    result.removeValueAt(i, false);
-                }
-            }
+        var previousResult;
+        if (aPreviousResult) {
+            previousResult = { searchString: aPreviousResult.searchString,
+                               logins: aPreviousResult.wrappedJSObject.logins };
         } else {
-            log("Creating new autocomplete search result.");
-
-            var doc = aElement.ownerDocument;
-            var origin = this._getPasswordOrigin(doc.documentURI);
-            var actionOrigin = this._getActionOrigin(aElement.form);
-
-            // This shouldn't trigger a master password prompt, because we
-            // don't attach to the input until after we successfully obtain
-            // logins for the form.
-            var logins = this.findLogins({}, origin, actionOrigin, null);
-            var matchingLogins = [];
-
-            // Filter out logins that don't match the search prefix. Also
-            // filter logins without a username, since that's confusing to see
-            // in the dropdown and we can't autocomplete them anyway.
-            for (i = 0; i < logins.length; i++) {
-                var username = logins[i].username.toLowerCase();
-                if (username &&
-                    aSearchString.length <= username.length &&
-                    aSearchString.toLowerCase() ==
-                        username.substr(0, aSearchString.length))
-                {
-                    matchingLogins.push(logins[i]);
-                }
-            }
-            log(matchingLogins.length, "autocomplete logins avail.");
-            result = new UserAutoCompleteResult(aSearchString, matchingLogins);
+            previousResult = null;
         }
 
-        return result;
+        let rect = BrowserUtils.getElementBoundingScreenRect(aElement);
+        LoginManagerContent._autoCompleteSearchAsync(aSearchString, previousResult,
+                                                     aElement, rect)
+                           .then(function(logins) {
+                               let results =
+                                   new UserAutoCompleteResult(aSearchString, logins);
+                               aCallback.onSearchCompletion(results);
+                           })
+                           .then(null, Cu.reportError);
     },
 
 
-
-
     /* ------- Internal methods / callbacks for document integration ------- */
 
 
-
-
     /*
      * _getPasswordOrigin
      *
@@ -545,102 +516,13 @@ LoginManager.prototype = {
      */
     fillForm : function (form) {
         log("fillForm processing form[ id:", form.id, "]");
-        return LoginManagerContent._fillForm(form, true, true, false, false, null)[0];
+        return LoginManagerContent._asyncFindLogins(form, { showMasterPassword: true })
+                                  .then(function({ form, loginsFound }) {
+                   return LoginManagerContent._fillForm(form, true, true,
+                                                        false, false, loginsFound)[0];
+               });
     },
 
 }; // end of LoginManager implementation
 
-
-
-
-// nsIAutoCompleteResult implementation
-function UserAutoCompleteResult (aSearchString, matchingLogins) {
-    function loginSort(a,b) {
-        var userA = a.username.toLowerCase();
-        var userB = b.username.toLowerCase();
-
-        if (userA < userB)
-            return -1;
-
-        if (userB > userA)
-            return  1;
-
-        return 0;
-    };
-
-    this.searchString = aSearchString;
-    this.logins = matchingLogins.sort(loginSort);
-    this.matchCount = matchingLogins.length;
-
-    if (this.matchCount > 0) {
-        this.searchResult = Ci.nsIAutoCompleteResult.RESULT_SUCCESS;
-        this.defaultIndex = 0;
-    }
-}
-
-UserAutoCompleteResult.prototype = {
-    QueryInterface : XPCOMUtils.generateQI([Ci.nsIAutoCompleteResult,
-                                            Ci.nsISupportsWeakReference]),
-
-    // private
-    logins : null,
-
-    // Allow autoCompleteSearch to get at the JS object so it can
-    // modify some readonly properties for internal use.
-    get wrappedJSObject() {
-        return this;
-    },
-
-    // Interfaces from idl...
-    searchString : null,
-    searchResult : Ci.nsIAutoCompleteResult.RESULT_NOMATCH,
-    defaultIndex : -1,
-    errorDescription : "",
-    matchCount : 0,
-
-    getValueAt : function (index) {
-        if (index < 0 || index >= this.logins.length)
-            throw "Index out of range.";
-
-        return this.logins[index].username;
-    },
-
-    getLabelAt: function(index) {
-        return this.getValueAt(index);
-    },
-
-    getCommentAt : function (index) {
-        return "";
-    },
-
-    getStyleAt : function (index) {
-        return "";
-    },
-
-    getImageAt : function (index) {
-        return "";
-    },
-
-    getFinalCompleteValueAt : function (index) {
-        return this.getValueAt(index);
-    },
-
-    removeValueAt : function (index, removeFromDB) {
-        if (index < 0 || index >= this.logins.length)
-            throw "Index out of range.";
-
-        var [removedLogin] = this.logins.splice(index, 1);
-
-        this.matchCount--;
-        if (this.defaultIndex > this.logins.length)
-            this.defaultIndex--;
-
-        if (removeFromDB) {
-            var pwmgr = Cc["@mozilla.org/login-manager;1"].
-                        getService(Ci.nsILoginManager);
-            pwmgr.removeLogin(removedLogin);
-        }
-    }
-};
-
 this.NSGetFactory = XPCOMUtils.generateNSGetFactory([LoginManager]);
