@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*-
  * vim: sw=2 ts=2 sts=2 expandtab
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -8,6 +8,10 @@ Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 Components.utils.import("resource://gre/modules/Services.jsm");
 XPCOMUtils.defineLazyModuleGetter(this, "PlacesUtils",
                                   "resource://gre/modules/PlacesUtils.jsm");
+XPCOMUtils.defineLazyModuleGetter(this, "TelemetryStopwatch",
+                                  "resource://gre/modules/TelemetryStopwatch.jsm");
+XPCOMUtils.defineLazyModuleGetter(this, "NetUtil",
+                                  "resource://gre/modules/NetUtil.jsm");
 
 ////////////////////////////////////////////////////////////////////////////////
 //// Constants
@@ -77,6 +81,9 @@ const kBrowserUrlbarAutofillPref = "autoFill";
 // Whether to search only typed entries.
 const kBrowserUrlbarAutofillTypedPref = "autoFill.typed";
 
+// The Telemetry histogram for urlInlineComplete query on domain
+const DOMAIN_QUERY_TELEMETRY = "PLACES_AUTOCOMPLETE_URLINLINE_DOMAIN_QUERY_TIME_MS";
+
 ////////////////////////////////////////////////////////////////////////////////
 //// Globals
 
@@ -202,14 +209,17 @@ function safePrefGetter(aPrefBranch, aName, aDefault) {
  * Wraps a callback and ensures that handleCompletion is not dispatched if the
  * query is no longer tracked.
  *
- * @param aCallback
+ * @param aAutocomplete
  *        A reference to a nsPlacesAutoComplete.
+ * @param aCallback
+ *        A reference to a mozIStorageStatementCallback
  * @param aDBConnection
  *        The database connection to execute the queries on.
  */
-function AutoCompleteStatementCallbackWrapper(aCallback,
+function AutoCompleteStatementCallbackWrapper(aAutocomplete, aCallback,
                                               aDBConnection)
 {
+  this._autocomplete = aAutocomplete;
   this._callback = aCallback;
   this._db = aDBConnection;
 }
@@ -232,9 +242,9 @@ AutoCompleteStatementCallbackWrapper.prototype = {
   {
     // Only dispatch handleCompletion if we are not done searching and are a
     // pending search.
-    let callback = this._callback;
-    if (!callback.isSearchComplete() && callback.isPendingSearch(this._handle)) {
-      callback.handleCompletion.apply(callback, arguments);
+    if (!this._autocomplete.isSearchComplete() &&
+        this._autocomplete.isPendingSearch(this._handle)) {
+      this._callback.handleCompletion.apply(this._callback, arguments);
     }
   },
 
@@ -266,6 +276,7 @@ AutoCompleteStatementCallbackWrapper.prototype = {
 
 ////////////////////////////////////////////////////////////////////////////////
 //// nsPlacesAutoComplete class
+//// @mozilla.org/autocomplete/search;1?name=history
 
 function nsPlacesAutoComplete()
 {
@@ -299,10 +310,7 @@ function nsPlacesAutoComplete()
     // to our own in-memory temp table, and having a cloned copy means we do not
     // run the risk of our queries taking longer due to the main database
     // connection performing a long-running task.
-    let db = Cc["@mozilla.org/browser/nav-history-service;1"].
-             getService(Ci.nsPIPlacesDatabase).
-             DBConnection.
-             clone(true);
+    let db = PlacesUtils.history.DBConnection.clone(true);
 
     // Autocomplete often fallbacks to a table scan due to lack of text indices.
     // In such cases a larger cache helps reducing IO.  The default Storage
@@ -334,22 +342,6 @@ function nsPlacesAutoComplete()
     return db;
   });
 
-  XPCOMUtils.defineLazyServiceGetter(this, "_bh",
-                                     "@mozilla.org/browser/global-history;2",
-                                     "nsIBrowserHistory");
-
-  XPCOMUtils.defineLazyServiceGetter(this, "_bs",
-                                     "@mozilla.org/browser/nav-bookmarks-service;1",
-                                     "nsINavBookmarksService");
-
-  XPCOMUtils.defineLazyServiceGetter(this, "_ioService",
-                                     "@mozilla.org/network/io-service;1",
-                                     "nsIIOService");
-
-  XPCOMUtils.defineLazyServiceGetter(this, "_faviconService",
-                                     "@mozilla.org/browser/favicon-service;1",
-                                     "nsIFaviconService");
-
   XPCOMUtils.defineLazyGetter(this, "_defaultQuery", function() {
     let replacementText = "";
     return this._db.createAsyncStatement(
@@ -576,7 +568,7 @@ nsPlacesAutoComplete.prototype = {
   onValueRemoved: function PAC_onValueRemoved(aResult, aURISpec, aRemoveFromDB)
   {
     if (aRemoveFromDB) {
-      this._bh.removePage(this._ioService.newURI(aURISpec, null, null));
+      PlacesUtils.history.removePage(NetUtil.newURI(aURISpec));
     }
   },
 
@@ -619,7 +611,7 @@ nsPlacesAutoComplete.prototype = {
   handleResult: function PAC_handleResult(aResultSet)
   {
     let row, haveMatches = false;
-    while (row = aResultSet.getNextRow()) {
+    while ((row = aResultSet.getNextRow())) {
       let match = this._processRow(row);
       haveMatches = haveMatches || match;
 
@@ -773,7 +765,7 @@ nsPlacesAutoComplete.prototype = {
     // handleCompletion implementation of AutoCompleteStatementCallbackWrapper).
 
     // Create our wrapper object and execute the queries.
-    let wrapper = new AutoCompleteStatementCallbackWrapper(this, this._db);
+    let wrapper = new AutoCompleteStatementCallbackWrapper(this, this, this._db);
     this._pendingQuery = wrapper.executeAsync(aQueries);
   },
 
@@ -956,7 +948,7 @@ nsPlacesAutoComplete.prototype = {
 
     // Bind the needed parameters to the query so consumers can use it.
     let (params = query.params) {
-      params.parent = this._bs.tagsFolder;
+      params.parent = PlacesUtils.tagsFolderId;
       params.query_type = kQueryTypeFiltered;
       params.matchBehavior = aMatchBehavior;
       params.searchBehavior = this._behavior;
@@ -1038,7 +1030,7 @@ nsPlacesAutoComplete.prototype = {
 
     let query = this._adaptiveQuery;
     let (params = query.params) {
-      params.parent = this._bs.tagsFolder;
+      params.parent = PlacesUtils.tagsFolderId;
       params.search_string = this._currentSearchString;
       params.query_type = kQueryTypeFiltered;
       params.matchBehavior = aMatchBehavior;
@@ -1189,10 +1181,10 @@ nsPlacesAutoComplete.prototype = {
     // Obtain the favicon for this URI.
     let favicon;
     if (aFaviconSpec) {
-      let uri = this._ioService.newURI(aFaviconSpec, null, null);
-      favicon = this._faviconService.getFaviconLinkForIcon(uri).spec;
+      let uri = NetUtil.newURI(aFaviconSpec);
+      favicon = PlacesUtils.favicons.getFaviconLinkForIcon(uri).spec;
     }
-    favicon = favicon || this._faviconService.defaultFavicon.spec;
+    favicon = favicon || PlacesUtils.favicons.defaultFavicon.spec;
 
     this._result.appendMatch(aURISpec, aTitle, favicon, aStyle);
   },
@@ -1267,6 +1259,7 @@ nsPlacesAutoComplete.prototype = {
 
 ////////////////////////////////////////////////////////////////////////////////
 //// urlInlineComplete class
+//// component @mozilla.org/autocomplete/search;1?name=urlinline
 
 function urlInlineComplete()
 {
@@ -1284,20 +1277,19 @@ urlInlineComplete.prototype = {
   get _db()
   {
     if (!this.__db && this._autofillEnabled) {
-      this.__db = PlacesUtils.history.QueryInterface(Ci.nsPIPlacesDatabase).
-                  DBConnection.clone(true);
+      this.__db = PlacesUtils.history.DBConnection.clone(true);
     }
     return this.__db;
   },
 
-  __syncQuery: null,
+  __hostQuery: null,
 
-  get _syncQuery()
+  get _hostQuery()
   {
-    if (!this.__syncQuery) {
+    if (!this.__hostQuery) {
       // Add a trailing slash at the end of the hostname, since we always
       // want to complete up to and including a URL separator.
-      this.__syncQuery = this._db.createStatement(
+      this.__hostQuery = this._db.createAsyncStatement(
           "/* do not warn (bug no): could index on (typed,frecency) but not worth it */ "
         + "SELECT host || '/', prefix || host || '/' "
         + "FROM moz_hosts "
@@ -1308,15 +1300,15 @@ urlInlineComplete.prototype = {
         + "LIMIT 1"
       );
     }
-    return this.__syncQuery;
+    return this.__hostQuery;
   },
 
-  __asyncQuery: null,
+  __urlQuery: null,
 
-  get _asyncQuery()
+  get _urlQuery()
   {
-    if (!this.__asyncQuery) {
-      this.__asyncQuery = this._db.createAsyncStatement(
+    if (!this.__urlQuery) {
+      this.__urlQuery = this._db.createAsyncStatement(
           "/* do not warn (bug no): can't use an index */ "
         + "SELECT h.url "
         + "FROM moz_places h "
@@ -1330,7 +1322,7 @@ urlInlineComplete.prototype = {
         + "LIMIT 1"
       );
     }
-    return this.__asyncQuery;
+    return this.__urlQuery;
   },
 
   //////////////////////////////////////////////////////////////////////////////
@@ -1349,18 +1341,17 @@ urlInlineComplete.prototype = {
     this._originalSearchString = aSearchString;
     this._currentSearchString =
       fixupSearchText(this._originalSearchString.toLowerCase());
-    // The protocol and the domain are lowercased by nsIURI, so it's fine to
+    // The protocol and the host are lowercased by nsIURI, so it's fine to
     // lowercase the typed prefix to add it back to the results later.
     this._strippedPrefix = this._originalSearchString.slice(
       0, this._originalSearchString.length - this._currentSearchString.length
     ).toLowerCase();
 
-    let result = Cc["@mozilla.org/autocomplete/simple-result;1"].
-                 createInstance(Ci.nsIAutoCompleteSimpleResult);
-    result.setSearchString(aSearchString);
-    result.setTypeAheadResult(true);
+    this._result = Cc["@mozilla.org/autocomplete/simple-result;1"].
+                   createInstance(Ci.nsIAutoCompleteSimpleResult);
+    this._result.setSearchString(aSearchString);
+    this._result.setTypeAheadResult(true);
 
-    this._result = result;
     this._listener = aListener;
 
     // Don't autoFill if the search term is recognized as a keyword, otherwise
@@ -1382,49 +1373,63 @@ urlInlineComplete.prototype = {
       return;
     }
 
-    // Do a synchronous search on the table of domains.
-    let query = this._syncQuery;
-    query.params.search_string = this._currentSearchString.toLowerCase();
-
-    // Domains have no "/" in them.
+    // Hosts have no "/" in them.
     let lastSlashIndex = this._currentSearchString.lastIndexOf("/");
-    if (lastSlashIndex == -1) {
-      var hasDomainResult = false;
-      var domain, untrimmedDomain;
-      try {
-        hasDomainResult = query.executeStep();
-        if (hasDomainResult) {
-          domain = query.getString(0);
-          untrimmedDomain = query.getString(1);
-        }
-      } finally {
-        query.reset();
-      }
-
-      if (hasDomainResult) {
-        // We got a match for a domain, we can add it immediately.
-        // TODO (bug 754265): this is a temporary solution introduced while
-        // waiting for a propert dedicated API.
-        result.appendMatch(this._strippedPrefix + domain, untrimmedDomain);
 
+    // Search only URLs if there's a slash in the search string...
+    if (lastSlashIndex != -1) {
+      // ...but not if it's exactly at the end of the search string.
+      if (lastSlashIndex < this._currentSearchString.length - 1)
+        this._queryURL();
+      else
         this._finishSearch();
-        return;
-      }
-    }
-
-    // We did not get a result from the synchronous domain search.
-    // We now do an asynchronous search through places, and complete
-    // up to the next URL separator.
-
-    // First, check if this is necessary.
-    // We don't need to search if we have no "/" separator, or if it's at
-    // the end of the search text.
-    if (lastSlashIndex == -1 ||
-        lastSlashIndex == this._currentSearchString.length - 1) {
-      this._finishSearch();
       return;
     }
 
+    // Do a synchronous search on the table of hosts.
+    let query = this._hostQuery;
+    query.params.search_string = this._currentSearchString.toLowerCase();
+    // This is just to measure the delay to reach the UI, not the query time.
+    TelemetryStopwatch.start(DOMAIN_QUERY_TELEMETRY);
+    let ac = this;
+    let wrapper = new AutoCompleteStatementCallbackWrapper(this, {
+      handleResult: function (aResultSet) {
+        let row = aResultSet.getNextRow();
+        let trimmedHost = row.getResultByIndex(0);
+        let untrimmedHost = row.getResultByIndex(1);
+        // If the untrimmed value doesn't preserve the user's input just
+        // ignore it and complete to the found host.
+        if (untrimmedHost &&
+            !untrimmedHost.toLowerCase().contains(ac._originalSearchString.toLowerCase())) {
+          untrimmedHost = null;
+        }
+
+        ac._result.appendMatch(ac._strippedPrefix + trimmedHost, "", "", "", untrimmedHost);
+
+        // handleCompletion() will cause the result listener to be called, and
+        // will display the result in the UI.
+      },
+
+      handleError: function (aError) {
+        Components.utils.reportError(
+          "URL Inline Complete: An async statement encountered an " +
+          "error: " + aError.result + ", '" + aError.message + "'");
+      },
+
+      handleCompletion: function (aReason) {
+        TelemetryStopwatch.finish(DOMAIN_QUERY_TELEMETRY);
+        ac._finishSearch();
+      }
+    }, this._db);
+    this._pendingQuery = wrapper.executeAsync([query]);
+  },
+
+  /**
+   * Execute an asynchronous search through places, and complete
+   * up to the next URL separator.
+   */
+  _queryURL: function UIC__queryURL()
+  {
     // The URIs in the database are fixed up, so we can match on a lowercased
     // host, but the path must be matched in a case sensitive way.
     let pathIndex =
@@ -1436,15 +1441,59 @@ urlInlineComplete.prototype = {
 
     // Within the standard autocomplete query, we only search the beginning
     // of URLs for 1 result.
-    let query = this._asyncQuery;
+    let query = this._urlQuery;
     let (params = query.params) {
       params.matchBehavior = MATCH_BEGINNING_CASE_SENSITIVE;
       params.searchBehavior = Ci.mozIPlacesAutoComplete["BEHAVIOR_URL"];
       params.searchString = this._currentSearchString;
     }
 
-    // Execute the async query
-    let wrapper = new AutoCompleteStatementCallbackWrapper(this, this._db);
+    // Execute the query.
+    let ac = this;
+    let wrapper = new AutoCompleteStatementCallbackWrapper(this, {
+      handleResult: function(aResultSet) {
+        let row = aResultSet.getNextRow();
+        let value = row.getResultByIndex(0);
+        let url = fixupSearchText(value);
+
+        let prefix = value.slice(0, value.length - stripPrefix(value).length);
+
+        // We must complete the URL up to the next separator (which is /, ? or #).
+        let separatorIndex = url.slice(ac._currentSearchString.length)
+                                .search(/[\/\?\#]/);
+        if (separatorIndex != -1) {
+          separatorIndex += ac._currentSearchString.length;
+          if (url[separatorIndex] == "/") {
+            separatorIndex++; // Include the "/" separator
+          }
+          url = url.slice(0, separatorIndex);
+        }
+
+        // Add the result.
+        // If the untrimmed value doesn't preserve the user's input just
+        // ignore it and complete to the found url.
+        let untrimmedURL = prefix + url;
+        if (untrimmedURL &&
+            !untrimmedURL.toLowerCase().contains(ac._originalSearchString.toLowerCase())) {
+          untrimmedURL = null;
+         }
+
+        ac._result.appendMatch(ac._strippedPrefix + url, "", "", "", untrimmedURL);
+
+        // handleCompletion() will cause the result listener to be called, and
+        // will display the result in the UI.
+      },
+
+      handleError: function(aError) {
+        Components.utils.reportError(
+          "URL Inline Complete: An async statement encountered an " +
+          "error: " + aError.result + ", '" + aError.message + "'");
+      },
+
+      handleCompletion: function(aReason) {
+        ac._finishSearch();
+      }
+    }, this._db);
     this._pendingQuery = wrapper.executeAsync([query]);
   },
 
@@ -1490,48 +1539,6 @@ urlInlineComplete.prototype = {
   //// nsIAutoCompleteSearchDescriptor
   get searchType() Ci.nsIAutoCompleteSearchDescriptor.SEARCH_TYPE_IMMEDIATE,
 
-  //////////////////////////////////////////////////////////////////////////////
-  //// mozIStorageStatementCallback
-
-  handleResult: function UIC_handleResult(aResultSet)
-  {
-    let row = aResultSet.getNextRow();
-    let value = row.getResultByIndex(0);
-    let url = fixupSearchText(value);
-
-    let prefix = value.slice(0, value.length - stripPrefix(value).length);
-
-    // We must complete the URL up to the next separator (which is /, ? or #).
-    let separatorIndex = url.slice(this._currentSearchString.length)
-                            .search(/[\/\?\#]/);
-    if (separatorIndex != -1) {
-      separatorIndex += this._currentSearchString.length;
-      if (url[separatorIndex] == "/") {
-        separatorIndex++; // Include the "/" separator
-      }
-      url = url.slice(0, separatorIndex);
-    }
-
-    // Add the result.
-    // TODO (bug 754265): this is a temporary solution introduced while
-    // waiting for a propert dedicated API.
-    this._result.appendMatch(this._strippedPrefix + url, prefix + url);
-
-    // handleCompletion() will cause the result listener to be called, and
-    // will display the result in the UI.
-  },
-
-  handleError: function UIC_handleError(aError)
-  {
-    Components.utils.reportError("URL Inline Complete: An async statement encountered an " +
-                                 "error: " + aError.result + ", '" + aError.message + "'");
-  },
-
-  handleCompletion: function UIC_handleCompletion(aReason)
-  {
-    this._finishSearch();
-  },
-
   //////////////////////////////////////////////////////////////////////////////
   //// nsIObserver
 
@@ -1564,8 +1571,8 @@ urlInlineComplete.prototype = {
   {
     // Finalize the statements that we have used.
     let stmts = [
-      "__syncQuery",
-      "__asyncQuery",
+      "__hostQuery",
+      "__urlQuery",
     ];
     for (let i = 0; i < stmts.length; i++) {
       // We do not want to create any query we haven't already created, so
@@ -1629,11 +1636,10 @@ urlInlineComplete.prototype = {
   QueryInterface: XPCOMUtils.generateQI([
     Ci.nsIAutoCompleteSearch,
     Ci.nsIAutoCompleteSearchDescriptor,
-    Ci.mozIStorageStatementCallback,
     Ci.nsIObserver,
     Ci.nsISupportsWeakReference,
   ])
 };
 
 let components = [nsPlacesAutoComplete, urlInlineComplete];
-const NSGetFactory = XPCOMUtils.generateNSGetFactory(components);
+this.NSGetFactory = XPCOMUtils.generateNSGetFactory(components);
