@@ -16,18 +16,15 @@
 
 'use strict';
 
-var promise = require('./util/promise');
+var Promise = require('./util/promise').Promise;
 var util = require('./util/util');
 var host = require('./util/host');
 var l10n = require('./util/l10n');
 
 var view = require('./ui/view');
-var converters = require('./converters/converters');
-var centralCanon = require('./commands/commands').centralCanon;
 var Parameter = require('./commands/commands').Parameter;
 var CommandOutputManager = require('./commands/commands').CommandOutputManager;
 
-var centralTypes = require('./types/types').centralTypes;
 var Status = require('./types/types').Status;
 var Conversion = require('./types/types').Conversion;
 var commandModule = require('./types/command');
@@ -40,7 +37,7 @@ var TrueNamedArgument = require('./types/types').TrueNamedArgument;
 var MergedArgument = require('./types/types').MergedArgument;
 var ScriptArgument = require('./types/types').ScriptArgument;
 
-var RESOLVED = promise.resolve(undefined);
+var RESOLVED = Promise.resolve(undefined);
 
 /**
  * This is a list of the known command line components to enable certain
@@ -106,9 +103,9 @@ var removeMapping = function(requisition) {
 /**
  * Some manual intervention is needed in parsing the { command.
  */
-function getEvalCommand(canon) {
+function getEvalCommand(commands) {
   if (getEvalCommand._cmd == null) {
-    getEvalCommand._cmd = canon.getCommand(evalCmd.name);
+    getEvalCommand._cmd = commands.get(evalCmd.name);
   }
   return getEvalCommand._cmd;
 }
@@ -196,7 +193,7 @@ Assignment.prototype.getPredictionRanked = function(context, rank) {
   }
 
   if (this.isInName()) {
-    return promise.resolve(undefined);
+    return Promise.resolve(undefined);
   }
 
   return this.getPredictions(context).then(function(predictions) {
@@ -347,7 +344,7 @@ function CommandAssignment(requisition) {
     },
     enumerable: true
   });
-  this.param = new Parameter(requisition.types, commandParamMetadata);
+  this.param = new Parameter(requisition.system.types, commandParamMetadata);
 }
 
 CommandAssignment.prototype = Object.create(Assignment.prototype);
@@ -368,7 +365,7 @@ exports.CommandAssignment = CommandAssignment;
  */
 function UnassignedAssignment(requisition, arg) {
   var isIncompleteName = (arg.text.charAt(0) === '-');
-  this.param = new Parameter(requisition.types, {
+  this.param = new Parameter(requisition.system.types, {
     name: '__unassigned',
     description: l10n.lookup('cliOptions'),
     type: {
@@ -426,19 +423,19 @@ Object.defineProperty(exports, 'logErrors', {
  * assignments of values to parameters, each handled by an instance of
  * Assignment.
  *
+ * @param system Allows access to the various plug-in points in GCLI. At a
+ * minimum it must contain commands and types objects.
  * @param options A set of options to customize how GCLI is used. Includes:
  * - environment An optional opaque object passed to commands in the
  *   Execution Context.
  * - document A DOM Document passed to commands using the Execution Context in
  *   order to allow creation of DOM nodes. If missing Requisition will use the
- *   global 'document'.
+ *   global 'document', or leave undefined.
  * - commandOutputManager A custom commandOutputManager to which output should
  *   be sent
- * - canon An instance of Canon that specifies the commands that are allowed in
- *   this Requisition
  * @constructor
  */
-function Requisition(options) {
+function Requisition(system, options) {
   options = options || {};
 
   this.environment = options.environment || {};
@@ -453,8 +450,7 @@ function Requisition(options) {
   }
 
   this.commandOutputManager = options.commandOutputManager || new CommandOutputManager();
-  this.canon = options.canon || centralCanon;
-  this.types = options.types || centralTypes;
+  this.system = system;
 
   this.shell = {
     cwd: '/', // Where we store the current working directory
@@ -492,6 +488,10 @@ function Requisition(options) {
 
   addMapping(this);
   this._setBlankAssignment(this.commandAssignment);
+
+  // If a command calls context.update then the UI needs some way to be
+  // informed of the change
+  this.onExternalUpdate = util.createEvent('Requisition.onExternalUpdate');
 }
 
 /**
@@ -550,7 +550,7 @@ Object.defineProperty(Requisition.prototype, 'executionContext', {
     if (this._executionContext == null) {
       this._executionContext = {
         defer: function() {
-          return promise.defer();
+          return Promise.defer();
         },
         typedData: function(type, data) {
           return {
@@ -578,14 +578,18 @@ Object.defineProperty(Requisition.prototype, 'executionContext', {
       });
       Object.defineProperty(this._executionContext, 'shell', {
         get: function() { return requisition.shell; },
-        enumerable : true
+        enumerable: true
+      });
+      Object.defineProperty(this._executionContext, 'system', {
+        get: function() { return requisition.system; },
+        enumerable: true
       });
 
       if (legacy) {
         this._executionContext.createView = view.createView;
         this._executionContext.exec = this.exec.bind(this);
-        this._executionContext.update = this.update.bind(this);
-        this._executionContext.updateExec = this.updateExec.bind(this);
+        this._executionContext.update = this._contextUpdate.bind(this);
+        this._executionContext.updateExec = this._contextUpdateExec.bind(this);
 
         Object.defineProperty(this._executionContext, 'document', {
           get: function() { return requisition.document; },
@@ -607,13 +611,13 @@ Object.defineProperty(Requisition.prototype, 'conversionContext', {
     if (this._conversionContext == null) {
       this._conversionContext = {
         defer: function() {
-          return promise.defer();
+          return Promise.defer();
         },
 
         createView: view.createView,
         exec: this.exec.bind(this),
-        update: this.update.bind(this),
-        updateExec: this.updateExec.bind(this)
+        update: this._contextUpdate.bind(this),
+        updateExec: this._contextUpdateExec.bind(this)
       };
 
       // Alias requisition so we're clear about what's what
@@ -627,6 +631,10 @@ Object.defineProperty(Requisition.prototype, 'conversionContext', {
         get: function() { return requisition.environment; },
         enumerable: true
       });
+      Object.defineProperty(this._conversionContext, 'system', {
+        get: function() { return requisition.system; },
+        enumerable: true
+      });
     }
 
     return this._conversionContext;
@@ -766,17 +774,36 @@ Requisition.prototype._getFirstBlankPositionalAssignment = function() {
   return reply;
 };
 
+/**
+ * The update process is asynchronous, so there is (unavoidably) a window
+ * where we've worked out the command but don't yet understand all the params.
+ * If we try to do things to a requisition in this window we may get
+ * inconsistent results. Asynchronous promises have made the window bigger.
+ * The only time we've seen this in practice is during focus events due to
+ * clicking on a shortcut. The focus want to check the cursor position while
+ * the shortcut is updating the command line.
+ * This function allows us to detect and back out of this problem.
+ * We should be able to remove this function when all the state in a
+ * requisition can be encapsulated and updated atomically.
+ */
+Requisition.prototype.isUpToDate = function() {
+  if (!this._args) {
+    return false;
+  }
+  for (var i = 0; i < this._args.length; i++) {
+    if (this._args[i].assignment == null) {
+      return false;
+    }
+  }
+  return true;
+};
+
 /**
  * Look through the arguments attached to our assignments for the assignment
  * at the given position.
  * @param {number} cursor The cursor position to query
  */
 Requisition.prototype.getAssignmentAt = function(cursor) {
-  if (!this._args) {
-    console.trace();
-    throw new Error('Missing args');
-  }
-
   // We short circuit this one because we may have no args, or no args with
   // any size and the alg below only finds arguments with size.
   if (cursor === 0) {
@@ -822,14 +849,7 @@ Requisition.prototype.getAssignmentAt = function(cursor) {
   // Possible shortcut, we don't really need to go through all the args
   // to work out the solution to this
 
-  var reply = assignForPos[cursor - 1];
-
-  if (!reply) {
-    throw new Error('Missing assignment.' +
-        ' cursor=' + cursor + ' text=' + this.toString());
-  }
-
-  return reply;
+  return assignForPos[cursor - 1];
 };
 
 /**
@@ -861,7 +881,7 @@ Requisition.prototype.toCanonicalString = function() {
     }
 
     var val = assignment.param.type.stringify(assignment.value, ctx);
-    return promise.resolve(val).then(function(str) {
+    return Promise.resolve(val).then(function(str) {
       return ' ' + str;
     }.bind(this));
   }.bind(this));
@@ -1017,7 +1037,7 @@ Requisition.prototype.setAssignment = function(assignment, arg, options) {
       this._endChangeCheckOrder(updateId);
     }
 
-    return promise.resolve(undefined);
+    return Promise.resolve(undefined);
   }.bind(this);
 
   if (arg == null) {
@@ -1210,7 +1230,7 @@ Requisition.prototype.getStateData = function(start, rank) {
   var context = this.executionContext;
   var predictionPromise = (typed.trim().length !== 0) ?
                           current.getPredictionRanked(context, rank) :
-                          promise.resolve(null);
+                          Promise.resolve(null);
 
   return predictionPromise.then(function(prediction) {
     // directTabText is for when the current input is a prefix of the completion
@@ -1349,7 +1369,7 @@ Requisition.prototype._addSpace = function(assignment) {
     return this.setAssignment(assignment, arg);
   }
   else {
-    return promise.resolve(undefined);
+    return Promise.resolve(undefined);
   }
 };
 
@@ -1426,7 +1446,7 @@ Requisition.prototype.complete = function(cursor, rank) {
       outstanding.push(assignPromise);
     }
 
-    return promise.all(outstanding).then(function() {
+    return Promise.all(outstanding).then(function() {
       return true;
     }.bind(this));
   }.bind(this));
@@ -1438,10 +1458,10 @@ Requisition.prototype.complete = function(cursor, rank) {
 Requisition.prototype.decrement = function(assignment) {
   var ctx = this.executionContext;
   var val = assignment.param.type.decrement(assignment.value, ctx);
-  return promise.resolve(val).then(function(replacement) {
+  return Promise.resolve(val).then(function(replacement) {
     if (replacement != null) {
       var val = assignment.param.type.stringify(replacement, ctx);
-      return promise.resolve(val).then(function(str) {
+      return Promise.resolve(val).then(function(str) {
         var arg = assignment.arg.beget({ text: str });
         return this.setAssignment(assignment, arg);
       }.bind(this));
@@ -1455,10 +1475,10 @@ Requisition.prototype.decrement = function(assignment) {
 Requisition.prototype.increment = function(assignment) {
   var ctx = this.executionContext;
   var val = assignment.param.type.increment(assignment.value, ctx);
-  return promise.resolve(val).then(function(replacement) {
+  return Promise.resolve(val).then(function(replacement) {
     if (replacement != null) {
       var val = assignment.param.type.stringify(replacement, ctx);
-      return promise.resolve(val).then(function(str) {
+      return Promise.resolve(val).then(function(str) {
         var arg = assignment.arg.beget({ text: str });
         return this.setAssignment(assignment, arg);
       }.bind(this));
@@ -1478,15 +1498,31 @@ function getDataCommandAttribute(element) {
   return command;
 }
 
+/**
+ * Designed to be called from context.update(). Acts just like update() except
+ * that it also calls onExternalUpdate() to inform the UI of an unexpected
+ * change to the current command.
+ */
+Requisition.prototype._contextUpdate = function(typed) {
+  return this.update(typed).then(function(reply) {
+    this.onExternalUpdate({ typed: typed });
+    return reply;
+  }.bind(this));
+};
+
 /**
  * Called by the UI when ever the user interacts with a command line input
- * @param typed The contents of the input field
+ * @param typed The contents of the input field OR an HTML element (or an event
+ * that targets an HTML element) which has a data-command attribute or a child
+ * with the same that contains the command to update with
  */
 Requisition.prototype.update = function(typed) {
-  if (typeof HTMLElement !== 'undefined' && typed instanceof HTMLElement) {
+  // Should be "if (typed instanceof HTMLElement)" except Gecko
+  if (typeof typed.querySelector === 'function') {
     typed = getDataCommandAttribute(typed);
   }
-  if (typeof Event !== 'undefined' && typed instanceof Event) {
+  // Should be "if (typed instanceof Event)" except Gecko
+  if (typeof typed.currentTarget === 'object') {
     typed = getDataCommandAttribute(typed.currentTarget);
   }
 
@@ -1739,7 +1775,7 @@ function isSimple(typed) {
 }
 
 /**
- * Looks in the canon for a command extension that matches what has been
+ * Looks in the commands for a command extension that matches what has been
  * typed at the command line.
  */
 Requisition.prototype._split = function(args) {
@@ -1750,7 +1786,8 @@ Requisition.prototype._split = function(args) {
   if (args[0].type === 'ScriptArgument') {
     // Special case: if the user enters { console.log('foo'); } then we need to
     // use the hidden 'eval' command
-    conversion = new Conversion(getEvalCommand(this.canon), new ScriptArgument());
+    conversion = new Conversion(getEvalCommand(this.system.commands),
+                                new ScriptArgument());
     this._setAssignmentInternal(this.commandAssignment, conversion);
     return;
   }
@@ -1987,7 +2024,7 @@ Requisition.prototype.exec = function(options) {
     if (options.command != null) {
       // Fast track by looking up the command directly since passed args
       // means there is no command line to parse.
-      command = this.canon.getCommand(options.command);
+      command = this.system.commands.get(options.command);
       if (!command) {
         console.error('Command not found: ' + options.command);
       }
@@ -2008,7 +2045,7 @@ Requisition.prototype.exec = function(options) {
     typed = typed.replace(/\s*}\s*$/, '');
   }
 
-  var output = new Output({
+  var output = new Output(this.conversionContext, {
     command: command,
     args: args,
     typed: typed,
@@ -2029,7 +2066,7 @@ Requisition.prototype.exec = function(options) {
         util.errorHandler(ex);
       }
       else {
-        console.log(data);
+        console.error(data);
       }
     }
 
@@ -2046,7 +2083,7 @@ Requisition.prototype.exec = function(options) {
     var ex = new Error(this.getStatusMessage());
     // We only reject a call to exec if GCLI breaks. Errors with commands are
     // exposed in the 'error' status of the Output object
-    return promise.resolve(onError(ex)).then(function(output) {
+    return Promise.resolve(onError(ex)).then(function(output) {
       this.clear();
       return output;
     }.bind(this));
@@ -2060,7 +2097,7 @@ Requisition.prototype.exec = function(options) {
     catch (ex) {
       var data = (typeof ex.message === 'string' && ex.stack != null) ?
                  ex.message : ex;
-      return promise.resolve(onError(data, ex));
+      return Promise.resolve(onError(data, ex));
     }
     finally {
       this.clear();
@@ -2068,6 +2105,18 @@ Requisition.prototype.exec = function(options) {
   }
 };
 
+/**
+ * Designed to be called from context.updateExec(). Acts just like updateExec()
+ * except that it also calls onExternalUpdate() to inform the UI of an
+ * unexpected change to the current command.
+ */
+Requisition.prototype._contextUpdateExec = function(typed, options) {
+  return this.updateExec(typed, options).then(function(reply) {
+    this.onExternalUpdate({ typed: typed });
+    return reply;
+  }.bind(this));
+};
+
 /**
  * A shortcut for calling update, resolving the promise and then exec.
  * @param input The string to execute
@@ -2085,13 +2134,14 @@ exports.Requisition = Requisition;
 /**
  * A simple object to hold information about the output of a command
  */
-function Output(options) {
+function Output(context, options) {
   options = options || {};
   this.command = options.command || '';
   this.args = options.args || {};
   this.typed = options.typed || '';
   this.canonical = options.canonical || '';
   this.hidden = options.hidden === true ? true : false;
+  this.converters = context.system.converters;
 
   this.type = undefined;
   this.data = undefined;
@@ -2099,8 +2149,9 @@ function Output(options) {
   this.error = false;
   this.start = new Date();
 
-  this._deferred = promise.defer();
-  this.promise = this._deferred.promise;
+  this.promise = new Promise(function(resolve, reject) {
+    this._resolve = resolve;
+  }.bind(this));
 }
 
 /**
@@ -2128,14 +2179,14 @@ Output.prototype.complete = function(data, error) {
     throw new Error('No type from output of ' + this.typed);
   }
 
-  this._deferred.resolve();
+  this._resolve();
 };
 
 /**
  * Call converters.convert using the data in this Output object
  */
 Output.prototype.convert = function(type, conversionContext) {
-  return converters.convert(this.data, this.type, type, conversionContext);
+  return this.converters.convert(this.data, this.type, type, conversionContext);
 };
 
 Output.prototype.toJson = function() {
