@@ -10,24 +10,30 @@ const Cr = Components.results;
 const { devtools } = Cu.import("resource://gre/modules/devtools/Loader.jsm", {});
 const { worker } = Cu.import("resource://gre/modules/devtools/worker-loader.js", {})
 const {Promise: promise} = Cu.import("resource://gre/modules/Promise.jsm", {});
+const { Task } = Cu.import("resource://gre/modules/Task.jsm", {});
+const { promiseInvoke } = devtools.require("devtools/async-utils");
 
 const Services = devtools.require("Services");
-const DevToolsUtils = devtools.require("devtools/toolkit/DevToolsUtils.js");
-const { DebuggerServer } = devtools.require("devtools/server/main");
-const { DebuggerServer: WorkerDebuggerServer } = worker.require("devtools/server/main");
-
 // Always log packets when running tests. runxpcshelltests.py will throw
 // the output away anyway, unless you give it the --verbose flag.
 Services.prefs.setBoolPref("devtools.debugger.log", true);
 // Enable remote debugging for the relevant tests.
 Services.prefs.setBoolPref("devtools.debugger.remote-enabled", true);
 
+const DevToolsUtils = devtools.require("devtools/toolkit/DevToolsUtils.js");
+const { DebuggerServer } = devtools.require("devtools/server/main");
+const { DebuggerServer: WorkerDebuggerServer } = worker.require("devtools/server/main");
+
+function dumpn(msg) {
+  dump("DBG-TEST: " + msg + "\n");
+}
+
 function tryImport(url) {
   try {
     Cu.import(url);
   } catch (e) {
-    dump("Error importing " + url + "\n");
-    dump(DevToolsUtils.safeErrorString(e) + "\n");
+    dumpn("Error importing " + url);
+    dumpn(DevToolsUtils.safeErrorString(e));
     throw e;
   }
 }
@@ -78,9 +84,9 @@ let listener = {
       // If we've been given an nsIScriptError, then we can print out
       // something nicely formatted, for tools like Emacs to pick up.
       var scriptError = aMessage.QueryInterface(Ci.nsIScriptError);
-      dump(aMessage.sourceName + ":" + aMessage.lineNumber + ": " +
-           scriptErrorFlagsToKind(aMessage.flags) + ": " +
-           aMessage.errorMessage + "\n");
+      dumpn(aMessage.sourceName + ":" + aMessage.lineNumber + ": " +
+            scriptErrorFlagsToKind(aMessage.flags) + ": " +
+            aMessage.errorMessage);
       var string = aMessage.errorMessage;
     } catch (x) {
       // Be a little paranoid with message, as the whole goal here is to lose
@@ -106,7 +112,7 @@ let listener = {
     // If we throw an error here because of them our tests start failing.
     // So, we'll just dump the message to the logs instead, to make sure the
     // information isn't lost.
-    dump("head_dbg.js observed a console message: " + string + "\n");
+    dumpn("head_dbg.js observed a console message: " + string);
   }
 };
 
@@ -121,7 +127,7 @@ function check_except(func)
     do_check_true(true);
     return;
   }
-  dump("Should have thrown an exception: " + func.toString());
+  dumpn("Should have thrown an exception: " + func.toString());
   do_check_true(false);
 }
 
@@ -172,7 +178,10 @@ function attachTestThread(aClient, aTitle, aCallback) {
     function onAttach(aResponse, aThreadClient) {
       aCallback(aResponse, aTabClient, aThreadClient);
     }
-    aTabClient.attachThread({ useSourceMaps: true }, onAttach);
+    aTabClient.attachThread({
+      useSourceMaps: true,
+      autoBlackBox: true
+    }, onAttach);
   });
 }
 
@@ -336,9 +345,9 @@ TracingTransport.prototype = {
   dumpLog: function() {
     for (let entry of this.packets) {
       if (entry.type === "sent") {
-        dump("trace.expectSend(" + entry.packet + ");\n");
+        dumpn("trace.expectSend(" + entry.packet + ");");
       } else {
-        dump("trace.expectReceive(" + entry.packet + ");\n");
+        dumpn("trace.expectReceive(" + entry.packet + ");");
       }
     }
   }
@@ -407,3 +416,171 @@ const Test = task => () => {
 };
 
 const assert = do_check_true;
+
+/**
+ * Create a promise that is resolved on the next occurence of the given event.
+ *
+ * @param DebuggerClient client
+ * @param String event
+ * @returns Promise
+ */
+function waitForEvent(client, event) {
+  dumpn("Waiting for event: " + event);
+  return new Promise((resolve, reject) => {
+    client.addOneTimeListener(event, (_, packet) => resolve(packet));
+  });
+}
+
+/**
+ * Create a promise that is resolved on the next pause.
+ *
+ * @param DebuggerClient client
+ * @returns Promise
+ */
+function waitForPause(client) {
+  return waitForEvent(client, "paused");
+}
+
+/**
+ * Execute the action on the next tick and return a promise that is resolved on
+ * the next pause.
+ *
+ * When using promises and Task.jsm, we often want to do an action that causes a
+ * pause and continue the task once the pause has ocurred. Unfortunately, if we
+ * do the action that causes the pause within the task's current tick we will
+ * pause before we have a chance to yield the promise that waits for the pause
+ * and we enter a dead lock. The solution is to create the promise that waits
+ * for the pause, schedule the action to run on the next tick of the event loop,
+ * and finally yield the promise.
+ *
+ * @param Function action
+ * @param DebuggerClient client
+ * @returns Promise
+ */
+function executeOnNextTickAndWaitForPause(action, client) {
+  const paused = waitForPause(client);
+  executeSoon(action);
+  return paused;
+}
+
+/**
+ * Create a promise that is resolved with the server's response to the client's
+ * Remote Debugger Protocol request. If a response with the `error` property is
+ * received, the promise is rejected. Any extra arguments passed in are
+ * forwarded to the method invocation.
+ *
+ * See `setBreakpoint` below, for example usage.
+ *
+ * @param DebuggerClient/ThreadClient/SourceClient/etc client
+ * @param Function method
+ * @param any args
+ * @returns Promise
+ */
+function rdpRequest(client, method, ...args) {
+  return promiseInvoke(client, method, ...args)
+    .then(response => {
+      const { error, message } = response;
+      if (error) {
+        throw new Error(error + ": " + message);
+      }
+      return response;
+    });
+}
+
+/**
+ * Set a breakpoint over the Remote Debugging Protocol.
+ *
+ * @param ThreadClient threadClient
+ * @param {url, line[, column[, condition]]} breakpointOptions
+ * @returns Promise
+ */
+function setBreakpoint(threadClient, breakpointOptions) {
+  dumpn("Setting a breakpoint: " + JSON.stringify(breakpointOptions, null, 2));
+  return rdpRequest(threadClient, threadClient.setBreakpoint, breakpointOptions);
+}
+
+/**
+ * Resume JS execution for the specified thread.
+ *
+ * @param ThreadClient threadClient
+ * @returns Promise
+ */
+function resume(threadClient) {
+  dumpn("Resuming.");
+  return rdpRequest(threadClient, threadClient.resume);
+}
+
+/**
+ * Resume JS execution for the specified thread and then wait for the next pause
+ * event.
+ *
+ * @param DebuggerClient client
+ * @param ThreadClient threadClient
+ * @returns Promise
+ */
+function resumeAndWaitForPause(client, threadClient) {
+  const paused = waitForPause(client);
+  return resume(threadClient).then(() => paused);
+}
+
+/**
+ * Get the list of sources for the specified thread.
+ *
+ * @param ThreadClient threadClient
+ * @returns Promise
+ */
+function getSources(threadClient) {
+  dumpn("Getting sources.");
+  return rdpRequest(threadClient, threadClient.getSources);
+}
+
+/**
+ * Resume JS execution for a single step and wait for the pause after the step
+ * has been taken.
+ *
+ * @param DebuggerClient client
+ * @param ThreadClient threadClient
+ * @returns Promise
+ */
+function stepIn(client, threadClient) {
+  dumpn("Stepping in.");
+  const paused = waitForPause(client);
+  return rdpRequest(threadClient, threadClient.stepIn)
+    .then(() => paused);
+}
+
+/**
+ * Get the list of `count` frames currently on stack, starting at the index
+ * `first` for the specified thread.
+ *
+ * @param ThreadClient threadClient
+ * @param Number first
+ * @param Number count
+ * @returns Promise
+ */
+function getFrames(threadClient, first, count) {
+  dumpn("Getting frames.");
+  return rdpRequest(threadClient, threadClient.getFrames, first, count);
+}
+
+/**
+ * Black box the specified source.
+ *
+ * @param SourceClient sourceClient
+ * @returns Promise
+ */
+function blackBox(sourceClient) {
+  dumpn("Black boxing source: " + sourceClient.actor);
+  return rdpRequest(sourceClient, sourceClient.blackBox);
+}
+
+/**
+ * Stop black boxing the specified source.
+ *
+ * @param SourceClient sourceClient
+ * @returns Promise
+ */
+function unBlackBox(sourceClient) {
+  dumpn("Un-black boxing source: " + sourceClient.actor);
+  return rdpRequest(sourceClient, sourceClient.unblackBox);
+}
