@@ -5,9 +5,6 @@
 
 #include <string>
 #include <sstream>
-#ifdef MOZ_INSTRUMENT_EVENT_LOOP
-#include "EventTracer.h"
-#endif
 #include "GeckoProfiler.h"
 #include "nsProfiler.h"
 #include "nsMemory.h"
@@ -23,7 +20,7 @@
 
 using std::string;
 
-NS_IMPL_ISUPPORTS1(nsProfiler, nsIProfiler)
+NS_IMPL_ISUPPORTS(nsProfiler, nsIProfiler)
 
 nsProfiler::nsProfiler()
   : mLockedForPrivateBrowsing(false)
@@ -81,10 +78,6 @@ nsProfiler::StartProfiler(uint32_t aEntries, double aInterval,
   profiler_start(aEntries, aInterval,
                  aFeatures, aFeatureCount,
                  aThreadNameFilters, aFilterCount);
-#ifdef MOZ_INSTRUMENT_EVENT_LOOP
-  bool printToConsole = false;
-  mozilla::InitEventTracing(printToConsole);
-#endif
   return NS_OK;
 }
 
@@ -204,10 +197,10 @@ nsProfiler::GetSharedLibraryInformation(nsAString& aOutString)
 NS_IMETHODIMP nsProfiler::GetProfileData(JSContext* aCx,
                                          JS::MutableHandle<JS::Value> aResult)
 {
-  JSObject *obj = profiler_get_profile_jsobject(aCx);
-  if (!obj)
+  JS::RootedObject obj(aCx, profiler_get_profile_jsobject(aCx));
+  if (!obj) {
     return NS_ERROR_FAILURE;
-
+  }
   aResult.setObject(*obj);
   return NS_OK;
 }
@@ -219,26 +212,6 @@ nsProfiler::IsActive(bool *aIsActive)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsProfiler::GetResponsivenessTimes(uint32_t *aCount, double **aResult)
-{
-  unsigned int len = 100;
-  const double* times = profiler_get_responsiveness();
-  if (!times) {
-    *aCount = 0;
-    *aResult = nullptr;
-    return NS_OK;
-  }
-
-  double *fs = static_cast<double *>
-                       (nsMemory::Clone(times, len * sizeof(double)));
-
-  *aCount = len;
-  *aResult = fs;
-
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsProfiler::GetFeatures(uint32_t *aCount, char ***aFeatures)
 {
