@@ -6,7 +6,7 @@
 #ifndef __mozilla_widget_GfxDriverInfo_h__
 #define __mozilla_widget_GfxDriverInfo_h__
 
-#include "mozilla/Util.h" // ArrayLength
+#include "mozilla/ArrayUtils.h" // ArrayLength
 #include "nsString.h"
 
 // Macros for adding a blocklist item to the static list.
@@ -35,6 +35,7 @@ enum OperatingSystem {
   DRIVER_OS_WINDOWS_VISTA,
   DRIVER_OS_WINDOWS_7,
   DRIVER_OS_WINDOWS_8,
+  DRIVER_OS_WINDOWS_8_1,
   DRIVER_OS_LINUX,
   DRIVER_OS_OS_X_10_5,
   DRIVER_OS_OS_X_10_6,
@@ -64,6 +65,7 @@ enum DeviceFamily {
   IntelGMA3150,
   IntelGMAX3000,
   IntelGMAX4500HD,
+  IntelHD3000,
   IntelMobileHDGraphics,
   NvidiaBlockD3D9Layers,
   RadeonX1000,
@@ -77,6 +79,7 @@ enum DeviceVendor {
   VendorNVIDIA,
   VendorAMD,
   VendorATI,
+  VendorMicrosoft,
   DeviceVendorMax
 };
 
@@ -212,8 +215,10 @@ inline void PadDriverDecimal(char *aString)
 }
 
 inline bool
-ParseDriverVersion(nsAString& aVersion, uint64_t *aNumericVersion)
+ParseDriverVersion(const nsAString& aVersion, uint64_t *aNumericVersion)
 {
+  *aNumericVersion = 0;
+
 #if defined(XP_WIN)
   int a, b, c, d;
   char aStr[8], bStr[8], cStr[8], dStr[8];
@@ -236,12 +241,15 @@ ParseDriverVersion(nsAString& aVersion, uint64_t *aNumericVersion)
   if (d < 0 || d > 0xffff) return false;
 
   *aNumericVersion = GFX_DRIVER_VERSION(a, b, c, d);
+  return true;
 #elif defined(ANDROID)
   // Can't use aVersion.ToInteger() because that's not compiled into our code
   // unless we have XPCOM_GLUE_AVOID_NSPR disabled.
   *aNumericVersion = atoi(NS_LossyConvertUTF16toASCII(aVersion).get());
-#endif
   return true;
+#else
+  return false;
+#endif
 }
 
 }
