@@ -6,19 +6,19 @@
 
 #include "mozilla/AvailableMemoryTracker.h"
 
+#if defined(XP_WIN)
 #include "prinrval.h"
 #include "prenv.h"
-
 #include "nsIMemoryReporter.h"
+#include "nsMemoryPressure.h"
+#endif
+
 #include "nsIObserver.h"
 #include "nsIObserverService.h"
 #include "nsIRunnable.h"
 #include "nsISupports.h"
-#include "nsMemoryPressure.h"
-#include "nsPrintfCString.h"
-#include "nsThread.h"
+#include "nsThreadUtils.h"
 
-#include "mozilla/Atomics.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/Services.h"
 
@@ -84,13 +84,15 @@ namespace {
 
 #endif
 
-void safe_write(const char *a)
+void
+safe_write(const char* aStr)
 {
   // Well, puts isn't exactly "safe", but at least it doesn't call malloc...
-  fputs(a, stdout);
+  fputs(aStr, stdout);
 }
 
-void safe_write(uint64_t x)
+void
+safe_write(uint64_t aNum)
 {
   // 2^64 is 20 decimal digits.
   const unsigned int max_len = 21;
@@ -98,10 +100,9 @@ void safe_write(uint64_t x)
   buf[max_len - 1] = '\0';
 
   uint32_t i;
-  for (i = max_len - 2; i < max_len && x > 0; i--)
-  {
-    buf[i] = "0123456789"[x % 10];
-    x /= 10;
+  for (i = max_len - 2; i < max_len && aNum > 0; i--) {
+    buf[i] = "0123456789"[aNum % 10];
+    aNum /= 10;
   }
 
   safe_write(&buf[i + 1]);
@@ -148,24 +149,25 @@ volatile PRIntervalTime sLastLowMemoryNotificationTime;
 
 // These are function pointers to the functions we wrap in Init().
 
-void* (WINAPI *sVirtualAllocOrig)
+void* (WINAPI* sVirtualAllocOrig)
   (LPVOID aAddress, SIZE_T aSize, DWORD aAllocationType, DWORD aProtect);
 
-void* (WINAPI *sMapViewOfFileOrig)
+void* (WINAPI* sMapViewOfFileOrig)
   (HANDLE aFileMappingObject, DWORD aDesiredAccess,
    DWORD aFileOffsetHigh, DWORD aFileOffsetLow,
    SIZE_T aNumBytesToMap);
 
-HBITMAP (WINAPI *sCreateDIBSectionOrig)
-  (HDC aDC, const BITMAPINFO *aBitmapInfo,
-   UINT aUsage, VOID **aBits,
+HBITMAP (WINAPI* sCreateDIBSectionOrig)
+  (HDC aDC, const BITMAPINFO* aBitmapInfo,
+   UINT aUsage, VOID** aBits,
    HANDLE aSection, DWORD aOffset);
 
 /**
  * Fire a memory pressure event if it's been long enough since the last one we
  * fired.
  */
-bool MaybeScheduleMemoryPressureEvent()
+bool
+MaybeScheduleMemoryPressureEvent()
 {
   // If this interval rolls over, we may fire an extra memory pressure
   // event, but that's not a big deal.
@@ -192,7 +194,8 @@ bool MaybeScheduleMemoryPressureEvent()
   return true;
 }
 
-void CheckMemAvailable()
+void
+CheckMemAvailable()
 {
   if (!sHooksActive) {
     return;
@@ -204,8 +207,7 @@ void CheckMemAvailable()
 
   DEBUG_WARN_IF_FALSE(success, "GlobalMemoryStatusEx failed.");
 
-  if (success)
-  {
+  if (success) {
     // sLowVirtualMemoryThreshold is in MB, but ullAvailVirtual is in bytes.
     if (stat.ullAvailVirtual < sLowVirtualMemoryThreshold * 1024 * 1024) {
       // If we're running low on virtual memory, unconditionally schedule the
@@ -214,14 +216,12 @@ void CheckMemAvailable()
       LOG("Detected low virtual memory.");
       ++sNumLowVirtualMemEvents;
       NS_DispatchEventualMemoryPressure(MemPressure_New);
-    }
-    else if (stat.ullAvailPageFile < sLowCommitSpaceThreshold * 1024 * 1024) {
+    } else if (stat.ullAvailPageFile < sLowCommitSpaceThreshold * 1024 * 1024) {
       LOG("Detected low available page file space.");
       if (MaybeScheduleMemoryPressureEvent()) {
         ++sNumLowCommitSpaceEvents;
       }
-    }
-    else if (stat.ullAvailPhys < sLowPhysicalMemoryThreshold * 1024 * 1024) {
+    } else if (stat.ullAvailPhys < sLowPhysicalMemoryThreshold * 1024 * 1024) {
       LOG("Detected low physical memory.");
       if (MaybeScheduleMemoryPressureEvent()) {
         ++sNumLowPhysicalMemEvents;
@@ -278,9 +278,9 @@ MapViewOfFileHook(HANDLE aFileMappingObject,
 
 HBITMAP WINAPI
 CreateDIBSectionHook(HDC aDC,
-                     const BITMAPINFO *aBitmapInfo,
+                     const BITMAPINFO* aBitmapInfo,
                      UINT aUsage,
-                     VOID **aBits,
+                     VOID** aBits,
                      HANDLE aSection,
                      DWORD aOffset)
 {
@@ -304,8 +304,9 @@ CreateDIBSectionHook(HDC aDC,
     // absolute value.
     int64_t size = bitCount * aBitmapInfo->bmiHeader.biWidth *
                               aBitmapInfo->bmiHeader.biHeight;
-    if (size < 0)
+    if (size < 0) {
       size *= -1;
+    }
 
     // If we're allocating more than 1MB, check how much memory is left after
     // the allocation.
@@ -325,63 +326,67 @@ CreateDIBSectionHook(HDC aDC,
   return result;
 }
 
-class LowMemoryEventsVirtualReporter MOZ_FINAL : public MemoryUniReporter
+static int64_t
+LowMemoryEventsVirtualDistinguishedAmount()
 {
+  return sNumLowVirtualMemEvents;
+}
+
+static int64_t
+LowMemoryEventsPhysicalDistinguishedAmount()
+{
+  return sNumLowPhysicalMemEvents;
+}
+
+class LowEventsReporter MOZ_FINAL : public nsIMemoryReporter
+{
+  ~LowEventsReporter() {}
+
 public:
-  LowMemoryEventsVirtualReporter()
-    : MemoryUniReporter("low-memory-events/virtual",
-                         KIND_OTHER, UNITS_COUNT_CUMULATIVE,
+  NS_DECL_ISUPPORTS
+
+  NS_IMETHOD CollectReports(nsIHandleReportCallback* aHandleReport,
+                            nsISupports* aData, bool aAnonymize)
+  {
+    nsresult rv;
+
+    // We only do virtual-memory tracking on 32-bit builds.
+    if (sizeof(void*) == 4) {
+      rv = MOZ_COLLECT_REPORT(
+        "low-memory-events/virtual", KIND_OTHER, UNITS_COUNT_CUMULATIVE,
+        LowMemoryEventsVirtualDistinguishedAmount(),
 "Number of low-virtual-memory events fired since startup. We fire such an "
 "event if we notice there is less than memory.low_virtual_mem_threshold_mb of "
-"virtual address space available (if zero, this behavior is disabled).  The "
+"virtual address space available (if zero, this behavior is disabled). The "
 "process will probably crash if it runs out of virtual address space, so "
-"this event is dire.")
-  {}
+"this event is dire.");
+      NS_ENSURE_SUCCESS(rv, rv);
+    }
 
-private:
-  int64_t Amount() MOZ_OVERRIDE
-  {
-    // This memory reporter shouldn't be installed on 64-bit machines, since we
-    // force-disable virtual-memory tracking there.
-    MOZ_ASSERT(sizeof(void*) == 4);
-
-    return sNumLowVirtualMemEvents;
-  }
-};
-
-class LowCommitSpaceEventsReporter MOZ_FINAL : public MemoryUniReporter
-{
-public:
-  LowCommitSpaceEventsReporter()
-    : MemoryUniReporter("low-commit-space-events",
-                         KIND_OTHER, UNITS_COUNT_CUMULATIVE,
+    rv = MOZ_COLLECT_REPORT(
+      "low-commit-space-events", KIND_OTHER, UNITS_COUNT_CUMULATIVE,
+      sNumLowCommitSpaceEvents,
 "Number of low-commit-space events fired since startup. We fire such an "
 "event if we notice there is less than memory.low_commit_space_threshold_mb of "
-"commit space available (if zero, this behavior is disabled).  Windows will "
+"commit space available (if zero, this behavior is disabled). Windows will "
 "likely kill the process if it runs out of commit space, so this event is "
-"dire.")
-  {}
+"dire.");
+    NS_ENSURE_SUCCESS(rv, rv);
 
-private:
-  int64_t Amount() MOZ_OVERRIDE { return sNumLowCommitSpaceEvents; }
-};
-
-class LowMemoryEventsPhysicalReporter MOZ_FINAL : public MemoryUniReporter
-{
-public:
-  LowMemoryEventsPhysicalReporter()
-    : MemoryUniReporter("low-memory-events/physical",
-                         KIND_OTHER, UNITS_COUNT_CUMULATIVE,
+    rv = MOZ_COLLECT_REPORT(
+      "low-memory-events/physical", KIND_OTHER, UNITS_COUNT_CUMULATIVE,
+      LowMemoryEventsPhysicalDistinguishedAmount(),
 "Number of low-physical-memory events fired since startup. We fire such an "
 "event if we notice there is less than memory.low_physical_memory_threshold_mb "
 "of physical memory available (if zero, this behavior is disabled).  The "
 "machine will start to page if it runs out of physical memory.  This may "
-"cause it to run slowly, but it shouldn't cause it to crash.")
-  {}
+"cause it to run slowly, but it shouldn't cause it to crash.");
+    NS_ENSURE_SUCCESS(rv, rv);
 
-private:
-  int64_t Amount() MOZ_OVERRIDE { return sNumLowPhysicalMemEvents; }
+    return NS_OK;
+  }
 };
+NS_IMPL_ISUPPORTS(LowEventsReporter, nsIMemoryReporter)
 
 #endif // defined(XP_WIN)
 
@@ -393,12 +398,14 @@ private:
  */
 class nsJemallocFreeDirtyPagesRunnable MOZ_FINAL : public nsIRunnable
 {
+  ~nsJemallocFreeDirtyPagesRunnable() {}
+
 public:
   NS_DECL_ISUPPORTS
   NS_DECL_NSIRUNNABLE
 };
 
-NS_IMPL_ISUPPORTS1(nsJemallocFreeDirtyPagesRunnable, nsIRunnable)
+NS_IMPL_ISUPPORTS(nsJemallocFreeDirtyPagesRunnable, nsIRunnable)
 
 NS_IMETHODIMP
 nsJemallocFreeDirtyPagesRunnable::Run()
@@ -419,6 +426,8 @@ nsJemallocFreeDirtyPagesRunnable::Run()
  */
 class nsMemoryPressureWatcher MOZ_FINAL : public nsIObserver
 {
+  ~nsMemoryPressureWatcher() {}
+
 public:
   NS_DECL_ISUPPORTS
   NS_DECL_NSIOBSERVER
@@ -429,7 +438,7 @@ private:
   static bool sFreeDirtyPages;
 };
 
-NS_IMPL_ISUPPORTS1(nsMemoryPressureWatcher, nsIObserver)
+NS_IMPL_ISUPPORTS(nsMemoryPressureWatcher, nsIObserver)
 
 bool nsMemoryPressureWatcher::sFreeDirtyPages = false;
 
@@ -456,10 +465,10 @@ nsMemoryPressureWatcher::Init()
  * free dirty pages held by jemalloc.
  */
 NS_IMETHODIMP
-nsMemoryPressureWatcher::Observe(nsISupports *subject, const char *topic,
-                                 const PRUnichar *data)
+nsMemoryPressureWatcher::Observe(nsISupports* aSubject, const char* aTopic,
+                                 const char16_t* aData)
 {
-  MOZ_ASSERT(!strcmp(topic, "memory-pressure"), "Unknown topic");
+  MOZ_ASSERT(!strcmp(aTopic, "memory-pressure"), "Unknown topic");
 
   if (sFreeDirtyPages) {
     nsRefPtr<nsIRunnable> runnable = new nsJemallocFreeDirtyPagesRunnable();
@@ -475,7 +484,8 @@ nsMemoryPressureWatcher::Observe(nsISupports *subject, const char *topic,
 namespace mozilla {
 namespace AvailableMemoryTracker {
 
-void Activate()
+void
+Activate()
 {
 #if defined(_M_IX86) && defined(XP_WIN)
   MOZ_ASSERT(sInitialized);
@@ -485,24 +495,23 @@ void Activate()
   // we're not going to run out of virtual memory!
   if (sizeof(void*) > 4) {
     sLowVirtualMemoryThreshold = 0;
-  }
-  else {
+  } else {
     Preferences::AddUintVarCache(&sLowVirtualMemoryThreshold,
-        "memory.low_virtual_mem_threshold_mb", 128);
+                                 "memory.low_virtual_mem_threshold_mb", 128);
   }
 
   Preferences::AddUintVarCache(&sLowPhysicalMemoryThreshold,
-      "memory.low_physical_memory_threshold_mb", 0);
+                               "memory.low_physical_memory_threshold_mb", 0);
   Preferences::AddUintVarCache(&sLowCommitSpaceThreshold,
-      "memory.low_commit_space_threshold_mb", 128);
+                               "memory.low_commit_space_threshold_mb", 128);
   Preferences::AddUintVarCache(&sLowMemoryNotificationIntervalMS,
-      "memory.low_memory_notification_interval_ms", 10000);
+                               "memory.low_memory_notification_interval_ms", 10000);
 
-  NS_RegisterMemoryReporter(new LowCommitSpaceEventsReporter());
-  NS_RegisterMemoryReporter(new LowMemoryEventsPhysicalReporter());
-  if (sizeof(void*) == 4) {
-    NS_RegisterMemoryReporter(new LowMemoryEventsVirtualReporter());
-  }
+  RegisterStrongMemoryReporter(new LowEventsReporter());
+  RegisterLowMemoryEventsVirtualDistinguishedAmount(
+    LowMemoryEventsVirtualDistinguishedAmount);
+  RegisterLowMemoryEventsPhysicalDistinguishedAmount(
+    LowMemoryEventsPhysicalDistinguishedAmount);
   sHooksActive = true;
 #endif
 
@@ -511,7 +520,8 @@ void Activate()
   watcher->Init();
 }
 
-void Init()
+void
+Init()
 {
   // Do nothing on x86-64, because nsWindowsDllInterceptor is not thread-safe
   // on 64-bit.  (On 32-bit, it's probably thread-safe.)  Even if we run Init()
@@ -530,16 +540,16 @@ void Init()
   if (!PR_GetEnv("MOZ_PGO_INSTRUMENTED")) {
     sKernel32Intercept.Init("Kernel32.dll");
     sKernel32Intercept.AddHook("VirtualAlloc",
-      reinterpret_cast<intptr_t>(VirtualAllocHook),
-      (void**) &sVirtualAllocOrig);
+                               reinterpret_cast<intptr_t>(VirtualAllocHook),
+                               reinterpret_cast<void**>(&sVirtualAllocOrig));
     sKernel32Intercept.AddHook("MapViewOfFile",
-      reinterpret_cast<intptr_t>(MapViewOfFileHook),
-      (void**) &sMapViewOfFileOrig);
+                               reinterpret_cast<intptr_t>(MapViewOfFileHook),
+                               reinterpret_cast<void**>(&sMapViewOfFileOrig));
 
     sGdi32Intercept.Init("Gdi32.dll");
     sGdi32Intercept.AddHook("CreateDIBSection",
-      reinterpret_cast<intptr_t>(CreateDIBSectionHook),
-      (void**) &sCreateDIBSectionOrig);
+                            reinterpret_cast<intptr_t>(CreateDIBSectionHook),
+                            reinterpret_cast<void**>(&sCreateDIBSectionOrig));
   }
 
   sInitialized = true;
