@@ -13,6 +13,16 @@
 #include "mozilla/Module.h"
 #include "prlink.h"
 
+class nsNativeModuleLoader;
+
+namespace mozilla {
+template<>
+struct HasDangerousPublicDestructor<nsNativeModuleLoader>
+{
+  static const bool value = true;
+};
+}
+
 class nsNativeModuleLoader : public mozilla::ModuleLoader
 {
  public:
@@ -21,7 +31,7 @@ class nsNativeModuleLoader : public mozilla::ModuleLoader
     nsNativeModuleLoader() {}
     ~nsNativeModuleLoader() {}
 
-    NS_OVERRIDE virtual const mozilla::Module* LoadModule(mozilla::FileLocation &aFile);
+    virtual const mozilla::Module* LoadModule(mozilla::FileLocation &aFile) MOZ_OVERRIDE;
 
     nsresult Init();
 
@@ -31,8 +41,8 @@ class nsNativeModuleLoader : public mozilla::ModuleLoader
     struct NativeLoadData
     {
         NativeLoadData()
-            : module(NULL)
-            , library(NULL)
+            : module(nullptr)
+            , library(nullptr)
         { }
 
         const mozilla::Module* module;
