@@ -1,4 +1,5 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -8,15 +9,12 @@
 /*
  * Double hashing, a la Knuth 6.
  */
+#include "mozilla/fallible.h"
 #include "mozilla/MemoryReporting.h"
 #include "mozilla/Types.h"
 #include "nscore.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-#if defined(__GNUC__) && defined(__i386__) && !defined(XP_OS2)
+#if defined(__GNUC__) && defined(__i386__)
 #define PL_DHASH_FASTCALL __attribute__ ((regparm (3),stdcall))
 #elif defined(XP_WIN)
 #define PL_DHASH_FASTCALL __fastcall
@@ -24,13 +22,15 @@ extern "C" {
 #define PL_DHASH_FASTCALL
 #endif
 
-#ifdef DEBUG_XXXbrendan
-#define PL_DHASHMETER 1
-#endif
-
-/* Table size limit, do not equal or exceed (see min&maxAlphaFrac, below). */
-#undef PL_DHASH_SIZE_LIMIT
-#define PL_DHASH_SIZE_LIMIT     ((uint32_t)1 << 24)
+/*
+ * Table size limit; do not exceed.  The max capacity used to be 1<<23 but that
+ * occasionally that wasn't enough.  Making it much bigger than 1<<26 probably
+ * isn't worthwhile -- tables that big are kind of ridiculous.  Also, the
+ * growth operation will (deliberately) fail if |capacity * entrySize|
+ * overflows a uint32_t, and entrySize is always at least 8 bytes.
+ */
+#undef PL_DHASH_MAX_SIZE
+#define PL_DHASH_MAX_SIZE     ((uint32_t)1 << 26)
 
 /* Minimum table size, or gross entry count (net is at most .75 loaded). */
 #ifndef PL_DHASH_MIN_SIZE
@@ -72,7 +72,7 @@ typedef struct PLDHashTableOps  PLDHashTableOps;
  * maintained automatically by PL_DHashTableOperate -- users should never set
  * it, and its only uses should be via the entry macros below.
  *
- * The PL_DHASH_ENTRY_IS_LIVE macro tests whether entry is neither free nor
+ * The PL_DHASH_ENTRY_IS_LIVE function tests whether entry is neither free nor
  * removed.  An entry may be either busy or free; if busy, it may be live or
  * removed.  Consumers of this API should not access members of entries that
  * are not live.
@@ -82,13 +82,28 @@ typedef struct PLDHashTableOps  PLDHashTableOps;
  * non-live, busy (i.e., removed) entry pointer to its caller.  See below for
  * more details on PL_DHashTableOperate's calling rules.
  */
-struct PLDHashEntryHdr {
-    PLDHashNumber       keyHash;        /* every entry must begin like this */
+struct PLDHashEntryHdr
+{
+  PLDHashNumber keyHash;  /* every entry must begin like this */
 };
 
-#define PL_DHASH_ENTRY_IS_FREE(entry)   ((entry)->keyHash == 0)
-#define PL_DHASH_ENTRY_IS_BUSY(entry)   (!PL_DHASH_ENTRY_IS_FREE(entry))
-#define PL_DHASH_ENTRY_IS_LIVE(entry)   ((entry)->keyHash >= 2)
+MOZ_ALWAYS_INLINE bool
+PL_DHASH_ENTRY_IS_FREE(PLDHashEntryHdr* aEntry)
+{
+  return aEntry->keyHash == 0;
+}
+
+MOZ_ALWAYS_INLINE bool
+PL_DHASH_ENTRY_IS_BUSY(PLDHashEntryHdr* aEntry)
+{
+  return !PL_DHASH_ENTRY_IS_FREE(aEntry);
+}
+
+MOZ_ALWAYS_INLINE bool
+PL_DHASH_ENTRY_IS_LIVE(PLDHashEntryHdr* aEntry)
+{
+  return aEntry->keyHash >= 2;
+}
 
 /*
  * A PLDHashTable is currently 8 words (without the PL_DHASHMETER overhead)
@@ -145,7 +160,7 @@ struct PLDHashEntryHdr {
  * assuming esize is not too large (in which case, chaining should probably be
  * used for any alpha).  For esize=2 and k=3, we want alpha >= .2; for esize=3
  * and k=2, we want alpha >= .4.  For k=4, esize could be 6, and alpha >= .5
- * would still obtain.  See the PL_DHASH_MIN_ALPHA macro further below.
+ * would still obtain.
  *
  * The current implementation uses a configurable lower bound on alpha, which
  * defaults to .25, when deciding to shrink the table (while still respecting
@@ -154,45 +169,52 @@ struct PLDHashEntryHdr {
  * Note a qualitative difference between chaining and double hashing: under
  * chaining, entry addresses are stable across table shrinks and grows.  With
  * double hashing, you can't safely hold an entry pointer and use it after an
- * ADD or REMOVE operation, unless you sample table->generation before adding
+ * ADD or REMOVE operation, unless you sample aTable->generation before adding
  * or removing, and compare the sample after, dereferencing the entry pointer
- * only if table->generation has not changed.
+ * only if aTable->generation has not changed.
  *
  * The moral of this story: there is no one-size-fits-all hash table scheme,
  * but for small table entry size, and assuming entry address stability is not
  * required, double hashing wins.
  */
-struct PLDHashTable {
-    const PLDHashTableOps *ops;         /* virtual operations, see below */
-    void                *data;          /* ops- and instance-specific data */
-    int16_t             hashShift;      /* multiplicative hash shift */
-    uint8_t             maxAlphaFrac;   /* 8-bit fixed point max alpha */
-    uint8_t             minAlphaFrac;   /* 8-bit fixed point min alpha */
-    uint32_t            entrySize;      /* number of bytes in an entry */
-    uint32_t            entryCount;     /* number of entries in table */
-    uint32_t            removedCount;   /* removed entry sentinels in table */
-    uint32_t            generation;     /* entry storage generation number */
-    char                *entryStore;    /* entry storage */
+struct PLDHashTable
+{
+  const PLDHashTableOps* ops;         /* virtual operations, see below */
+  void*               data;           /* ops- and instance-specific data */
+  int16_t             hashShift;      /* multiplicative hash shift */
+  /*
+   * |recursionLevel| is only used in debug builds, but is present in opt
+   * builds to avoid binary compatibility problems when mixing DEBUG and
+   * non-DEBUG components.  (Actually, even if it were removed,
+   * sizeof(PLDHashTable) wouldn't change, due to struct padding.)
+   */
+  uint16_t            recursionLevel; /* used to detect unsafe re-entry */
+  uint32_t            entrySize;      /* number of bytes in an entry */
+  uint32_t            entryCount;     /* number of entries in table */
+  uint32_t            removedCount;   /* removed entry sentinels in table */
+  uint32_t            generation;     /* entry storage generation number */
+  char*               entryStore;     /* entry storage */
 #ifdef PL_DHASHMETER
-    struct PLDHashStats {
-        uint32_t        searches;       /* total number of table searches */
-        uint32_t        steps;          /* hash chain links traversed */
-        uint32_t        hits;           /* searches that found key */
-        uint32_t        misses;         /* searches that didn't find key */
-        uint32_t        lookups;        /* number of PL_DHASH_LOOKUPs */
-        uint32_t        addMisses;      /* adds that miss, and do work */
-        uint32_t        addOverRemoved; /* adds that recycled a removed entry */
-        uint32_t        addHits;        /* adds that hit an existing entry */
-        uint32_t        addFailures;    /* out-of-memory during add growth */
-        uint32_t        removeHits;     /* removes that hit, and do work */
-        uint32_t        removeMisses;   /* useless removes that miss */
-        uint32_t        removeFrees;    /* removes that freed entry directly */
-        uint32_t        removeEnums;    /* removes done by Enumerate */
-        uint32_t        grows;          /* table expansions */
-        uint32_t        shrinks;        /* table contractions */
-        uint32_t        compresses;     /* table compressions */
-        uint32_t        enumShrinks;    /* contractions after Enumerate */
-    } stats;
+  struct PLDHashStats
+  {
+    uint32_t        searches;       /* total number of table searches */
+    uint32_t        steps;          /* hash chain links traversed */
+    uint32_t        hits;           /* searches that found key */
+    uint32_t        misses;         /* searches that didn't find key */
+    uint32_t        lookups;        /* number of PL_DHASH_LOOKUPs */
+    uint32_t        addMisses;      /* adds that miss, and do work */
+    uint32_t        addOverRemoved; /* adds that recycled a removed entry */
+    uint32_t        addHits;        /* adds that hit an existing entry */
+    uint32_t        addFailures;    /* out-of-memory during add growth */
+    uint32_t        removeHits;     /* removes that hit, and do work */
+    uint32_t        removeMisses;   /* useless removes that miss */
+    uint32_t        removeFrees;    /* removes that freed entry directly */
+    uint32_t        removeEnums;    /* removes done by Enumerate */
+    uint32_t        grows;          /* table expansions */
+    uint32_t        shrinks;        /* table contractions */
+    uint32_t        compresses;     /* table compressions */
+    uint32_t        enumShrinks;    /* contractions after Enumerate */
+  } stats;
 #endif
 };
 
@@ -206,66 +228,62 @@ struct PLDHashTable {
 
 /*
  * Table space at entryStore is allocated and freed using these callbacks.
- * The allocator should return null on error only (not if called with nbytes
- * equal to 0; but note that pldhash.c code will never call with 0 nbytes).
+ * The allocator should return null on error only (not if called with aNBytes
+ * equal to 0; but note that pldhash.c code will never call with 0 aNBytes).
  */
-typedef void *
-(* PLDHashAllocTable)(PLDHashTable *table, uint32_t nbytes);
+typedef void* (*PLDHashAllocTable)(PLDHashTable* aTable, uint32_t aNBytes);
 
-typedef void
-(* PLDHashFreeTable) (PLDHashTable *table, void *ptr);
+typedef void (*PLDHashFreeTable)(PLDHashTable* aTable, void* aPtr);
 
 /*
  * Compute the hash code for a given key to be looked up, added, or removed
- * from table.  A hash code may have any PLDHashNumber value.
+ * from aTable.  A hash code may have any PLDHashNumber value.
  */
-typedef PLDHashNumber
-(* PLDHashHashKey)   (PLDHashTable *table, const void *key);
+typedef PLDHashNumber (*PLDHashHashKey)(PLDHashTable* aTable,
+                                        const void* aKey);
 
 /*
- * Compare the key identifying entry in table with the provided key parameter.
+ * Compare the key identifying aEntry in aTable with the provided key parameter.
  * Return true if keys match, false otherwise.
  */
-typedef bool
-(* PLDHashMatchEntry)(PLDHashTable *table, const PLDHashEntryHdr *entry,
-                      const void *key);
+typedef bool (*PLDHashMatchEntry)(PLDHashTable* aTable,
+                                  const PLDHashEntryHdr* aEntry,
+                                  const void* aKey);
 
 /*
- * Copy the data starting at from to the new entry storage at to.  Do not add
+ * Copy the data starting at aFrom to the new entry storage at aTo. Do not add
  * reference counts for any strong references in the entry, however, as this
  * is a "move" operation: the old entry storage at from will be freed without
  * any reference-decrementing callback shortly.
  */
-typedef void
-(* PLDHashMoveEntry)(PLDHashTable *table, const PLDHashEntryHdr *from,
-                     PLDHashEntryHdr *to);
+typedef void (*PLDHashMoveEntry)(PLDHashTable* aTable,
+                                 const PLDHashEntryHdr* aFrom,
+                                 PLDHashEntryHdr* aTo);
 
 /*
  * Clear the entry and drop any strong references it holds.  This callback is
  * invoked during a PL_DHASH_REMOVE operation (see below for operation codes),
  * but only if the given key is found in the table.
  */
-typedef void
-(* PLDHashClearEntry)(PLDHashTable *table, PLDHashEntryHdr *entry);
+typedef void (*PLDHashClearEntry)(PLDHashTable* aTable,
+                                  PLDHashEntryHdr* aEntry);
 
 /*
  * Called when a table (whether allocated dynamically by itself, or nested in
  * a larger structure, or allocated on the stack) is finished.  This callback
- * allows table->ops-specific code to finalize table->data.
+ * allows aTable->ops-specific code to finalize aTable->data.
  */
-typedef void
-(* PLDHashFinalize)  (PLDHashTable *table);
+typedef void (*PLDHashFinalize)(PLDHashTable* aTable);
 
 /*
  * Initialize a new entry, apart from keyHash.  This function is called when
  * PL_DHashTableOperate's PL_DHASH_ADD case finds no existing entry for the
- * given key, and must add a new one.  At that point, entry->keyHash is not
+ * given key, and must add a new one.  At that point, aEntry->keyHash is not
  * set yet, to avoid claiming the last free entry in a severely overloaded
  * table.
  */
-typedef bool
-(* PLDHashInitEntry)(PLDHashTable *table, PLDHashEntryHdr *entry,
-                     const void *key);
+typedef bool (*PLDHashInitEntry)(PLDHashTable* aTable, PLDHashEntryHdr* aEntry,
+                                 const void* aKey);
 
 /*
  * Finally, the "vtable" structure for PLDHashTable.  The first eight hooks
@@ -280,7 +298,7 @@ typedef bool
  *  moveEntry           Call placement new using copy ctor, run dtor on old
  *                      entry storage.
  *  clearEntry          Run dtor on entry.
- *  finalize            Stub unless table->data was initialized and needs to
+ *  finalize            Stub unless aTable->data was initialized and needs to
  *                      be finalized.
  *
  * Note the reason why initEntry is optional: the default hooks (stubs) clear
@@ -294,154 +312,112 @@ typedef bool
  *
  * XXX assumes 0 is null for pointer types.
  */
-struct PLDHashTableOps {
-    /* Mandatory hooks.  All implementations must provide these. */
-    PLDHashAllocTable   allocTable;
-    PLDHashFreeTable    freeTable;
-    PLDHashHashKey      hashKey;
-    PLDHashMatchEntry   matchEntry;
-    PLDHashMoveEntry    moveEntry;
-    PLDHashClearEntry   clearEntry;
-    PLDHashFinalize     finalize;
+struct PLDHashTableOps
+{
+  /* Mandatory hooks.  All implementations must provide these. */
+  PLDHashAllocTable   allocTable;
+  PLDHashFreeTable    freeTable;
+  PLDHashHashKey      hashKey;
+  PLDHashMatchEntry   matchEntry;
+  PLDHashMoveEntry    moveEntry;
+  PLDHashClearEntry   clearEntry;
+  PLDHashFinalize     finalize;
 
-    /* Optional hooks start here.  If null, these are not called. */
-    PLDHashInitEntry    initEntry;
+  /* Optional hooks start here.  If null, these are not called. */
+  PLDHashInitEntry    initEntry;
 };
 
 /*
  * Default implementations for the above ops.
  */
-NS_COM_GLUE void *
-PL_DHashAllocTable(PLDHashTable *table, uint32_t nbytes);
+NS_COM_GLUE void* PL_DHashAllocTable(PLDHashTable* aTable, uint32_t aNBytes);
 
-NS_COM_GLUE void
-PL_DHashFreeTable(PLDHashTable *table, void *ptr);
+NS_COM_GLUE void PL_DHashFreeTable(PLDHashTable* aTable, void* aPtr);
 
-NS_COM_GLUE PLDHashNumber
-PL_DHashStringKey(PLDHashTable *table, const void *key);
+NS_COM_GLUE PLDHashNumber PL_DHashStringKey(PLDHashTable* aTable,
+                                            const void* aKey);
 
 /* A minimal entry contains a keyHash header and a void key pointer. */
-struct PLDHashEntryStub {
-    PLDHashEntryHdr hdr;
-    const void      *key;
+struct PLDHashEntryStub
+{
+  PLDHashEntryHdr hdr;
+  const void*     key;
 };
 
-NS_COM_GLUE PLDHashNumber
-PL_DHashVoidPtrKeyStub(PLDHashTable *table, const void *key);
+NS_COM_GLUE PLDHashNumber PL_DHashVoidPtrKeyStub(PLDHashTable* aTable,
+                                                 const void* aKey);
 
-NS_COM_GLUE bool
-PL_DHashMatchEntryStub(PLDHashTable *table,
-                       const PLDHashEntryHdr *entry,
-                       const void *key);
+NS_COM_GLUE bool PL_DHashMatchEntryStub(PLDHashTable* aTable,
+                                        const PLDHashEntryHdr* aEntry,
+                                        const void* aKey);
 
-NS_COM_GLUE bool
-PL_DHashMatchStringKey(PLDHashTable *table,
-                       const PLDHashEntryHdr *entry,
-                       const void *key);
+NS_COM_GLUE bool PL_DHashMatchStringKey(PLDHashTable* aTable,
+                                        const PLDHashEntryHdr* aEntry,
+                                        const void* aKey);
 
 NS_COM_GLUE void
-PL_DHashMoveEntryStub(PLDHashTable *table,
-                      const PLDHashEntryHdr *from,
-                      PLDHashEntryHdr *to);
+PL_DHashMoveEntryStub(PLDHashTable* aTable,
+                      const PLDHashEntryHdr* aFrom,
+                      PLDHashEntryHdr* aTo);
 
-NS_COM_GLUE void
-PL_DHashClearEntryStub(PLDHashTable *table, PLDHashEntryHdr *entry);
+NS_COM_GLUE void PL_DHashClearEntryStub(PLDHashTable* aTable,
+                                        PLDHashEntryHdr* aEntry);
 
-NS_COM_GLUE void
-PL_DHashFreeStringKey(PLDHashTable *table, PLDHashEntryHdr *entry);
+NS_COM_GLUE void PL_DHashFreeStringKey(PLDHashTable* aTable,
+                                       PLDHashEntryHdr* aEntry);
 
-NS_COM_GLUE void
-PL_DHashFinalizeStub(PLDHashTable *table);
+NS_COM_GLUE void PL_DHashFinalizeStub(PLDHashTable* aTable);
 
 /*
  * If you use PLDHashEntryStub or a subclass of it as your entry struct, and
  * if your entries move via memcpy and clear via memset(0), you can use these
  * stub operations.
  */
-NS_COM_GLUE const PLDHashTableOps *
-PL_DHashGetStubOps(void);
+NS_COM_GLUE const PLDHashTableOps* PL_DHashGetStubOps(void);
 
 /*
  * Dynamically allocate a new PLDHashTable using malloc, initialize it using
  * PL_DHashTableInit, and return its address.  Return null on malloc failure.
- * Note that the entry storage at table->entryStore will be allocated using
- * the ops->allocTable callback.
+ * Note that the entry storage at aTable->entryStore will be allocated using
+ * the aOps->allocTable callback.
  */
-NS_COM_GLUE PLDHashTable *
-PL_NewDHashTable(const PLDHashTableOps *ops, void *data, uint32_t entrySize,
-                 uint32_t capacity);
+NS_COM_GLUE PLDHashTable* PL_NewDHashTable(const PLDHashTableOps* aOps,
+                                           void* aData, uint32_t aEntrySize,
+                                           uint32_t aCapacity);
 
 /*
- * Finalize table's data, free its entry storage (via table->ops->freeTable),
- * and return the memory starting at table to the malloc heap.
+ * Finalize aTable's data, free its entry storage (via aTable->ops->freeTable),
+ * and return the memory starting at aTable to the malloc heap.
  */
-NS_COM_GLUE void
-PL_DHashTableDestroy(PLDHashTable *table);
+NS_COM_GLUE void PL_DHashTableDestroy(PLDHashTable* aTable);
 
 /*
- * Initialize table with ops, data, entrySize, and capacity.  Capacity is a
- * guess for the smallest table size at which the table will usually be less
+ * Initialize aTable with aOps, aData, aEntrySize, and aCapacity. Capacity is
+ * a guess for the smallest table size at which the table will usually be less
  * than 75% loaded (the table will grow or shrink as needed; capacity serves
- * only to avoid inevitable early growth from PL_DHASH_MIN_SIZE).
+ * only to avoid inevitable early growth from PL_DHASH_MIN_SIZE).  This will
+ * crash if it can't allocate enough memory, or if entrySize or capacity are
+ * too large.
  */
-NS_COM_GLUE bool
-PL_DHashTableInit(PLDHashTable *table, const PLDHashTableOps *ops, void *data,
-                  uint32_t entrySize, uint32_t capacity);
+NS_COM_GLUE void PL_DHashTableInit(PLDHashTable* aTable,
+                                   const PLDHashTableOps* aOps, void* aData,
+                                   uint32_t aEntrySize, uint32_t aCapacity);
 
 /*
- * Set maximum and minimum alpha for table.  The defaults are 0.75 and .25.
- * maxAlpha must be in [0.5, 0.9375] for the default PL_DHASH_MIN_SIZE; or if
- * MinSize=PL_DHASH_MIN_SIZE <= 256, in [0.5, (float)(MinSize-1)/MinSize]; or
- * else in [0.5, 255.0/256].  minAlpha must be in [0, maxAlpha / 2), so that
- * we don't shrink on the very next remove after growing a table upon adding
- * an entry that brings entryCount past maxAlpha * tableSize.
+ * Initialize aTable. This is the same as PL_DHashTableInit, except that it
+ * returns a boolean indicating success, rather than crashing on failure.
  */
-NS_COM_GLUE void
-PL_DHashTableSetAlphaBounds(PLDHashTable *table,
-                            float maxAlpha,
-                            float minAlpha);
+MOZ_WARN_UNUSED_RESULT NS_COM_GLUE bool PL_DHashTableInit(
+  PLDHashTable* aTable, const PLDHashTableOps* aOps, void* aData,
+  uint32_t aEntrySize, uint32_t aCapacity, const mozilla::fallible_t&);
 
 /*
- * Call this macro with k, the number of pointer-sized words wasted per entry
- * under chaining, to compute the minimum alpha at which double hashing still
- * beats chaining.
- */
-#define PL_DHASH_MIN_ALPHA(table, k)                                          \
-    ((float)((table)->entrySize / sizeof(void *) - 1)                         \
-     / ((table)->entrySize / sizeof(void *) + (k)))
-
-/*
- * Default max/min alpha, and macros to compute the value for the |capacity|
- * parameter to PL_NewDHashTable and PL_DHashTableInit, given default or any
- * max alpha, such that adding entryCount entries right after initializing the
- * table will not require a reallocation (so PL_DHASH_ADD can't fail for those
- * PL_DHashTableOperate calls).
- *
- * NB: PL_DHASH_CAP is a helper macro meant for use only in PL_DHASH_CAPACITY.
- * Don't use it directly!
- */
-#define PL_DHASH_DEFAULT_MAX_ALPHA 0.75
-#define PL_DHASH_DEFAULT_MIN_ALPHA 0.25
-
-#define PL_DHASH_CAP(entryCount, maxAlpha)                                    \
-    ((uint32_t)((double)(entryCount) / (maxAlpha)))
-
-#define PL_DHASH_CAPACITY(entryCount, maxAlpha)                               \
-    (PL_DHASH_CAP(entryCount, maxAlpha) +                                     \
-     (((PL_DHASH_CAP(entryCount, maxAlpha) * (uint8_t)(0x100 * (maxAlpha)))     \
-       >> 8) < (entryCount)))
-
-#define PL_DHASH_DEFAULT_CAPACITY(entryCount)                                 \
-    PL_DHASH_CAPACITY(entryCount, PL_DHASH_DEFAULT_MAX_ALPHA)
-
-/*
- * Finalize table's data, free its entry storage using table->ops->freeTable,
+ * Finalize aTable's data, free its entry storage using aTable->ops->freeTable,
  * and leave its members unchanged from their last live values (which leaves
- * pointers dangling).  If you want to burn cycles clearing table, it's up to
+ * pointers dangling).  If you want to burn cycles clearing aTable, it's up to
  * your code to call memset.
  */
-NS_COM_GLUE void
-PL_DHashTableFinish(PLDHashTable *table);
+NS_COM_GLUE void PL_DHashTableFinish(PLDHashTable* aTable);
 
 /*
  * To consolidate keyHash computation and table grow/shrink code, we use a
@@ -449,12 +425,13 @@ PL_DHashTableFinish(PLDHashTable *table);
  * codes are declared here, along with codes returned by PLDHashEnumerator
  * functions, which control PL_DHashTableEnumerate's behavior.
  */
-typedef enum PLDHashOperator {
-    PL_DHASH_LOOKUP = 0,        /* lookup entry */
-    PL_DHASH_ADD = 1,           /* add entry */
-    PL_DHASH_REMOVE = 2,        /* remove entry, or enumerator says remove */
-    PL_DHASH_NEXT = 0,          /* enumerator says continue */
-    PL_DHASH_STOP = 1           /* enumerator says stop */
+typedef enum PLDHashOperator
+{
+  PL_DHASH_LOOKUP = 0,        /* lookup entry */
+  PL_DHASH_ADD = 1,           /* add entry */
+  PL_DHASH_REMOVE = 2,        /* remove entry, or enumerator says remove */
+  PL_DHASH_NEXT = 0,          /* enumerator says continue */
+  PL_DHASH_STOP = 1           /* enumerator says stop */
 } PLDHashOperator;
 
 /*
@@ -470,11 +447,11 @@ typedef enum PLDHashOperator {
  *  entry = PL_DHashTableOperate(table, key, PL_DHASH_ADD);
  *
  * If entry is null upon return, then either the table is severely overloaded,
- * and memory can't be allocated for entry storage via table->ops->allocTable;
- * Or if table->ops->initEntry is non-null, the table->ops->initEntry op may
+ * and memory can't be allocated for entry storage via aTable->ops->allocTable;
+ * Or if aTable->ops->initEntry is non-null, the aTable->ops->initEntry op may
  * have returned false.
  *
- * Otherwise, entry->keyHash has been set so that PL_DHASH_ENTRY_IS_BUSY(entry)
+ * Otherwise, aEntry->keyHash has been set so that PL_DHASH_ENTRY_IS_BUSY(entry)
  * is true, and it is up to the caller to initialize the key and value parts
  * of the entry sub-type, if they have not been set already (i.e. if entry was
  * not already in the table, and if the optional initEntry hook was not used).
@@ -487,8 +464,9 @@ typedef enum PLDHashOperator {
  * the entry is marked so that PL_DHASH_ENTRY_IS_FREE(entry).  This operation
  * returns null unconditionally; you should ignore its return value.
  */
-NS_COM_GLUE PLDHashEntryHdr * PL_DHASH_FASTCALL
-PL_DHashTableOperate(PLDHashTable *table, const void *key, PLDHashOperator op);
+NS_COM_GLUE PLDHashEntryHdr* PL_DHASH_FASTCALL
+PL_DHashTableOperate(PLDHashTable* aTable, const void* aKey,
+                     PLDHashOperator aOp);
 
 /*
  * Remove an entry already accessed via LOOKUP or ADD.
@@ -499,8 +477,8 @@ PL_DHashTableOperate(PLDHashTable *table, const void *key, PLDHashOperator op);
  * shrink the table if it is underloaded.  It does not update stats #ifdef
  * PL_DHASHMETER, either.
  */
-NS_COM_GLUE void
-PL_DHashTableRawRemove(PLDHashTable *table, PLDHashEntryHdr *entry);
+NS_COM_GLUE void PL_DHashTableRawRemove(PLDHashTable* aTable,
+                                        PLDHashEntryHdr* aEntry);
 
 /*
  * Enumerate entries in table using etor:
@@ -512,11 +490,11 @@ PL_DHashTableRawRemove(PLDHashTable *table, PLDHashEntryHdr *entry);
  *   op = etor(table, entry, number, arg);
  *
  * where number is a zero-based ordinal assigned to live entries according to
- * their order in table->entryStore.
+ * their order in aTable->entryStore.
  *
  * The return value, op, is treated as a set of flags.  If op is PL_DHASH_NEXT,
  * then continue enumerating.  If op contains PL_DHASH_REMOVE, then clear (via
- * table->ops->clearEntry) and free entry.  Then we check whether op contains
+ * aTable->ops->clearEntry) and free entry.  Then we check whether op contains
  * PL_DHASH_STOP; if so, stop enumerating and return the number of live entries
  * that were enumerated so far.  Return the total number of live entries when
  * enumeration completes normally.
@@ -524,7 +502,7 @@ PL_DHashTableRawRemove(PLDHashTable *table, PLDHashEntryHdr *entry);
  * If etor calls PL_DHashTableOperate on table with op != PL_DHASH_LOOKUP, it
  * must return PL_DHASH_STOP; otherwise undefined behavior results.
  *
- * If any enumerator returns PL_DHASH_REMOVE, table->entryStore may be shrunk
+ * If any enumerator returns PL_DHASH_REMOVE, aTable->entryStore may be shrunk
  * or compressed after enumeration, but before PL_DHashTableEnumerate returns.
  * Such an enumerator therefore can't safely set aside entry pointers, but an
  * enumerator that never returns PL_DHASH_REMOVE can set pointers to entries
@@ -541,38 +519,35 @@ PL_DHashTableRawRemove(PLDHashTable *table, PLDHashEntryHdr *entry);
  * to be shrunk or compressed, you can call PL_DHashTableRawRemove safely on
  * the entry being enumerated, rather than returning PL_DHASH_REMOVE.
  */
-typedef PLDHashOperator
-(* PLDHashEnumerator)(PLDHashTable *table, PLDHashEntryHdr *hdr, uint32_t number,
-                      void *arg);
+typedef PLDHashOperator (*PLDHashEnumerator)(PLDHashTable* aTable,
+                                             PLDHashEntryHdr* aHdr,
+                                             uint32_t aNumber, void* aArg);
 
 NS_COM_GLUE uint32_t
-PL_DHashTableEnumerate(PLDHashTable *table, PLDHashEnumerator etor, void *arg);
+PL_DHashTableEnumerate(PLDHashTable* aTable, PLDHashEnumerator aEtor,
+                       void* aArg);
 
-typedef size_t
-(* PLDHashSizeOfEntryExcludingThisFun)(PLDHashEntryHdr *hdr,
-                                       mozilla::MallocSizeOf mallocSizeOf,
-                                       void *arg);
+typedef size_t (*PLDHashSizeOfEntryExcludingThisFun)(
+  PLDHashEntryHdr* aHdr, mozilla::MallocSizeOf aMallocSizeOf, void* aArg);
 
 /**
  * Measure the size of the table's entry storage, and if
- * |sizeOfEntryExcludingThis| is non-NULL, measure the size of things pointed
- * to by entries.  Doesn't measure |ops| because it's often shared between
- * tables, nor |data| because it's opaque.
+ * |aSizeOfEntryExcludingThis| is non-nullptr, measure the size of things
+ * pointed to by entries.  Doesn't measure |ops| because it's often shared
+ * between tables, nor |data| because it's opaque.
  */
-NS_COM_GLUE size_t
-PL_DHashTableSizeOfExcludingThis(const PLDHashTable *table,
-                                 PLDHashSizeOfEntryExcludingThisFun sizeOfEntryExcludingThis,
-                                 mozilla::MallocSizeOf mallocSizeOf,
-                                 void *arg = NULL);
+NS_COM_GLUE size_t PL_DHashTableSizeOfExcludingThis(
+  const PLDHashTable* aTable,
+  PLDHashSizeOfEntryExcludingThisFun aSizeOfEntryExcludingThis,
+  mozilla::MallocSizeOf aMallocSizeOf, void* aArg = nullptr);
 
 /**
  * Like PL_DHashTableSizeOfExcludingThis, but includes sizeof(*this).
  */
-NS_COM_GLUE size_t
-PL_DHashTableSizeOfIncludingThis(const PLDHashTable *table,
-                                 PLDHashSizeOfEntryExcludingThisFun sizeOfEntryExcludingThis,
-                                 mozilla::MallocSizeOf mallocSizeOf,
-                                 void *arg = NULL);
+NS_COM_GLUE size_t PL_DHashTableSizeOfIncludingThis(
+  const PLDHashTable* aTable,
+  PLDHashSizeOfEntryExcludingThisFun aSizeOfEntryExcludingThis,
+  mozilla::MallocSizeOf aMallocSizeOf, void* aArg = nullptr);
 
 #ifdef DEBUG
 /**
@@ -589,19 +564,14 @@ PL_DHashTableSizeOfIncludingThis(const PLDHashTable *table,
  * no longer trigger erroneously due to multi-threaded access.  Instead,
  * mutations will cause assertions.
  */
-NS_COM_GLUE void
-PL_DHashMarkTableImmutable(PLDHashTable *table);
+NS_COM_GLUE void PL_DHashMarkTableImmutable(PLDHashTable* aTable);
 #endif
 
 #ifdef PL_DHASHMETER
 #include <stdio.h>
 
-NS_COM_GLUE void
-PL_DHashTableDumpMeter(PLDHashTable *table, PLDHashEnumerator dump, FILE *fp);
-#endif
-
-#ifdef __cplusplus
-}
+NS_COM_GLUE void PL_DHashTableDumpMeter(PLDHashTable* aTable,
+                                        PLDHashEnumerator aDump, FILE* aFp);
 #endif
 
 #endif /* pldhash_h___ */
