@@ -1,5 +1,5 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* vim:set ts=4 sw=4 et cindent: */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -25,14 +25,12 @@ static bool do_preload = false;
 
 #if defined(XP_WIN)
 #define READ_TEXTMODE L"rt"
-#elif defined(XP_OS2)
-#define READ_TEXTMODE "rt"
 #else
 #define READ_TEXTMODE "r"
 #endif
 
 #if defined(SUNOS4) || defined(NEXTSTEP) || \
-    defined(XP_DARWIN) || defined(XP_OS2) || \
+    defined(XP_DARWIN) || \
     (defined(OPENBSD) || defined(NETBSD)) && !defined(__ELF__)
 #define LEADING_UNDERSCORE "_"
 #else
@@ -87,38 +85,6 @@ CloseLibHandle(LibHandleType aLibHandle)
     FreeLibrary(aLibHandle);
 }
 
-#elif defined(XP_OS2)
-#define INCL_DOS
-#define INCL_DOSERRORS
-#include <os2.h>
-
-typedef HMODULE LibHandleType;
-
-static LibHandleType
-GetLibHandle(pathstr_t aDependentLib)
-{
-    CHAR pszError[_MAX_PATH];
-    ULONG ulrc = NO_ERROR;
-    LibHandleType libHandle;
-    ulrc = DosLoadModule(pszError, _MAX_PATH, aDependentLib, &libHandle);
-    return (ulrc == NO_ERROR) ? libHandle : nullptr;
-}
-
-static NSFuncPtr
-GetSymbol(LibHandleType aLibHandle, const char *aSymbol)
-{
-    ULONG ulrc = NO_ERROR;
-    GetFrozenFunctionsFunc sym;
-    ulrc = DosQueryProcAddr(aLibHandle, 0, aSymbol, (PFN*)&sym);
-    return (ulrc == NO_ERROR) ? sym : nullptr;
-}
-
-static void
-CloseLibHandle(LibHandleType aLibHandle)
-{
-    DosFreeModule(aLibHandle);
-}
-
 #elif defined(XP_MACOSX)
 #include <mach-o/dyld.h>
 
@@ -348,8 +314,7 @@ typedef Scoped<ScopedCloseFileTraits> ScopedCloseFile;
 static void
 XPCOMGlueUnload()
 {
-#if !defined(XP_WIN) && !defined(XP_OS2) && !defined(XP_MACOSX) \
-  && defined(NS_TRACE_MALLOC)
+#if !defined(XP_WIN) && !defined(XP_MACOSX) && defined(NS_TRACE_MALLOC)
     if (sTop) {
         _malloc = __libc_malloc;
         _calloc = __libc_calloc;
@@ -370,7 +335,7 @@ XPCOMGlueUnload()
     }
 }
 
-#if defined(XP_WIN) || defined(XP_OS2)
+#if defined(XP_WIN)
 // like strpbrk but finds the *last* char, not the first
 static const char*
 ns_strrpbrk(const char *string, const char *strCharSet)
@@ -394,7 +359,7 @@ static GetFrozenFunctionsFunc
 XPCOMGlueLoad(const char *xpcomFile)
 {
     char xpcomDir[MAXPATHLEN];
-#if defined(XP_WIN) || defined(XP_OS2)
+#if defined(XP_WIN)
     const char *lastSlash = ns_strrpbrk(xpcomFile, "/\\");
 #else
     const char *lastSlash = strrchr(xpcomFile, '/');
@@ -416,6 +381,10 @@ XPCOMGlueLoad(const char *xpcomFile)
         cursor = xpcomDir;
     }
 
+    if (getenv("MOZ_RUN_GTEST")) {
+        strcat(xpcomDir, ".gtest");
+    }
+
     ScopedCloseFile flist;
     flist = TS_tfopen(xpcomDir, READ_TEXTMODE);
     if (!flist) {
@@ -457,8 +426,7 @@ XPCOMGlueLoad(const char *xpcomFile)
         return nullptr;
     }
 
-#if !defined(XP_WIN) && !defined(XP_OS2) && !defined(XP_MACOSX) \
-  && defined(NS_TRACE_MALLOC)
+#if !defined(XP_WIN) && !defined(XP_MACOSX) && defined(NS_TRACE_MALLOC)
     _malloc = (__ptr_t(*)(size_t)) GetSymbol(sTop->libHandle, "malloc");
     _calloc = (__ptr_t(*)(size_t, size_t)) GetSymbol(sTop->libHandle, "calloc");
     _realloc = (__ptr_t(*)(__ptr_t, size_t)) GetSymbol(sTop->libHandle, "realloc");
@@ -592,15 +560,6 @@ NS_GetDebug(nsIDebug* *result)
 }
 
 
-XPCOM_API(nsresult)
-NS_GetTraceRefcnt(nsITraceRefcnt* *result)
-{
-    if (!xpcomFunctions.getTraceRefcnt)
-        return NS_ERROR_NOT_INITIALIZED;
-    return xpcomFunctions.getTraceRefcnt(result);
-}
-
-
 XPCOM_API(nsresult)
 NS_StringContainerInit(nsStringContainer &aStr)
 {
@@ -611,7 +570,7 @@ NS_StringContainerInit(nsStringContainer &aStr)
 
 XPCOM_API(nsresult)
 NS_StringContainerInit2(nsStringContainer &aStr,
-                        const PRUnichar   *aData,
+                        const char16_t   *aData,
                         uint32_t           aDataLength,
                         uint32_t           aFlags)
 {
@@ -628,7 +587,7 @@ NS_StringContainerFinish(nsStringContainer &aStr)
 }
 
 XPCOM_API(uint32_t)
-NS_StringGetData(const nsAString &aStr, const PRUnichar **aBuf, bool *aTerm)
+NS_StringGetData(const nsAString &aStr, const char16_t **aBuf, bool *aTerm)
 {
     if (!xpcomFunctions.stringGetData) {
         *aBuf = nullptr;
@@ -638,7 +597,7 @@ NS_StringGetData(const nsAString &aStr, const PRUnichar **aBuf, bool *aTerm)
 }
 
 XPCOM_API(uint32_t)
-NS_StringGetMutableData(nsAString &aStr, uint32_t aLen, PRUnichar **aBuf)
+NS_StringGetMutableData(nsAString &aStr, uint32_t aLen, char16_t **aBuf)
 {
     if (!xpcomFunctions.stringGetMutableData) {
         *aBuf = nullptr;
@@ -647,7 +606,7 @@ NS_StringGetMutableData(nsAString &aStr, uint32_t aLen, PRUnichar **aBuf)
     return xpcomFunctions.stringGetMutableData(aStr, aLen, aBuf);
 }
 
-XPCOM_API(PRUnichar*)
+XPCOM_API(char16_t*)
 NS_StringCloneData(const nsAString &aStr)
 {
     if (!xpcomFunctions.stringCloneData)
@@ -656,7 +615,7 @@ NS_StringCloneData(const nsAString &aStr)
 }
 
 XPCOM_API(nsresult)
-NS_StringSetData(nsAString &aStr, const PRUnichar *aBuf, uint32_t aCount)
+NS_StringSetData(nsAString &aStr, const char16_t *aBuf, uint32_t aCount)
 {
     if (!xpcomFunctions.stringSetData)
         return NS_ERROR_NOT_INITIALIZED;
@@ -666,7 +625,7 @@ NS_StringSetData(nsAString &aStr, const PRUnichar *aBuf, uint32_t aCount)
 
 XPCOM_API(nsresult)
 NS_StringSetDataRange(nsAString &aStr, uint32_t aCutStart, uint32_t aCutLength,
-                      const PRUnichar *aBuf, uint32_t aCount)
+                      const char16_t *aBuf, uint32_t aCount)
 {
     if (!xpcomFunctions.stringSetDataRange)
         return NS_ERROR_NOT_INITIALIZED;
@@ -950,6 +909,15 @@ NS_CycleCollectorSuspect2(void* obj, nsCycleCollectionParticipant *p)
     return xpcomFunctions.cycleSuspect2Func(obj, p);
 }
 
+XPCOM_API(void)
+NS_CycleCollectorSuspect3(void* obj, nsCycleCollectionParticipant *p,
+                          nsCycleCollectingAutoRefCnt* aRefCnt,
+                          bool* aShouldDelete)
+{
+    if (xpcomFunctions.cycleSuspect3Func)
+        xpcomFunctions.cycleSuspect3Func(obj, p, aRefCnt, aShouldDelete);
+}
+
 XPCOM_API(bool)
 NS_CycleCollectorForget2(nsPurpleBufferEntry* e)
 {
